@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: yasm.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: yasm.c 1760 2007-02-08 05:02:37Z peter $");
 
 #include <ctype.h>
 #include <libyasm/compat-queue.h>
@@ -40,8 +40,6 @@
 
 #include "license.c"
 
-#define DEFAULT_OBJFMT_MODULE	"bin"
-
 /* Preprocess-only buffer size */
 #define PREPROC_BUF_SIZE    16384
 
@@ -61,8 +59,6 @@ static int special_options = 0;
 /*@null@*/ /*@dependent@*/ static yasm_objfmt *cur_objfmt = NULL;
 /*@null@*/ /*@dependent@*/ static const yasm_objfmt_module *
     cur_objfmt_module = NULL;
-/*@null@*/ /*@dependent@*/ static const yasm_optimizer_module *
-    cur_optimizer_module = NULL;
 /*@null@*/ /*@dependent@*/ static yasm_dbgfmt *cur_dbgfmt = NULL;
 /*@null@*/ /*@dependent@*/ static const yasm_dbgfmt_module *
     cur_dbgfmt_module = NULL;
@@ -70,6 +66,7 @@ static int special_options = 0;
 /*@null@*/ /*@dependent@*/ static const yasm_listfmt_module *
     cur_listfmt_module = NULL;
 static int preproc_only = 0;
+static unsigned int force_strict = 0;
 static int generate_make_dependencies = 0;
 static int warning_error = 0;	/* warnings being treated as errors */
 static enum {
@@ -79,7 +76,9 @@ static enum {
 
 /*@null@*/ /*@dependent@*/ static FILE *open_file(const char *filename,
 						  const char *mode);
-static void cleanup(/*@null@*/ yasm_object *object);
+static void check_errors(/*@only@*/ yasm_errwarns *errwarns,
+			 /*@only@*/ yasm_object *object);
+static void cleanup(/*@null@*/ /*@only@*/ yasm_object *object);
 
 /* Forward declarations: cmd line parser handlers */
 static int opt_special_handler(char *cmd, /*@null@*/ char *param, int extra);
@@ -92,8 +91,10 @@ static int opt_listfmt_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_listfile_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_objfile_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_machine_handler(char *cmd, /*@null@*/ char *param, int extra);
+static int opt_strict_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_warning_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int preproc_only_handler(char *cmd, /*@null@*/ char *param, int extra);
+static int opt_include_option(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_preproc_option(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_ewmsg_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_makedep_handler(char *cmd, /*@null@*/ char *param, int extra);
@@ -108,7 +109,9 @@ static /*@exits@*/ void handle_yasm_int_error(const char *file,
 static /*@exits@*/ void handle_yasm_fatal(const char *message, va_list va);
 static const char *handle_yasm_gettext(const char *msgid);
 static void print_yasm_error(const char *filename, unsigned long line,
-			     const char *msg);
+			     const char *msg, /*@null@*/ const char *xref_fn,
+			     unsigned long xref_line,
+			     /*@null@*/ const char *xref_msg);
 static void print_yasm_warning(const char *filename, unsigned long line,
 			       const char *msg);
 
@@ -149,6 +152,8 @@ static opt_option options[] =
       N_("name of object-file output"), N_("filename") },
     { 'm', "machine", 1, opt_machine_handler, 0,
       N_("select machine (list with -m help)"), N_("machine") },
+    { 0, "force-strict", 0, opt_strict_handler, 0,
+      N_("treat all sized operands as if `strict' was used"), NULL },
     { 'w', NULL, 0, opt_warning_handler, 1,
       N_("inhibits warning messages"), NULL },
     { 'W', NULL, 0, opt_warning_handler, 0,
@@ -157,15 +162,15 @@ static opt_option options[] =
       N_("generate Makefile dependencies on stdout"), NULL },
     { 'e', "preproc-only", 0, preproc_only_handler, 0,
       N_("preprocess only (writes output to stdout by default)"), NULL },
-    { 'i', NULL, 1, opt_preproc_option, 0,
+    { 'i', NULL, 1, opt_include_option, 0,
       N_("add include path"), N_("path") },
-    { 'I', NULL, 1, opt_preproc_option, 0,
+    { 'I', NULL, 1, opt_include_option, 0,
       N_("add include path"), N_("path") },
-    { 'P', NULL, 1, opt_preproc_option, 1,
+    { 'P', NULL, 1, opt_preproc_option, 0,
       N_("pre-include file"), N_("filename") },
-    { 'D', NULL, 1, opt_preproc_option, 2,
+    { 'D', NULL, 1, opt_preproc_option, 1,
       N_("pre-define a macro, optionally to value"), N_("macro[=value]") },
-    { 'U', NULL, 1, opt_preproc_option, 3,
+    { 'U', NULL, 1, opt_preproc_option, 2,
       N_("undefine a macro"), N_("macro") },
     { 'X', NULL, 1, opt_ewmsg_handler, 0,
       N_("select error/warning message style (`gnu' or `vc')"), N_("style") },
@@ -175,15 +180,15 @@ static opt_option options[] =
 /*@observer@*/ static const char *version_msg[] = {
     PACKAGE_NAME " " PACKAGE_INTVER "." PACKAGE_BUILD,
     "Compiled on " __DATE__ ".",
-    "Copyright (c) 2001-2006 Peter Johnson and other Yasm developers.",
+    "Copyright (c) 2001-2007 Peter Johnson and other Yasm developers.",
     "Run yasm --license for licensing overview and summary."
 };
 
 /* help messages */
-/*@observer@*/ static const char help_head[] = N_(
+/*@observer@*/ static const char *help_head = N_(
     "usage: yasm [option]* file\n"
     "Options:\n");
-/*@observer@*/ static const char help_tail[] = N_(
+/*@observer@*/ static const char *help_tail = N_(
     "\n"
     "Files are asm sources to be assembled.\n"
     "\n"
@@ -203,17 +208,345 @@ typedef struct constcharparam {
 
 static constcharparam_head preproc_options;
 
+static int
+do_preproc_only(FILE *in)
+{
+    yasm_linemap *linemap;
+    char *preproc_buf = yasm_xmalloc(PREPROC_BUF_SIZE);
+    size_t got;
+    const char *base_filename;
+    FILE *out = NULL;
+    yasm_errwarns *errwarns = yasm_errwarns_create();
+
+    /* Initialize line manager */
+    linemap = yasm_linemap_create();
+    yasm_linemap_set(linemap, in_filename, 1, 1);
+
+    /* Default output to stdout if not specified or generating dependency
+       makefiles */
+    if (!obj_filename || generate_make_dependencies) {
+	out = stdout;
+
+        /* determine the object filename if not specified, but we need a
+            file name for the makefile rule */
+        if (generate_make_dependencies && !obj_filename) {
+            if (in == stdin)
+                /* Default to yasm.out if no obj filename specified */
+		obj_filename = yasm__xstrdup("yasm.out");
+            else {
+                /* replace (or add) extension to base filename */
+		yasm__splitpath(in_filename, &base_filename);
+		if (base_filename[0] == '\0')
+		    obj_filename = yasm__xstrdup("yasm.out");
+		else
+		    obj_filename = replace_extension(base_filename,
+                        cur_objfmt_module->extension, "yasm.out");
+	    }
+        }
+    } else {
+	/* Open output (object) file */
+	out = open_file(obj_filename, "wt");
+	if (!out) {
+	    yasm_xfree(preproc_buf);
+	    return EXIT_FAILURE;
+	}
+    }
+
+    /* Pre-process until done */
+    cur_preproc = yasm_preproc_create(cur_preproc_module, in, in_filename,
+				      linemap, errwarns);
+
+    apply_preproc_builtins();
+    apply_preproc_saved_options();
+
+    if (generate_make_dependencies) {
+	size_t totlen;
+
+        fprintf(stdout, "%s: %s", obj_filename, in_filename);
+	totlen = strlen(obj_filename)+2+strlen(in_filename);
+
+	while ((got = yasm_preproc_get_included_file(cur_preproc, preproc_buf,
+						     PREPROC_BUF_SIZE)) != 0) {
+	    totlen += got;
+	    if (totlen > 72) {
+		fputs(" \\\n  ", stdout);
+		totlen = 2;
+	    }
+	    fputc(' ', stdout);
+            fwrite(preproc_buf, got, 1, stdout);
+	}
+	fputc('\n', stdout);
+    } else {
+        while ((got = yasm_preproc_input(cur_preproc, preproc_buf,
+					 PREPROC_BUF_SIZE)) != 0)
+	    fwrite(preproc_buf, got, 1, out);
+    }
+
+    if (in != stdin)
+	fclose(in);
+
+    if (out != stdout)
+	fclose(out);
+
+    if (yasm_errwarns_num_errors(errwarns, warning_error) > 0) {
+	yasm_errwarns_output_all(errwarns, linemap, warning_error,
+				 print_yasm_error, print_yasm_warning);
+	if (out != stdout)
+	    remove(obj_filename);
+	yasm_xfree(preproc_buf);
+	yasm_linemap_destroy(linemap);
+	yasm_errwarns_destroy(errwarns);
+	cleanup(NULL);
+	return EXIT_FAILURE;
+    }
+
+    yasm_errwarns_output_all(errwarns, linemap, warning_error,
+			     print_yasm_error, print_yasm_warning);
+    yasm_xfree(preproc_buf);
+    yasm_linemap_destroy(linemap);
+    yasm_errwarns_destroy(errwarns);
+    cleanup(NULL);
+    return EXIT_SUCCESS;
+}
+
+static int
+do_assemble(FILE *in)
+{
+    yasm_object *object;
+    yasm_section *def_sect;
+    const char *base_filename;
+    /*@null@*/ FILE *obj = NULL;
+    yasm_arch_create_error arch_error;
+    yasm_errwarns *errwarns = yasm_errwarns_create();
+    int i, matched;
+
+    /* determine the object filename if not specified */
+    if (!obj_filename) {
+	if (in == stdin)
+	    /* Default to yasm.out if no obj filename specified */
+	    obj_filename = yasm__xstrdup("yasm.out");
+	else {
+	    /* replace (or add) extension to base filename */
+	    yasm__splitpath(in_filename, &base_filename);
+	    if (base_filename[0] == '\0')
+		obj_filename = yasm__xstrdup("yasm.out");
+	    else
+		obj_filename = replace_extension(base_filename,
+						 cur_objfmt_module->extension,
+						 "yasm.out");
+	}
+    }
+
+    /* Set up architecture using machine and parser. */
+    if (!machine_name) {
+	/* If we're using x86 and the default objfmt bits is 64, default the
+	 * machine to amd64.  When we get more arches with multiple machines,
+	 * we should do this in a more modular fashion.
+	 */
+	if (strcmp(cur_arch_module->keyword, "x86") == 0 &&
+	    cur_objfmt_module->default_x86_mode_bits == 64)
+	    machine_name = yasm__xstrdup("amd64");
+	else
+	    machine_name =
+		yasm__xstrdup(cur_arch_module->default_machine_keyword);
+    }
+
+    cur_arch = cur_arch_module->create(machine_name,
+				       cur_parser_module->keyword,
+				       &arch_error);
+    if (!cur_arch) {
+	switch (arch_error) {
+	    case YASM_ARCH_CREATE_BAD_MACHINE:
+		print_error(_("%s: `%s' is not a valid %s for %s `%s'"),
+			    _("FATAL"), machine_name, _("machine"),
+			    _("architecture"), cur_arch_module->keyword);
+		break;
+	    case YASM_ARCH_CREATE_BAD_PARSER:
+		print_error(_("%s: `%s' is not a valid %s for %s `%s'"),
+			    _("FATAL"), cur_parser_module->keyword,
+			    _("parser"), _("architecture"),
+			    cur_arch_module->keyword);
+		break;
+	    default:
+		print_error(_("%s: unknown architecture error"), _("FATAL"));
+	}
+
+	return EXIT_FAILURE;
+    }
+
+    /* Create object */
+    object = yasm_object_create(in_filename, obj_filename);
+    yasm_linemap_set(yasm_object_get_linemap(object), in_filename, 1, 1);
+
+    /* Initialize the object format */
+    cur_objfmt = yasm_objfmt_create(cur_objfmt_module, object, cur_arch);
+    if (!cur_objfmt) {
+	print_error(
+	    _("%s: object format `%s' does not support architecture `%s' machine `%s'"),
+	    _("FATAL"), cur_objfmt_module->keyword, cur_arch_module->keyword,
+	    machine_name);
+	if (in != stdin)
+	    fclose(in);
+	cleanup(object);
+	return EXIT_FAILURE;
+    }
+
+    /* Get a fresh copy of objfmt_module as it may have changed. */
+    cur_objfmt_module = ((yasm_objfmt_base *)cur_objfmt)->module;
+
+    /* Add an initial "default" section to object */
+    def_sect = yasm_objfmt_add_default_section(cur_objfmt);
+
+    /* Check to see if the requested debug format is in the allowed list
+     * for the active object format.
+     */
+    matched = 0;
+    for (i=0; cur_objfmt_module->dbgfmt_keywords[i]; i++)
+	if (yasm__strcasecmp(cur_objfmt_module->dbgfmt_keywords[i],
+			     cur_dbgfmt_module->keyword) == 0)
+	    matched = 1;
+    if (!matched) {
+	print_error(_("%s: `%s' is not a valid %s for %s `%s'"),
+	    _("FATAL"), cur_dbgfmt_module->keyword, _("debug format"),
+	    _("object format"), cur_objfmt_module->keyword);
+	if (in != stdin)
+	    fclose(in);
+	cleanup(object);
+	return EXIT_FAILURE;
+    }
+
+    /* Initialize the debug format */
+    cur_dbgfmt = yasm_dbgfmt_create(cur_dbgfmt_module, object, cur_objfmt,
+				    cur_arch);
+    if (!cur_dbgfmt) {
+	print_error(
+	    _("%s: debug format `%s' does not work with object format `%s'"),
+	    _("FATAL"), cur_dbgfmt_module->keyword,
+	    cur_objfmt_module->keyword);
+	if (in != stdin)
+	    fclose(in);
+	return EXIT_FAILURE;
+    }
+
+    /* Check to see if the requested preprocessor is in the allowed list
+     * for the active parser.
+     */
+    matched = 0;
+    for (i=0; cur_parser_module->preproc_keywords[i]; i++)
+	if (yasm__strcasecmp(cur_parser_module->preproc_keywords[i],
+			     cur_preproc_module->keyword) == 0)
+	    matched = 1;
+    if (!matched) {
+	print_error(_("%s: `%s' is not a valid %s for %s `%s'"), _("FATAL"),
+		    cur_preproc_module->keyword, _("preprocessor"),
+		    _("parser"), cur_parser_module->keyword);
+	if (in != stdin)
+	    fclose(in);
+	cleanup(object);
+	return EXIT_FAILURE;
+    }
+
+    cur_preproc = cur_preproc_module->create(in, in_filename,
+					     yasm_object_get_linemap(object),
+					     errwarns);
+
+    apply_preproc_builtins();
+    apply_preproc_saved_options();
+
+    /* Get initial x86 BITS setting from object format */
+    if (strcmp(cur_arch_module->keyword, "x86") == 0) {
+	yasm_arch_set_var(cur_arch, "mode_bits",
+			  cur_objfmt_module->default_x86_mode_bits);
+    }
+
+    yasm_arch_set_var(cur_arch, "force_strict", force_strict);
+
+    /* Parse! */
+    cur_parser_module->do_parse(object, cur_preproc, cur_arch, cur_objfmt,
+				cur_dbgfmt, in, in_filename,
+				list_filename != NULL, def_sect, errwarns);
+
+    /* Close input file */
+    if (in != stdin)
+	fclose(in);
+
+    check_errors(errwarns, object);
+
+    /* Check for undefined symbols */
+    yasm_symtab_parser_finalize(yasm_object_get_symtab(object),
+				strcmp(cur_parser_module->keyword, "gas")==0 ||
+				strcmp(cur_parser_module->keyword, "gnu")==0,
+				cur_objfmt, errwarns);
+    check_errors(errwarns, object);
+
+    /* Finalize parse */
+    yasm_object_finalize(object, errwarns);
+    check_errors(errwarns, object);
+
+    /* Optimize */
+    yasm_object_optimize(object, cur_arch, errwarns);
+    check_errors(errwarns, object);
+
+    /* generate any debugging information */
+    yasm_dbgfmt_generate(cur_dbgfmt, errwarns);
+    check_errors(errwarns, object);
+
+    /* open the object file for output (if not already opened by dbg objfmt) */
+    if (!obj && strcmp(cur_objfmt_module->keyword, "dbg") != 0) {
+	obj = open_file(obj_filename, "wb");
+	if (!obj) {
+	    cleanup(object);
+	    return EXIT_FAILURE;
+	}
+    }
+
+    /* Write the object file */
+    yasm_objfmt_output(cur_objfmt, obj?obj:stderr,
+		       strcmp(cur_dbgfmt_module->keyword, "null"), cur_dbgfmt,
+		       errwarns);
+
+    /* Close object file */
+    if (obj)
+	fclose(obj);
+
+    /* If we had an error at this point, we also need to delete the output
+     * object file (to make sure it's not left newer than the source).
+     */
+    if (yasm_errwarns_num_errors(errwarns, warning_error) > 0)
+	remove(obj_filename);
+    check_errors(errwarns, object);
+
+    /* Open and write the list file */
+    if (list_filename) {
+	FILE *list = open_file(list_filename, "wt");
+	if (!list) {
+	    cleanup(object);
+	    return EXIT_FAILURE;
+	}
+	/* Initialize the list format */
+	cur_listfmt = yasm_listfmt_create(cur_listfmt_module, in_filename,
+					  obj_filename);
+	yasm_listfmt_output(cur_listfmt, list,
+			    yasm_object_get_linemap(object), cur_arch);
+	fclose(list);
+    }
+
+    yasm_errwarns_output_all(errwarns, yasm_object_get_linemap(object),
+			     warning_error, print_yasm_error,
+			     print_yasm_warning);
+
+    yasm_errwarns_destroy(errwarns);
+    cleanup(object);
+    return EXIT_SUCCESS;
+}
+
 /* main function */
 /*@-globstate -unrecog@*/
 int
 main(int argc, char *argv[])
 {
-    /*@null@*/ FILE *in = NULL, *obj = NULL;
-    yasm_object *object = NULL;
-    yasm_section *def_sect;
+    /*@null@*/ FILE *in = NULL;
     size_t i;
-    yasm_arch_create_error arch_error;
-    const char *base_filename;
 
 #if defined(HAVE_SETLOCALE) && defined(HAVE_LC_MESSAGES)
     setlocale(LC_MESSAGES, "");
@@ -253,168 +586,28 @@ main(int argc, char *argv[])
 	    return EXIT_SUCCESS;
     }
 
-    /* Initialize BitVector (needed for floating point). */
+    /* Initialize BitVector (needed for intnum/floatnum). */
     if (BitVector_Boot() != ErrCode_Ok) {
 	print_error(_("%s: could not initialize BitVector"), _("FATAL"));
 	return EXIT_FAILURE;
     }
 
-    if (in_filename && strcmp(in_filename, "-") != 0) {
-	/* Open the input file (if not standard input) */
-	in = fopen(in_filename, "rt");
-	if (!in) {
-	    print_error(_("%s: could not open file `%s'"), _("FATAL"),
-			in_filename);
-	    yasm_xfree(in_filename);
-	    if (obj_filename)
-		yasm_xfree(obj_filename);
-	    return EXIT_FAILURE;
-	}
-    } else {
-	/* If no files were specified or filename was "-", read stdin */
-	in = stdin;
-	if (!in_filename)
-	    in_filename = yasm__xstrdup("-");
-    }
-
     /* Initialize intnum and floatnum */
     yasm_intnum_initialize();
     yasm_floatnum_initialize();
     
     /* If not already specified, default to bin as the object format. */
-    if (!cur_objfmt_module)
-	cur_objfmt_module = yasm_load_objfmt(DEFAULT_OBJFMT_MODULE);
-
     if (!cur_objfmt_module) {
-	print_error(_("%s: could not load default %s"), _("FATAL"),
-		    _("object format"));
-	return EXIT_FAILURE;
-    }
-
-    /* handle preproc-only case here */
-    if (preproc_only) {
-	yasm_linemap *linemap;
-	char *preproc_buf = yasm_xmalloc(PREPROC_BUF_SIZE);
-	size_t got;
-
-	/* Initialize line manager */
-	linemap = yasm_linemap_create();
-	yasm_linemap_set(linemap, in_filename, 1, 1);
-
-	/* Default output to stdout if not specified or generating dependency
-           makefiles */
-	if (!obj_filename || generate_make_dependencies) {
-	    obj = stdout;
-
-            /* determine the object filename if not specified, but we need a
-               file name for the makefile rule */
-            if (generate_make_dependencies && !obj_filename) {
-                if (in == stdin)
-                    /* Default to yasm.out if no obj filename specified */
-                    obj_filename = yasm__xstrdup("yasm.out");
-                else {
-                    /* replace (or add) extension to base filename */
-		    yasm__splitpath(in_filename, &base_filename);
-		    if (base_filename[0] == '\0')
-			obj_filename = yasm__xstrdup("yasm.out");
-		    else
-			obj_filename = replace_extension(base_filename,
-                            cur_objfmt_module->extension, "yasm.out");
-		}
-            }
-	} else {
-	    /* Open output (object) file */
-	    obj = open_file(obj_filename, "wt");
-	    if (!obj) {
-		yasm_xfree(preproc_buf);
-		return EXIT_FAILURE;
-	    }
-	}
-
-	/* If not already specified, default to nasm preproc. */
-	if (!cur_preproc_module)
-	    cur_preproc_module = yasm_load_preproc("nasm");
-
-	if (!cur_preproc_module) {
+	if (!objfmt_keyword)
+	    objfmt_keyword = yasm__xstrdup("bin");
+	cur_objfmt_module = yasm_load_objfmt(objfmt_keyword);
+	if (!cur_objfmt_module) {
 	    print_error(_("%s: could not load default %s"), _("FATAL"),
-			_("preprocessor"));
-	    cleanup(NULL);
+			_("object format"));
 	    return EXIT_FAILURE;
 	}
-	
-	/* Pre-process until done */
-	cur_preproc = yasm_preproc_create(cur_preproc_module, in, in_filename,
-					  linemap);
-
-	apply_preproc_builtins();
-	apply_preproc_saved_options();
-
-        if (generate_make_dependencies) {
-	    size_t totlen;
-
-            fprintf(stdout, "%s: %s", obj_filename, in_filename);
-	    totlen = strlen(obj_filename)+2+strlen(in_filename);
-
-            while ((got = yasm_preproc_get_included_file(cur_preproc,
-				preproc_buf, PREPROC_BUF_SIZE)) != 0) {
-		totlen += got;
-		if (totlen > 72) {
-		    fputs(" \\\n  ", stdout);
-		    totlen = 2;
-		}
-		fputc(' ', stdout);
-                fwrite(preproc_buf, got, 1, stdout);
-	    }
-	    fputc('\n', stdout);
-        } else {
-            while ((got = yasm_preproc_input(cur_preproc, preproc_buf,
-					     PREPROC_BUF_SIZE)) != 0)
-	        fwrite(preproc_buf, got, 1, obj);
-        }
-
-	if (in != stdin)
-	    fclose(in);
-
-	if (obj != stdout)
-	    fclose(obj);
-
-	if (yasm_get_num_errors(warning_error) > 0) {
-	    yasm_errwarn_output_all(linemap, warning_error, print_yasm_error,
-				    print_yasm_warning);
-	    if (obj != stdout)
-		remove(obj_filename);
-	    yasm_xfree(preproc_buf);
-	    yasm_linemap_destroy(linemap);
-	    cleanup(NULL);
-	    return EXIT_FAILURE;
-	}
-	yasm_xfree(preproc_buf);
-	yasm_linemap_destroy(linemap);
-	cleanup(NULL);
-	return EXIT_SUCCESS;
     }
 
-    /* determine the object filename if not specified */
-    if (!obj_filename) {
-	if (in == stdin)
-	    /* Default to yasm.out if no obj filename specified */
-	    obj_filename = yasm__xstrdup("yasm.out");
-	else {
-	    /* replace (or add) extension to base filename */
-	    yasm__splitpath(in_filename, &base_filename);
-	    if (base_filename[0] == '\0')
-		obj_filename = yasm__xstrdup("yasm.out");
-	    else
-		obj_filename = replace_extension(base_filename,
-						 cur_objfmt_module->extension,
-						 "yasm.out");
-	}
-    }
-
-    /* Create object */
-    object = yasm_object_create(in_filename, obj_filename);
-    yasm_linemap_set(yasm_object_get_linemap(object), in_filename, 1, 1);
-
     /* Default to x86 as the architecture */
     if (!cur_arch_module) {
 	cur_arch_module = yasm_load_arch("x86");
@@ -425,6 +618,18 @@ main(int argc, char *argv[])
 	}
     }
 
+    /* Check for arch help */
+    if (machine_name && strcmp(machine_name, "help") == 0) {
+	yasm_arch_machine *m = cur_arch_module->machines;
+	printf(_("Available %s for %s `%s':\n"), _("machines"),
+	       _("architecture"), cur_arch_module->keyword);
+	while (m->keyword && m->name) {
+	    print_list_keyword_desc(m->name, m->keyword);
+	    m++;
+	}
+	return EXIT_SUCCESS;
+    }
+
     /* Default to NASM as the parser */
     if (!cur_parser_module) {
 	cur_parser_module = yasm_load_parser("nasm");
@@ -436,267 +641,66 @@ main(int argc, char *argv[])
 	}
     }
 
-    /* Set up architecture using the selected (or default) machine and
-     * selected (or default nasm) parser.
-     */
-    if (!machine_name) {
-	/* If we're using x86 and the default objfmt bits is 64, default the
-	 * machine to amd64.  When we get more arches with multiple machines,
-	 * we should do this in a more modular fashion.
-	 */
-	if (strcmp(cur_arch_module->keyword, "x86") == 0 &&
-	    cur_objfmt_module->default_x86_mode_bits == 64)
-	    machine_name = yasm__xstrdup("amd64");
-	else
-	    machine_name =
-		yasm__xstrdup(cur_arch_module->default_machine_keyword);
-    }
-
-    cur_arch = cur_arch_module->create(machine_name,
-				       cur_parser_module->keyword,
-				       &arch_error);
-    if (!cur_arch) {
-	if (strcmp(machine_name, "help") == 0) {
-	    yasm_arch_machine *m = cur_arch_module->machines;
-	    printf(_("Available %s for %s `%s':\n"), _("machines"),
-		   _("architecture"), cur_arch_module->keyword);
-	    while (m->keyword && m->name) {
-		print_list_keyword_desc(m->name, m->keyword);
-		m++;
-	    }
-	    return EXIT_SUCCESS;
-	}
-
-	switch (arch_error) {
-	    case YASM_ARCH_CREATE_BAD_MACHINE:
-		print_error(_("%s: `%s' is not a valid %s for %s `%s'"),
-			    _("FATAL"), machine_name, _("machine"),
-			    _("architecture"), cur_arch_module->keyword);
-		break;
-	    case YASM_ARCH_CREATE_BAD_PARSER:
-		print_error(_("%s: `%s' is not a valid %s for %s `%s'"),
-			    _("FATAL"), machine_name, _("machine"),
-			    _("architecture"), cur_arch_module->keyword);
-		break;
-	    default:
-		print_error(_("%s: unknown architecture error"), _("FATAL"));
-	}
-
-	return EXIT_FAILURE;
-    }
-
-    /* Set basic as the optimizer (TODO: user choice) */
-    cur_optimizer_module = yasm_load_optimizer("basic");
-
-    if (!cur_optimizer_module) {
-	print_error(_("%s: could not load default %s"), _("FATAL"),
-		    _("optimizer"));
-	return EXIT_FAILURE;
-    }
-
-    /* If list file enabled, make sure we have a list format loaded. */
-    if (list_filename) {
-	/* If not already specified, default to nasm as the list format. */
-	if (!cur_listfmt_module)
-	    cur_listfmt_module = yasm_load_listfmt("nasm");
-    }
-
-    /* If not already specified, default to null as the debug format. */
-    if (!cur_dbgfmt_module)
-	cur_dbgfmt_module = yasm_load_dbgfmt("null");
-    else {
-	int matched_dbgfmt = 0;
-	/* Check to see if the requested debug format is in the allowed list
-	 * for the active object format.
-	 */
-	for (i=0; cur_objfmt_module->dbgfmt_keywords[i]; i++)
-	    if (yasm__strcasecmp(cur_objfmt_module->dbgfmt_keywords[i],
-				 cur_dbgfmt_module->keyword) == 0)
-		matched_dbgfmt = 1;
-	if (!matched_dbgfmt) {
-	    print_error(_("%s: `%s' is not a valid %s for %s `%s'"),
-		_("FATAL"), cur_dbgfmt_module->keyword, _("debug format"),
-		_("object format"), cur_objfmt_module->keyword);
-	    if (in != stdin)
-		fclose(in);
-	    /*cleanup(NULL);*/
-	    return EXIT_FAILURE;
-	}
-    }
-
-    if (!cur_dbgfmt_module) {
-	print_error(_("%s: could not load default %s"), _("FATAL"),
-		    _("debug format"));
-	return EXIT_FAILURE;
-    }
-
-    /* Initialize the object format */
-    cur_objfmt = yasm_objfmt_create(cur_objfmt_module, object, cur_arch);
-    if (!cur_objfmt) {
-	print_error(
-	    _("%s: object format `%s' does not support architecture `%s' machine `%s'"),
-	    _("FATAL"), cur_objfmt_module->keyword, cur_arch_module->keyword,
-	    machine_name);
-	if (in != stdin)
-	    fclose(in);
-	return EXIT_FAILURE;
-    }
-
-    /* Get a fresh copy of objfmt_module as it may have changed. */
-    cur_objfmt_module = ((yasm_objfmt_base *)cur_objfmt)->module;
-
-    /* Add an initial "default" section to object */
-    def_sect = yasm_objfmt_add_default_section(cur_objfmt);
-
-    /* Initialize the debug format */
-    cur_dbgfmt = yasm_dbgfmt_create(cur_dbgfmt_module, object, cur_objfmt,
-				    cur_arch);
-    if (!cur_dbgfmt) {
-	print_error(
-	    _("%s: debug format `%s' does not work with object format `%s'"),
-	    _("FATAL"), cur_dbgfmt_module->keyword,
-	    cur_objfmt_module->keyword);
-	if (in != stdin)
-	    fclose(in);
-	return EXIT_FAILURE;
-    }
-
     /* If not already specified, default to the parser's default preproc. */
-    if (!cur_preproc_module)
+    if (!cur_preproc_module) {
 	cur_preproc_module =
 	    yasm_load_preproc(cur_parser_module->default_preproc_keyword);
-    else {
-	int matched_preproc = 0;
-	/* Check to see if the requested preprocessor is in the allowed list
-	 * for the active parser.
-	 */
-	for (i=0; cur_parser_module->preproc_keywords[i]; i++)
-	    if (yasm__strcasecmp(cur_parser_module->preproc_keywords[i],
-				 cur_preproc_module->keyword) == 0)
-		matched_preproc = 1;
-	if (!matched_preproc) {
-	    print_error(_("%s: `%s' is not a valid %s for %s `%s'"),
-			_("FATAL"), cur_preproc_module->keyword,
-			_("preprocessor"), _("parser"),
-			cur_parser_module->keyword);
-	    if (in != stdin)
-		fclose(in);
+	if (!cur_preproc_module) {
+	    print_error(_("%s: could not load default %s"), _("FATAL"),
+			_("preprocessor"));
 	    cleanup(NULL);
 	    return EXIT_FAILURE;
 	}
     }
 
-    if (!cur_preproc_module) {
-	print_error(_("%s: could not load default %s"), _("FATAL"),
-		    _("preprocessor"));
-	cleanup(NULL);
+    /* Determine input filename and open input file. */
+    if (!in_filename) {
+	print_error(_("No input files specified"));
 	return EXIT_FAILURE;
-    }
-
-    cur_preproc = cur_preproc_module->create(in, in_filename,
-					     yasm_object_get_linemap(object));
-
-    apply_preproc_builtins();
-    apply_preproc_saved_options();
-
-    /* Get initial x86 BITS setting from object format */
-    if (strcmp(cur_arch_module->keyword, "x86") == 0) {
-	yasm_arch_set_var(cur_arch, "mode_bits",
-			  cur_objfmt_module->default_x86_mode_bits);
-    }
-
-    /* Parse! */
-    cur_parser_module->do_parse(object, cur_preproc, cur_arch, cur_objfmt,
-				cur_dbgfmt, in, in_filename,
-				list_filename != NULL, def_sect);
-
-    /* Close input file */
-    if (in != stdin)
-	fclose(in);
-
-    /* Check for undefined symbols */
-    yasm_symtab_parser_finalize(yasm_object_get_symtab(object),
-				strcmp(cur_parser_module->keyword, "gas")==0,
-				cur_objfmt);
-
-    if (yasm_get_num_errors(warning_error) > 0) {
-	yasm_errwarn_output_all(yasm_object_get_linemap(object), warning_error,
-				print_yasm_error, print_yasm_warning);
-	cleanup(object);
-	return EXIT_FAILURE;
-    }
-
-    /* Finalize parse */
-    yasm_object_finalize(object);
-
-    if (yasm_get_num_errors(warning_error) > 0) {
-	yasm_errwarn_output_all(yasm_object_get_linemap(object), warning_error,
-				print_yasm_error, print_yasm_warning);
-	cleanup(object);
-	return EXIT_FAILURE;
-    }
-
-    /* Optimize */
-    cur_optimizer_module->optimize(object);
-
-    if (yasm_get_num_errors(warning_error) > 0) {
-	yasm_errwarn_output_all(yasm_object_get_linemap(object), warning_error,
-				print_yasm_error, print_yasm_warning);
-	cleanup(object);
-	return EXIT_FAILURE;
-    }
-
-    /* generate any debugging information */
-    yasm_dbgfmt_generate(cur_dbgfmt);
-
-    /* open the object file for output (if not already opened by dbg objfmt) */
-    if (!obj && strcmp(cur_objfmt_module->keyword, "dbg") != 0) {
-	obj = open_file(obj_filename, "wb");
-	if (!obj) {
-	    cleanup(object);
+    } else if (strcmp(in_filename, "-") != 0) {
+	/* Open the input file (if not standard input) */
+	in = fopen(in_filename, "rt");
+	if (!in) {
+	    print_error(_("%s: could not open file `%s'"), _("FATAL"),
+			in_filename);
+	    yasm_xfree(in_filename);
+	    if (obj_filename)
+		yasm_xfree(obj_filename);
 	    return EXIT_FAILURE;
 	}
+    } else {
+	/* Filename was "-", read stdin */
+	in = stdin;
     }
 
-    /* Write the object file */
-    yasm_objfmt_output(cur_objfmt, obj?obj:stderr,
-		       strcmp(cur_dbgfmt_module->keyword, "null"), cur_dbgfmt);
+    /* handle preproc-only case here */
+    if (preproc_only)
+	return do_preproc_only(in);
 
-    /* Close object file */
-    if (obj)
-	fclose(obj);
-
-    /* If we had an error at this point, we also need to delete the output
-     * object file (to make sure it's not left newer than the source).
-     */
-    if (yasm_get_num_errors(warning_error) > 0) {
-	yasm_errwarn_output_all(yasm_object_get_linemap(object), warning_error,
-				print_yasm_error, print_yasm_warning);
-	remove(obj_filename);
-	cleanup(object);
-	return EXIT_FAILURE;
-    }
-
-    /* Open and write the list file */
+    /* If list file enabled, make sure we have a list format loaded. */
     if (list_filename) {
-	FILE *list = open_file(list_filename, "wt");
-	if (!list) {
-	    cleanup(object);
-	    return EXIT_FAILURE;
+	/* If not already specified, default to nasm as the list format. */
+	if (!cur_listfmt_module) {
+	    cur_listfmt_module = yasm_load_listfmt("nasm");
+	    if (!cur_listfmt_module) {
+		print_error(_("%s: could not load default %s"), _("FATAL"),
+			    _("list format"));
+		return EXIT_FAILURE;
+	    }
 	}
-	/* Initialize the list format */
-	cur_listfmt = yasm_listfmt_create(cur_listfmt_module, in_filename,
-					  obj_filename);
-	yasm_listfmt_output(cur_listfmt, list,
-			    yasm_object_get_linemap(object), cur_arch);
-	fclose(list);
     }
 
-    yasm_errwarn_output_all(yasm_object_get_linemap(object), warning_error,
-			    print_yasm_error, print_yasm_warning);
+    /* If not already specified, default to null as the debug format. */
+    if (!cur_dbgfmt_module) {
+	cur_dbgfmt_module = yasm_load_dbgfmt("null");
+	if (!cur_dbgfmt_module) {
+	    print_error(_("%s: could not load default %s"), _("FATAL"),
+			_("debug format"));
+	    return EXIT_FAILURE;
+	}
+    }
 
-    cleanup(object);
-    return EXIT_SUCCESS;
+    return do_assemble(in);
 }
 /*@=globstate =unrecog@*/
 
@@ -712,6 +716,19 @@ open_file(const char *filename, const char *mode)
     return f;
 }
 
+static void
+check_errors(yasm_errwarns *errwarns, yasm_object *object)
+{
+    if (yasm_errwarns_num_errors(errwarns, warning_error) > 0) {
+	yasm_errwarns_output_all(errwarns, yasm_object_get_linemap(object),
+				 warning_error, print_yasm_error,
+				 print_yasm_warning);
+	yasm_errwarns_destroy(errwarns);
+	cleanup(object);
+	exit(EXIT_FAILURE);
+    }
+}
+
 /* Define DO_FREE to 1 to enable deallocation of all data structures.
  * Useful for detecting memory leaks, but slows down execution unnecessarily
  * (as the OS will free everything we miss here).
@@ -775,6 +792,23 @@ not_an_option_handler(char *param)
     return 0;
 }
 
+int
+other_option_handler(char *option)
+{
+    /* Accept, but ignore, -O and -Onnn, for compatibility with NASM. */
+    if (option[0] == '-' && option[1] == 'O') {
+	int n = 2;
+	for (;;) {
+	    if (option[n] == '\0')
+		return 0;
+	    if (!isdigit(option[n]))
+		return 1;
+	    n++;
+	}
+    }
+    return 1;
+}
+
 static int
 opt_special_handler(/*@unused@*/ char *cmd, /*@unused@*/ char *param, int extra)
 {
@@ -949,6 +983,15 @@ opt_machine_handler(/*@unused@*/ char *cmd, char *param,
     return 0;
 }
 
+static int
+opt_strict_handler(/*@unused@*/ char *cmd,
+		   /*@unused@*/ /*@null@*/ char *param,
+		   /*@unused@*/ int extra)
+{
+    force_strict = 1;
+    return 0;
+}
+
 static int
 opt_warning_handler(char *cmd, /*@unused@*/ char *param, int extra)
 {
@@ -995,6 +1038,13 @@ preproc_only_handler(/*@unused@*/ char *cmd, /*@unused@*/ char *param,
     return 0;
 }
 
+static int
+opt_include_option(/*@unused@*/ char *cmd, char *param, /*@unused@*/ int extra)
+{
+    yasm_add_include_path(param);
+    return 0;
+}
+
 static int
 opt_preproc_option(/*@unused@*/ char *cmd, char *param, int extra)
 {
@@ -1036,9 +1086,6 @@ apply_preproc_builtins()
 {
     char *predef;
 
-    if (!objfmt_keyword)
-	objfmt_keyword = yasm__xstrdup(DEFAULT_OBJFMT_MODULE);
-
     /* Define standard YASM assembly-time macro constants */
     predef = yasm_xmalloc(strlen("__YASM_OBJFMT__=")
 			  + strlen(objfmt_keyword) + 1);
@@ -1053,14 +1100,13 @@ apply_preproc_saved_options()
 {
     constcharparam *cp, *cpnext;
 
-    void (*funcs[4])(yasm_preproc *, const char *);
-    funcs[0] = cur_preproc_module->add_include_path;
-    funcs[1] = cur_preproc_module->add_include_file;
-    funcs[2] = cur_preproc_module->predefine_macro;
-    funcs[3] = cur_preproc_module->undefine_macro;
+    void (*funcs[3])(yasm_preproc *, const char *);
+    funcs[0] = cur_preproc_module->add_include_file;
+    funcs[1] = cur_preproc_module->predefine_macro;
+    funcs[2] = cur_preproc_module->undefine_macro;
 
     STAILQ_FOREACH(cp, &preproc_options, link) {
-	if (0 <= cp->id && cp->id < 4 && funcs[cp->id])
+	if (0 <= cp->id && cp->id < 3 && funcs[cp->id])
 	    funcs[cp->id](cur_preproc, cp->param);
     }
 
@@ -1172,19 +1218,39 @@ handle_yasm_gettext(const char *msgid)
     return gettext(msgid);
 }
 
-const char *fmt[2] = {
+static const char *fmt[2] = {
 	"%s:%lu: %s%s\n",	/* GNU */
 	"%s(%lu) : %s%s\n"	/* VC */
 };
 
+static const char *fmt_noline[2] = {
+	"%s: %s%s\n",	/* GNU */
+	"%s : %s%s\n"	/* VC */
+};
+
 static void
-print_yasm_error(const char *filename, unsigned long line, const char *msg)
+print_yasm_error(const char *filename, unsigned long line, const char *msg,
+		 const char *xref_fn, unsigned long xref_line,
+		 const char *xref_msg)
 {
-    fprintf(stderr, fmt[ewmsg_style], filename, line, "", msg);
+    if (line)
+	fprintf(stderr, fmt[ewmsg_style], filename, line, "", msg);
+    else
+	fprintf(stderr, fmt_noline[ewmsg_style], filename, "", msg);
+
+    if (xref_fn && xref_msg) {
+	if (xref_line)
+	    fprintf(stderr, fmt[ewmsg_style], xref_fn, xref_line, "", xref_msg);
+	else
+	    fprintf(stderr, fmt_noline[ewmsg_style], xref_fn, "", xref_msg);
+    }
 }
 
 static void
 print_yasm_warning(const char *filename, unsigned long line, const char *msg)
 {
-    fprintf(stderr, fmt[ewmsg_style], filename, line, _("warning: "), msg);
+    if (line)
+	fprintf(stderr, fmt[ewmsg_style], filename, line, _("warning: "), msg);
+    else
+	fprintf(stderr, fmt_noline[ewmsg_style], filename, _("warning: "), msg);
 }
