@@ -3,7 +3,7 @@
  * \brief YASM library primary header file.
  *
  * \rcs
- * $Id: libyasm.h 1591 2006-07-13 03:25:38Z peter $
+ * $Id: libyasm.h 1774 2007-02-18 20:42:55Z peter $
  * \endrcs
  *
  * \license
@@ -44,8 +44,16 @@
 #ifndef YASM_LIB_H
 #define YASM_LIB_H
 
+#ifdef YASM_PYXELATOR
+typedef struct __FILE FILE;
+typedef struct __va_list va_list;
+typedef unsigned long size_t;
+typedef unsigned long uintptr_t;
+#else
 #include <stdio.h>
 #include <stdarg.h>
+#include <libyasm-stdint.h>
+#endif
 
 #ifdef YASM_LIB_INTERNAL
 # include <libyasm/compat-queue.h>
@@ -54,7 +62,7 @@
 #include <libyasm/coretype.h>
 #include <libyasm/valparam.h>
 
-#include <libyasm/linemgr.h>
+#include <libyasm/linemap.h>
 
 #include <libyasm/errwarn.h>
 #include <libyasm/intnum.h>
@@ -70,7 +78,6 @@
 #include <libyasm/dbgfmt.h>
 #include <libyasm/objfmt.h>
 #include <libyasm/listfmt.h>
-#include <libyasm/optimizer.h>
 #include <libyasm/parser.h>
 #include <libyasm/preproc.h>
 
