@@ -27,8 +27,9 @@
 #define YASM_LIB_INTERNAL
 #define YASM_ARCH_INTERNAL
 #include "util.h"
-/*@unused@*/ RCSID("$Id: arch.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: arch.c 1786 2007-02-24 04:16:05Z peter $");
 
+#include "libyasm-stdint.h"
 #include "coretype.h"
 
 #include "expr.h"
@@ -39,7 +40,7 @@
 
 
 yasm_insn_operand *
-yasm_operand_create_reg(unsigned long reg)
+yasm_operand_create_reg(uintptr_t reg)
 {
     yasm_insn_operand *retval = yasm_xmalloc(sizeof(yasm_insn_operand));
 
@@ -48,12 +49,13 @@ yasm_operand_create_reg(unsigned long reg)
     retval->targetmod = 0;
     retval->size = 0;
     retval->deref = 0;
+    retval->strict = 0;
 
     return retval;
 }
 
 yasm_insn_operand *
-yasm_operand_create_segreg(unsigned long segreg)
+yasm_operand_create_segreg(uintptr_t segreg)
 {
     yasm_insn_operand *retval = yasm_xmalloc(sizeof(yasm_insn_operand));
 
@@ -62,6 +64,7 @@ yasm_operand_create_segreg(unsigned long segreg)
     retval->targetmod = 0;
     retval->size = 0;
     retval->deref = 0;
+    retval->strict = 0;
 
     return retval;
 }
@@ -76,6 +79,7 @@ yasm_operand_create_mem(/*@only@*/ yasm_effaddr *ea)
     retval->targetmod = 0;
     retval->size = 0;
     retval->deref = 0;
+    retval->strict = 0;
 
     return retval;
 }
@@ -84,7 +88,7 @@ yasm_insn_operand *
 yasm_operand_create_imm(/*@only@*/ yasm_expr *val)
 {
     yasm_insn_operand *retval;
-    const unsigned long *reg;
+    const uintptr_t *reg;
 
     reg = yasm_expr_get_reg(&val, 0);
     if (reg) {
@@ -97,6 +101,7 @@ yasm_operand_create_imm(/*@only@*/ yasm_expr *val)
 	retval->targetmod = 0;
 	retval->size = 0;
 	retval->deref = 0;
+	retval->strict = 0;
     }
 
     return retval;
@@ -127,8 +132,11 @@ yasm_operand_print(const yasm_insn_operand *op, FILE *f, int indent_level,
 	    fprintf(f, "\n");
 	    break;
     }
-    fprintf(f, "%*sTargetMod=%lx\n", indent_level+1, "", op->targetmod);
+    fprintf(f, "%*sTargetMod=%lx\n", indent_level+1, "",
+	    (unsigned long)op->targetmod);
     fprintf(f, "%*sSize=%u\n", indent_level+1, "", op->size);
+    fprintf(f, "%*sDeref=%d, Strict=%d\n", indent_level+1, "", (int)op->deref,
+	    (int)op->strict);
 }
 
 void
