@@ -3,7 +3,7 @@
  * \brief YASM architecture interface.
  *
  * \rcs
- * $Id: arch.h 1591 2006-07-13 03:25:38Z peter $
+ * $Id: arch.h 1774 2007-02-18 20:42:55Z peter $
  * \endrcs
  *
  * \license
@@ -35,14 +35,14 @@
 #define YASM_ARCH_H
 
 /** Errors that may be returned by yasm_arch_module::create(). */
-typedef enum {
+typedef enum yasm_arch_create_error {
     YASM_ARCH_CREATE_OK = 0,		/**< No error. */
     YASM_ARCH_CREATE_BAD_MACHINE,	/**< Unrecognized machine name. */
     YASM_ARCH_CREATE_BAD_PARSER		/**< Unrecognized parser name. */
 } yasm_arch_create_error;
 
 /** Return values for yasm_arch_module::parse_check_insnprefix(). */
-typedef enum {
+typedef enum yasm_arch_insnprefix {
     YASM_ARCH_NOTINSNPREFIX = 0,	/**< Unrecognized */
     YASM_ARCH_INSN,			/**< An instruction */
     YASM_ARCH_PREFIX			/**< An instruction prefix */
@@ -51,7 +51,7 @@ typedef enum {
 /** Types of registers / target modifiers that may be returned by
  * yasm_arch_module::parse_check_regtmod().
  */
-typedef enum {
+typedef enum yasm_arch_regtmod {
     YASM_ARCH_NOTREGTMOD = 0,		/**< Unrecognized */
     YASM_ARCH_REG,			/**< A "normal" register */
     YASM_ARCH_REGGROUP,			/**< A group of indexable registers */
@@ -136,28 +136,27 @@ typedef struct yasm_arch_module {
     /** Module-level implementation of yasm_arch_parse_cpu().
      * Call yasm_arch_parse_cpu() instead of calling this function.
      */
-    void (*parse_cpu) (yasm_arch *arch, const char *cpuid, size_t cpuid_len,
-		       unsigned long line);
+    void (*parse_cpu) (yasm_arch *arch, const char *cpuid, size_t cpuid_len);
 
     /** Module-level implementation of yasm_arch_parse_check_insnprefix().
      * Call yasm_arch_parse_check_insnprefix() instead of calling this function.
      */
     yasm_arch_insnprefix (*parse_check_insnprefix)
-	(yasm_arch *arch, /*@out@*/ unsigned long data[4], const char *id,
-	 size_t id_len, unsigned long line);
+	(yasm_arch *arch, /*@out@*/ uintptr_t data[4], const char *id,
+	 size_t id_len);
 
     /** Module-level implementation of yasm_arch_parse_check_regtmod().
      * Call yasm_arch_parse_check_regtmod() instead of calling this function.
      */
     yasm_arch_regtmod (*parse_check_regtmod)
-	(yasm_arch *arch, /*@out@*/ unsigned long *data, const char *id,
-	 size_t id_len, unsigned long line);
+	(yasm_arch *arch, /*@out@*/ uintptr_t *data, const char *id,
+	 size_t id_len);
 
     /** Module-level implementation of yasm_arch_parse_directive().
      * Call yasm_arch_parse_directive() instead of calling this function.
      */
     int (*parse_directive) (yasm_arch *arch, const char *name,
-			    yasm_valparamhead *valparams,
+			    /*@null@*/ yasm_valparamhead *valparams,
 			    /*@null@*/ yasm_valparamhead *objext_valparams,
 			    yasm_object *object, unsigned long line);
 
@@ -171,18 +170,16 @@ typedef struct yasm_arch_module {
      */
     void (*finalize_insn)
 	(yasm_arch *arch, yasm_bytecode *bc, yasm_bytecode *prev_bc,
-	 const unsigned long data[4], int num_operands,
+	 const uintptr_t data[4], int num_operands,
 	 /*@null@*/ yasm_insn_operands *operands, int num_prefixes,
-	 unsigned long **prefixes, int num_segregs,
-	 const unsigned long *segregs);
+	 uintptr_t **prefixes, int num_segregs, const uintptr_t *segregs);
 
     /** Module-level implementation of yasm_arch_floatnum_tobytes().
      * Call yasm_arch_floatnum_tobytes() instead of calling this function.
      */
     int (*floatnum_tobytes) (yasm_arch *arch, const yasm_floatnum *flt,
 			     unsigned char *buf, size_t destsize,
-			     size_t valsize, size_t shift, int warn,
-			     unsigned long line);
+			     size_t valsize, size_t shift, int warn);
 
     /** Module-level implementation of yasm_arch_intnum_tobytes().
      * Call yasm_arch_intnum_tobytes() instead of calling this function.
@@ -190,28 +187,28 @@ typedef struct yasm_arch_module {
     int (*intnum_tobytes) (yasm_arch *arch, const yasm_intnum *intn,
 			   unsigned char *buf, size_t destsize, size_t valsize,
 			   int shift, const yasm_bytecode *bc,
-			   int warn, unsigned long line);
+			   int warn);
 
     /** Module-level implementation of yasm_arch_get_reg_size().
      * Call yasm_arch_get_reg_size() instead of calling this function.
      */
-    unsigned int (*get_reg_size) (yasm_arch *arch, unsigned long reg);
+    unsigned int (*get_reg_size) (yasm_arch *arch, uintptr_t reg);
 
     /** Module-level implementation of yasm_arch_reggroup_get_reg().
      * Call yasm_arch_reggroup_get_reg() instead of calling this function.
      */
-    unsigned long (*reggroup_get_reg) (yasm_arch *arch, unsigned long reggroup,
-				       unsigned long regindex);
+    uintptr_t (*reggroup_get_reg) (yasm_arch *arch, uintptr_t reggroup,
+				   unsigned long regindex);
 
     /** Module-level implementation of yasm_arch_reg_print().
      * Call yasm_arch_reg_print() instead of calling this function.
      */
-    void (*reg_print) (yasm_arch *arch, unsigned long reg, FILE *f);
+    void (*reg_print) (yasm_arch *arch, uintptr_t reg, FILE *f);
 
     /** Module-level implementation of yasm_arch_segreg_print().
      * Call yasm_arch_segreg_print() instead of calling this function.
      */
-    void (*segreg_print) (yasm_arch *arch, unsigned long segreg, FILE *f);
+    void (*segreg_print) (yasm_arch *arch, uintptr_t segreg, FILE *f);
 
     /** Module-level implementation of yasm_arch_ea_create().
      * Call yasm_arch_ea_create() instead of calling this function.
@@ -230,7 +227,7 @@ typedef struct yasm_arch_module {
      */
     const char *default_machine_keyword;
 
-    /** Canonical "word" size in bytes.
+    /** Canonical "word" size in bits.
      * Call yasm_arch_wordsize() to get the word size of a particular
      * #yasm_arch.
      */
@@ -250,7 +247,7 @@ struct yasm_insn_operand {
     /*@reldef@*/ STAILQ_ENTRY(yasm_insn_operand) link;
 
     /** Operand type. */
-    enum {
+    enum yasm_insn_operand_type {
 	YASM_INSN__OPERAND_REG = 1,	/**< A register. */
 	YASM_INSN__OPERAND_SEGREG,	/**< A segment register. */
 	YASM_INSN__OPERAND_MEMORY,	/**< An effective address
@@ -260,15 +257,15 @@ struct yasm_insn_operand {
 
     /** Operand data. */
     union {
-	unsigned long reg;  /**< Arch data for reg/segreg. */
+	uintptr_t reg;	    /**< Arch data for reg/segreg. */
 	yasm_effaddr *ea;   /**< Effective address for memory references. */
 	yasm_expr *val;	    /**< Value of immediate or jump target. */
     } data;
 
-    unsigned long targetmod;	/**< Arch target modifier, 0 if none. */
+    uintptr_t targetmod;	/**< Arch target modifier, 0 if none. */
 
     /** Specified size of the operand, in bytes.  0 if not user-specified. */
-    unsigned int size;
+    unsigned int size:8;
 
     /** Nonzero if dereference.  Used for "*foo" in GAS.
      * The reason for this is that by default in GAS, an unprefixed value
@@ -278,7 +275,18 @@ struct yasm_insn_operand {
      * to indicate the "*" prefix has been used, and the arch needs to adjust
      * the operand type appropriately depending on the instruction type.
      */
-    int deref;
+    unsigned int deref:1;
+
+    /** Nonzero if strict.  Used for "strict foo" in NASM.
+     * This is used to inhibit optimization on otherwise "sized" values.
+     * For example, the user may just want to be explicit with the size on
+     * "push dword 4", but not actually want to force the immediate size to
+     * 4 bytes (rather wanting the optimizer to optimize it down to 1 byte as
+     * though "dword" was not specified).  To indicate the immediate should
+     * actually be forced to 4 bytes, the user needs to write
+     * "push strict dword 4", which sets this flag.
+     */
+    unsigned int strict:1;
 };
 #endif
 
@@ -350,10 +358,8 @@ int yasm_arch_set_var(yasm_arch *arch, const char *var, unsigned long val);
  * \param arch		architecture
  * \param cpuid		cpu identifier as in the input file
  * \param cpuid_len	length of cpu identifier string
- * \param line		virtual line (from yasm_linemap)
  */
-void yasm_arch_parse_cpu(yasm_arch *arch, const char *cpuid, size_t cpuid_len,
-			 unsigned long line);
+void yasm_arch_parse_cpu(yasm_arch *arch, const char *cpuid, size_t cpuid_len);
 
 /** Check an generic identifier to see if it matches architecture specific
  * names for instructions or instruction prefixes.  Unrecognized identifiers
@@ -365,12 +371,11 @@ void yasm_arch_parse_cpu(yasm_arch *arch, const char *cpuid, size_t cpuid_len,
  *			[output]
  * \param id		identifier as in the input file
  * \param id_len	length of id string
- * \param line		virtual line (from yasm_linemap)
  * \return Identifier type (#YASM_ARCH_NOTINSNPREFIX if unrecognized)
  */
 yasm_arch_insnprefix yasm_arch_parse_check_insnprefix
-    (yasm_arch *arch, /*@out@*/ unsigned long data[4], const char *id,
-     size_t id_len, unsigned long line);
+    (yasm_arch *arch, /*@out@*/ uintptr_t data[4], const char *id,
+     size_t id_len);
 
 /** Check an generic identifier to see if it matches architecture specific
  * names for registers or target modifiers.  Unrecognized identifiers should
@@ -382,12 +387,11 @@ yasm_arch_insnprefix yasm_arch_parse_check_insnprefix
  *			[output]
  * \param id		identifier as in the input file
  * \param id_len	length of id string
- * \param line		virtual line (from yasm_linemap)
  * \return Identifier type (#YASM_ARCH_NOTREGTMOD if unrecognized)
  */
 yasm_arch_regtmod yasm_arch_parse_check_regtmod
-    (yasm_arch *arch, /*@out@*/ unsigned long *data, const char *id,
-     size_t id_len, unsigned long line);
+    (yasm_arch *arch, /*@out@*/ uintptr_t *data, const char *id,
+     size_t id_len);
 
 /** Handle architecture-specific directives.
  * Should modify behavior ONLY of parse functions, much like parse_cpu().
@@ -402,7 +406,7 @@ yasm_arch_regtmod yasm_arch_parse_check_regtmod
  *	   recognized, even if it wasn't valid.
  */
 int yasm_arch_parse_directive(yasm_arch *arch, const char *name,
-			      yasm_valparamhead *valparams,
+			      /*@null@*/ yasm_valparamhead *valparams,
 			      /*@null@*/ yasm_valparamhead *objext_valparams,
 			      yasm_object *object, unsigned long line);
 
@@ -431,10 +435,9 @@ const unsigned char **yasm_arch_get_fill(const yasm_arch *arch);
  */
 void yasm_arch_finalize_insn
     (yasm_arch *arch, yasm_bytecode *bc, yasm_bytecode *prev_bc,
-     const unsigned long data[4], int num_operands,
+     const uintptr_t data[4], int num_operands,
      /*@null@*/ yasm_insn_operands *operands, int num_prefixes,
-     const unsigned long **prefixes, int num_segregs,
-     const unsigned long *segregs);
+     const uintptr_t **prefixes, int num_segregs, const uintptr_t *segregs);
 
 /** Output #yasm_floatnum to buffer.  Puts the value into the least
  * significant bits of the destination, or may be shifted into more
@@ -448,13 +451,11 @@ void yasm_arch_finalize_insn
  * \param valsize	size (in bits)
  * \param shift		left shift (in bits)
  * \param warn		enables standard overflow/underflow warnings
- * \param line		virtual line; may be 0 if warn is 0.
  * \return Nonzero on error.
  */
 int yasm_arch_floatnum_tobytes(yasm_arch *arch, const yasm_floatnum *flt,
 			       unsigned char *buf, size_t destsize,
-			       size_t valsize, size_t shift, int warn,
-			       unsigned long line);
+			       size_t valsize, size_t shift, int warn);
 
 /** Output #yasm_intnum to buffer.  Puts the value into the least
  * significant bits of the destination, or may be shifted into more
@@ -470,21 +471,19 @@ int yasm_arch_floatnum_tobytes(yasm_arch *arch, const yasm_floatnum *flt,
  * \param bc		bytecode being output ("parent" of value)
  * \param warn		enables standard warnings (value doesn't fit into
  *			valsize bits)
- * \param line		virtual line; may be 0 if warn is 0
  * \return Nonzero on error.
  */
 int yasm_arch_intnum_tobytes(yasm_arch *arch, const yasm_intnum *intn,
 			     unsigned char *buf, size_t destsize,
 			     size_t valsize, int shift,
-			     const yasm_bytecode *bc, int warn,
-			     unsigned long line);
+			     const yasm_bytecode *bc, int warn);
 
-/** Get the equivalent byte size of a register.
+/** Get the equivalent size of a register in bits.
  * \param arch	architecture
  * \param reg	register
  * \return 0 if there is no suitable equivalent size, otherwise the size.
  */
-unsigned int yasm_arch_get_reg_size(yasm_arch *arch, unsigned long reg);
+unsigned int yasm_arch_get_reg_size(yasm_arch *arch, uintptr_t reg);
 
 /** Get a specific register of a register group, based on the register
  * group and the index within the group.
@@ -494,23 +493,22 @@ unsigned int yasm_arch_get_reg_size(yasm_arch *arch, unsigned long reg);
  * \return 0 if regindex is not valid for that register group, otherwise the
  *         specific register value.
  */
-unsigned long yasm_arch_reggroup_get_reg(yasm_arch *arch,
-					 unsigned long reggroup,
-					 unsigned long regindex);
+uintptr_t yasm_arch_reggroup_get_reg(yasm_arch *arch, uintptr_t reggroup,
+				     unsigned long regindex);
 
 /** Print a register.  For debugging purposes.
  * \param arch		architecture
  * \param reg		register
  * \param f		file
  */
-void yasm_arch_reg_print(yasm_arch *arch, unsigned long reg, FILE *f);
+void yasm_arch_reg_print(yasm_arch *arch, uintptr_t reg, FILE *f);
 
 /** Print a segment register.  For debugging purposes.
  * \param arch		architecture
  * \param segreg	segment register
  * \param f		file
  */
-void yasm_arch_segreg_print(yasm_arch *arch, unsigned long segreg, FILE *f);
+void yasm_arch_segreg_print(yasm_arch *arch, uintptr_t segreg, FILE *f);
 
 /** Create an effective address from an expression.
  * \param arch	architecture
@@ -544,14 +542,14 @@ yasm_effaddr *yasm_arch_ea_create(yasm_arch *arch, /*@keep@*/ yasm_expr *e);
     ((yasm_arch_base *)arch)->module->get_address_size(arch)
 #define yasm_arch_set_var(arch, var, val) \
     ((yasm_arch_base *)arch)->module->set_var(arch, var, val)
-#define yasm_arch_parse_cpu(arch, cpuid, cpuid_len, line) \
-    ((yasm_arch_base *)arch)->module->parse_cpu(arch, cpuid, cpuid_len, line)
-#define yasm_arch_parse_check_insnprefix(arch, data, id, id_len, line) \
+#define yasm_arch_parse_cpu(arch, cpuid, cpuid_len) \
+    ((yasm_arch_base *)arch)->module->parse_cpu(arch, cpuid, cpuid_len)
+#define yasm_arch_parse_check_insnprefix(arch, data, id, id_len) \
     ((yasm_arch_base *)arch)->module->parse_check_insnprefix(arch, data, id, \
-							     id_len, line)
-#define yasm_arch_parse_check_regtmod(arch, data, id, id_len, line) \
+							     id_len)
+#define yasm_arch_parse_check_regtmod(arch, data, id, id_len) \
     ((yasm_arch_base *)arch)->module->parse_check_regtmod(arch, data, id, \
-							  id_len, line)
+							  id_len)
 #define yasm_arch_parse_directive(arch, name, valparams, objext_valparams, \
 				  object, line) \
     ((yasm_arch_base *)arch)->module->parse_directive \
@@ -565,13 +563,13 @@ yasm_effaddr *yasm_arch_ea_create(yasm_arch *arch, /*@keep@*/ yasm_expr *e);
 	(arch, bc, prev_bc, data, num_operands, operands, num_prefixes, \
 	 prefixes, num_segregs, segregs)
 #define yasm_arch_floatnum_tobytes(arch, flt, buf, destsize, valsize, shift, \
-				   warn, line) \
+				   warn) \
     ((yasm_arch_base *)arch)->module->floatnum_tobytes \
-	(arch, flt, buf, destsize, valsize, shift, warn, line)
+	(arch, flt, buf, destsize, valsize, shift, warn)
 #define yasm_arch_intnum_tobytes(arch, intn, buf, destsize, valsize, shift, \
-				 bc, warn, line) \
+				 bc, warn) \
     ((yasm_arch_base *)arch)->module->intnum_tobytes \
-	(arch, intn, buf, destsize, valsize, shift, bc, warn, line)
+	(arch, intn, buf, destsize, valsize, shift, bc, warn)
 #define yasm_arch_get_reg_size(arch, reg) \
     ((yasm_arch_base *)arch)->module->get_reg_size(arch, reg)
 #define yasm_arch_reggroup_get_reg(arch, regg, regi) \
@@ -589,13 +587,13 @@ yasm_effaddr *yasm_arch_ea_create(yasm_arch *arch, /*@keep@*/ yasm_expr *e);
  * \param reg	register
  * \return Newly allocated operand.
  */
-yasm_insn_operand *yasm_operand_create_reg(unsigned long reg);
+yasm_insn_operand *yasm_operand_create_reg(uintptr_t reg);
 
 /** Create an instruction operand from a segment register.
  * \param segreg	segment register
  * \return Newly allocated operand.
  */
-yasm_insn_operand *yasm_operand_create_segreg(unsigned long segreg);
+yasm_insn_operand *yasm_operand_create_segreg(uintptr_t segreg);
 
 /** Create an instruction operand from an effective address.
  * \param ea	effective address
