@@ -3,7 +3,7 @@
  * \brief YASM bytecode interface.
  *
  * \rcs
- * $Id: bytecode.h 1591 2006-07-13 03:25:38Z peter $
+ * $Id: bytecode.h 1774 2007-02-18 20:42:55Z peter $
  * \endrcs
  *
  * \license
@@ -58,11 +58,9 @@ struct yasm_effaddr {
 
     yasm_value disp;		/**< address displacement */
 
-    unsigned long segreg;	/**< segment register override (0 if none) */
+    uintptr_t segreg;		/**< segment register override (0 if none) */
 
-    unsigned char disp_len;	/**< length of disp (in bytes), 0 if unknown,
-				 *   0xff if unknown and required to be >0.
-				 */
+    unsigned char need_nonzero_len; /**< 1 if length of disp must be >0. */
     unsigned char need_disp;	/**< 1 if a displacement should be present
 				 *   in the output.
 				 */
@@ -75,14 +73,6 @@ struct yasm_effaddr {
 				 */
 };
 
-/** An immediate value. */
-typedef struct yasm_immval {
-    yasm_value val;		/**< the immediate value itself */
-
-    unsigned char len;		/**< final length (in bytes), 0 if unknown */
-    unsigned char sign;		/**< 1 if final imm is treated as signed */
-} yasm_immval;
-
 /** A data value (opaque type). */
 typedef struct yasm_dataval yasm_dataval;
 /** A list of data values (opaque type). */
@@ -92,20 +82,6 @@ typedef struct yasm_datavalhead yasm_datavalhead;
 /*@reldef@*/ STAILQ_HEAD(yasm_datavalhead, yasm_dataval);
 #endif
 
-/** Return value flags for yasm_bc_resolve(). */
-typedef enum {
-    YASM_BC_RESOLVE_NONE = 0,		/**< Ok, but length is not minimum. */
-    YASM_BC_RESOLVE_ERROR = 1<<0,	/**< Error found, output. */
-    YASM_BC_RESOLVE_MIN_LEN = 1<<1,	/**< Length is minimum possible. */
-    YASM_BC_RESOLVE_UNKNOWN_LEN = 1<<2	/**< Length indeterminate. */
-} yasm_bc_resolve_flags;
-
-/** Create an immediate value from an expression.
- * \param e	expression (kept, do not free).
- * \return Newly allocated immediate value.
- */
-/*@only@*/ yasm_immval *yasm_imm_create_expr(/*@keep@*/ yasm_expr *e);
-
 /** Get the displacement portion of an effective address.
  * \param ea	effective address
  * \return Expression representing the displacement (read-only).
@@ -113,9 +89,9 @@ typedef enum {
 /*@observer@*/ const yasm_expr *yasm_ea_get_disp(const yasm_effaddr *ea);
 
 /** Set the length of the displacement portion of an effective address.
- * The length is specified in bytes.
+ * The length is specified in bits.
  * \param ea	effective address
- * \param len	length in bytes
+ * \param len	length in bits
  */
 void yasm_ea_set_len(yasm_effaddr *ea, unsigned int len);
 
@@ -144,10 +120,8 @@ void yasm_ea_set_strong(yasm_effaddr *ea, unsigned int strong);
  * addresses.  A override of an override will result in a warning.
  * \param ea		effective address
  * \param segreg	segment register (0 if none)
- * \param line		virtual line number
  */
-void yasm_ea_set_segreg(yasm_effaddr *ea, unsigned long segreg,
-			unsigned long line);
+void yasm_ea_set_segreg(yasm_effaddr *ea, uintptr_t segreg);
 
 /** Delete (free allocated memory for) an effective address.
  * \param ea	effective address (only pointer to it).
@@ -174,12 +148,14 @@ void yasm_bc_set_multiple(yasm_bytecode *bc, /*@keep@*/ yasm_expr *e);
  * \param size		storage size (in bytes) for each data value
  * \param append_zero	append a single zero byte after each data value
  *			(if non-zero)
+ * \param arch		architecture (optional); if provided, data items
+ *			are directly simplified to bytes if possible
  * \param line		virtual line (from yasm_linemap)
  * \return Newly allocated bytecode.
  */
 /*@only@*/ yasm_bytecode *yasm_bc_create_data
     (yasm_datavalhead *datahead, unsigned int size, int append_zero,
-     unsigned long line);
+     /*@null@*/ yasm_arch *arch, unsigned long line);
 
 /** Create a bytecode containing LEB128-encoded data value(s).
  * \param datahead	list of data values (kept, do not free)
@@ -201,17 +177,19 @@ void yasm_bc_set_multiple(yasm_bytecode *bc, /*@keep@*/ yasm_expr *e);
      unsigned long line);
 
 /** Create a bytecode that includes a binary file verbatim.
- * \param filename	full path to binary file (kept, do not free)
+ * \param filename	path to binary file (kept, do not free)
  * \param start		starting location in file (in bytes) to read data from
  *			(kept, do not free); may be NULL to indicate 0
  * \param maxlen	maximum number of bytes to read from the file (kept, do
  *			do not free); may be NULL to indicate no maximum
+ * \param linemap	line mapping repository
  * \param line		virtual line (from yasm_linemap) for the bytecode
  * \return Newly allocated bytecode.
  */
 /*@only@*/ yasm_bytecode *yasm_bc_create_incbin
     (/*@only@*/ char *filename, /*@only@*/ /*@null@*/ yasm_expr *start,
-     /*@only@*/ /*@null@*/ yasm_expr *maxlen, unsigned long line);
+     /*@only@*/ /*@null@*/ yasm_expr *maxlen, yasm_linemap *linemap,
+     unsigned long line);
 
 /** Create a bytecode that aligns the following bytecode to a boundary.
  * \param boundary	byte alignment (must be a power of two)
@@ -250,7 +228,7 @@ void yasm_bc_set_multiple(yasm_bytecode *bc, /*@keep@*/ yasm_expr *e);
  *       giving operands to this function.
  */
 /*@only@*/ yasm_bytecode *yasm_bc_create_insn
-    (yasm_arch *arch, const unsigned long insn_data[4], int num_operands,
+    (yasm_arch *arch, const uintptr_t insn_data[4], int num_operands,
      /*@null@*/ yasm_insn_operands *operands, unsigned long line);
 
 /** Create a bytecode that represents a single empty (0 length) instruction.
@@ -267,13 +245,13 @@ void yasm_bc_set_multiple(yasm_bytecode *bc, /*@keep@*/ yasm_expr *e);
  * \param prefix_data	data the identifies the prefix
  */
 void yasm_bc_insn_add_prefix(yasm_bytecode *bc,
-			     const unsigned long prefix_data[4]);
+			     const uintptr_t prefix_data[4]);
 
 /** Associate a segment prefix with an instruction bytecode.
  * \param bc		instruction bytecode
  * \param segreg	data the identifies the segment register
  */
-void yasm_bc_insn_add_seg_prefix(yasm_bytecode *bc, unsigned long segreg);
+void yasm_bc_insn_add_seg_prefix(yasm_bytecode *bc, uintptr_t segreg);
 
 /** Get the section that contains a particular bytecode.
  * \param bc	bytecode
@@ -310,32 +288,69 @@ void yasm_bc_print(const yasm_bytecode *bc, FILE *f, int indent_level);
  */
 void yasm_bc_finalize(yasm_bytecode *bc, yasm_bytecode *prev_bc);
 
-/** Common version of calc_bc_dist that takes offsets from bytecodes.
- * Should be used for the final stages of optimizers as well as in yasm_objfmt
- * yasm_expr output functions.
- * \see yasm_calc_bc_dist_func for parameter descriptions.
+/** Determine the distance between the starting offsets of two bytecodes.
+ * \param precbc1	preceding bytecode to the first bytecode
+ * \param precbc2	preceding bytecode to the second bytecode
+ * \return Distance in bytes between the two bytecodes (bc2-bc1), or NULL if
+ *	   the distance was indeterminate.
+ * \warning Only valid /after/ optimization.
  */
-/*@null@*/ /*@only@*/ yasm_intnum *yasm_common_calc_bc_dist
+/*@null@*/ /*@only@*/ yasm_intnum *yasm_calc_bc_dist
     (yasm_bytecode *precbc1, yasm_bytecode *precbc2);
 
-/** Resolve labels in a bytecode, and calculate its length.
- * Tries to minimize the length as much as possible.
- * \note Sometimes it's impossible to determine if a length is the minimum
- *       possible.  In this case, this function returns that the length is NOT
- *       the minimum.
- * \param bc		bytecode
- * \param save		when zero, this function does \em not modify bc other
- *			than the length/size values (i.e. it doesn't keep the
- *			values returned by calc_bc_dist except temporarily to
- *			try to minimize the length); when nonzero, all fields
- *			in bc may be modified by this function
- * \param calc_bc_dist	function used to determine bytecode distance
- * \return Flags indicating whether the length is the minimum possible,
- *	   indeterminate, and if there was an error recognized (and output)
- *	   during execution.
+/** Get the offset of the next bytecode (the next bytecode doesn't have to
+ * actually exist).
+ * \param precbc	preceding bytecode
+ * \return Offset of the next bytecode in bytes.
+ * \warning Only valid /after/ optimization.
  */
-yasm_bc_resolve_flags yasm_bc_resolve(yasm_bytecode *bc, int save,
-				      yasm_calc_bc_dist_func calc_bc_dist);
+unsigned long yasm_bc_next_offset(yasm_bytecode *precbc);
+
+/** Add a dependent span for a bytecode.
+ * \param add_span_data	add_span_data passed into bc_calc_len()
+ * \param bc		bytecode containing span
+ * \param id		non-zero identifier for span; may be any non-zero value
+ *			if <0, expand is called for any change;
+ *			if >0, expand is only called when exceeds threshold
+ * \param value		dependent value for bytecode expansion
+ * \param neg_thres	negative threshold for long/short decision
+ * \param pos_thres	positive threshold for long/short decision
+ */
+typedef void (*yasm_bc_add_span_func)
+    (void *add_span_data, yasm_bytecode *bc, int id, const yasm_value *value,
+     long neg_thres, long pos_thres);
+
+/** Resolve EQUs in a bytecode and calculate its minimum size.
+ * Generates dependent bytecode spans for cases where, if the length spanned
+ * increases, it could cause the bytecode size to increase.
+ * Any bytecode multiple is NOT included in the length or spans generation;
+ * this must be handled at a higher level.
+ * \param bc		bytecode
+ * \param add_span	function to call to add a span
+ * \param add_span_data	extra data to be passed to add_span function
+ * \return 0 if no error occurred, nonzero if there was an error recognized
+ *         (and output) during execution.
+ * \note May store to bytecode updated expressions and the short length.
+ */
+int yasm_bc_calc_len(yasm_bytecode *bc, yasm_bc_add_span_func add_span,
+		     void *add_span_data);
+
+/** Recalculate a bytecode's length based on an expanded span length.
+ * \param bc		bytecode
+ * \param span		span ID (as given to yasm_bc_add_span_func in
+ *                      yasm_bc_calc_len)
+ * \param old_val	previous span value
+ * \param new_val	new span value
+ * \param neg_thres	negative threshold for long/short decision (returned)
+ * \param pos_thres	postivie threshold for long/short decision (returned)
+ * \return 0 if bc no longer dependent on this span's length, negative if
+ *         there was an error recognized (and output) during execution, and
+ *         positive if bc size may increase for this span further based on the
+ *         new negative and positive thresholds returned.
+ * \note May store to bytecode updated expressions and the updated length.
+ */
+int yasm_bc_expand(yasm_bytecode *bc, int span, long old_val, long new_val,
+		   /*@out@*/ long *neg_thres, /*@out@*/ long *pos_thres);
 
 /** Convert a bytecode into its byte representation.
  * \param bc	 	bytecode
@@ -363,14 +378,15 @@ yasm_bc_resolve_flags yasm_bc_resolve(yasm_bytecode *bc, int save,
      /*@null@*/ yasm_output_reloc_func output_reloc)
     /*@sets *buf@*/;
 
-/** Get the bytecode multiple value as an unsigned long integer.
+/** Get the bytecode multiple value as an integer.
  * \param bc		bytecode
  * \param multiple	multiple value (output)
- * \param calc_bc_dist	bytecode distance calculation function (optional)
- * \return 1 on error (set with yasm__error), 0 on success.
+ * \param calc_bc_dist	nonzero if distances between bytecodes should be
+ *			calculated, 0 if error should be returned in this case
+ * \return 1 on error (set with yasm_error_set), 0 on success.
  */
-int yasm_bc_get_multiple(yasm_bytecode *bc, /*@out@*/ unsigned long *multiple,
-			 /*@null@*/ yasm_calc_bc_dist_func calc_bc_dist);
+int yasm_bc_get_multiple(yasm_bytecode *bc, /*@out@*/ long *multiple,
+			 int calc_bc_dist);
 
 /** Create a new data value from an expression.
  * \param expn	expression
@@ -379,12 +395,25 @@ int yasm_bc_get_multiple(yasm_bytecode *bc, /*@out@*/ unsigned long *multiple,
 yasm_dataval *yasm_dv_create_expr(/*@keep@*/ yasm_expr *expn);
 
 /** Create a new data value from a string.
- * \param contents	string (raw, may contain NULs)
+ * \param contents	string (may contain NULs)
  * \param len		length of string
  * \return Newly allocated data value.
  */
 yasm_dataval *yasm_dv_create_string(/*@keep@*/ char *contents, size_t len);
 
+/** Create a new data value from raw bytes data.
+ * \param contents	raw data (may contain NULs)
+ * \param len		length
+ * \return Newly allocated data value.
+ */
+yasm_dataval *yasm_dv_create_raw(/*@keep@*/ unsigned char *contents,
+				 unsigned long len);
+
+#ifndef YASM_DOXYGEN
+#define yasm_dv_create_string(s, l) yasm_dv_create_raw((unsigned char *)(s), \
+						       (unsigned long)(l))
+#endif
+
 /** Initialize a list of data values.
  * \param headp	list of data values
  */
@@ -396,7 +425,7 @@ void yasm_dvs_initialize(yasm_datavalhead *headp);
 /** Delete (free allocated memory for) a list of data values.
  * \param headp	list of data values
  */
-void yasm_dvs_destroy(yasm_datavalhead *headp);
+void yasm_dvs_delete(yasm_datavalhead *headp);
 
 /** Add data value to the end of a list of data values.
  * \note Does not make a copy of the data value; so don't pass this function
