@@ -3,7 +3,7 @@
  * \brief YASM core types and utility functions.
  *
  * \rcs
- * $Id: coretype.h 1591 2006-07-13 03:25:38Z peter $
+ * $Id: coretype.h 1795 2007-02-24 22:39:55Z peter $
  * \endrcs
  *
  * \license
@@ -40,8 +40,6 @@ typedef struct yasm_arch yasm_arch;
 typedef struct yasm_preproc yasm_preproc;
 /** Parser instance (mostly opaque type).  \see parser.h for details. */
 typedef struct yasm_parser yasm_parser;
-/** Optimizer interface.  \see optimizer.h for details. */
-typedef struct yasm_optimizer yasm_optimizer;
 /** Object format interface.  \see objfmt.h for details. */
 typedef struct yasm_objfmt yasm_objfmt;
 /** Debug format interface.  \see dbgfmt.h for details. */
@@ -66,6 +64,11 @@ typedef struct yasm_assoc_data_callback {
     void (*print) (void *data, FILE *f, int indent_level);
 } yasm_assoc_data_callback;
 
+/** Set of collected error/warnings (opaque type).
+ * \see errwarn.h for details.
+ */
+typedef struct yasm_errwarns yasm_errwarns;
+
 /** Bytecode (opaque type).
  * \see bytecode.h for related functions.
  * Define YASM_BC_INTERNAL to get visible internals.
@@ -140,6 +143,12 @@ typedef struct yasm_value {
      */
     unsigned int ip_rel : 1;
 
+    /** Indicates the value is a jump target address (rather than a simple
+     * data address).  In some objfmt/arch combinations (e.g. macho/amd64)
+     * this info is needed to generate special relocations.
+     */
+    unsigned int jump_target : 1;
+
     /** Indicates the relative portion of the value should be relocated
      * relative to its own section start rather than relative to the
      * section start of the bytecode containing this value.  E.g. the value
@@ -147,12 +156,20 @@ typedef struct yasm_value {
      * section start.  Boolean.
      */
     unsigned int section_rel : 1;
+
+    /** Sign of the value.  Nonzero if the final value should be treated as
+     * signed, 0 if it should be treated as signed.
+     */
+    unsigned int sign : 1;
+
+    /** Size of the value, in bits. */
+    unsigned int size : 8;
 } yasm_value;
 
 /** Maximum value of #yasm_value.rshift */
 #define YASM_VALUE_RSHIFT_MAX	127
 
-/** Line number mapping repository (opaque type).  \see linemgr.h for related
+/** Line number mapping repository (opaque type).  \see linemap.h for related
  * functions.
  */
 typedef struct yasm_linemap yasm_linemap;
@@ -173,7 +190,7 @@ typedef struct yasm_valparamhead yasm_valparamhead;
 typedef struct yasm_insn_operands yasm_insn_operands;
 
 /** Expression operators usable in #yasm_expr expressions. */
-typedef enum {
+typedef enum yasm_expr_op {
     YASM_EXPR_IDENT,	/**< No operation, just a value. */
     YASM_EXPR_ADD,	/**< Arithmetic addition (+). */
     YASM_EXPR_SUB,	/**< Arithmetic subtraction (-). */
@@ -187,12 +204,16 @@ typedef enum {
     YASM_EXPR_OR,	/**< Bitwise OR. */
     YASM_EXPR_AND,	/**< Bitwise AND. */
     YASM_EXPR_XOR,	/**< Bitwise XOR. */
+    YASM_EXPR_XNOR,	/**< Bitwise XNOR. */
     YASM_EXPR_NOR,	/**< Bitwise NOR. */
     YASM_EXPR_SHL,	/**< Shift left (logical). */
     YASM_EXPR_SHR,	/**< Shift right (logical). */
     YASM_EXPR_LOR,	/**< Logical OR. */
     YASM_EXPR_LAND,	/**< Logical AND. */
     YASM_EXPR_LNOT,	/**< Logical negation. */
+    YASM_EXPR_LXOR,	/**< Logical XOR. */
+    YASM_EXPR_LXNOR,	/**< Logical XNOR. */
+    YASM_EXPR_LNOR,	/**< Logical NOR. */
     YASM_EXPR_LT,	/**< Less than comparison. */
     YASM_EXPR_GT,	/**< Greater than comparison. */
     YASM_EXPR_EQ,	/**< Equality comparison. */
@@ -210,7 +231,7 @@ typedef enum {
  * \see symrec.h for related functions.
  * \note YASM_SYM_EXTERN and YASM_SYM_COMMON are mutually exclusive.
  */
-typedef enum {
+typedef enum yasm_sym_vis {
     YASM_SYM_LOCAL = 0,		/**< Default, local only */
     YASM_SYM_GLOBAL = 1 << 0,	/**< If symbol is declared GLOBAL */
     YASM_SYM_COMMON = 1 << 1,	/**< If symbol is declared COMMON */
@@ -218,15 +239,6 @@ typedef enum {
     YASM_SYM_DLOCAL = 1 << 3	/**< If symbol is explicitly declared LOCAL */
 } yasm_sym_vis;
 
-/** Determine the distance between the starting offsets of two bytecodes.
- * \param precbc1	preceding bytecode to the first bytecode
- * \param precbc2	preceding bytecode to the second bytecode
- * \return Distance in bytes between the two bytecodes (bc2-bc1), or NULL if
- *	   the distance was indeterminate.
- */
-typedef /*@null@*/ /*@only@*/ yasm_intnum * (*yasm_calc_bc_dist_func)
-    (yasm_bytecode *precbc1, yasm_bytecode *precbc2);
-
 /** Convert yasm_value to its byte representation.  Usually implemented by
  * object formats to keep track of relocations and verify legal expressions.
  * Must put the value into the least significant bits of the destination,
@@ -235,9 +247,6 @@ typedef /*@null@*/ /*@only@*/ yasm_intnum * (*yasm_calc_bc_dist_func)
  * \param value		value
  * \param buf		buffer for byte representation
  * \param destsize	destination size (in bytes)
- * \param valsize	size (in bits)
- * \param shift		left shift (in bits); may be negative to specify right
- *			shift (standard warnings include truncation to boundary)
  * \param offset	offset (in bytes) of the expr contents from the start
  *			of the bytecode (needed for relative)
  * \param bc		current bytecode (usually passed into higher-level
@@ -251,9 +260,8 @@ typedef /*@null@*/ /*@only@*/ yasm_intnum * (*yasm_calc_bc_dist_func)
  * \return Nonzero if an error occurred, 0 otherwise.
  */
 typedef int (*yasm_output_value_func)
-    (yasm_value *value, /*@out@*/ unsigned char *buf, size_t destsize,
-     size_t valsize, int shift, unsigned long offset, yasm_bytecode *bc,
-     int warn, /*@null@*/ void *d) /*@uses *ep@*/;
+    (yasm_value *value, /*@out@*/ unsigned char *buf, unsigned int destsize,
+     unsigned long offset, yasm_bytecode *bc, int warn, /*@null@*/ void *d);
 
 /** Convert a symbol reference to its byte representation.  Usually implemented
  * by object formats and debug formats to keep track of relocations generated
@@ -273,8 +281,8 @@ typedef int (*yasm_output_value_func)
  * \return Nonzero if an error occurred, 0 otherwise.
  */
 typedef int (*yasm_output_reloc_func)
-    (yasm_symrec *sym, yasm_bytecode *bc, unsigned char *buf, size_t destsize,
-     size_t valsize, int warn, void *d);
+    (yasm_symrec *sym, yasm_bytecode *bc, unsigned char *buf,
+     unsigned int destsize, unsigned int valsize, int warn, void *d);
 
 /** Sort an array using merge sort algorithm.
  * \internal
