@@ -3,7 +3,7 @@
  * \brief YASM debug format interface.
  *
  * \rcs
- * $Id: dbgfmt.h 1591 2006-07-13 03:25:38Z peter $
+ * $Id: dbgfmt.h 1757 2007-02-07 08:29:41Z peter $
  * \endrcs
  *
  * \license
@@ -72,13 +72,14 @@ typedef struct yasm_dbgfmt_module {
      * Call yasm_dbgfmt_directive() instead of calling this function.
      */
     int (*directive) (yasm_dbgfmt *dbgfmt, const char *name,
-		      yasm_section *sect, yasm_valparamhead *valparams,
+		      yasm_section *sect,
+		      /*@null@*/ yasm_valparamhead *valparams,
 		      unsigned long line);
 
     /** Module-level implementation of yasm_dbgfmt_generate().
      * Call yasm_dbgfmt_generate() instead of calling this function.
      */
-    void (*generate) (yasm_dbgfmt *dbgfmt);
+    void (*generate) (yasm_dbgfmt *dbgfmt, yasm_errwarns *errwarns);
 } yasm_dbgfmt_module;
 
 /** Get the keyword used to select a debug format.
@@ -115,13 +116,16 @@ void yasm_dbgfmt_destroy(/*@only@*/ yasm_dbgfmt *dbgfmt);
  *	       recognized even if it wasn't valid.
  */
 int yasm_dbgfmt_directive(yasm_dbgfmt *dbgfmt, const char *name,
-			  yasm_section *sect, yasm_valparamhead *valparams,
+			  yasm_section *sect,
+			  /*@null@*/ yasm_valparamhead *valparams,
 			  unsigned long line);
 
 /** Generate debugging information bytecodes.
  * \param dbgfmt	debug format
+ * \param errwarns	error/warning set
+ * \note Errors and warnings are stored into errwarns.
  */
-void yasm_dbgfmt_generate(yasm_dbgfmt *dbgfmt);
+void yasm_dbgfmt_generate(yasm_dbgfmt *dbgfmt, yasm_errwarns *errwarns);
 
 #ifndef YASM_DOXYGEN
 
@@ -138,8 +142,8 @@ void yasm_dbgfmt_generate(yasm_dbgfmt *dbgfmt);
 #define yasm_dbgfmt_directive(dbgfmt, name, sect, valparams, line) \
     ((yasm_dbgfmt_base *)dbgfmt)->module->directive(dbgfmt, name, sect, \
 						    valparams, line)
-#define yasm_dbgfmt_generate(dbgfmt) \
-    ((yasm_dbgfmt_base *)dbgfmt)->module->generate(dbgfmt)
+#define yasm_dbgfmt_generate(dbgfmt, ews) \
+    ((yasm_dbgfmt_base *)dbgfmt)->module->generate(dbgfmt, ews)
 
 #endif
 
