@@ -1,4 +1,4 @@
-/* $Id: expr-int.h 1591 2006-07-13 03:25:38Z peter $
+/* $Id: expr-int.h 1786 2007-02-24 04:16:05Z peter $
  * Expressions internal structures header file
  *
  *  Copyright (C) 2001  Peter Johnson
@@ -27,25 +27,31 @@
 #ifndef YASM_EXPR_INT_H
 #define YASM_EXPR_INT_H
 
-/* Types listed in canonical sorting order.  See expr_order_terms(). */
-typedef enum {
+/* Types listed in canonical sorting order.  See expr_order_terms().
+ * Note precbc must be used carefully (in a-b pairs), as only symrecs can
+ * become the relative term in a #yasm_value.
+ */
+typedef enum yasm_expr__type {
     YASM_EXPR_NONE = 0,
     YASM_EXPR_REG = 1<<0,
     YASM_EXPR_INT = 1<<1,
-    YASM_EXPR_FLOAT = 1<<2,
-    YASM_EXPR_SYM = 1<<3,
-    YASM_EXPR_SYMEXP = 1<<4, /* post-expanded sym (due to absolute expansion) */
-    YASM_EXPR_EXPR = 1<<5
+    YASM_EXPR_SUBST = 1<<2,
+    YASM_EXPR_FLOAT = 1<<3,
+    YASM_EXPR_SYM = 1<<4,
+    YASM_EXPR_PRECBC = 1<<5, /* direct bytecode ref (rather than via symrec) */
+    YASM_EXPR_EXPR = 1<<6
 } yasm_expr__type;
 
 struct yasm_expr__item {
     yasm_expr__type type;
     union {
+	yasm_bytecode *precbc;
 	yasm_symrec *sym;
 	yasm_expr *expn;
 	yasm_intnum *intn;
 	yasm_floatnum *flt;
-	unsigned long reg;
+	uintptr_t reg;
+	unsigned int subst;
     } data;
 };
 
@@ -85,4 +91,28 @@ yasm_expr *yasm_expr__copy_except(const yasm_expr *e, int except);
 
 int yasm_expr__contains(const yasm_expr *e, yasm_expr__type t);
 
+/** Transform symrec-symrec terms in expression into YASM_EXPR_SUBST items.
+ * Calls the callback function for each symrec-symrec term.
+ * \param ep		expression (pointer to)
+ * \param cbd		callback data passed to callback function
+ * \param callback	callback function: given subst index for bytecode
+ *			pair, bytecode pair (bc2-bc1), and cbd (callback data)
+ * \return Number of transformations made.
+ */
+int yasm_expr__bc_dist_subst(yasm_expr **ep, void *cbd,
+			     void (*callback) (unsigned int subst,
+					       yasm_bytecode *precbc,
+					       yasm_bytecode *precbc2,
+					       void *cbd));
+
+/** Substitute items into expr YASM_EXPR_SUBST items (by index).  Items are
+ * copied, so caller is responsible for freeing array of items.
+ * \param e		expression
+ * \param num_items	number of items in items array
+ * \param items		items array
+ * \return 1 on error (index out of range).
+ */
+int yasm_expr__subst(yasm_expr *e, unsigned int num_items,
+		     const yasm_expr__item *items);
+
 #endif
