@@ -26,8 +26,9 @@
  */
 #define YASM_LIB_INTERNAL
 #include "util.h"
-/*@unused@*/ RCSID("$Id: expr.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: expr.c 1795 2007-02-24 22:39:55Z peter $");
 
+#include "libyasm-stdint.h"
 #include "coretype.h"
 #include "bitvect.h"
 
@@ -49,6 +50,13 @@ static int expr_traverse_nodes_post(/*@null@*/ yasm_expr *e,
 				    /*@null@*/ void *d,
 				    int (*func) (/*@null@*/ yasm_expr *e,
 						 /*@null@*/ void *d));
+static void expr_delete_term(yasm_expr__item *term, int recurse);
+
+/* Bitmap of used items.  We should really never need more than 2 at a time,
+ * so 31 is pretty much overkill.
+ */
+static unsigned long itempool_used = 0;
+static yasm_expr__item itempool[31];
 
 /* allocate a new expression node, with children as defined.
  * If it's a unary operator, put the element in left and set right=NULL. */
@@ -58,6 +66,7 @@ yasm_expr_create(yasm_expr_op op, yasm_expr__item *left,
 		 yasm_expr__item *right, unsigned long line)
 {
     yasm_expr *ptr, *sube;
+    unsigned long z;
     ptr = yasm_xmalloc(sizeof(yasm_expr));
 
     ptr->op = op;
@@ -66,7 +75,10 @@ yasm_expr_create(yasm_expr_op op, yasm_expr__item *left,
     ptr->terms[1].type = YASM_EXPR_NONE;
     if (left) {
 	ptr->terms[0] = *left;	/* structure copy */
-	yasm_xfree(left);
+	z = (unsigned long)(left-itempool);
+	if (z>=31)
+	    yasm_internal_error(N_("could not find expritem in pool"));
+	itempool_used &= ~(1<<z);
 	ptr->numterms++;
 
 	/* Search downward until we find something *other* than an
@@ -86,7 +98,10 @@ yasm_expr_create(yasm_expr_op op, yasm_expr__item *left,
 
     if (right) {
 	ptr->terms[1] = *right;	/* structure copy */
-	yasm_xfree(right);
+	z = (unsigned long)(right-itempool);
+	if (z>=31)
+	    yasm_internal_error(N_("could not find expritem in pool"));
+	itempool_used &= ~(1<<z);
 	ptr->numterms++;
 
 	/* Search downward until we find something *other* than an
@@ -109,10 +124,35 @@ yasm_expr_create(yasm_expr_op op, yasm_expr__item *left,
 /*@=compmempass@*/
 
 /* helpers */
+static yasm_expr__item *
+expr_get_item(void)
+{
+    int z = 0;
+    unsigned long v = itempool_used & 0x7fffffff;
+
+    while (v & 1) {
+	v >>= 1;
+	z++;
+    }
+    if (z>=31)
+	yasm_internal_error(N_("too many expritems"));
+    itempool_used |= 1<<z;
+    return &itempool[z];
+}
+
+yasm_expr__item *
+yasm_expr_precbc(yasm_bytecode *precbc)
+{
+    yasm_expr__item *e = expr_get_item();
+    e->type = YASM_EXPR_PRECBC;
+    e->data.precbc = precbc;
+    return e;
+}
+
 yasm_expr__item *
 yasm_expr_sym(yasm_symrec *s)
 {
-    yasm_expr__item *e = yasm_xmalloc(sizeof(yasm_expr__item));
+    yasm_expr__item *e = expr_get_item();
     e->type = YASM_EXPR_SYM;
     e->data.sym = s;
     return e;
@@ -121,7 +161,7 @@ yasm_expr_sym(yasm_symrec *s)
 yasm_expr__item *
 yasm_expr_expr(yasm_expr *x)
 {
-    yasm_expr__item *e = yasm_xmalloc(sizeof(yasm_expr__item));
+    yasm_expr__item *e = expr_get_item();
     e->type = YASM_EXPR_EXPR;
     e->data.expn = x;
     return e;
@@ -130,7 +170,7 @@ yasm_expr_expr(yasm_expr *x)
 yasm_expr__item *
 yasm_expr_int(yasm_intnum *i)
 {
-    yasm_expr__item *e = yasm_xmalloc(sizeof(yasm_expr__item));
+    yasm_expr__item *e = expr_get_item();
     e->type = YASM_EXPR_INT;
     e->data.intn = i;
     return e;
@@ -139,32 +179,37 @@ yasm_expr_int(yasm_intnum *i)
 yasm_expr__item *
 yasm_expr_float(yasm_floatnum *f)
 {
-    yasm_expr__item *e = yasm_xmalloc(sizeof(yasm_expr__item));
+    yasm_expr__item *e = expr_get_item();
     e->type = YASM_EXPR_FLOAT;
     e->data.flt = f;
     return e;
 }
 
 yasm_expr__item *
-yasm_expr_reg(unsigned long reg)
+yasm_expr_reg(uintptr_t reg)
 {
-    yasm_expr__item *e = yasm_xmalloc(sizeof(yasm_expr__item));
+    yasm_expr__item *e = expr_get_item();
     e->type = YASM_EXPR_REG;
     e->data.reg = reg;
     return e;
 }
 
-/* Transforms instances of symrec-symrec [symrec+(-1*symrec)] into integers if
- * possible.  Uses a simple n^2 algorithm because n is usually quite small.
+/* Transforms instances of symrec-symrec [symrec+(-1*symrec)] into single
+ * expritems if possible.  Uses a simple n^2 algorithm because n is usually
+ * quite small.  Also works for precbc-precbc (or symrec-precbc,
+ * precbc-symrec).
  */
 static /*@only@*/ yasm_expr *
-expr_xform_bc_dist(/*@returned@*/ /*@only@*/ yasm_expr *e,
-		   yasm_calc_bc_dist_func calc_bc_dist)
+expr_xform_bc_dist_base(/*@returned@*/ /*@only@*/ yasm_expr *e,
+			/*@null@*/ void *cbd,
+			int (*callback) (yasm_expr__item *ei,
+					 yasm_bytecode *precbc,
+					 yasm_bytecode *precbc2,
+					 void *cbd))
 {
     int i;
     /*@dependent@*/ yasm_section *sect;
     /*@dependent@*/ /*@null@*/ yasm_bytecode *precbc;
-    /*@null@*/ yasm_intnum *dist;
     int numterms;
 
     /* Handle symrec-symrec in ADD exprs by looking for (-1*symrec) and
@@ -177,7 +222,7 @@ expr_xform_bc_dist(/*@returned@*/ /*@only@*/ yasm_expr *e,
 	int j;
 	yasm_expr *sube;
 	yasm_intnum *intn;
-	yasm_symrec *sym;
+	yasm_symrec *sym = NULL;
 	/*@dependent@*/ yasm_section *sect2;
 	/*@dependent@*/ /*@null@*/ yasm_bytecode *precbc2;
 
@@ -190,13 +235,19 @@ expr_xform_bc_dist(/*@returned@*/ /*@only@*/ yasm_expr *e,
 
 	if (sube->terms[0].type == YASM_EXPR_INT &&
 	    (sube->terms[1].type == YASM_EXPR_SYM ||
-	     sube->terms[1].type == YASM_EXPR_SYMEXP)) {
+	     sube->terms[1].type == YASM_EXPR_PRECBC)) {
 	    intn = sube->terms[0].data.intn;
-	    sym = sube->terms[1].data.sym;
+	    if (sube->terms[1].type == YASM_EXPR_PRECBC)
+		precbc = sube->terms[1].data.precbc;
+	    else
+		sym = sube->terms[1].data.sym;
 	} else if ((sube->terms[0].type == YASM_EXPR_SYM ||
-		    sube->terms[0].type == YASM_EXPR_SYMEXP) &&
+		    sube->terms[0].type == YASM_EXPR_PRECBC) &&
 		   sube->terms[1].type == YASM_EXPR_INT) {
-	    sym = sube->terms[0].data.sym;
+	    if (sube->terms[0].type == YASM_EXPR_PRECBC)
+		precbc = sube->terms[0].data.precbc;
+	    else
+		sym = sube->terms[0].data.sym;
 	    intn = sube->terms[1].data.intn;
 	} else
 	    continue;
@@ -204,21 +255,19 @@ expr_xform_bc_dist(/*@returned@*/ /*@only@*/ yasm_expr *e,
 	if (!yasm_intnum_is_neg1(intn))
 	    continue;
 
-	if (!yasm_symrec_get_label(sym, &precbc))
+	if (sym && !yasm_symrec_get_label(sym, &precbc))
 	    continue;
 	sect2 = yasm_bc_get_section(precbc);
 
 	/* Now look for a symrec term in the same segment */
 	for (j=0; j<e->numterms; j++) {
-	    if ((e->terms[j].type == YASM_EXPR_SYM ||
-		 e->terms[j].type == YASM_EXPR_SYMEXP) &&
-		yasm_symrec_get_label(e->terms[j].data.sym, &precbc2) &&
+	    if (((e->terms[j].type == YASM_EXPR_SYM &&
+		  yasm_symrec_get_label(e->terms[j].data.sym, &precbc2)) ||
+	         (e->terms[j].type == YASM_EXPR_PRECBC &&
+		  (precbc2 = e->terms[j].data.precbc)))	&&
 		(sect = yasm_bc_get_section(precbc2)) &&
 		sect == sect2 &&
-		(dist = calc_bc_dist(precbc, precbc2))) {
-		/* Change the symrec term to an integer */
-		e->terms[j].type = YASM_EXPR_INT;
-		e->terms[j].data.intn = dist;
+		callback(&e->terms[j], precbc, precbc2, cbd)) {
 		/* Delete the matching (-1*symrec) term */
 		yasm_expr_destroy(sube);
 		e->terms[i].type = YASM_EXPR_NONE;
@@ -244,6 +293,72 @@ expr_xform_bc_dist(/*@returned@*/ /*@only@*/ yasm_expr *e,
     return e;
 }
 
+static int
+expr_xform_bc_dist_cb(yasm_expr__item *ei, yasm_bytecode *precbc,
+		      yasm_bytecode *precbc2, /*@null@*/ void *d)
+{
+    yasm_intnum *dist = yasm_calc_bc_dist(precbc, precbc2);
+    if (!dist)
+	return 0;
+    /* Change the term to an integer */
+    ei->type = YASM_EXPR_INT;
+    ei->data.intn = dist;
+    return 1;
+}
+
+/* Transforms instances of symrec-symrec [symrec+(-1*symrec)] into integers if
+ * possible.
+ */
+static /*@only@*/ yasm_expr *
+expr_xform_bc_dist(/*@returned@*/ /*@only@*/ yasm_expr *e)
+{
+    return expr_xform_bc_dist_base(e, NULL, expr_xform_bc_dist_cb);
+}
+
+typedef struct bc_dist_subst_cbd {
+    void (*callback) (unsigned int subst, yasm_bytecode *precbc,
+		      yasm_bytecode *precbc2, void *cbd);
+    void *cbd;
+    unsigned int subst;
+} bc_dist_subst_cbd;
+
+static int
+expr_bc_dist_subst_cb(yasm_expr__item *ei, yasm_bytecode *precbc,
+		      yasm_bytecode *precbc2, /*@null@*/ void *d)
+{
+    bc_dist_subst_cbd *my_cbd = d;
+    assert(my_cbd != NULL);
+    /* Call higher-level callback */
+    my_cbd->callback(my_cbd->subst, precbc, precbc2, my_cbd->cbd);
+    /* Change the term to an subst */
+    ei->type = YASM_EXPR_SUBST;
+    ei->data.subst = my_cbd->subst;
+    my_cbd->subst++;
+    return 1;
+}
+
+static yasm_expr *
+expr_xform_bc_dist_subst(yasm_expr *e, void *d)
+{
+    return expr_xform_bc_dist_base(e, d, expr_bc_dist_subst_cb);
+}
+
+int
+yasm_expr__bc_dist_subst(yasm_expr **ep, void *cbd,
+			 void (*callback) (unsigned int subst,
+					   yasm_bytecode *precbc,
+					   yasm_bytecode *precbc2,
+					   void *cbd))
+{
+    bc_dist_subst_cbd my_cbd;	/* callback info for low-level callback */
+    my_cbd.callback = callback;
+    my_cbd.cbd = cbd;
+    my_cbd.subst = 0;
+    *ep = yasm_expr__level_tree(*ep, 1, 1, 1, 0, &expr_xform_bc_dist_subst,
+				&my_cbd);
+    return my_cbd.subst;
+}
+
 /* Negate just a single ExprItem by building a -1*ei subexpression */
 static void
 expr_xform_neg_item(yasm_expr *e, yasm_expr__item *ei)
@@ -304,8 +419,7 @@ expr_xform_neg_helper(/*@returned@*/ /*@only@*/ yasm_expr *e)
 	     * floatnums present below; if there ARE floatnums, recurse.
 	     */
 	    if (e->terms[0].type == YASM_EXPR_FLOAT)
-		yasm_floatnum_calc(e->terms[0].data.flt, YASM_EXPR_NEG, NULL,
-				   e->line);
+		yasm_floatnum_calc(e->terms[0].data.flt, YASM_EXPR_NEG, NULL);
 	    else if (e->terms[0].type == YASM_EXPR_EXPR &&
 		yasm_expr__contains(e->terms[0].data.expn, YASM_EXPR_FLOAT))
 		    expr_xform_neg_helper(e->terms[0].data.expn);
@@ -373,8 +487,10 @@ expr_xform_neg(/*@returned@*/ /*@only@*/ yasm_expr *e)
 static int
 expr_is_constant(yasm_expr_op op, yasm_intnum *intn)
 {
-    return ((yasm_intnum_is_zero(intn) && op == YASM_EXPR_MUL) ||
-	    (yasm_intnum_is_zero(intn) && op == YASM_EXPR_AND) ||
+    int iszero = yasm_intnum_is_zero(intn);
+    return ((iszero && op == YASM_EXPR_MUL) ||
+	    (iszero && op == YASM_EXPR_AND) ||
+	    (iszero && op == YASM_EXPR_LAND) ||
 	    (yasm_intnum_is_neg1(intn) && op == YASM_EXPR_OR));
 }
 
@@ -382,24 +498,31 @@ expr_is_constant(yasm_expr_op op, yasm_intnum *intn)
 static int
 expr_can_destroy_int_left(yasm_expr_op op, yasm_intnum *intn)
 {
+    int iszero = yasm_intnum_is_zero(intn);
     return ((yasm_intnum_is_pos1(intn) && op == YASM_EXPR_MUL) ||
-	    (yasm_intnum_is_zero(intn) && op == YASM_EXPR_ADD) ||
+	    (iszero && op == YASM_EXPR_ADD) ||
 	    (yasm_intnum_is_neg1(intn) && op == YASM_EXPR_AND) ||
-	    (yasm_intnum_is_zero(intn) && op == YASM_EXPR_OR));
+	    (!iszero && op == YASM_EXPR_LAND) ||
+	    (iszero && op == YASM_EXPR_OR) ||
+	    (iszero && op == YASM_EXPR_LOR));
 }
 
 /* Look for simple "right" identities like x+|-0, x*&/1 */
 static int
 expr_can_destroy_int_right(yasm_expr_op op, yasm_intnum *intn)
 {
-    return ((yasm_intnum_is_pos1(intn) && op == YASM_EXPR_MUL) ||
-	    (yasm_intnum_is_pos1(intn) && op == YASM_EXPR_DIV) ||
-	    (yasm_intnum_is_zero(intn) && op == YASM_EXPR_ADD) ||
-	    (yasm_intnum_is_zero(intn) && op == YASM_EXPR_SUB) ||
+    int iszero = yasm_intnum_is_zero(intn);
+    int ispos1 = yasm_intnum_is_pos1(intn);
+    return ((ispos1 && op == YASM_EXPR_MUL) ||
+	    (ispos1 && op == YASM_EXPR_DIV) ||
+	    (iszero && op == YASM_EXPR_ADD) ||
+	    (iszero && op == YASM_EXPR_SUB) ||
 	    (yasm_intnum_is_neg1(intn) && op == YASM_EXPR_AND) ||
-	    (yasm_intnum_is_zero(intn) && op == YASM_EXPR_OR) ||
-	    (yasm_intnum_is_zero(intn) && op == YASM_EXPR_SHL) ||
-	    (yasm_intnum_is_zero(intn) && op == YASM_EXPR_SHR));
+	    (!iszero && op == YASM_EXPR_LAND) ||
+	    (iszero && op == YASM_EXPR_OR) ||
+	    (iszero && op == YASM_EXPR_LOR) ||
+	    (iszero && op == YASM_EXPR_SHL) ||
+	    (iszero && op == YASM_EXPR_SHR));
 }
 
 /* Check for and simplify identities.  Returns new number of expr terms.
@@ -454,19 +577,7 @@ expr_simplify_identity(yasm_expr *e, int numterms, int int_term,
 	/* Loop through, deleting everything but the integer term */
 	for (i=0; i<e->numterms; i++)
 	    if (i != int_term)
-		switch (e->terms[i].type) {
-		    case YASM_EXPR_INT:
-			yasm_intnum_destroy(e->terms[i].data.intn);
-			break;
-		    case YASM_EXPR_FLOAT:
-			yasm_floatnum_destroy(e->terms[i].data.flt);
-			break;
-		    case YASM_EXPR_EXPR:
-			yasm_expr_destroy(e->terms[i].data.expn);
-			break;
-		    default:
-			break;
-		}
+		expr_delete_term(&e->terms[i], 1);
 
 	/* Move integer term to the first term (if not already there) */
 	if (int_term != 0)
@@ -476,10 +587,11 @@ expr_simplify_identity(yasm_expr *e, int numterms, int int_term,
 	numterms = 1;
     }
 
-    /* Compute NOT and NEG on single intnum. */
+    /* Compute NOT, NEG, and LNOT on single intnum. */
     if (numterms == 1 && int_term == 0 &&
-	(e->op == YASM_EXPR_NOT || e->op == YASM_EXPR_NEG))
-	yasm_intnum_calc(e->terms[0].data.intn, e->op, NULL, e->line);
+	(e->op == YASM_EXPR_NOT || e->op == YASM_EXPR_NEG ||
+	 e->op == YASM_EXPR_LNOT))
+	yasm_intnum_calc(e->terms[0].data.intn, e->op, NULL);
 
     /* Change expression to IDENT if possible. */
     if (numterms == 1)
@@ -568,7 +680,7 @@ expr_level_op(/*@returned@*/ /*@only@*/ yasm_expr *e, int fold_const,
 	for (i=first_int_term+1, o=first_int_term+1; i<e->numterms; i++) {
 	    if (e->terms[i].type == YASM_EXPR_INT) {
 		yasm_intnum_calc(e->terms[first_int_term].data.intn, e->op,
-				 e->terms[i].data.intn, e->line);
+				 e->terms[i].data.intn);
 		fold_numterms--;
 		level_numterms--;
 		/* make sure to delete folded intnum */
@@ -598,7 +710,8 @@ expr_level_op(/*@returned@*/ /*@only@*/ yasm_expr *e, int fold_const,
      */
     if ((e->op != YASM_EXPR_ADD && e->op != YASM_EXPR_MUL &&
 	 e->op != YASM_EXPR_OR && e->op != YASM_EXPR_AND &&
-	 e->op != YASM_EXPR_XOR) ||
+	 e->op != YASM_EXPR_LOR && e->op != YASM_EXPR_LAND &&
+	 e->op != YASM_EXPR_LXOR && e->op != YASM_EXPR_XOR) ||
 	level_numterms <= fold_numterms) {
 	/* Downsize e if necessary */
 	if (fold_numterms < e->numterms && e->numterms > 2)
@@ -643,8 +756,7 @@ expr_level_op(/*@returned@*/ /*@only@*/ yasm_expr *e, int fold_const,
 			e->terms[first_int_term] = sube->terms[j];  /* struc */
 		    } else {
 			yasm_intnum_calc(e->terms[first_int_term].data.intn,
-					 e->op, sube->terms[j].data.intn,
-					 e->line);
+					 e->op, sube->terms[j].data.intn);
 			/* make sure to delete folded intnum */
 			yasm_intnum_destroy(sube->terms[j].data.intn);
 		    }
@@ -685,136 +797,114 @@ expr_level_op(/*@returned@*/ /*@only@*/ yasm_expr *e, int fold_const,
 }
 /*@=mustfree@*/
 
+typedef SLIST_HEAD(yasm__exprhead, yasm__exprentry) yasm__exprhead;
 typedef struct yasm__exprentry {
     /*@reldef@*/ SLIST_ENTRY(yasm__exprentry) next;
     /*@null@*/ const yasm_expr *e;
 } yasm__exprentry;
 
-/* Level an entire expn tree, expanding equ's as we go */
-yasm_expr *
-yasm_expr__level_tree(yasm_expr *e, int fold_const, int simplify_ident,
-		      int simplify_reg_mul, yasm_calc_bc_dist_func calc_bc_dist,
-		      yasm_expr_xform_func expr_xform_extra,
-		      void *expr_xform_extra_data, yasm__exprhead *eh,
-		      int *error)
+static yasm_expr *
+expr_expand_equ(yasm_expr *e, yasm__exprhead *eh)
 {
     int i;
-    yasm__exprhead eh_local;
     yasm__exprentry ee;
 
-    if (!e)
-	return 0;
-
-    if (!eh) {
-	eh = &eh_local;
-	SLIST_INIT(eh);
-    }
-
-    e = expr_xform_neg(e);
-
-    ee.e = NULL;
-
     /* traverse terms */
     for (i=0; i<e->numterms; i++) {
-	/* Expansion stage first: expand equ's, and expand symrecs that
-	 * reference absolute sections into
-	 * absolute start expr + (symrec - first bc in abs section).
-	 */
-	if (e->terms[i].type == YASM_EXPR_SYM) {
+	const yasm_expr *equ_expr;
+
+	/* Expand equ's. */
+	if (e->terms[i].type == YASM_EXPR_SYM &&
+	    (equ_expr = yasm_symrec_get_equ(e->terms[i].data.sym))) {
 	    yasm__exprentry *np;
-	    const yasm_expr *equ_expr =
-		yasm_symrec_get_equ(e->terms[i].data.sym);
-	    /*@dependent@*/ yasm_section *sect;
-	    /*@dependent@*/ /*@null@*/ yasm_bytecode *precbc;
 
-	    if (equ_expr) {
-		/* Check for circular reference */
-		SLIST_FOREACH(np, eh, next) {
-		    if (np->e == equ_expr) {
-			yasm__error(e->line,
-				    N_("circular reference detected"));
-			if (error)
-			    *error = 1;
-			return e;
-		    }
+	    /* Check for circular reference */
+	    SLIST_FOREACH(np, eh, next) {
+		if (np->e == equ_expr) {
+		    yasm_error_set(YASM_ERROR_TOO_COMPLEX,
+				   N_("circular reference detected"));
+		    return e;
 		}
-
-		e->terms[i].type = YASM_EXPR_EXPR;
-		e->terms[i].data.expn = yasm_expr_copy(equ_expr);
-
-		ee.e = equ_expr;
-		SLIST_INSERT_HEAD(eh, &ee, next);
-	    } else if (yasm_symrec_get_label(e->terms[i].data.sym, &precbc) &&
-		       (sect = yasm_bc_get_section(precbc)) &&
-		       yasm_section_is_absolute(sect)) {
-		const yasm_expr *start = yasm_section_get_start(sect);
-		yasm_expr *sube, *sube2;
-
-		/* Check for circular reference */
-		SLIST_FOREACH(np, eh, next) {
-		    if (np->e == start) {
-			yasm__error(e->line,
-				    N_("circular reference detected"));
-			if (error)
-			    *error = 1;
-			return e;
-		    }
-		}
-
-		sube = yasm_xmalloc(sizeof(yasm_expr));
-		sube->op = YASM_EXPR_SUB;
-		sube->line = e->line;
-		sube->numterms = 2;
-		sube->terms[0].type = YASM_EXPR_SYMEXP;
-		sube->terms[0].data.sym = e->terms[i].data.sym;
-		sube->terms[1].type = YASM_EXPR_SYMEXP;
-		sube->terms[1].data.sym = yasm_section_abs_get_sym(sect);
-		assert(sube->terms[1].data.sym != NULL);
-
-		sube2 = yasm_xmalloc(sizeof(yasm_expr));
-		sube2->op = YASM_EXPR_ADD;
-		sube2->line = e->line;
-		sube2->numterms = 2;
-		sube2->terms[0].type = YASM_EXPR_EXPR;
-		sube2->terms[0].data.expn = yasm_expr_copy(start);
-		sube2->terms[1].type = YASM_EXPR_EXPR;
-		sube2->terms[1].data.expn = sube;
-
-		e->terms[i].type = YASM_EXPR_EXPR;
-		e->terms[i].data.expn = sube2;
-
-		ee.e = start;
-		SLIST_INSERT_HEAD(eh, &ee, next);
 	    }
-	}
 
+	    e->terms[i].type = YASM_EXPR_EXPR;
+	    e->terms[i].data.expn = yasm_expr_copy(equ_expr);
+
+	    /* Remember we saw this equ and recurse */
+	    ee.e = equ_expr;
+	    SLIST_INSERT_HEAD(eh, &ee, next);
+	    e->terms[i].data.expn = expr_expand_equ(e->terms[i].data.expn, eh);
+	    SLIST_REMOVE_HEAD(eh, next);
+	} else if (e->terms[i].type == YASM_EXPR_EXPR)
+	    /* Recurse */
+	    e->terms[i].data.expn = expr_expand_equ(e->terms[i].data.expn, eh);
+    }
+
+    return e;
+}
+
+static yasm_expr *
+expr_level_tree(yasm_expr *e, int fold_const, int simplify_ident,
+		int simplify_reg_mul, int calc_bc_dist,
+		yasm_expr_xform_func expr_xform_extra,
+		void *expr_xform_extra_data)
+{
+    int i;
+
+    e = expr_xform_neg(e);
+
+    /* traverse terms */
+    for (i=0; i<e->numterms; i++) {
+	/* Recurse */
 	if (e->terms[i].type == YASM_EXPR_EXPR)
 	    e->terms[i].data.expn =
-		yasm_expr__level_tree(e->terms[i].data.expn, fold_const,
-				      simplify_ident, simplify_reg_mul,
-				      calc_bc_dist, expr_xform_extra,
-				      expr_xform_extra_data, eh, error);
+		expr_level_tree(e->terms[i].data.expn, fold_const,
+				simplify_ident, simplify_reg_mul, calc_bc_dist,
+				expr_xform_extra, expr_xform_extra_data);
+    }
 
-	if (ee.e) {
-	    SLIST_REMOVE_HEAD(eh, next);
-	    ee.e = NULL;
-	}
+    /* Check for SEG of SEG:OFF, if we match, simplify to just the segment */
+    if (e->op == YASM_EXPR_SEG && e->terms[0].type == YASM_EXPR_EXPR &&
+	e->terms[0].data.expn->op == YASM_EXPR_SEGOFF) {
+	e->op = YASM_EXPR_IDENT;
+	e->terms[0].data.expn->op = YASM_EXPR_IDENT;
+	/* Destroy the second (offset) term */
+	expr_delete_term(&e->terms[1], 1);
     }
 
     /* do callback */
     e = expr_level_op(e, fold_const, simplify_ident, simplify_reg_mul);
     if (calc_bc_dist || expr_xform_extra) {
 	if (calc_bc_dist)
-	    e = expr_xform_bc_dist(e, calc_bc_dist);
+	    e = expr_xform_bc_dist(e);
 	if (expr_xform_extra)
 	    e = expr_xform_extra(e, expr_xform_extra_data);
-	e = yasm_expr__level_tree(e, fold_const, simplify_ident,
-				  simplify_reg_mul, NULL, NULL, NULL, NULL,
-				  error);
+	e = expr_level_tree(e, fold_const, simplify_ident, simplify_reg_mul,
+			    0, NULL, NULL);
     }
     return e;
 }
 
+/* Level an entire expn tree, expanding equ's as we go */
+yasm_expr *
+yasm_expr__level_tree(yasm_expr *e, int fold_const, int simplify_ident,
+		      int simplify_reg_mul, int calc_bc_dist,
+		      yasm_expr_xform_func expr_xform_extra,
+		      void *expr_xform_extra_data)
+{
+    yasm__exprhead eh;
+    SLIST_INIT(&eh);
+
+    if (!e)
+	return 0;
+
+    e = expr_expand_equ(e, &eh);
+    e = expr_level_tree(e, fold_const, simplify_ident, simplify_reg_mul,
+			calc_bc_dist, expr_xform_extra, expr_xform_extra_data);
+
+    return e;
+}
+
 /* Comparison function for expr_order_terms().
  * Assumes ExprType enum is in canonical order.
  */
@@ -846,6 +936,9 @@ yasm_expr__order_terms(yasm_expr *e)
 	case YASM_EXPR_OR:
 	case YASM_EXPR_AND:
 	case YASM_EXPR_XOR:
+	case YASM_EXPR_LOR:
+	case YASM_EXPR_LAND:
+	case YASM_EXPR_LXOR:
 	    /* Use mergesort to sort.  It's fast on already sorted values and a
 	     * stable sort (multiple terms of same type are kept in the same
 	     * order).
@@ -858,6 +951,39 @@ yasm_expr__order_terms(yasm_expr *e)
     }
 }
 
+static void
+expr_item_copy(yasm_expr__item *dest, const yasm_expr__item *src)
+{
+    dest->type = src->type;
+    switch (src->type) {
+	case YASM_EXPR_SYM:
+	    /* Symbols don't need to be copied */
+	    dest->data.sym = src->data.sym;
+	    break;
+	case YASM_EXPR_PRECBC:
+	    /* Nor do direct bytecode references */
+	    dest->data.precbc = src->data.precbc;
+	    break;
+	case YASM_EXPR_EXPR:
+	    dest->data.expn = yasm_expr__copy_except(src->data.expn, -1);
+	    break;
+	case YASM_EXPR_INT:
+	    dest->data.intn = yasm_intnum_copy(src->data.intn);
+	    break;
+	case YASM_EXPR_FLOAT:
+	    dest->data.flt = yasm_floatnum_copy(src->data.flt);
+	    break;
+	case YASM_EXPR_REG:
+	    dest->data.reg = src->data.reg;
+	    break;
+	case YASM_EXPR_SUBST:
+	    dest->data.subst = src->data.subst;
+	    break;
+	default:
+	    break;
+    }
+}
+
 /* Copy entire expression EXCEPT for index "except" at *top level only*. */
 yasm_expr *
 yasm_expr__copy_except(const yasm_expr *e, int except)
@@ -872,34 +998,8 @@ yasm_expr__copy_except(const yasm_expr *e, int except)
     n->line = e->line;
     n->numterms = e->numterms;
     for (i=0; i<e->numterms; i++) {
-	yasm_expr__item *dest = &n->terms[i];
-	const yasm_expr__item *src = &e->terms[i];
-
-	if (i != except) {
-	    dest->type = src->type;
-	    switch (src->type) {
-		case YASM_EXPR_SYM:
-		case YASM_EXPR_SYMEXP:
-		    /* Symbols don't need to be copied */
-		    dest->data.sym = src->data.sym;
-		    break;
-		case YASM_EXPR_EXPR:
-		    dest->data.expn =
-			yasm_expr__copy_except(src->data.expn, -1);
-		    break;
-		case YASM_EXPR_INT:
-		    dest->data.intn = yasm_intnum_copy(src->data.intn);
-		    break;
-		case YASM_EXPR_FLOAT:
-		    dest->data.flt = yasm_floatnum_copy(src->data.flt);
-		    break;
-		case YASM_EXPR_REG:
-		    dest->data.reg = src->data.reg;
-		    break;
-		default:
-		    break;
-	    }
-	}
+	if (i != except)
+	    expr_item_copy(&n->terms[i], &e->terms[i]);
     }
 
     return n;
@@ -911,22 +1011,31 @@ yasm_expr_copy(const yasm_expr *e)
     return yasm_expr__copy_except(e, -1);
 }
 
+static void
+expr_delete_term(yasm_expr__item *term, int recurse)
+{
+    switch (term->type) {
+	case YASM_EXPR_INT:
+	    yasm_intnum_destroy(term->data.intn);
+	    break;
+	case YASM_EXPR_FLOAT:
+	    yasm_floatnum_destroy(term->data.flt);
+	    break;
+	case YASM_EXPR_EXPR:
+	    if (recurse)
+		yasm_expr_destroy(term->data.expn);
+	    break;
+	default:
+	    break;
+    }
+}
+
 static int
 expr_destroy_each(/*@only@*/ yasm_expr *e, /*@unused@*/ void *d)
 {
     int i;
-    for (i=0; i<e->numterms; i++) {
-	switch (e->terms[i].type) {
-	    case YASM_EXPR_INT:
-		yasm_intnum_destroy(e->terms[i].data.intn);
-		break;
-	    case YASM_EXPR_FLOAT:
-		yasm_floatnum_destroy(e->terms[i].data.flt);
-		break;
-	    default:
-		break;	/* none of the other types needs to be deleted */
-	}
-    }
+    for (i=0; i<e->numterms; i++)
+	expr_delete_term(&e->terms[i], 0);
     yasm_xfree(e);	/* free ourselves */
     return 0;	/* don't stop recursion */
 }
@@ -958,6 +1067,33 @@ yasm_expr__contains(const yasm_expr *e, yasm_expr__type t)
     return yasm_expr__traverse_leaves_in_const(e, &t, expr_contains_callback);
 }
 
+typedef struct subst_cbd {
+    unsigned int num_items;
+    const yasm_expr__item *items;
+} subst_cbd;
+
+static int
+expr_subst_callback(yasm_expr__item *ei, void *d)
+{
+    subst_cbd *cbd = d;
+    if (ei->type != YASM_EXPR_SUBST)
+	return 0;
+    if (ei->data.subst >= cbd->num_items)
+	return 1;   /* error */
+    expr_item_copy(ei, &cbd->items[ei->data.subst]);
+    return 0;
+}
+
+int
+yasm_expr__subst(yasm_expr *e, unsigned int num_items,
+		 const yasm_expr__item *items)
+{
+    subst_cbd cbd;
+    cbd.num_items = num_items;
+    cbd.items = items;
+    return yasm_expr__traverse_leaves_in(e, &cbd, expr_subst_callback);
+}
+
 /* Traverse over expression tree, calling func for each operation AFTER the
  * branches (if expressions) have been traversed (eg, postorder
  * traversal).  The data pointer d is passed to each func call.
@@ -1038,6 +1174,31 @@ yasm_expr__traverse_leaves_in(yasm_expr *e, void *d,
     return 0;
 }
 
+yasm_expr *
+yasm_expr_extract_deep_segoff(yasm_expr **ep)
+{
+    yasm_expr *retval;
+    yasm_expr *e = *ep;
+    int i;
+
+    /* Try to extract at this level */
+    retval = yasm_expr_extract_segoff(ep);
+    if (retval)
+	return retval;
+
+    /* Not at this level?  Search any expr children. */
+    for (i=0; i<e->numterms; i++) {
+	if (e->terms[i].type == YASM_EXPR_EXPR) {
+	    retval = yasm_expr_extract_deep_segoff(&e->terms[i].data.expn);
+	    if (retval)
+		return retval;
+	}
+    }
+
+    /* Didn't find one */
+    return NULL;
+}
+
 yasm_expr *
 yasm_expr_extract_segoff(yasm_expr **ep)
 {
@@ -1097,7 +1258,7 @@ yasm_expr_extract_wrt(yasm_expr **ep)
 
 /*@-unqualifiedtrans -nullderef -nullstate -onlytrans@*/
 yasm_intnum *
-yasm_expr_get_intnum(yasm_expr **ep, yasm_calc_bc_dist_func calc_bc_dist)
+yasm_expr_get_intnum(yasm_expr **ep, int calc_bc_dist)
 {
     *ep = yasm_expr_simplify(*ep, calc_bc_dist);
 
@@ -1113,11 +1274,9 @@ const yasm_symrec *
 yasm_expr_get_symrec(yasm_expr **ep, int simplify)
 {
     if (simplify)
-	*ep = yasm_expr_simplify(*ep, NULL);
+	*ep = yasm_expr_simplify(*ep, 0);
 
-    if ((*ep)->op == YASM_EXPR_IDENT &&
-	((*ep)->terms[0].type == YASM_EXPR_SYM ||
-	 (*ep)->terms[0].type == YASM_EXPR_SYMEXP))
+    if ((*ep)->op == YASM_EXPR_IDENT && (*ep)->terms[0].type == YASM_EXPR_SYM)
 	return (*ep)->terms[0].data.sym;
     else
 	return (yasm_symrec *)NULL;
@@ -1125,11 +1284,11 @@ yasm_expr_get_symrec(yasm_expr **ep, int simplify)
 /*@=unqualifiedtrans =nullderef -nullstate -onlytrans@*/
 
 /*@-unqualifiedtrans -nullderef -nullstate -onlytrans@*/
-const unsigned long *
+const uintptr_t *
 yasm_expr_get_reg(yasm_expr **ep, int simplify)
 {
     if (simplify)
-	*ep = yasm_expr_simplify(*ep, NULL);
+	*ep = yasm_expr_simplify(*ep, 0);
 
     if ((*ep)->op == YASM_EXPR_IDENT && (*ep)->terms[0].type == YASM_EXPR_REG)
 	return &((*ep)->terms[0].data.reg);
@@ -1188,6 +1347,9 @@ yasm_expr_print(const yasm_expr *e, FILE *f)
 	case YASM_EXPR_XOR:
 	    strcpy(opstr, "^");
 	    break;
+	case YASM_EXPR_XNOR:
+	    strcpy(opstr, "XNOR");
+	    break;
 	case YASM_EXPR_NOR:
 	    strcpy(opstr, "NOR");
 	    break;
@@ -1206,6 +1368,15 @@ yasm_expr_print(const yasm_expr *e, FILE *f)
 	case YASM_EXPR_LNOT:
 	    strcpy(opstr, "!");
 	    break;
+	case YASM_EXPR_LXOR:
+	    strcpy(opstr, "^^");
+	    break;
+	case YASM_EXPR_LXNOR:
+	    strcpy(opstr, "LXNOR");
+	    break;
+	case YASM_EXPR_LNOR:
+	    strcpy(opstr, "LNOR");
+	    break;
 	case YASM_EXPR_LT:
 	    strcpy(opstr, "<");
 	    break;
@@ -1243,8 +1414,11 @@ yasm_expr_print(const yasm_expr *e, FILE *f)
     }
     for (i=0; i<e->numterms; i++) {
 	switch (e->terms[i].type) {
+	    case YASM_EXPR_PRECBC:
+		fprintf(f, "{%lx}",
+			yasm_bc_next_offset(e->terms[i].data.precbc));
+		break;
 	    case YASM_EXPR_SYM:
-	    case YASM_EXPR_SYMEXP:
 		fprintf(f, "%s", yasm_symrec_get_name(e->terms[i].data.sym));
 		break;
 	    case YASM_EXPR_EXPR:
@@ -1262,6 +1436,9 @@ yasm_expr_print(const yasm_expr *e, FILE *f)
 		/* FIXME */
 		/*yasm_arch_reg_print(arch, e->terms[i].data.reg, f);*/
 		break;
+	    case YASM_EXPR_SUBST:
+		fprintf(f, "[%u]", e->terms[i].data.subst);
+		break;
 	    case YASM_EXPR_NONE:
 		break;
 	}
