@@ -3,7 +3,7 @@
  * \brief YASM expression interface.
  *
  * \rcs
- * $Id: expr.h 1591 2006-07-13 03:25:38Z peter $
+ * $Id: expr.h 1786 2007-02-24 04:16:05Z peter $
  * \endrcs
  *
  * \license
@@ -48,6 +48,12 @@ typedef struct yasm_expr__item yasm_expr__item;
     (yasm_expr_op op, /*@only@*/ yasm_expr__item *a,
      /*@only@*/ /*@null@*/ yasm_expr__item *b, unsigned long line);
 
+/** Create a new preceding-bytecode expression item.
+ * \param precbc    preceding bytecode
+ * \return Newly allocated expression item.
+ */
+/*@only@*/ yasm_expr__item *yasm_expr_precbc(/*@keep@*/ yasm_bytecode *precbc);
+
 /** Create a new symbol expression item.
  * \param sym	    symbol
  * \return Newly allocated expression item.
@@ -76,7 +82,7 @@ typedef struct yasm_expr__item yasm_expr__item;
  * \param reg	    register
  * \return Newly allocated expression item.
  */
-/*@only@*/ yasm_expr__item *yasm_expr_reg(unsigned long reg);
+/*@only@*/ yasm_expr__item *yasm_expr_reg(uintptr_t reg);
 
 /** Create a new expression tree e=l op r.
  * \param l	expression for left side of new expression
@@ -133,46 +139,43 @@ int yasm_expr_is_op(const yasm_expr *e, yasm_expr_op op);
 typedef /*@only@*/ yasm_expr * (*yasm_expr_xform_func)
     (/*@returned@*/ /*@only@*/ yasm_expr *e, /*@null@*/ void *d);
 
-/** Linked list of expression entries.
- * \internal
- * Used internally by yasm_expr__level_tree().
- */
-typedef struct yasm__exprhead yasm__exprhead;
-#ifdef YASM_LIB_INTERNAL
-SLIST_HEAD(yasm__exprhead, yasm__exprentry);
-#endif
-
 /** Level an entire expression tree.
  * \internal
  * \param e		    expression
  * \param fold_const	    enable constant folding if nonzero
  * \param simplify_ident    simplify identities
  * \param simplify_reg_mul  simplify REG*1 identities
- * \param calc_bc_dist	    bytecode distance-calculation function
+ * \param calc_bc_dist	    nonzero if distances between bytecodes should be
+ *			    calculated, 0 if they should be left intact
  * \param expr_xform_extra  extra transformation function
  * \param expr_xform_extra_data	data to pass to expr_xform_extra
- * \param eh		    call with NULL (for internal use in recursion)
- * \param error		    if non-NULL, set to 1 if an error is encountered
- *			    (e.g. circular reference errors)
  * \return Leveled expression.
  */
 /*@only@*/ /*@null@*/ yasm_expr *yasm_expr__level_tree
     (/*@returned@*/ /*@only@*/ /*@null@*/ yasm_expr *e, int fold_const,
-     int simplify_ident, int simplify_reg_mul,
-     /*@null@*/ yasm_calc_bc_dist_func calc_bc_dist,
+     int simplify_ident, int simplify_reg_mul, int calc_bc_dist,
      /*@null@*/ yasm_expr_xform_func expr_xform_extra,
-     /*@null@*/ void *expr_xform_extra_data, /*@null@*/ yasm__exprhead *eh,
-     /*@null@*/ int *error);
+     /*@null@*/ void *expr_xform_extra_data);
 
 /** Simplify an expression as much as possible.  Eliminates extraneous
  * branches and simplifies integer-only subexpressions.  Simplified version
  * of yasm_expr__level_tree().
  * \param e	expression
- * \param cbd	bytecode distance-calculation function
+ * \param cbd	if distance between bytecodes should be calculated
  * \return Simplified expression.
  */
 #define yasm_expr_simplify(e, cbd) \
-    yasm_expr__level_tree(e, 1, 1, 1, cbd, NULL, NULL, NULL, NULL)
+    yasm_expr__level_tree(e, 1, 1, 1, cbd, NULL, NULL)
+
+/** Extract the segment portion of an expression containing SEG:OFF, leaving
+ * the offset.
+ * \param ep		expression (pointer to)
+ * \return NULL if unable to extract a segment (expr does not contain a
+ *         YASM_EXPR_SEGOFF operator), otherwise the segment expression.
+ *         The input expression is modified such that on return, it's the
+ *         offset expression.
+ */
+/*@only@*/ /*@null@*/ yasm_expr *yasm_expr_extract_deep_segoff(yasm_expr **ep);
 
 /** Extract the segment portion of a SEG:OFF expression, leaving the offset.
  * \param ep		expression (pointer to)
@@ -195,13 +198,14 @@ SLIST_HEAD(yasm__exprhead, yasm__exprentry);
 
 /** Get the integer value of an expression if it's just an integer.
  * \param ep		expression (pointer to)
- * \param calc_bc_dist	bytecode distance-calculation function
+ * \param calc_bc_dist	nonzero if distances between bytecodes should be
+ *			calculated, 0 if NULL should be returned in this case
  * \return NULL if the expression is too complex (contains anything other than
  *         integers, ie floats, non-valued labels, registers); otherwise the
  *         intnum value of the expression.
  */
 /*@dependent@*/ /*@null@*/ yasm_intnum *yasm_expr_get_intnum
-    (yasm_expr **ep, /*@null@*/ yasm_calc_bc_dist_func calc_bc_dist);
+    (yasm_expr **ep, int calc_bc_dist);
 
 /** Get the symbol value of an expression if it's just a symbol.
  * \param ep		expression (pointer to)
@@ -218,7 +222,7 @@ SLIST_HEAD(yasm__exprhead, yasm__exprentry);
  * \return NULL if the expression is too complex; otherwise the register value
  *         of the expression.
  */
-/*@dependent@*/ /*@null@*/ const unsigned long *yasm_expr_get_reg
+/*@dependent@*/ /*@null@*/ const uintptr_t *yasm_expr_get_reg
     (yasm_expr **ep, int simplify);
 
 /** Print an expression.  For debugging purposes.
