@@ -24,10 +24,8 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#undef HAVE_CONFIG_H
-#endif
+#include <util.h>
+/*@unused@*/ RCSID("$Id: file.c 1771 2007-02-15 03:05:12Z peter $");
 
 /* Need either unistd.h or direct.h (on Windows) to prototype getcwd() */
 #ifdef HAVE_UNISTD_H
@@ -38,12 +36,139 @@
 
 #include <ctype.h>
 
-#define YASM_LIB_INTERNAL
-#include "util.h"
-/*@unused@*/ RCSID("$Id: file.c 1591 2006-07-13 03:25:38Z peter $");
-
+#include "errwarn.h"
 #include "file.h"
 
+#define BSIZE	8192	    /* Fill block size */
+
+
+void
+yasm_scanner_initialize(yasm_scanner *s)
+{
+    s->bot = NULL;
+    s->tok = NULL;
+    s->ptr = NULL;
+    s->cur = NULL;
+    s->lim = NULL;
+    s->top = NULL;
+    s->eof = NULL;
+}
+
+void
+yasm_scanner_delete(yasm_scanner *s)
+{
+    if (s->bot) {
+	yasm_xfree(s->bot);
+	s->bot = NULL;
+    }
+}
+
+int
+yasm_fill_helper(yasm_scanner *s, unsigned char **cursor,
+		 size_t (*input_func) (void *d, unsigned char *buf,
+				       size_t max),
+		 void *input_func_data)
+{
+    size_t cnt;
+    int first = 0;
+
+    if (s->eof)
+	return 0;
+
+    cnt = s->tok - s->bot;
+    if (cnt > 0) {
+	memmove(s->bot, s->tok, (size_t)(s->lim - s->tok));
+	s->tok = s->bot;
+	s->ptr -= cnt;
+	*cursor -= cnt;
+	s->lim -= cnt;
+    }
+    if (!s->bot)
+	first = 1;
+    if ((s->top - s->lim) < BSIZE) {
+	unsigned char *buf = yasm_xmalloc((size_t)(s->lim - s->bot) + BSIZE);
+	memcpy(buf, s->tok, (size_t)(s->lim - s->tok));
+	s->tok = buf;
+	s->ptr = &buf[s->ptr - s->bot];
+	*cursor = &buf[*cursor - s->bot];
+	s->lim = &buf[s->lim - s->bot];
+	s->top = &s->lim[BSIZE];
+	if (s->bot)
+	    yasm_xfree(s->bot);
+	s->bot = buf;
+    }
+    if ((cnt = input_func(input_func_data, s->lim, BSIZE)) == 0) {
+	s->eof = &s->lim[cnt];
+	*s->eof++ = '\n';
+    }
+    s->lim += cnt;
+    return first;
+}
+
+void
+yasm_unescape_cstring(unsigned char *str, size_t *len)
+{
+    unsigned char *s = str;
+    unsigned char *o = str;
+    unsigned char t[4];
+
+    while ((size_t)(s-str)<*len) {
+	if (*s == '\\' && (size_t)(&s[1]-str)<*len) {
+	    s++;
+	    switch (*s) {
+		case 'b': *o = '\b'; s++; break;
+		case 'f': *o = '\f'; s++; break;
+		case 'n': *o = '\n'; s++; break;
+		case 'r': *o = '\r'; s++; break;
+		case 't': *o = '\t'; s++; break;
+		case 'x':
+		    /* hex escape; grab last two digits */
+		    s++;
+		    while ((size_t)(&s[2]-str)<*len && isxdigit(s[0])
+			   && isxdigit(s[1]) && isxdigit(s[2]))
+			s++;
+		    if ((size_t)(s-str)<*len && isxdigit(*s)) {
+			t[0] = *s++;
+			t[1] = '\0';
+			t[2] = '\0';
+			if ((size_t)(s-str)<*len && isxdigit(*s))
+			    t[1] = *s++;
+			*o = (unsigned char)strtoul((char *)t, NULL, 16);
+		    } else
+			*o = '\0';
+		    break;
+		default:
+		    if (isdigit(*s)) {
+			int warn = 0;
+			/* octal escape */
+			if (*s > '7')
+			    warn = 1;
+			*o = *s++ - '0';
+			if ((size_t)(s-str)<*len && isdigit(*s)) {
+			    if (*s > '7')
+				warn = 1;
+			    *o <<= 3;
+			    *o += *s++ - '0';
+			    if ((size_t)(s-str)<*len && isdigit(*s)) {
+				if (*s > '7')
+				    warn = 1;
+				*o <<= 3;
+				*o += *s++ - '0';
+			    }
+			}
+			if (warn)
+			    yasm_warn_set(YASM_WARN_GENERAL,
+					  N_("octal value out of range"));
+		    } else
+			*o = *s++;
+		    break;
+	    }
+	    o++;
+	} else
+	    *o++ = *s++;
+    }
+    *len = o-str;
+}
 
 size_t
 yasm__splitpath_unix(const char *path, /*@out@*/ const char **tail)
@@ -331,32 +456,85 @@ yasm__combpath_win(const char *from, const char *to)
     return out;
 }
 
+typedef struct incpath {
+    STAILQ_ENTRY(incpath) link;
+    /*@owned@*/ char *path;
+} incpath;
+
+STAILQ_HEAD(, incpath) incpaths = STAILQ_HEAD_INITIALIZER(incpaths);
+
 FILE *
-yasm__fopen_include(const char *iname, const char *from, const char **paths,
-		    const char *mode, char **oname)
+yasm_fopen_include(const char *iname, const char *from, const char *mode,
+		   char **oname)
 {
     FILE *f;
     char *combine;
-    const char *path;
+    incpath *np;
 
     /* Try directly relative to from first, then each of the include paths */
-    path = from;
-    while (path) {
-	combine = yasm__combpath(path, iname);
+    if (from) {
+	combine = yasm__combpath(from, iname);
 	f = fopen(combine, mode);
 	if (f) {
-	    *oname = combine;
+	    if (oname)
+		*oname = combine;
+	    else
+		yasm_xfree(combine);
 	    return f;
 	}
 	yasm_xfree(combine);
-	if (!paths)
-	    break;
-	path = *paths++;
     }
 
+    STAILQ_FOREACH(np, &incpaths, link) {
+	combine = yasm__combpath(np->path, iname);
+	f = fopen(combine, mode);
+	if (f) {
+	    if (oname)
+		*oname = combine;
+	    else
+		yasm_xfree(combine);
+	    return f;
+	}
+	yasm_xfree(combine);
+    }
+
+    if (oname)
+	*oname = NULL;
     return NULL;
 }
 
+void
+yasm_delete_include_paths(void)
+{
+    incpath *n1, *n2;
+
+    n1 = STAILQ_FIRST(&incpaths);
+    while (n1) {
+	n2 = STAILQ_NEXT(n1, link);
+	yasm_xfree(n1->path);
+	yasm_xfree(n1);
+	n1 = n2;
+    }
+    STAILQ_INIT(&incpaths);
+}
+
+void
+yasm_add_include_path(const char *path)
+{
+    incpath *np = yasm_xmalloc(sizeof(incpath));
+    size_t len = strlen(path);
+
+    np->path = yasm_xmalloc(len+2);
+    memcpy(np->path, path, len+1);
+    /* Add trailing slash if it is missing */
+    if (path[len-1] != '\\' && path[len-1] != '/') {
+	np->path[len] = '/';
+	np->path[len+1] = '\0';
+    }
+
+    STAILQ_INSERT_TAIL(&incpaths, np, link);
+}
+
 size_t
 yasm_fwrite_16_l(unsigned short val, FILE *f)
 {
