@@ -31,17 +31,17 @@
  */
 #define YASM_LIB_INTERNAL
 #include "util.h"
-/*@unused@*/ RCSID("$Id: hamt.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: hamt.c 1786 2007-02-24 04:16:05Z peter $");
 
+#include "libyasm-stdint.h"
 #include "coretype.h"
 #include "hamt.h"
-#include "_stdint.h"			/* for uintptr_t */
 
-typedef struct HAMTEntry {
+struct HAMTEntry {
     STAILQ_ENTRY(HAMTEntry) next;	/* next hash table entry */
     /*@dependent@*/ const char *str;	/* string being hashed */
     /*@owned@*/ void *data;		/* data pointer being stored */
-} HAMTEntry;
+};
 
 typedef struct HAMTNode {
     unsigned long BitMapKey;		/* 32 bits, bitmap or hash key */
@@ -160,6 +160,24 @@ HAMT_traverse(HAMT *hamt, void *d,
     return 0;
 }
 
+const HAMTEntry *
+HAMT_first(const HAMT *hamt)
+{
+    return STAILQ_FIRST(&hamt->entries);
+}
+
+const HAMTEntry *
+HAMT_next(const HAMTEntry *prev)
+{
+    return STAILQ_NEXT(prev, next);
+}
+
+void *
+HAMTEntry_get_data(const HAMTEntry *entry)
+{
+    return entry->data;
+}
+
 /*@-temptrans -kepttrans -mustfree@*/
 void *
 HAMT_insert(HAMT *hamt, const char *str, void *data, int *replace,
