@@ -26,7 +26,7 @@
  */
 #define YASM_LIB_INTERNAL
 #include "util.h"
-/*@unused@*/ RCSID("$Id: intnum.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: intnum.c 1795 2007-02-24 22:39:55Z peter $");
 
 #include <ctype.h>
 #include <limits.h>
@@ -48,7 +48,6 @@ struct yasm_intnum {
 	wordptr bv;		/* bit vector (for integers >32 bits) */
     } val;
     enum { INTNUM_UL, INTNUM_BV } type;
-    unsigned char origsize;	/* original (parsed) size, in bits */
 };
 
 /* static bitvect used for conversions */
@@ -83,16 +82,22 @@ yasm_intnum_cleanup(void)
 }
 
 yasm_intnum *
-yasm_intnum_create_dec(char *str, unsigned long line)
+yasm_intnum_create_dec(char *str)
 {
     yasm_intnum *intn = yasm_xmalloc(sizeof(yasm_intnum));
 
-    intn->origsize = 0;	    /* no reliable way to figure this out */
-
-    if (BitVector_from_Dec_static(from_dec_data, conv_bv,
-				  (unsigned char *)str) == ErrCode_Ovfl)
-	yasm__warning(YASM_WARN_GENERAL, line,
-		      N_("Numeric constant too large for internal format"));
+    switch (BitVector_from_Dec_static(from_dec_data, conv_bv,
+				      (unsigned char *)str)) {
+	case ErrCode_Pars:
+	    yasm_error_set(YASM_ERROR_VALUE, N_("invalid decimal literal"));
+	    break;
+	case ErrCode_Ovfl:
+	    yasm_error_set(YASM_ERROR_OVERFLOW,
+		N_("Numeric constant too large for internal format"));
+	    break;
+	default:
+	    break;
+    }
     if (Set_Max(conv_bv) < 32) {
 	intn->type = INTNUM_UL;
 	intn->val.ul = BitVector_Chunk_Read(conv_bv, 32, 0);
@@ -105,17 +110,21 @@ yasm_intnum_create_dec(char *str, unsigned long line)
 }
 
 yasm_intnum *
-yasm_intnum_create_bin(char *str, unsigned long line)
+yasm_intnum_create_bin(char *str)
 {
     yasm_intnum *intn = yasm_xmalloc(sizeof(yasm_intnum));
 
-    intn->origsize = (unsigned char)strlen(str);
-
-    if(intn->origsize > BITVECT_NATIVE_SIZE)
-	yasm__warning(YASM_WARN_GENERAL, line,
-		      N_("Numeric constant too large for internal format"));
-
-    BitVector_from_Bin(conv_bv, (unsigned char *)str);
+    switch (BitVector_from_Bin(conv_bv, (unsigned char *)str)) {
+	case ErrCode_Pars:
+	    yasm_error_set(YASM_ERROR_VALUE, N_("invalid binary literal"));
+	    break;
+	case ErrCode_Ovfl:
+	    yasm_error_set(YASM_ERROR_OVERFLOW,
+		N_("Numeric constant too large for internal format"));
+	    break;
+	default:
+	    break;
+    }
     if (Set_Max(conv_bv) < 32) {
 	intn->type = INTNUM_UL;
 	intn->val.ul = BitVector_Chunk_Read(conv_bv, 32, 0);
@@ -128,17 +137,21 @@ yasm_intnum_create_bin(char *str, unsigned long line)
 }
 
 yasm_intnum *
-yasm_intnum_create_oct(char *str, unsigned long line)
+yasm_intnum_create_oct(char *str)
 {
     yasm_intnum *intn = yasm_xmalloc(sizeof(yasm_intnum));
 
-    intn->origsize = strlen(str)*3;
-
-    if(intn->origsize > BITVECT_NATIVE_SIZE)
-	yasm__warning(YASM_WARN_GENERAL, line,
-		      N_("Numeric constant too large for internal format"));
-
-    BitVector_from_Oct(conv_bv, (unsigned char *)str);
+    switch (BitVector_from_Oct(conv_bv, (unsigned char *)str)) {
+	case ErrCode_Pars:
+	    yasm_error_set(YASM_ERROR_VALUE, N_("invalid octal literal"));
+	    break;
+	case ErrCode_Ovfl:
+	    yasm_error_set(YASM_ERROR_OVERFLOW,
+		N_("Numeric constant too large for internal format"));
+	    break;
+	default:
+	    break;
+    }
     if (Set_Max(conv_bv) < 32) {
 	intn->type = INTNUM_UL;
 	intn->val.ul = BitVector_Chunk_Read(conv_bv, 32, 0);
@@ -151,17 +164,21 @@ yasm_intnum_create_oct(char *str, unsigned long line)
 }
 
 yasm_intnum *
-yasm_intnum_create_hex(char *str, unsigned long line)
+yasm_intnum_create_hex(char *str)
 {
     yasm_intnum *intn = yasm_xmalloc(sizeof(yasm_intnum));
 
-    intn->origsize = strlen(str)*4;
-
-    if(intn->origsize > BITVECT_NATIVE_SIZE)
-	yasm__warning(YASM_WARN_GENERAL, line,
-		      N_("Numeric constant too large for internal format"));
-
-    BitVector_from_Hex(conv_bv, (unsigned char *)str);
+    switch (BitVector_from_Hex(conv_bv, (unsigned char *)str)) {
+	case ErrCode_Pars:
+	    yasm_error_set(YASM_ERROR_VALUE, N_("invalid hex literal"));
+	    break;
+	case ErrCode_Ovfl:
+	    yasm_error_set(YASM_ERROR_OVERFLOW,
+			   N_("Numeric constant too large for internal format"));
+	    break;
+	default:
+	    break;
+    }
     if (Set_Max(conv_bv) < 32) {
 	intn->type = INTNUM_UL;
 	intn->val.ul = BitVector_Chunk_Read(conv_bv, 32, 0);
@@ -175,16 +192,14 @@ yasm_intnum_create_hex(char *str, unsigned long line)
 
 /*@-usedef -compdef -uniondef@*/
 yasm_intnum *
-yasm_intnum_create_charconst_nasm(const char *str, unsigned long line)
+yasm_intnum_create_charconst_nasm(const char *str)
 {
     yasm_intnum *intn = yasm_xmalloc(sizeof(yasm_intnum));
     size_t len = strlen(str);
 
-    intn->origsize = len*8;
-
-    if(intn->origsize > BITVECT_NATIVE_SIZE)
-	yasm__warning(YASM_WARN_GENERAL, line,
-		      N_("Character constant too large for internal format"));
+    if(len*8 > BITVECT_NATIVE_SIZE)
+	yasm_error_set(YASM_ERROR_OVERFLOW,
+		       N_("Character constant too large for internal format"));
 
     if (len > 4) {
 	BitVector_Empty(conv_bv);
@@ -196,19 +211,19 @@ yasm_intnum_create_charconst_nasm(const char *str, unsigned long line)
 
     switch (len) {
 	case 4:
-	    intn->val.ul |= (unsigned long)str[3];
+	    intn->val.ul |= ((unsigned long)str[3]) & 0xff;
 	    intn->val.ul <<= 8;
 	    /*@fallthrough@*/
 	case 3:
-	    intn->val.ul |= (unsigned long)str[2];
+	    intn->val.ul |= ((unsigned long)str[2]) & 0xff;
 	    intn->val.ul <<= 8;
 	    /*@fallthrough@*/
 	case 2:
-	    intn->val.ul |= (unsigned long)str[1];
+	    intn->val.ul |= ((unsigned long)str[1]) & 0xff;
 	    intn->val.ul <<= 8;
 	    /*@fallthrough@*/
 	case 1:
-	    intn->val.ul |= (unsigned long)str[0];
+	    intn->val.ul |= ((unsigned long)str[0]) & 0xff;
 	case 0:
 	    break;
 	default:
@@ -232,7 +247,6 @@ yasm_intnum_create_uint(unsigned long i)
 
     intn->val.ul = i;
     intn->type = INTNUM_UL;
-    intn->origsize = 0;
 
     return intn;
 }
@@ -253,21 +267,18 @@ yasm_intnum_create_int(long i)
     intn = yasm_xmalloc(sizeof(yasm_intnum));
     intn->val.bv = BitVector_Clone(conv_bv);
     intn->type = INTNUM_BV;
-    intn->origsize = 0;
 
     return intn;
 }
 
 yasm_intnum *
 yasm_intnum_create_leb128(const unsigned char *ptr, int sign,
-			  unsigned long *size, unsigned long line)
+			  unsigned long *size)
 {
     yasm_intnum *intn = yasm_xmalloc(sizeof(yasm_intnum));
     const unsigned char *ptr_orig = ptr;
     unsigned long i = 0;
 
-    intn->origsize = 0;
-
     BitVector_Empty(conv_bv);
     for (;;) {
 	BitVector_Chunk_Store(conv_bv, 7, i, *ptr);
@@ -277,11 +288,11 @@ yasm_intnum_create_leb128(const unsigned char *ptr, int sign,
 	ptr++;
     }
 
-    *size = (ptr-ptr_orig)+1;
+    *size = (unsigned long)(ptr-ptr_orig)+1;
 
     if(i > BITVECT_NATIVE_SIZE)
-	yasm__warning(YASM_WARN_GENERAL, line,
-		      N_("Numeric constant too large for internal format"));
+	yasm_error_set(YASM_ERROR_OVERFLOW,
+		       N_("Numeric constant too large for internal format"));
     else if (sign && (*ptr & 0x40) == 0x40)
 	BitVector_Interval_Fill(conv_bv, i, BITVECT_NATIVE_SIZE-1);
 
@@ -296,6 +307,42 @@ yasm_intnum_create_leb128(const unsigned char *ptr, int sign,
     return intn;
 }
 
+yasm_intnum *
+yasm_intnum_create_sized(unsigned char *ptr, int sign, size_t srcsize,
+			 int bigendian)
+{
+    yasm_intnum *intn = yasm_xmalloc(sizeof(yasm_intnum));
+    unsigned long i = 0;
+
+    if (srcsize*8 > BITVECT_NATIVE_SIZE)
+	yasm_error_set(YASM_ERROR_OVERFLOW,
+		       N_("Numeric constant too large for internal format"));
+
+    /* Read the buffer into a bitvect */
+    BitVector_Empty(conv_bv);
+    if (bigendian) {
+	/* TODO */
+	yasm_internal_error(N_("big endian not implemented"));
+    } else {
+	for (i = 0; i < srcsize; i++)
+	    BitVector_Chunk_Store(conv_bv, 8, i*8, ptr[i]);
+    }
+
+    /* Sign extend if needed */
+    if (srcsize*8 < BITVECT_NATIVE_SIZE && sign && (ptr[i] & 0x80) == 0x80)
+	BitVector_Interval_Fill(conv_bv, i*8, BITVECT_NATIVE_SIZE-1);
+
+    if (Set_Max(conv_bv) < 32) {
+	intn->type = INTNUM_UL;
+	intn->val.ul = BitVector_Chunk_Read(conv_bv, 32, 0);
+    } else {
+	intn->type = INTNUM_BV;
+	intn->val.bv = BitVector_Clone(conv_bv);
+    }
+
+    return intn;
+}
+
 yasm_intnum *
 yasm_intnum_copy(const yasm_intnum *intn)
 {
@@ -310,7 +357,6 @@ yasm_intnum_copy(const yasm_intnum *intn)
 	    break;
     }
     n->type = intn->type;
-    n->origsize = intn->origsize;
 
     return n;
 }
@@ -324,12 +370,12 @@ yasm_intnum_destroy(yasm_intnum *intn)
 }
 
 /*@-nullderef -nullpass -branchstate@*/
-void
-yasm_intnum_calc(yasm_intnum *acc, yasm_expr_op op, yasm_intnum *operand,
-		 unsigned long line)
+int
+yasm_intnum_calc(yasm_intnum *acc, yasm_expr_op op, yasm_intnum *operand)
 {
     boolean carry = 0;
     wordptr op1, op2 = NULL;
+    N_int count;
 
     /* Always do computations with in full bit vector.
      * Bit vector results must be calculated through intermediate storage.
@@ -353,8 +399,12 @@ yasm_intnum_calc(yasm_intnum *acc, yasm_expr_op op, yasm_intnum *operand,
     }
 
     if (!operand && op != YASM_EXPR_NEG && op != YASM_EXPR_NOT &&
-	op != YASM_EXPR_LNOT)
-	yasm_internal_error(N_("Operation needs an operand"));
+	op != YASM_EXPR_LNOT) {
+	yasm_error_set(YASM_ERROR_ARITHMETIC,
+		       N_("operation needs an operand"));
+	BitVector_Empty(result);
+	return 1;
+    }
 
     /* A operation does a bitvector computation if result is allocated. */
     switch (op) {
@@ -369,17 +419,37 @@ yasm_intnum_calc(yasm_intnum *acc, yasm_expr_op op, yasm_intnum *operand,
 	    break;
 	case YASM_EXPR_DIV:
 	    /* TODO: make sure op1 and op2 are unsigned */
-	    BitVector_Divide(result, op1, op2, spare);
+	    if (BitVector_is_empty(op2)) {
+		yasm_error_set(YASM_ERROR_ZERO_DIVISION, N_("divide by zero"));
+		BitVector_Empty(result);
+		return 1;
+	    } else
+		BitVector_Divide(result, op1, op2, spare);
 	    break;
 	case YASM_EXPR_SIGNDIV:
-	    BitVector_Divide(result, op1, op2, spare);
+	    if (BitVector_is_empty(op2)) {
+		yasm_error_set(YASM_ERROR_ZERO_DIVISION, N_("divide by zero"));
+		BitVector_Empty(result);
+		return 1;
+	    } else
+		BitVector_Divide(result, op1, op2, spare);
 	    break;
 	case YASM_EXPR_MOD:
 	    /* TODO: make sure op1 and op2 are unsigned */
-	    BitVector_Divide(spare, op1, op2, result);
+	    if (BitVector_is_empty(op2)) {
+		yasm_error_set(YASM_ERROR_ZERO_DIVISION, N_("divide by zero"));
+		BitVector_Empty(result);
+		return 1;
+	    } else
+		BitVector_Divide(spare, op1, op2, result);
 	    break;
 	case YASM_EXPR_SIGNMOD:
-	    BitVector_Divide(spare, op1, op2, result);
+	    if (BitVector_is_empty(op2)) {
+		yasm_error_set(YASM_ERROR_ZERO_DIVISION, N_("divide by zero"));
+		BitVector_Empty(result);
+		return 1;
+	    } else
+		BitVector_Divide(spare, op1, op2, result);
 	    break;
 	case YASM_EXPR_NEG:
 	    BitVector_Negate(result, op1);
@@ -396,6 +466,10 @@ yasm_intnum_calc(yasm_intnum *acc, yasm_expr_op op, yasm_intnum *operand,
 	case YASM_EXPR_XOR:
 	    Set_ExclusiveOr(result, op1, op2);
 	    break;
+	case YASM_EXPR_XNOR:
+	    Set_ExclusiveOr(result, op1, op2);
+	    Set_Complement(result, result);
+	    break;
 	case YASM_EXPR_NOR:
 	    Set_Union(result, op1, op2);
 	    Set_Complement(result, result);
@@ -410,7 +484,10 @@ yasm_intnum_calc(yasm_intnum *acc, yasm_expr_op op, yasm_intnum *operand,
 	case YASM_EXPR_SHR:
 	    if (operand->type == INTNUM_UL) {
 		BitVector_Copy(result, op1);
-		BitVector_Move_Right(result, (N_int)operand->val.ul);
+		carry = BitVector_msb_(op1);
+		count = (N_int)operand->val.ul;
+		while (count-- > 0)
+		    BitVector_shift_right(result, carry);
 	    } else	/* don't even bother, just zero result */
 		BitVector_Empty(result);
 	    break;
@@ -428,45 +505,66 @@ yasm_intnum_calc(yasm_intnum *acc, yasm_expr_op op, yasm_intnum *operand,
 	    BitVector_Empty(result);
 	    BitVector_LSB(result, BitVector_is_empty(op1));
 	    break;
+	case YASM_EXPR_LXOR:
+	    BitVector_Empty(result);
+	    BitVector_LSB(result, !BitVector_is_empty(op1) ^
+			  !BitVector_is_empty(op2));
+	    break;
+	case YASM_EXPR_LXNOR:
+	    BitVector_Empty(result);
+	    BitVector_LSB(result, !(!BitVector_is_empty(op1) ^
+			  !BitVector_is_empty(op2)));
+	    break;
+	case YASM_EXPR_LNOR:
+	    BitVector_Empty(result);
+	    BitVector_LSB(result, !(!BitVector_is_empty(op1) ||
+			  !BitVector_is_empty(op2)));
+	    break;
 	case YASM_EXPR_EQ:
 	    BitVector_Empty(result);
 	    BitVector_LSB(result, BitVector_equal(op1, op2));
 	    break;
 	case YASM_EXPR_LT:
 	    BitVector_Empty(result);
-	    BitVector_LSB(result, BitVector_Lexicompare(op1, op2) < 0);
+	    BitVector_LSB(result, BitVector_Compare(op1, op2) < 0);
 	    break;
 	case YASM_EXPR_GT:
 	    BitVector_Empty(result);
-	    BitVector_LSB(result, BitVector_Lexicompare(op1, op2) > 0);
+	    BitVector_LSB(result, BitVector_Compare(op1, op2) > 0);
 	    break;
 	case YASM_EXPR_LE:
 	    BitVector_Empty(result);
-	    BitVector_LSB(result, BitVector_Lexicompare(op1, op2) <= 0);
+	    BitVector_LSB(result, BitVector_Compare(op1, op2) <= 0);
 	    break;
 	case YASM_EXPR_GE:
 	    BitVector_Empty(result);
-	    BitVector_LSB(result, BitVector_Lexicompare(op1, op2) >= 0);
+	    BitVector_LSB(result, BitVector_Compare(op1, op2) >= 0);
 	    break;
 	case YASM_EXPR_NE:
 	    BitVector_Empty(result);
 	    BitVector_LSB(result, !BitVector_equal(op1, op2));
 	    break;
 	case YASM_EXPR_SEG:
-	    yasm__error(line, N_("invalid use of '%s'"), "SEG");
+	    yasm_error_set(YASM_ERROR_ARITHMETIC, N_("invalid use of '%s'"),
+			   "SEG");
 	    break;
 	case YASM_EXPR_WRT:
-	    yasm__error(line, N_("invalid use of '%s'"), "WRT");
+	    yasm_error_set(YASM_ERROR_ARITHMETIC, N_("invalid use of '%s'"),
+			   "WRT");
 	    break;
 	case YASM_EXPR_SEGOFF:
-	    yasm__error(line, N_("invalid use of '%s'"), ":");
+	    yasm_error_set(YASM_ERROR_ARITHMETIC, N_("invalid use of '%s'"),
+			   ":");
 	    break;
 	case YASM_EXPR_IDENT:
 	    if (result)
 		BitVector_Copy(result, op1);
 	    break;
 	default:
-	    yasm_internal_error(N_("invalid operation in intnum calculation"));
+	    yasm_error_set(YASM_ERROR_ARITHMETIC,
+			   N_("invalid operation in intnum calculation"));
+	    BitVector_Empty(result);
+	    return 1;
     }
 
     /* Try to fit the result into 32 bits if possible */
@@ -484,6 +582,7 @@ yasm_intnum_calc(yasm_intnum *acc, yasm_expr_op op, yasm_intnum *operand,
 	    acc->val.bv = BitVector_Clone(result);
 	}
     }
+    return 0;
 }
 /*@=nullderef =nullpass =branchstate@*/
 
@@ -503,6 +602,27 @@ yasm_intnum_set_uint(yasm_intnum *intn, unsigned long val)
     intn->val.ul = val;
 }
 
+void
+yasm_intnum_set_int(yasm_intnum *intn, long val)
+{
+    /* positive numbers can go through the uint() function */
+    if (val >= 0) {
+	yasm_intnum_set_uint(intn, (unsigned long)val);
+	return;
+    }
+
+    BitVector_Empty(conv_bv);
+    BitVector_Chunk_Store(conv_bv, 32, 0, (unsigned long)(-val));
+    BitVector_Negate(conv_bv, conv_bv);
+
+    if (intn->type == INTNUM_BV)
+	BitVector_Copy(intn->val.bv, conv_bv);
+    else {
+	intn->val.bv = BitVector_Clone(conv_bv);
+	intn->type = INTNUM_BV;
+    }
+}
+
 int
 yasm_intnum_is_zero(const yasm_intnum *intn)
 {
@@ -584,7 +704,7 @@ yasm_intnum_get_int(const yasm_intnum *intn)
 void
 yasm_intnum_get_sized(const yasm_intnum *intn, unsigned char *ptr,
 		      size_t destsize, size_t valsize, int shift,
-		      int bigendian, int warn, unsigned long line)
+		      int bigendian, int warn)
 {
     wordptr op1 = op1static, op2;
     unsigned char *buf;
@@ -598,11 +718,11 @@ yasm_intnum_get_sized(const yasm_intnum *intn, unsigned char *ptr,
 
     /* General size warnings */
     if (warn<0 && !yasm_intnum_check_size(intn, valsize, rshift, 1))
-	yasm__warning(YASM_WARN_GENERAL, line,
+	yasm_warn_set(YASM_WARN_GENERAL,
 		      N_("value does not fit in signed %d bit field"),
 		      valsize);
     if (warn>0 && !yasm_intnum_check_size(intn, valsize, rshift, 2))
-	yasm__warning(YASM_WARN_GENERAL, line,
+	yasm_warn_set(YASM_WARN_GENERAL,
 		      N_("value does not fit in %d bit field"), valsize);
 
     /* Read the original data into a bitvect */
@@ -610,7 +730,7 @@ yasm_intnum_get_sized(const yasm_intnum *intn, unsigned char *ptr,
 	/* TODO */
 	yasm_internal_error(N_("big endian not implemented"));
     } else
-	BitVector_Block_Store(op1, ptr, destsize);
+	BitVector_Block_Store(op1, ptr, (N_int)destsize);
 
     /* If not already a bitvect, convert value to be written to a bitvect */
     if (intn->type == INTNUM_BV)
@@ -624,9 +744,9 @@ yasm_intnum_get_sized(const yasm_intnum *intn, unsigned char *ptr,
     /* Check low bits if right shifting and warnings enabled */
     if (warn && rshift > 0) {
 	BitVector_Copy(conv_bv, op2);
-	BitVector_Move_Left(conv_bv, BITVECT_NATIVE_SIZE-rshift);
+	BitVector_Move_Left(conv_bv, (N_int)(BITVECT_NATIVE_SIZE-rshift));
 	if (!BitVector_is_empty(conv_bv))
-	    yasm__warning(YASM_WARN_GENERAL, line,
+	    yasm_warn_set(YASM_WARN_GENERAL,
 			  N_("misaligned value, truncating to boundary"));
     }
 
@@ -639,7 +759,7 @@ yasm_intnum_get_sized(const yasm_intnum *intn, unsigned char *ptr,
     }
 
     /* Write the new value into the destination bitvect */
-    BitVector_Interval_Copy(op1, op2, (unsigned int)shift, 0, valsize);
+    BitVector_Interval_Copy(op1, op2, (unsigned int)shift, 0, (N_int)valsize);
 
     /* Write out the new data */
     buf = BitVector_Block_Read(op1, &len);
@@ -698,6 +818,43 @@ yasm_intnum_check_size(const yasm_intnum *intn, size_t size, size_t rshift,
     return (Set_Max(val) < (long)size);
 }
 
+int
+yasm_intnum_in_range(const yasm_intnum *intn, long low, long high)
+{
+    wordptr val = result;
+    wordptr lval = op1static;
+    wordptr hval = op2static;
+
+    /* If not already a bitvect, convert value to be written to a bitvect */
+    if (intn->type == INTNUM_BV)
+	val = intn->val.bv;
+    else {
+	BitVector_Empty(val);
+	BitVector_Chunk_Store(val, 32, 0, intn->val.ul);
+    }
+
+    /* Convert high and low to bitvects */
+    BitVector_Empty(lval);
+    if (low >= 0)
+	BitVector_Chunk_Store(lval, 32, 0, (unsigned long)low);
+    else {
+	BitVector_Chunk_Store(lval, 32, 0, (unsigned long)(-low));
+	BitVector_Negate(lval, lval);
+    }
+
+    BitVector_Empty(hval);
+    if (high >= 0)
+	BitVector_Chunk_Store(hval, 32, 0, (unsigned long)high);
+    else {
+	BitVector_Chunk_Store(hval, 32, 0, (unsigned long)(-high));
+	BitVector_Negate(hval, hval);
+    }
+
+    /* Compare! */
+    return (BitVector_Compare(val, lval) >= 0
+	    && BitVector_Compare(val, hval) <= 0);
+}
+
 static unsigned long
 get_leb128(wordptr val, unsigned char *ptr, int sign)
 {
@@ -859,6 +1016,25 @@ yasm_size_uleb128(unsigned long v)
     return size_leb128(val, 0);
 }
 
+char *
+yasm_intnum_get_str(const yasm_intnum *intn)
+{
+    unsigned char *s;
+
+    switch (intn->type) {
+	case INTNUM_UL:
+	    s = yasm_xmalloc(16);
+	    sprintf((char *)s, "%lu", intn->val.ul);
+	    return (char *)s;
+	    break;
+	case INTNUM_BV:
+	    return (char *)BitVector_to_Dec(intn->val.bv);
+	    break;
+    }
+    /*@notreached@*/
+    return NULL;
+}
+
 void
 yasm_intnum_print(const yasm_intnum *intn, FILE *f)
 {
@@ -866,11 +1042,11 @@ yasm_intnum_print(const yasm_intnum *intn, FILE *f)
 
     switch (intn->type) {
 	case INTNUM_UL:
-	    fprintf(f, "0x%lx/%u", intn->val.ul, (unsigned int)intn->origsize);
+	    fprintf(f, "0x%lx", intn->val.ul);
 	    break;
 	case INTNUM_BV:
 	    s = BitVector_to_Hex(intn->val.bv);
-	    fprintf(f, "0x%s/%u", (char *)s, (unsigned int)intn->origsize);
+	    fprintf(f, "0x%s", (char *)s);
 	    yasm_xfree(s);
 	    break;
     }
