@@ -3,7 +3,7 @@
  * \brief YASM preprocessor module interface.
  *
  * \rcs
- * $Id: preproc.h 1591 2006-07-13 03:25:38Z peter $
+ * $Id: preproc.h 1669 2006-10-28 21:31:38Z peter $
  * \endrcs
  *
  * \license
@@ -56,16 +56,17 @@ typedef struct yasm_preproc_module {
      * Module-level implementation of yasm_preproc_create().
      * Call yasm_preproc_create() instead of calling this function.
      *
-     * The preprocessor needs access to the object format module to find out
-     * any output format specific macros.
-     *
      * \param f			initial starting file
      * \param in_filename	initial starting filename
      * \param lm		line mapping repository
+     * \param errwarns		error/warnning set.
      * \return New preprocessor.
+     *
+     * \note Any preprocessor errors and warnings are stored into errwarns.
      */
     /*@only@*/ yasm_preproc * (*create) (FILE *f, const char *in_filename,
-					 yasm_linemap *lm);
+					 yasm_linemap *lm,
+					 yasm_errwarns *errwarns);
 
     /** Module-level implementation of yasm_preproc_destroy().
      * Call yasm_preproc_destroy() instead of calling this function.
@@ -84,11 +85,6 @@ typedef struct yasm_preproc_module {
     size_t (*get_included_file) (yasm_preproc *preproc, /*@out@*/ char *buf,
                                  size_t max_size);
 
-    /** Module-level implementation of yasm_preproc_add_include_path().
-     * Call yasm_preproc_add_include_path() instead of calling this function.
-     */
-    void (*add_include_path) (yasm_preproc *preproc, const char *path);
-
     /** Module-level implementation of yasm_preproc_add_include_file().
      * Call yasm_preproc_add_include_file() instead of calling this function.
      */
@@ -117,11 +113,13 @@ typedef struct yasm_preproc_module {
  * \param f		initial starting file
  * \param in_filename	initial starting file filename
  * \param lm		line mapping repository
+ * \param errwarns	error/warning set
  * \return New preprocessor.
+ * \note Errors/warnings are stored into errwarns.
  */
 /*@only@*/ yasm_preproc *yasm_preproc_create
     (yasm_preproc_module *module, FILE *f, const char *in_filename,
-     yasm_linemap *lm);
+     yasm_linemap *lm, yasm_errwarns *errwarns);
 
 /** Cleans up any allocated preproc memory.
  * \param preproc	preprocessor
@@ -147,12 +145,6 @@ size_t yasm_preproc_input(yasm_preproc *preproc, /*@out@*/ char *buf,
 size_t yasm_preproc_get_included_file(yasm_preproc *preproc,
 				      /*@out@*/ char *buf, size_t max_size);
 
-/** Add a directory to the include search path.
- * \param preproc	preprocessor
- * \param path		pathname
- */
-void yasm_preproc_add_include_path(yasm_preproc *preproc, const char *path);
-
 /** Pre-include a file.
  * \param preproc	preprocessor
  * \param filename	filename
@@ -177,15 +169,15 @@ void yasm_preproc_undefine_macro(yasm_preproc *preproc, const char *macroname);
  * \param preproc	preprocessor
  * \param macronameval	"name=value" string
  */
-void yasm_preproc_builtin_define(yasm_preproc *preproc,
+void yasm_preproc_define_builtin(yasm_preproc *preproc,
 				 const char *macronameval);
 
 #ifndef YASM_DOXYGEN
 
 /* Inline macro implementations for preproc functions */
 
-#define yasm_preproc_create(module, f, in_filename, lm) \
-    module->create(f, in_filename, lm)
+#define yasm_preproc_create(module, f, in_filename, lm, ews) \
+    module->create(f, in_filename, lm, ews)
 
 #define yasm_preproc_destroy(preproc) \
     ((yasm_preproc_base *)preproc)->module->destroy(preproc)
@@ -193,8 +185,6 @@ void yasm_preproc_builtin_define(yasm_preproc *preproc,
     ((yasm_preproc_base *)preproc)->module->input(preproc, buf, max_size)
 #define yasm_preproc_get_included_file(preproc, buf, max_size) \
     ((yasm_preproc_base *)preproc)->module->get_included_file(preproc, buf, max_size)
-#define yasm_preproc_add_include_path(preproc, path) \
-    ((yasm_preproc_base *)preproc)->module->add_include_path(preproc, path)
 #define yasm_preproc_add_include_file(preproc, filename) \
     ((yasm_preproc_base *)preproc)->module->add_include_file(preproc, filename)
 #define yasm_preproc_predefine_macro(preproc, macronameval) \
