@@ -3,7 +3,7 @@
  * \brief YASM section interface.
  *
  * \rcs
- * $Id: section.h 1591 2006-07-13 03:25:38Z peter $
+ * $Id: section.h 1598 2006-08-10 04:02:59Z peter $
  * \endrcs
  *
  * \license
@@ -103,8 +103,10 @@ void yasm_object_print(const yasm_object *object, FILE *f, int indent_level);
 
 /** Finalize an object after parsing.
  * \param object	object
+ * \param errwarns	error/warning set
+ * \note Errors/warnings are stored into errwarns.
  */
-void yasm_object_finalize(yasm_object *object);
+void yasm_object_finalize(yasm_object *object, yasm_errwarns *errwarns);
 
 /** Traverses all sections in an object, calling a function on each section.
  * \param object	object
@@ -156,6 +158,16 @@ const char *yasm_object_get_object_fn(const yasm_object *object);
 /*@dependent@*/ yasm_linemap *yasm_object_get_linemap
     (const yasm_object *object);
 
+/** Optimize an object.  Takes the unoptimized object and optimizes it.
+ * If successful, the object is ready for output to an object file.
+ * \param object	object
+ * \param arch		architecture
+ * \param errwarns	error/warning set
+ * \note Optimization failures are stored into errwarns.
+ */
+void yasm_object_optimize(yasm_object *object, yasm_arch *arch,
+			  yasm_errwarns *errwarns);
+
 /** Determine if a section is absolute or general.
  * \param sect	    section
  * \return Nonzero if section is absolute.
@@ -281,15 +293,18 @@ yasm_bytecode *yasm_section_bcs_last(yasm_section *sect);
      /*@returned@*/ /*@only@*/ /*@null@*/ yasm_bytecode *bc);
 
 /** Traverses all bytecodes in a section, calling a function on each bytecode.
- * \param sect	section
- * \param d	data pointer passed to func on each call
- * \param func	function
+ * \param sect	    section
+ * \param errwarns  error/warning set (may be NULL)
+ * \param d	    data pointer passed to func on each call (may be NULL)
+ * \param func	    function
  * \return Stops early (and returns func's return value) if func returns a
  *	   nonzero value; otherwise 0.
+ * \note If errwarns is non-NULL, yasm_errwarn_propagate() is called after
+ *       each call to func (with the bytecode's line number).
  */
 int yasm_section_bcs_traverse
-    (yasm_section *sect, /*@null@*/ void *d,
-     int (*func) (yasm_bytecode *bc, /*@null@*/ void *d));
+    (yasm_section *sect, /*@null@*/ yasm_errwarns *errwarns,
+     /*@null@*/ void *d, int (*func) (yasm_bytecode *bc, /*@null@*/ void *d));
 
 /** Get name of a section.
  * \param   sect    section
