@@ -26,17 +26,19 @@
  */
 #define YASM_LIB_INTERNAL
 #include "util.h"
-/*@unused@*/ RCSID("$Id: symrec.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: symrec.c 1786 2007-02-24 04:16:05Z peter $");
 
 #ifdef STDC_HEADERS
 # include <limits.h>
 #endif
 
+#include "libyasm-stdint.h"
 #include "coretype.h"
 #include "hamt.h"
 #include "assocdat.h"
 
 #include "errwarn.h"
+#include "intnum.h"
 #include "floatnum.h"
 #include "expr.h"
 #include "symrec.h"
@@ -46,15 +48,6 @@
 #include "objfmt.h"
 
 
-/* DEFINED is set with EXTERN and COMMON below */
-typedef enum {
-    SYM_NOSTATUS = 0,
-    SYM_USED = 1 << 0,		/* for using variables before definition */
-    SYM_DEFINED = 1 << 1,	/* once it's been defined in the file */
-    SYM_VALUED = 1 << 2,	/* once its value has been determined */
-    SYM_NOTINTABLE = 1 << 3	/* if it's not in sym_table (ex. '$') */
-} sym_status;
-
 typedef enum {
     SYM_UNKNOWN,		/* for unknown type (COMMON/EXTERN) */
     SYM_EQU,			/* for EQU defined symbols (expressions) */
@@ -69,7 +62,7 @@ typedef enum {
 struct yasm_symrec {
     char *name;
     sym_type type;
-    sym_status status;
+    yasm_sym_status status;
     yasm_sym_vis visibility;
     unsigned long line;		/*  symbol was first declared or used on */
     union {
@@ -135,7 +128,7 @@ symtab_get_or_new_in_table(yasm_symtab *symtab, /*@only@*/ char *name)
     yasm_symrec *rec = symrec_new_common(name);
     int replace = 0;
 
-    rec->status = SYM_NOSTATUS;
+    rec->status = YASM_SYM_NOSTATUS;
 
     return HAMT_insert(symtab->sym_table, name, rec, &replace,
 		       symrec_destroy_one);
@@ -147,7 +140,7 @@ symtab_get_or_new_not_in_table(yasm_symtab *symtab, /*@only@*/ char *name)
     non_table_symrec *sym = yasm_xmalloc(sizeof(non_table_symrec));
     sym->rec = symrec_new_common(name);
 
-    sym->rec->status = SYM_NOTINTABLE;
+    sym->rec->status = YASM_SYM_NOTINTABLE;
 
     SLIST_INSERT_HEAD(&symtab->non_table_syms, sym, link);
 
@@ -175,13 +168,43 @@ yasm_symtab_traverse(yasm_symtab *symtab, void *d,
     return HAMT_traverse(symtab->sym_table, d, (int (*) (void *, void *))func);
 }
 
+const yasm_symtab_iter *
+yasm_symtab_first(const yasm_symtab *symtab)
+{
+    return (const yasm_symtab_iter *)HAMT_first(symtab->sym_table);
+}
+
+/*@null@*/ const yasm_symtab_iter *
+yasm_symtab_next(const yasm_symtab_iter *prev)
+{
+    return (const yasm_symtab_iter *)HAMT_next((const HAMTEntry *)prev);
+}
+
+yasm_symrec *
+yasm_symtab_iter_value(const yasm_symtab_iter *cur)
+{
+    return (yasm_symrec *)HAMTEntry_get_data((const HAMTEntry *)cur);
+}
+
+yasm_symrec *
+yasm_symtab_abs_sym(yasm_symtab *symtab)
+{
+    yasm_symrec *rec = symtab_get_or_new(symtab, "", 1);
+    rec->line = 0;
+    rec->type = SYM_EQU;
+    rec->value.expn =
+	yasm_expr_create_ident(yasm_expr_int(yasm_intnum_create_uint(0)), 0);
+    rec->status |= YASM_SYM_DEFINED|YASM_SYM_VALUED|YASM_SYM_USED;
+    return rec;
+}
+
 yasm_symrec *
 yasm_symtab_use(yasm_symtab *symtab, const char *name, unsigned long line)
 {
     yasm_symrec *rec = symtab_get_or_new(symtab, name, 1);
     if (rec->line == 0)
 	rec->line = line;	/* set line number of first use */
-    rec->status |= SYM_USED;
+    rec->status |= YASM_SYM_USED;
     return rec;
 }
 
@@ -198,17 +221,18 @@ symtab_define(yasm_symtab *symtab, const char *name, sym_type type,
     yasm_symrec *rec = symtab_get_or_new(symtab, name, in_table);
 
     /* Has it been defined before (either by DEFINED or COMMON/EXTERN)? */
-    if (rec->status & SYM_DEFINED) {
-	yasm__error(line, N_("redefinition of `%s'"), name);
-	yasm__error_at(line, rec->line, N_("`%s' previously defined here"),
+    if (rec->status & YASM_SYM_DEFINED) {
+	yasm_error_set_xref(rec->line, N_("`%s' previously defined here"),
+			    name);
+	yasm_error_set(YASM_ERROR_GENERAL, N_("redefinition of `%s'"),
 		       name);
     } else {
 	if (rec->visibility & YASM_SYM_EXTERN)
-	    yasm__warning(YASM_WARN_GENERAL, line,
+	    yasm_warn_set(YASM_WARN_GENERAL,
 			  N_("`%s' both defined and declared extern"), name);
 	rec->line = line;	/* set line number of definition */
 	rec->type = type;
-	rec->status |= SYM_DEFINED;
+	rec->status |= YASM_SYM_DEFINED;
     }
     return rec;
 }
@@ -219,7 +243,7 @@ yasm_symtab_define_equ(yasm_symtab *symtab, const char *name, yasm_expr *e,
 {
     yasm_symrec *rec = symtab_define(symtab, name, SYM_EQU, 1, line);
     rec->value.expn = e;
-    rec->status |= SYM_VALUED;
+    rec->status |= YASM_SYM_VALUED;
     return rec;
 }
 
@@ -251,7 +275,7 @@ yasm_symtab_define_special(yasm_symtab *symtab, const char *name,
 			   yasm_sym_vis vis)
 {
     yasm_symrec *rec = symtab_define(symtab, name, SYM_SPECIAL, 1, 0);
-    rec->status |= SYM_VALUED;
+    rec->status |= YASM_SYM_VALUED;
     rec->visibility = vis;
     return rec;
 }
@@ -280,13 +304,13 @@ yasm_symrec_declare(yasm_symrec *rec, yasm_sym_vis vis, unsigned long line)
      * X   1      -      1      -
      */
     if ((vis == YASM_SYM_GLOBAL) ||
-	(!(rec->status & SYM_DEFINED) &&
+	(!(rec->status & YASM_SYM_DEFINED) &&
 	 (!(rec->visibility & (YASM_SYM_COMMON | YASM_SYM_EXTERN)) ||
 	  ((rec->visibility & YASM_SYM_COMMON) && (vis == YASM_SYM_COMMON)) ||
 	  ((rec->visibility & YASM_SYM_EXTERN) && (vis == YASM_SYM_EXTERN)))))
 	rec->visibility |= vis;
     else
-	yasm__error(line,
+	yasm_error_set(YASM_ERROR_GENERAL,
 	    N_("duplicate definition of `%s'; first defined on line %lu"),
 	    rec->name, rec->line);
 }
@@ -295,40 +319,66 @@ typedef struct symtab_finalize_info {
     unsigned long firstundef_line;
     int undef_extern;
     /*@null@*/ yasm_objfmt *objfmt;
+    yasm_errwarns *errwarns;
 } symtab_finalize_info;
 
 static int
 symtab_parser_finalize_checksym(yasm_symrec *sym, /*@null@*/ void *d)
 {
     symtab_finalize_info *info = (symtab_finalize_info *)d;
+    yasm_section *sect;
+
     /* error if a symbol is used but never defined or extern/common declared */
-    if ((sym->status & SYM_USED) && !(sym->status & SYM_DEFINED) &&
+    if ((sym->status & YASM_SYM_USED) && !(sym->status & YASM_SYM_DEFINED) &&
 	!(sym->visibility & (YASM_SYM_EXTERN | YASM_SYM_COMMON))) {
 	if (info->undef_extern && info->objfmt)
 	    yasm_objfmt_extern_declare(info->objfmt, sym->name, NULL, 1);
 	else {
-	    yasm__error(sym->line, N_("undefined symbol `%s' (first use)"),
-			sym->name);
+	    yasm_error_set(YASM_ERROR_GENERAL,
+			   N_("undefined symbol `%s' (first use)"), sym->name);
+	    yasm_errwarn_propagate(info->errwarns, sym->line);
 	    if (sym->line < info->firstundef_line)
 		info->firstundef_line = sym->line;
 	}
     }
 
+    /* Change labels in absolute sections into EQUs with value
+     * absolute start expr + (label bc - first bc in abs section).
+     * Don't worry about possible circular references because that will get
+     * caught in EQU expansion.
+     */
+    if (sym->type == SYM_LABEL && sym->value.precbc
+	&& (sect = yasm_bc_get_section(sym->value.precbc))
+	&& yasm_section_is_absolute(sect)) {
+	sym->type = SYM_EQU;
+	sym->value.expn = yasm_expr_create_tree(
+	    yasm_expr_create(YASM_EXPR_SUB,
+			     yasm_expr_precbc(sym->value.precbc),
+			     yasm_expr_precbc(yasm_section_bcs_first(sect)),
+			     sym->line),
+	    YASM_EXPR_ADD,
+	    yasm_expr_copy(yasm_section_get_start(sect)),
+	    sym->line);
+    }
+
     return 0;
 }
 
 void
 yasm_symtab_parser_finalize(yasm_symtab *symtab, int undef_extern,
-			    yasm_objfmt *objfmt)
+			    yasm_objfmt *objfmt, yasm_errwarns *errwarns)
 {
     symtab_finalize_info info;
     info.firstundef_line = ULONG_MAX;
     info.undef_extern = undef_extern;
     info.objfmt = objfmt;
+    info.errwarns = errwarns;
     yasm_symtab_traverse(symtab, &info, symtab_parser_finalize_checksym);
-    if (info.firstundef_line < ULONG_MAX)
-	yasm__error(info.firstundef_line,
-		    N_(" (Each undefined symbol is reported only once.)"));
+    if (info.firstundef_line < ULONG_MAX) {
+	yasm_error_set(YASM_ERROR_GENERAL,
+		       N_(" (Each undefined symbol is reported only once.)"));
+	yasm_errwarn_propagate(errwarns, info.firstundef_line);
+    }
 }
 
 void
@@ -384,6 +434,18 @@ yasm_symrec_get_visibility(const yasm_symrec *sym)
     return sym->visibility;
 }
 
+yasm_sym_status
+yasm_symrec_get_status(const yasm_symrec *sym)
+{
+    return sym->status;
+}
+
+unsigned long
+yasm_symrec_get_line(const yasm_symrec *sym)
+{
+    return sym->line;
+}
+
 const yasm_expr *
 yasm_symrec_get_equ(const yasm_symrec *sym)
 {
@@ -405,6 +467,12 @@ yasm_symrec_get_label(const yasm_symrec *sym,
     return 1;
 }
 
+int
+yasm_symrec_is_abs(const yasm_symrec *sym)
+{
+    return (sym->line == 0 && sym->type == SYM_EQU && sym->name[0] == '\0');
+}
+
 int
 yasm_symrec_is_special(const yasm_symrec *sym)
 {
@@ -460,16 +528,16 @@ yasm_symrec_print(const yasm_symrec *sym, FILE *f, int indent_level)
     }
 
     fprintf(f, "%*sStatus=", indent_level, "");
-    if (sym->status == SYM_NOSTATUS)
+    if (sym->status == YASM_SYM_NOSTATUS)
 	fprintf(f, "None\n");
     else {
-	if (sym->status & SYM_USED)
+	if (sym->status & YASM_SYM_USED)
 	    fprintf(f, "Used,");
-	if (sym->status & SYM_DEFINED)
+	if (sym->status & YASM_SYM_DEFINED)
 	    fprintf(f, "Defined,");
-	if (sym->status & SYM_VALUED)
+	if (sym->status & YASM_SYM_VALUED)
 	    fprintf(f, "Valued,");
-	if (sym->status & SYM_NOTINTABLE)
+	if (sym->status & YASM_SYM_NOTINTABLE)
 	    fprintf(f, "Not in Table,");
 	fprintf(f, "\n");
     }
