@@ -3,7 +3,7 @@
  * \brief YASM symbol table interface.
  *
  * \rcs
- * $Id: symrec.h 1591 2006-07-13 03:25:38Z peter $
+ * $Id: symrec.h 1745 2007-01-29 03:32:37Z peter $
  * \endrcs
  *
  * \license
@@ -34,6 +34,19 @@
 #ifndef YASM_SYMREC_H
 #define YASM_SYMREC_H
 
+/** Symbol status.  YASM_SYM_DEFINED is set by yasm_symtab_define_label(),
+ * yasm_symtab_define_equ(), or yasm_symtab_declare()/yasm_symrec_declare()
+ * with a visibility of #YASM_SYM_EXTERN or #YASM_SYM_COMMON.
+ */
+typedef enum yasm_sym_status {
+    YASM_SYM_NOSTATUS = 0,	    /**< no status */
+    YASM_SYM_USED = 1 << 0,	    /**< for use before definition */
+    YASM_SYM_DEFINED = 1 << 1,	    /**< once it's been defined in the file */
+    YASM_SYM_VALUED = 1 << 2,	    /**< once its value has been determined */
+    YASM_SYM_NOTINTABLE = 1 << 3    /**< if it's not in sym_table (ex. '$') */
+} yasm_sym_status;
+
+
 /** Create a new symbol table. */
 yasm_symtab *yasm_symtab_create(void);
 
@@ -44,6 +57,15 @@ yasm_symtab *yasm_symtab_create(void);
  */
 void yasm_symtab_destroy(/*@only@*/ yasm_symtab *symtab);
 
+/** Get a reference to the symbol table's "absolute" symbol.  This is
+ * essentially an EQU with no name and value 0, and is used for relocating
+ * absolute current-position-relative values.
+ * \see yasm_value_set_curpos_rel().
+ * \param symtab    symbol table
+ * \return Absolute symbol (dependent pointer, do not free).
+ */
+/*@dependent@*/ yasm_symrec *yasm_symtab_abs_sym(yasm_symtab *symtab);
+
 /** Get a reference to (use) a symbol.  The symbol does not necessarily need to
  * be defined before it is used.
  * \param symtab    symbol table
@@ -154,15 +176,40 @@ int /*@alt void@*/ yasm_symtab_traverse
     (yasm_symtab *symtab, /*@null@*/ void *d,
      yasm_symtab_traverse_callback func);
 
+/** Symbol table iterator (opaque type). */
+typedef struct yasm_symtab_iter yasm_symtab_iter;
+
+/** Get an iterator pointing to the first symbol in the symbol table.
+ * \param symtab    symbol table
+ * \return Iterator for the symbol table.
+ */
+const yasm_symtab_iter *yasm_symtab_first(const yasm_symtab *symtab);
+
+/** Move a symbol table iterator to the next symbol in the symbol table.
+ * \param prev		Previous iterator value
+ * \return Next iterator value, or NULL if no more symbols in the table.
+ */
+/*@null@*/ const yasm_symtab_iter *yasm_symtab_next
+    (const yasm_symtab_iter *prev);
+
+/** Get the symbol corresponding to the current symbol table iterator value.
+ * \param cur		iterator value
+ * \return Corresponding symbol.
+ */
+yasm_symrec *yasm_symtab_iter_value(const yasm_symtab_iter *cur);
+
 /** Finalize symbol table after parsing stage.  Checks for symbols that are
  * used but never defined or declared #YASM_SYM_EXTERN or #YASM_SYM_COMMON.
  * \param symtab	symbol table
  * \param undef_extern	if nonzero, all undef syms should be declared extern
  * \param objfmt	object format to notify about new extern decls
  *			(may be NULL if undef_extern is 0)
+ * \param errwarns	error/warning set
+ * \note Errors/warnings are stored into errwarns.
  */
 void yasm_symtab_parser_finalize(yasm_symtab *symtab, int undef_extern,
-				 /*@null@*/ yasm_objfmt *objfmt);
+				 /*@null@*/ yasm_objfmt *objfmt,
+				 yasm_errwarns *errwarns);
 
 /** Print the symbol table.  For debugging purposes.
  * \param symtab	symbol table
@@ -183,6 +230,18 @@ void yasm_symtab_print(yasm_symtab *symtab, FILE *f, int indent_level);
  */
 yasm_sym_vis yasm_symrec_get_visibility(const yasm_symrec *sym);
 
+/** Get the status of a symbol.
+ * \param sym	    symbol
+ * \return Symbol status.
+ */
+yasm_sym_status yasm_symrec_get_status(const yasm_symrec *sym);
+
+/** Get the virtual line of a symbol (where it was first declared or used).
+ * \param sym	    symbol
+ * \return line	    virtual line
+ */
+unsigned long yasm_symrec_get_line(const yasm_symrec *sym);
+
 /** Get EQU value of a symbol.
  * \param sym	    symbol
  * \return EQU value, or NULL if symbol is not an EQU or is not defined.
@@ -202,6 +261,13 @@ typedef /*@dependent@*/ yasm_bytecode *yasm_symrec_get_label_bytecodep;
 int yasm_symrec_get_label(const yasm_symrec *sym,
 			  /*@out@*/ yasm_symrec_get_label_bytecodep *precbc);
 
+/** Determine if symbol is the "absolute" symbol created by
+ * yasm_symtab_abs_sym().
+ * \param sym	    symbol
+ * \return 0 if symbol is not the "absolute" symbol, nonzero otherwise.
+ */
+int yasm_symrec_is_abs(const yasm_symrec *sym);
+
 /** Determine if symbol is a special symbol.
  * \param sym	    symbol
  * \return 0 if symbol is not a special symbol, nonzero otherwise.
