@@ -26,8 +26,9 @@
  */
 #define YASM_LIB_INTERNAL
 #include "util.h"
-/*@unused@*/ RCSID("$Id: value.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: value.c 1786 2007-02-24 04:16:05Z peter $");
 
+#include "libyasm-stdint.h"
 #include "coretype.h"
 #include "bitvect.h"
 
@@ -46,8 +47,84 @@
 #include "expr-int.h"
 #include "bc-int.h"
 
+void
+yasm_value_initialize(/*@out@*/ yasm_value *value,
+		      /*@null@*/ /*@kept@*/ yasm_expr *e, unsigned int size)
+{
+    value->abs = e;
+    value->rel = NULL;
+    value->wrt = NULL;
+    value->seg_of = 0;
+    value->rshift = 0;
+    value->curpos_rel = 0;
+    value->ip_rel = 0;
+    value->jump_target = 0;
+    value->section_rel = 0;
+    value->sign = 0;
+    value->size = size;
+}
+
+void
+yasm_value_init_sym(/*@out@*/ yasm_value *value, /*@null@*/ yasm_symrec *sym,
+		    unsigned int size)
+{
+    value->abs = NULL;
+    value->rel = sym;
+    value->wrt = NULL;
+    value->seg_of = 0;
+    value->rshift = 0;
+    value->curpos_rel = 0;
+    value->ip_rel = 0;
+    value->jump_target = 0;
+    value->section_rel = 0;
+    value->sign = 0;
+    value->size = size;
+}
+
+void
+yasm_value_init_copy(yasm_value *value, const yasm_value *orig)
+{
+    value->abs = orig->abs ? yasm_expr_copy(orig->abs) : NULL;
+    value->rel = orig->rel;
+    value->wrt = orig->wrt;
+    value->seg_of = orig->seg_of;
+    value->rshift = orig->rshift;
+    value->curpos_rel = orig->curpos_rel;
+    value->ip_rel = orig->ip_rel;
+    value->jump_target = orig->jump_target;
+    value->section_rel = orig->section_rel;
+    value->sign = orig->sign;
+    value->size = orig->size;
+}
+
+void
+yasm_value_delete(yasm_value *value)
+{
+    if (value->abs)
+	yasm_expr_destroy(value->abs);
+    value->abs = NULL;
+    value->rel = NULL;
+}
+
+void
+yasm_value_set_curpos_rel(yasm_value *value, yasm_bytecode *bc,
+			  unsigned int ip_rel)
+{
+    value->curpos_rel = 1;
+    value->ip_rel = ip_rel;
+    /* In order for us to correctly output curpos-relative values, we must
+     * have a relative portion of the value.  If one doesn't exist, point
+     * to a custom absolute symbol.
+     */
+    if (!value->rel) {
+	value->rel = yasm_symtab_abs_sym(yasm_object_get_symtab(
+	    yasm_section_get_object(yasm_bc_get_section(bc))));
+    }
+}
+
 static int
-value_finalize_scan(yasm_value *value, yasm_expr *e, int ssym_not_ok)
+value_finalize_scan(yasm_value *value, yasm_expr *e,
+		    /*@null@*/ yasm_bytecode *expr_precbc, int ssym_not_ok)
 {
     int i;
     /*@dependent@*/ yasm_section *sect;
@@ -106,35 +183,37 @@ value_finalize_scan(yasm_value *value, yasm_expr *e, int ssym_not_ok)
 
 		if (sube->op != YASM_EXPR_MUL || sube->numterms != 2) {
 		    /* recurse instead */
-		    if (value_finalize_scan(value, sube, ssym_not_ok))
+		    if (value_finalize_scan(value, sube, expr_precbc,
+					    ssym_not_ok))
 			return 1;
 		    continue;
 		}
 
 		if (sube->terms[0].type == YASM_EXPR_INT &&
-		    (sube->terms[1].type == YASM_EXPR_SYM ||
-		     sube->terms[1].type == YASM_EXPR_SYMEXP)) {
+		    sube->terms[1].type == YASM_EXPR_SYM) {
 		    intn = sube->terms[0].data.intn;
 		    sym = sube->terms[1].data.sym;
-		} else if ((sube->terms[0].type == YASM_EXPR_SYM ||
-			    sube->terms[0].type == YASM_EXPR_SYMEXP) &&
+		} else if (sube->terms[0].type == YASM_EXPR_SYM &&
 			   sube->terms[1].type == YASM_EXPR_INT) {
 		    sym = sube->terms[0].data.sym;
 		    intn = sube->terms[1].data.intn;
 		} else {
-		    if (value_finalize_scan(value, sube, ssym_not_ok))
+		    if (value_finalize_scan(value, sube, expr_precbc,
+					    ssym_not_ok))
 			return 1;
 		    continue;
 		}
 
 		if (!yasm_intnum_is_neg1(intn)) {
-		    if (value_finalize_scan(value, sube, ssym_not_ok))
+		    if (value_finalize_scan(value, sube, expr_precbc,
+					    ssym_not_ok))
 			return 1;
 		    continue;
 		}
 
 		if (!yasm_symrec_get_label(sym, &precbc)) {
-		    if (value_finalize_scan(value, sube, ssym_not_ok))
+		    if (value_finalize_scan(value, sube, expr_precbc,
+					    ssym_not_ok))
 			return 1;
 		    continue;
 		}
@@ -142,8 +221,7 @@ value_finalize_scan(yasm_value *value, yasm_expr *e, int ssym_not_ok)
 
 		/* Now look for a unused symrec term in the same segment */
 		for (j=0; j<e->numterms; j++) {
-		    if ((e->terms[j].type == YASM_EXPR_SYM
-			 || e->terms[j].type == YASM_EXPR_SYMEXP)
+		    if (e->terms[j].type == YASM_EXPR_SYM
 			&& yasm_symrec_get_label(e->terms[j].data.sym,
 						 &precbc2)
 			&& (sect = yasm_bc_get_section(precbc2))
@@ -160,14 +238,21 @@ value_finalize_scan(yasm_value *value, yasm_expr *e, int ssym_not_ok)
 		 * unused symrec terms in other segments and generate
 		 * a curpos-relative reloc.
 		 *
+		 * Similarly, handle -1*symrec in other segment via the
+		 * following transformation:
+		 * other-this = (other-.)+(.-this)
+		 * We can only do this transformation if "this" is in
+		 * this expr's segment.
+		 *
 		 * Don't do this if we've already become curpos-relative.
 		 * The unmatched symrec will be caught below.
 		 */
-		if (j == e->numterms && yasm_symrec_is_curpos(sym)
-		    && !value->curpos_rel) {
+		if (j == e->numterms && !value->curpos_rel
+		    && (yasm_symrec_is_curpos(sym)
+			|| (expr_precbc
+			    && sect2 == yasm_bc_get_section(expr_precbc)))) {
 		    for (j=0; j<e->numterms; j++) {
-			if ((e->terms[j].type == YASM_EXPR_SYM
-			    || e->terms[j].type == YASM_EXPR_SYMEXP)
+			if (e->terms[j].type == YASM_EXPR_SYM
 			    && yasm_symrec_get_label(e->terms[j].data.sym,
 						     &precbc2)
 			    && (used & (1<<j)) == 0) {
@@ -178,17 +263,30 @@ value_finalize_scan(yasm_value *value, yasm_expr *e, int ssym_not_ok)
 				return 1;
 			    value->rel = e->terms[j].data.sym;
 			    value->curpos_rel = 1;
-			    /* Replace both symrec portions with 0 */
-			    yasm_expr_destroy(sube);
-			    e->terms[i].type = YASM_EXPR_INT;
-			    e->terms[i].data.intn = yasm_intnum_create_uint(0);
-			    e->terms[j].type = YASM_EXPR_INT;
-			    e->terms[j].data.intn = yasm_intnum_create_uint(0);
+			    if (yasm_symrec_is_curpos(sym)) {
+				/* Replace both symrec portions with 0 */
+				yasm_expr_destroy(sube);
+				e->terms[i].type = YASM_EXPR_INT;
+				e->terms[i].data.intn =
+				    yasm_intnum_create_uint(0);
+				e->terms[j].type = YASM_EXPR_INT;
+				e->terms[j].data.intn =
+				    yasm_intnum_create_uint(0);
+			    } else {
+				/* Replace positive portion with curpos */
+				yasm_symtab *symtab =
+				    yasm_object_get_symtab(
+					yasm_section_get_object(sect2));
+				e->terms[j].data.sym =
+				    yasm_symtab_define_curpos
+				    (symtab, ".", expr_precbc, e->line);
+			    }
 			    break;	/* stop looking */
 			}
 		    }
 		}
 
+
 		if (j == e->numterms)
 		    return 1;	/* We didn't find a match! */
 	    }
@@ -197,8 +295,7 @@ value_finalize_scan(yasm_value *value, yasm_expr *e, int ssym_not_ok)
 	     * we don't WANT to find one, error out.
 	     */
 	    for (i=0; i<e->numterms; i++) {
-		if ((e->terms[i].type == YASM_EXPR_SYM
-		     || e->terms[i].type == YASM_EXPR_SYMEXP)
+		if (e->terms[i].type == YASM_EXPR_SYM
 		    && (used & (1<<i)) == 0) {
 		    if (value->rel || ssym_not_ok)
 			return 1;
@@ -222,10 +319,10 @@ value_finalize_scan(yasm_value *value, yasm_expr *e, int ssym_not_ok)
 		case YASM_EXPR_FLOAT:
 		    return 1;		/* not legal */
 		case YASM_EXPR_SYM:
-		case YASM_EXPR_SYMEXP:
 		    return 1;
 		case YASM_EXPR_EXPR:
-		    if (value_finalize_scan(value, e->terms[1].data.expn, 1))
+		    if (value_finalize_scan(value, e->terms[1].data.expn,
+					    expr_precbc, 1))
 			return 1;
 		    break;
 		default:
@@ -238,7 +335,6 @@ value_finalize_scan(yasm_value *value, yasm_expr *e, int ssym_not_ok)
 		case YASM_EXPR_FLOAT:
 		    return 1;		/* not legal */
 		case YASM_EXPR_SYM:
-		case YASM_EXPR_SYMEXP:
 		    if (value->rel || ssym_not_ok)
 			return 1;
 		    value->rel = e->terms[0].data.sym;
@@ -249,7 +345,7 @@ value_finalize_scan(yasm_value *value, yasm_expr *e, int ssym_not_ok)
 		case YASM_EXPR_EXPR:
 		    /* recurse */
 		    if (value_finalize_scan(value, e->terms[0].data.expn,
-					    ssym_not_ok))
+					    expr_precbc, ssym_not_ok))
 			return 1;
 		    break;
 		default:
@@ -273,8 +369,7 @@ value_finalize_scan(yasm_value *value, yasm_expr *e, int ssym_not_ok)
 	     * Not okay for anything BUT a single symrec as an immediate
 	     * child.
 	     */
-	    if (e->terms[0].type != YASM_EXPR_SYM
-		&& e->terms[0].type != YASM_EXPR_SYMEXP)
+	    if (e->terms[0].type != YASM_EXPR_SYM)
 		return 1;
 
 	    if (value->seg_of)
@@ -300,7 +395,6 @@ value_finalize_scan(yasm_value *value, yasm_expr *e, int ssym_not_ok)
 	    /* Handle RHS */
 	    switch (e->terms[1].type) {
 		case YASM_EXPR_SYM:
-		case YASM_EXPR_SYMEXP:
 		    if (value->wrt)
 			return 1;
 		    value->wrt = e->terms[1].data.sym;
@@ -317,7 +411,6 @@ value_finalize_scan(yasm_value *value, yasm_expr *e, int ssym_not_ok)
 	    /* Handle LHS */
 	    switch (e->terms[0].type) {
 		case YASM_EXPR_SYM:
-		case YASM_EXPR_SYMEXP:
 		    if (value->rel || ssym_not_ok)
 			return 1;
 		    value->rel = e->terms[0].data.sym;
@@ -328,7 +421,7 @@ value_finalize_scan(yasm_value *value, yasm_expr *e, int ssym_not_ok)
 		case YASM_EXPR_EXPR:
 		    /* recurse */
 		    return value_finalize_scan(value, e->terms[0].data.expn,
-					       ssym_not_ok);
+					       expr_precbc, ssym_not_ok);
 		default:
 		    break;  /* ignore */
 	    }
@@ -339,12 +432,12 @@ value_finalize_scan(yasm_value *value, yasm_expr *e, int ssym_not_ok)
 	    for (i=0; i<e->numterms; i++) {
 		switch (e->terms[i].type) {
 		    case YASM_EXPR_SYM:
-		    case YASM_EXPR_SYMEXP:
 			return 1;
 		    case YASM_EXPR_EXPR:
 			/* recurse */
 			return value_finalize_scan(value,
-						   e->terms[i].data.expn, 1);
+						   e->terms[i].data.expn,
+						   expr_precbc, 1);
 		    default:
 			break;
 		}
@@ -356,20 +449,20 @@ value_finalize_scan(yasm_value *value, yasm_expr *e, int ssym_not_ok)
 }
 
 int
-yasm_value_finalize_expr(yasm_value *value, yasm_expr *e)
+yasm_value_finalize_expr(yasm_value *value, yasm_expr *e,
+			 yasm_bytecode *precbc, unsigned int size)
 {
-    int error = 0;
-
     if (!e) {
-	yasm_value_initialize(value, NULL);
+	yasm_value_initialize(value, NULL, size);
 	return 0;
     }
 
-    yasm_value_initialize(value, yasm_expr__level_tree
-			  (e, 1, 1, 0, NULL, NULL, NULL, NULL, &error));
+    yasm_value_initialize(value,
+			  yasm_expr__level_tree(e, 1, 1, 0, 0, NULL, NULL),
+			  size);
 
     /* quit early if there was an issue in simplify() */
-    if (error)
+    if (yasm_error_occurred())
 	return 1;
 
     /* Handle trivial (IDENT) cases immediately */
@@ -385,7 +478,6 @@ yasm_value_finalize_expr(yasm_value *value, yasm_expr *e)
 	    case YASM_EXPR_FLOAT:
 		return 0;
 	    case YASM_EXPR_SYM:
-	    case YASM_EXPR_SYMEXP:
 		value->rel = value->abs->terms[0].data.sym;
 		yasm_expr_destroy(value->abs);
 		value->abs = NULL;
@@ -404,11 +496,10 @@ yasm_value_finalize_expr(yasm_value *value, yasm_expr *e)
 	}
     }
 
-    if (value_finalize_scan(value, value->abs, 0))
+    if (value_finalize_scan(value, value->abs, precbc, 0))
 	return 1;
 
-    value->abs = yasm_expr__level_tree(value->abs, 1, 1, 0, NULL, NULL, NULL,
-				       NULL, NULL);
+    value->abs = yasm_expr__level_tree(value->abs, 1, 1, 0, 0, NULL, NULL);
 
     /* Simplify 0 in abs to NULL */
     if (value->abs->op == YASM_EXPR_IDENT
@@ -420,27 +511,93 @@ yasm_value_finalize_expr(yasm_value *value, yasm_expr *e)
     return 0;
 }
 
+int
+yasm_value_finalize(yasm_value *value, yasm_bytecode *precbc)
+{
+    unsigned int valsize = value->size;
+    return yasm_value_finalize_expr(value, value->abs, precbc, valsize);
+}
+
+yasm_intnum *
+yasm_value_get_intnum(yasm_value *value, yasm_bytecode *bc, int calc_bc_dist)
+{
+    /*@dependent@*/ /*@null@*/ yasm_intnum *intn = NULL;
+    /*@only@*/ yasm_intnum *outval;
+    int sym_local;
+
+    if (value->abs) {
+	/* Handle integer expressions, if non-integer or too complex, return
+	 * NULL.
+	 */
+	intn = yasm_expr_get_intnum(&value->abs, calc_bc_dist);
+	if (!intn)
+	    return NULL;
+    }
+
+    if (value->rel) {
+	/* If relative portion is not in bc section, return NULL.
+	 * Otherwise get the relative portion's offset.
+	 */
+	/*@dependent@*/ yasm_bytecode *rel_prevbc;
+	unsigned long dist;
+
+	if (!bc)
+	    return NULL;    /* Can't calculate relative value */
+
+	sym_local = yasm_symrec_get_label(value->rel, &rel_prevbc);
+	if (value->wrt || value->seg_of || value->section_rel || !sym_local)
+	    return NULL;    /* we can't handle SEG, WRT, or external symbols */
+	if (rel_prevbc->section != bc->section)
+	    return NULL;    /* not in this section */
+	if (!value->curpos_rel)
+	    return NULL;    /* not PC-relative */
+
+	/* Calculate value relative to current assembly position */
+	dist = yasm_bc_next_offset(rel_prevbc);
+	if (dist < bc->offset) {
+	    outval = yasm_intnum_create_uint(bc->offset - dist);
+	    yasm_intnum_calc(outval, YASM_EXPR_NEG, NULL);
+	} else {
+	    dist -= bc->offset;
+	    outval = yasm_intnum_create_uint(dist);
+	}
+
+	if (value->rshift > 0) {
+	    /*@only@*/ yasm_intnum *shamt =
+		yasm_intnum_create_uint((unsigned long)value->rshift);
+	    yasm_intnum_calc(outval, YASM_EXPR_SHR, shamt);
+	    yasm_intnum_destroy(shamt);
+	}
+	/* Add in absolute portion */
+	if (intn)
+	    yasm_intnum_calc(outval, YASM_EXPR_ADD, intn);
+	return outval;
+    }
+
+    if (intn)
+	return yasm_intnum_copy(intn);
+    
+    /* No absolute or relative portions: output 0 */
+    return yasm_intnum_create_uint(0);
+}
+
 int
 yasm_value_output_basic(yasm_value *value, /*@out@*/ unsigned char *buf,
-			size_t destsize, size_t valsize, int shift,
-			yasm_bytecode *bc, int warn, yasm_arch *arch,
-			yasm_calc_bc_dist_func calc_bc_dist)
+			size_t destsize, yasm_bytecode *bc, int warn,
+			yasm_arch *arch)
 {
     /*@dependent@*/ /*@null@*/ yasm_intnum *intn = NULL;
     /*@only@*/ yasm_intnum *outval;
     int sym_local;
     int retval = 1;
+    unsigned int valsize = value->size;
 
     if (value->abs) {
 	/* Handle floating point expressions */
 	if (!value->rel && value->abs->op == YASM_EXPR_IDENT
 	    && value->abs->terms[0].type == YASM_EXPR_FLOAT) {
-	    if (shift < 0)
-		yasm_internal_error(N_("attempting to negative shift a float"));
 	    if (yasm_arch_floatnum_tobytes(arch, value->abs->terms[0].data.flt,
-					   buf, destsize, valsize,
-					   (unsigned int)shift, warn,
-					   bc->line))
+					   buf, destsize, valsize, 0, warn))
 		return -1;
 	    else
 		return 1;
@@ -448,18 +605,37 @@ yasm_value_output_basic(yasm_value *value, /*@out@*/ unsigned char *buf,
 
 	/* Check for complex float expressions */
 	if (yasm_expr__contains(value->abs, YASM_EXPR_FLOAT)) {
-	    yasm__error(bc->line, N_("floating point expression too complex"));
+	    yasm_error_set(YASM_ERROR_FLOATING_POINT,
+			   N_("floating point expression too complex"));
 	    return -1;
 	}
 
-	/* Handle integer expressions */
-	intn = yasm_expr_get_intnum(&value->abs, calc_bc_dist);
+	/* Handle normal integer expressions */
+	intn = yasm_expr_get_intnum(&value->abs, 1);
+
 	if (!intn) {
-	    yasm__error(bc->line, N_("expression too complex"));
+	    /* Second try before erroring: yasm_expr_get_intnum doesn't handle
+	     * SEG:OFF, so try simplifying out any to just the OFF portion,
+	     * then getting the intnum again.
+	     */
+	    yasm_expr *seg = yasm_expr_extract_deep_segoff(&value->abs);
+	    if (seg)
+		yasm_expr_destroy(seg);
+	    intn = yasm_expr_get_intnum(&value->abs, 1);
+	}
+
+	if (!intn) {
+	    /* Still don't have an integer! */
+	    yasm_error_set(YASM_ERROR_TOO_COMPLEX,
+			   N_("expression too complex"));
 	    return -1;
 	}
     }
 
+    /* Adjust warn for signed/unsigned integer warnings */
+    if (warn != 0)
+	warn = value->sign ? -1 : 1;
+
     if (value->rel) {
 	/* If relative portion is not in bc section, don't try to handle it
 	 * here.  Otherwise get the relative portion's offset.
@@ -476,10 +652,10 @@ yasm_value_output_basic(yasm_value *value, /*@out@*/ unsigned char *buf,
 	    return 0;	    /* not PC-relative */
 
 	/* Calculate value relative to current assembly position */
-	dist = rel_prevbc->offset + rel_prevbc->len;
+	dist = yasm_bc_next_offset(rel_prevbc);
 	if (dist < bc->offset) {
 	    outval = yasm_intnum_create_uint(bc->offset - dist);
-	    yasm_intnum_calc(outval, YASM_EXPR_NEG, NULL, bc->line);
+	    yasm_intnum_calc(outval, YASM_EXPR_NEG, NULL);
 	} else {
 	    dist -= bc->offset;
 	    outval = yasm_intnum_create_uint(dist);
@@ -488,27 +664,34 @@ yasm_value_output_basic(yasm_value *value, /*@out@*/ unsigned char *buf,
 	if (value->rshift > 0) {
 	    /*@only@*/ yasm_intnum *shamt =
 		yasm_intnum_create_uint((unsigned long)value->rshift);
-	    yasm_intnum_calc(outval, YASM_EXPR_SHR, shamt, bc->line);
+	    yasm_intnum_calc(outval, YASM_EXPR_SHR, shamt);
 	    yasm_intnum_destroy(shamt);
 	}
 	/* Add in absolute portion */
 	if (intn)
-	    yasm_intnum_calc(outval, YASM_EXPR_ADD, intn, bc->line);
+	    yasm_intnum_calc(outval, YASM_EXPR_ADD, intn);
 	/* Output! */
-	if (yasm_arch_intnum_tobytes(arch, outval, buf, destsize, valsize,
-				     shift, bc, warn, bc->line))
+	if (yasm_arch_intnum_tobytes(arch, outval, buf, destsize, valsize, 0,
+				     bc, warn))
 	    retval = -1;
 	yasm_intnum_destroy(outval);
-    } else if (intn) {
+	return retval;
+    }
+
+    if (value->seg_of || value->rshift || value->curpos_rel || value->ip_rel
+	|| value->section_rel)
+	return 0;   /* We can't handle this with just an absolute */
+
+    if (intn) {
 	/* Output just absolute portion */
-	if (yasm_arch_intnum_tobytes(arch, intn, buf, destsize, valsize,
-				     shift, bc, warn, bc->line))
+	if (yasm_arch_intnum_tobytes(arch, intn, buf, destsize, valsize, 0, bc,
+				     warn))
 	    retval = -1;
     } else {
 	/* No absolute or relative portions: output 0 */
 	outval = yasm_intnum_create_uint(0);
-	if (yasm_arch_intnum_tobytes(arch, outval, buf, destsize, valsize,
-				     shift, bc, warn, bc->line))
+	if (yasm_arch_intnum_tobytes(arch, outval, buf, destsize, valsize, 0,
+				     bc, warn))
 	    retval = -1;
 	yasm_intnum_destroy(outval);
     }
@@ -518,6 +701,8 @@ yasm_value_output_basic(yasm_value *value, /*@out@*/ unsigned char *buf,
 void
 yasm_value_print(const yasm_value *value, FILE *f, int indent_level)
 {
+    fprintf(f, "%*s%u-bit, %ssigned", indent_level, "", value->size,
+	    value->sign ? "" : "un");
     fprintf(f, "%*sAbsolute portion=", indent_level, "");
     yasm_expr_print(value->abs, f);
     fprintf(f, "\n");
@@ -534,5 +719,11 @@ yasm_value_print(const yasm_value *value, FILE *f, int indent_level)
 	if (value->curpos_rel)
 	    fprintf(f, "%*s(Relative to current position)\n", indent_level,
 		    "");
+	if (value->ip_rel)
+	    fprintf(f, "%*s(IP-relative)\n", indent_level, "");
+	if (value->jump_target)
+	    fprintf(f, "%*s(Jump target)\n", indent_level, "");
+	if (value->section_rel)
+	    fprintf(f, "%*s(Section-relative)\n", indent_level, "");
     }
 }
