@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: x86arch.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: x86arch.c 1774 2007-02-18 20:42:55Z peter $");
 
 #define YASM_LIB_INTERNAL
 #include <libyasm.h>
@@ -61,10 +61,12 @@ x86_create(const char *machine, const char *parser,
     arch_x86->cpu_enabled = ~CPU_Any;
     arch_x86->amd64_machine = amd64_machine;
     arch_x86->mode_bits = 0;
+    arch_x86->force_strict = 0;
 
     if (yasm__strcasecmp(parser, "nasm") == 0)
 	arch_x86->parser = X86_PARSER_NASM;
-    else if (yasm__strcasecmp(parser, "gas") == 0)
+    else if (yasm__strcasecmp(parser, "gas") == 0
+	     || yasm__strcasecmp(parser, "gnu") == 0)
 	arch_x86->parser = X86_PARSER_GAS;
     else {
 	yasm_xfree(arch_x86);
@@ -108,7 +110,9 @@ x86_set_var(yasm_arch *arch, const char *var, unsigned long val)
 {
     yasm_arch_x86 *arch_x86 = (yasm_arch_x86 *)arch;
     if (yasm__strcasecmp(var, "mode_bits") == 0)
-	arch_x86->mode_bits = val;
+	arch_x86->mode_bits = (unsigned char)val;
+    else if (yasm__strcasecmp(var, "force_strict") == 0)
+	arch_x86->force_strict = (unsigned char)val;
     else
 	return 1;
     return 0;
@@ -116,10 +120,11 @@ x86_set_var(yasm_arch *arch, const char *var, unsigned long val)
 
 static int
 x86_parse_directive(yasm_arch *arch, const char *name,
-		    yasm_valparamhead *valparams,
+		    /*@null@*/ yasm_valparamhead *valparams,
 		    /*@unused@*/ /*@null@*/
 		    yasm_valparamhead *objext_valparams,
-		    /*@unused@*/ yasm_object *object, unsigned long line)
+		    /*@unused@*/ yasm_object *object,
+		    /*@unused@*/ unsigned long line)
 {
     yasm_arch_x86 *arch_x86 = (yasm_arch_x86 *)arch;
     yasm_valparam *vp;
@@ -127,14 +132,18 @@ x86_parse_directive(yasm_arch *arch, const char *name,
     long lval;
 
     if (yasm__strcasecmp(name, "bits") == 0) {
-	if ((vp = yasm_vps_first(valparams)) && !vp->val &&
+	if (!valparams)
+	    yasm_error_set(YASM_ERROR_VALUE, N_("[%s] requires an argument"),
+			   "BITS");
+	else if ((vp = yasm_vps_first(valparams)) && !vp->val &&
 	    vp->param != NULL &&
-	    (intn = yasm_expr_get_intnum(&vp->param, NULL)) != NULL &&
+	    (intn = yasm_expr_get_intnum(&vp->param, 0)) != NULL &&
 	    (lval = yasm_intnum_get_int(intn)) &&
 	    (lval == 16 || lval == 32 || lval == 64))
 	    arch_x86->mode_bits = (unsigned char)lval;
 	else
-	    yasm__error(line, N_("invalid argument to [%s]"), "BITS");
+	    yasm_error_set(YASM_ERROR_VALUE, N_("invalid argument to [%s]"),
+			   "BITS");
 	return 0;
     } else
 	return 1;
@@ -146,148 +155,199 @@ x86_get_fill(const yasm_arch *arch)
     const yasm_arch_x86 *arch_x86 = (const yasm_arch_x86 *)arch;
 
     /* Fill patterns that GAS uses. */
-    static const char *fill16[16] = {
+    static const unsigned char *fill16[16] = {
 	NULL,				/* unused			*/
+	(const unsigned char *)
 	"\x90",				/* 1 - nop			*/
+	(const unsigned char *)
 	"\x89\xf6",			/* 2 - mov si, si		*/
+	(const unsigned char *)
 	"\x8d\x74\x00",			/* 3 - lea si, [si+byte 0]	*/
+	(const unsigned char *)
 	"\x8d\xb4\x00\x00",		/* 4 - lea si, [si+word 0]	*/
+	(const unsigned char *)
 	"\x90"				/* 5 - nop			*/
 	"\x8d\xb4\x00\x00",		/*     lea si, [si+word 0]	*/
+	(const unsigned char *)
 	"\x89\xf6"			/* 6 - mov si, si		*/
 	"\x8d\xbd\x00\x00",		/*     lea di, [di+word 0]	*/
+	(const unsigned char *)
 	"\x8d\x74\x00"			/* 7 - lea si, [si+byte 0]	*/
 	"\x8d\xbd\x00\x00",		/*     lea di, [di+word 0]	*/
+	(const unsigned char *)
 	"\x8d\xb4\x00\x00"		/* 8 - lea si, [si+word 0]	*/
 	"\x8d\xbd\x00\x00",		/*     lea di, [di+word 0]	*/
+	(const unsigned char *)
 	"\xeb\x07\x90\x90\x90\x90\x90"	/* 9 - jmp $+9; nop fill	*/
 	"\x90\x90",
+	(const unsigned char *)
 	"\xeb\x08\x90\x90\x90\x90\x90"	/* 10 - jmp $+10; nop fill	*/
 	"\x90\x90\x90",
+	(const unsigned char *)
 	"\xeb\x09\x90\x90\x90\x90\x90"	/* 11 - jmp $+11; nop fill	*/
 	"\x90\x90\x90\x90",
+	(const unsigned char *)
 	"\xeb\x0a\x90\x90\x90\x90\x90"	/* 12 - jmp $+12; nop fill	*/
 	"\x90\x90\x90\x90\x90",
+	(const unsigned char *)
 	"\xeb\x0b\x90\x90\x90\x90\x90"	/* 13 - jmp $+13; nop fill	*/
 	"\x90\x90\x90\x90\x90\x90",
+	(const unsigned char *)
 	"\xeb\x0c\x90\x90\x90\x90\x90"	/* 14 - jmp $+14; nop fill	*/
 	"\x90\x90\x90\x90\x90\x90\x90",
+	(const unsigned char *)
 	"\xeb\x0d\x90\x90\x90\x90\x90"	/* 15 - jmp $+15; nop fill	*/
 	"\x90\x90\x90\x90\x90\x90\x90\x90"
     };
-    static const char *fill32[16] = {
+    static const unsigned char *fill32[16] = {
 	NULL,				/* unused			*/
+	(const unsigned char *)
 	"\x90",				/* 1 - nop			*/
+	(const unsigned char *)
 	"\x89\xf6",			/* 2 - mov esi, esi		*/
+	(const unsigned char *)
 	"\x8d\x76\x00",			/* 3 - lea esi, [esi+byte 0]	*/
+	(const unsigned char *)
 	"\x8d\x74\x26\x00",		/* 4 - lea esi, [esi*1+byte 0]	*/
+	(const unsigned char *)
 	"\x90"				/* 5 - nop			*/
 	"\x8d\x74\x26\x00",		/*     lea esi, [esi*1+byte 0]	*/
+	(const unsigned char *)
 	"\x8d\xb6\x00\x00\x00\x00",	/* 6 - lea esi, [esi+dword 0]	*/
+	(const unsigned char *)
 	"\x8d\xb4\x26\x00\x00\x00\x00",	/* 7 - lea esi, [esi*1+dword 0]	*/
+	(const unsigned char *)
 	"\x90"				/* 8 - nop			*/
 	"\x8d\xb4\x26\x00\x00\x00\x00",	/*     lea esi, [esi*1+dword 0]	*/
 #if 0
 	/* GAS uses these */
+	(const unsigned char *)
 	"\x89\xf6"			/* 9 - mov esi, esi		*/
 	"\x8d\xbc\x27\x00\x00\x00\x00",	/*     lea edi, [edi*1+dword 0]	*/
+	(const unsigned char *)
 	"\x8d\x76\x00"			/* 10 - lea esi, [esi+byte 0]	*/
 	"\x8d\xbc\x27\x00\x00\x00\x00",	/*      lea edi, [edi+dword 0]	*/
+	(const unsigned char *)
 	"\x8d\x74\x26\x00"		/* 11 - lea esi, [esi*1+byte 0]	*/
 	"\x8d\xbc\x27\x00\x00\x00\x00",	/*      lea edi, [edi*1+dword 0]*/
+	(const unsigned char *)
 	"\x8d\xb6\x00\x00\x00\x00"	/* 12 - lea esi, [esi+dword 0]	*/
 	"\x8d\xbf\x00\x00\x00\x00",	/*      lea edi, [edi+dword 0]	*/
+	(const unsigned char *)
 	"\x8d\xb6\x00\x00\x00\x00"	/* 13 - lea esi, [esi+dword 0]	*/
 	"\x8d\xbc\x27\x00\x00\x00\x00",	/*      lea edi, [edi*1+dword 0]*/
+	(const unsigned char *)
 	"\x8d\xb4\x26\x00\x00\x00\x00"	/* 14 - lea esi, [esi*1+dword 0]*/
 	"\x8d\xbc\x27\x00\x00\x00\x00",	/*      lea edi, [edi*1+dword 0]*/
 #else
 	/* But on newer processors, these are recommended */
+	(const unsigned char *)
 	"\xeb\x07\x90\x90\x90\x90\x90"	/* 9 - jmp $+9; nop fill	*/
 	"\x90\x90",
+	(const unsigned char *)
 	"\xeb\x08\x90\x90\x90\x90\x90"	/* 10 - jmp $+10; nop fill	*/
 	"\x90\x90\x90",
+	(const unsigned char *)
 	"\xeb\x09\x90\x90\x90\x90\x90"	/* 11 - jmp $+11; nop fill	*/
 	"\x90\x90\x90\x90",
+	(const unsigned char *)
 	"\xeb\x0a\x90\x90\x90\x90\x90"	/* 12 - jmp $+12; nop fill	*/
 	"\x90\x90\x90\x90\x90",
+	(const unsigned char *)
 	"\xeb\x0b\x90\x90\x90\x90\x90"	/* 13 - jmp $+13; nop fill	*/
 	"\x90\x90\x90\x90\x90\x90",
+	(const unsigned char *)
 	"\xeb\x0c\x90\x90\x90\x90\x90"	/* 14 - jmp $+14; nop fill	*/
 	"\x90\x90\x90\x90\x90\x90\x90",
 #endif
+	(const unsigned char *)
 	"\xeb\x0d\x90\x90\x90\x90\x90"	/* 15 - jmp $+15; nop fill	*/
 	"\x90\x90\x90\x90\x90\x90\x90\x90"
     };
-    static const char *fill64[16] = {
+    static const unsigned char *fill64[16] = {
 	NULL,				/* unused			*/
+	(const unsigned char *)
 	"\x90",				/* 1 - nop			*/
+	(const unsigned char *)
 	"\x66\x90",			/* 2 - o16; nop			*/
+	(const unsigned char *)
 	"\x66\x66\x90",			/* 3 - o16; o16; nop		*/
+	(const unsigned char *)
 	"\x66\x66\x66\x90",		/* 4 - o16; o16; o16; nop	*/
+	(const unsigned char *)
 	"\x66\x66\x90\x66\x90",		/* 5 */
+	(const unsigned char *)
 	"\x66\x66\x90\x66\x66\x90",	/* 6 */
+	(const unsigned char *)
 	"\x66\x66\x66\x90\x66\x66\x90",	/* 7 */
+	(const unsigned char *)
 	"\x66\x66\x66\x90\x66\x66\x66"	/* 8 */
 	"\x90",
+	(const unsigned char *)
 	"\x66\x66\x90\x66\x66\x90\x66"	/* 9 */
 	"\x66\x90",
+	(const unsigned char *)
 	"\x66\x66\x66\x90\x66\x66\x90"	/* 10 */
 	"\x66\x66\x90",
+	(const unsigned char *)
 	"\x66\x66\x66\x90\x66\x66\x66"	/* 11 */
 	"\x90\x66\x66\x90",
+	(const unsigned char *)
 	"\x66\x66\x66\x90\x66\x66\x66"	/* 12 */
 	"\x90\x66\x66\x66\x90",
+	(const unsigned char *)
 	"\x66\x66\x66\x90\x66\x66\x90"	/* 13 */
 	"\x66\x66\x90\x66\x66\x90",
+	(const unsigned char *)
 	"\x66\x66\x66\x90\x66\x66\x66"	/* 14 */
 	"\x90\x66\x66\x90\x66\x66\x90",
+	(const unsigned char *)
 	"\x66\x66\x66\x90\x66\x66\x66"	/* 15 */
 	"\x90\x66\x66\x66\x90\x66\x66\x90"
     };
     switch (arch_x86->mode_bits) {
 	case 16:
-	    return (const unsigned char **)fill16;
+	    return fill16;
 	case 32:
-	    return (const unsigned char **)fill32;
+	    return fill32;
 	case 64:
-	    return (const unsigned char **)fill64;
+	    return fill64;
 	default:
-	    yasm_internal_error(N_("Invalid mode_bits in x86_get_fill"));
-	    /*@notreached@*/
+	    yasm_error_set(YASM_ERROR_VALUE,
+			   N_("Invalid mode_bits in x86_get_fill"));
 	    return NULL;
     }
 }
 
 unsigned int
-yasm_x86__get_reg_size(yasm_arch *arch, unsigned long reg)
+yasm_x86__get_reg_size(yasm_arch *arch, uintptr_t reg)
 {
     switch ((x86_expritem_reg_size)(reg & ~0xFUL)) {
 	case X86_REG8:
 	case X86_REG8X:
-	    return 1;
+	    return 8;
 	case X86_REG16:
-	    return 2;
+	    return 16;
 	case X86_REG32:
 	case X86_CRREG:
 	case X86_DRREG:
 	case X86_TRREG:
-	    return 4;
+	    return 32;
 	case X86_REG64:
 	case X86_MMXREG:
-	    return 8;
+	    return 64;
 	case X86_XMMREG:
-	    return 16;
+	    return 128;
 	case X86_FPUREG:
-	    return 10;
+	    return 80;
 	default:
-	    yasm_internal_error(N_("unknown register size"));
+	    yasm_error_set(YASM_ERROR_VALUE, N_("unknown register size"));
     }
     return 0;
 }
 
-static unsigned long
-x86_reggroup_get_reg(yasm_arch *arch, unsigned long reggroup,
+static uintptr_t
+x86_reggroup_get_reg(yasm_arch *arch, uintptr_t reggroup,
 		     unsigned long regindex)
 {
     yasm_arch_x86 *arch_x86 = (yasm_arch_x86 *)arch;
@@ -305,13 +365,13 @@ x86_reggroup_get_reg(yasm_arch *arch, unsigned long reggroup,
 		return 0;
 	    return reggroup | (regindex & 7);
 	default:
-	    yasm_internal_error(N_("bad register group"));
+	    yasm_error_set(YASM_ERROR_VALUE, N_("bad register group"));
     }
     return 0;
 }
 
 static void
-x86_reg_print(yasm_arch *arch, unsigned long reg, FILE *f)
+x86_reg_print(yasm_arch *arch, uintptr_t reg, FILE *f)
 {
     static const char *name8[] = {"al","cl","dl","bl","ah","ch","dh","bh"};
     static const char *name8x[] = {
@@ -366,12 +426,12 @@ x86_reg_print(yasm_arch *arch, unsigned long reg, FILE *f)
 	    fprintf(f, "st%d", (int)(reg&0xF));
 	    break;
 	default:
-	    yasm_internal_error(N_("unknown register size"));
+	    yasm_error_set(YASM_ERROR_VALUE, N_("unknown register size"));
     }
 }
 
 static void
-x86_segreg_print(yasm_arch *arch, unsigned long segreg, FILE *f)
+x86_segreg_print(yasm_arch *arch, uintptr_t segreg, FILE *f)
 {
     static const char *name[] = {"es","cs","ss","ds","fs","gs"};
     fprintf(f, "%s", name[segreg&7]);
@@ -408,6 +468,6 @@ yasm_arch_module yasm_x86_LTX_arch = {
     yasm_x86__ea_create_expr,
     x86_machines,
     "x86",
-    2,
+    16,
     1
 };
