@@ -1,4 +1,4 @@
-/* $Id: x86arch.h 1591 2006-07-13 03:25:38Z peter $
+/* $Id: x86arch.h 1774 2007-02-18 20:42:55Z peter $
  * x86 Architecture header file
  *
  *  Copyright (C) 2001  Peter Johnson
@@ -58,12 +58,13 @@
 #define CPU_SVM	    (1UL<<25)	/* Secure Virtual Machine instruction */
 #define CPU_PadLock (1UL<<25)	/* VIA PadLock instruction */
 #define CPU_EM64T   (1UL<<26)	/* Intel EM64T or better */
+#define CPU_SSE4    (1UL<<27)   /* Streaming SIMD extensions 4 required */
 
 /* Technically not CPU capabilities, they do affect what instructions are
  * available.  These are tested against BITS==64.
  */
-#define CPU_64	    (1UL<<26)	/* Only available in 64-bit mode */
-#define CPU_Not64   (1UL<<27)	/* Not available (invalid) in 64-bit mode */
+#define CPU_64	    (1UL<<30)	/* Only available in 64-bit mode */
+#define CPU_Not64   (1UL<<31)	/* Not available (invalid) in 64-bit mode */
 
 typedef struct yasm_arch_x86 {
     yasm_arch_base arch;	/* base structure */
@@ -76,6 +77,7 @@ typedef struct yasm_arch_x86 {
 	X86_PARSER_GAS
     } parser;
     unsigned char mode_bits;
+    unsigned char force_strict;
 } yasm_arch_x86;
 
 /* 0-15 (low 4 bits) used for register number, stored in same data area.
@@ -108,8 +110,7 @@ typedef enum {
     X86_NEAR = 1,
     X86_SHORT,
     X86_FAR,
-    X86_TO,
-    X86_FAR_SEGOFF	    /* FAR due to SEG:OFF immediate */
+    X86_TO
 } x86_parse_targetmod;
 
 typedef enum {
@@ -133,7 +134,7 @@ typedef enum {
  * in 64-bit mode or if it wasn't needed to express reg.
  */
 int yasm_x86__set_rex_from_reg(unsigned char *rex, unsigned char *low3,
-			       unsigned long reg, unsigned int bits,
+			       uintptr_t reg, unsigned int bits,
 			       x86_rex_bit_pos rexbit);
 
 /* Effective address type */
@@ -155,7 +156,7 @@ typedef struct x86_effaddr {
 } x86_effaddr;
 
 void yasm_x86__ea_init(x86_effaddr *x86_ea, unsigned int spare,
-		       unsigned long line);
+		       yasm_bytecode *precbc);
 
 void yasm_x86__ea_set_disponly(x86_effaddr *x86_ea);
 x86_effaddr *yasm_x86__ea_create_reg(unsigned long reg, unsigned char *rex,
@@ -170,8 +171,7 @@ void yasm_x86__bc_insn_opersize_override(yasm_bytecode *bc,
 void yasm_x86__bc_insn_addrsize_override(yasm_bytecode *bc,
 					 unsigned int addrsize);
 void yasm_x86__bc_insn_set_lockrep_prefix(yasm_bytecode *bc,
-					  unsigned int prefix,
-					  unsigned long line);
+					  unsigned int prefix);
 
 /* Bytecode types */
 typedef struct x86_common {
@@ -193,7 +193,7 @@ typedef struct x86_insn {
 
     /*@null@*/ x86_effaddr *x86_ea; /* effective address */
 
-    /*@null@*/ yasm_immval *imm;    /* immediate or relative value */
+    /*@null@*/ yasm_value *imm;	    /* immediate or relative value */
 
     unsigned char def_opersize_64;  /* default operand size in 64-bit mode */
     unsigned char special_prefix;   /* "special" prefix (0=none) */
@@ -206,43 +206,18 @@ typedef struct x86_insn {
 	/* None */
 	X86_POSTOP_NONE = 0,
 
-	/* Shift opcodes have an immediate form and a ,1 form (with no
-	 * immediate).  In the parser, we set this and opcode_len=1, but store
-	 * the ,1 version in the second byte of the opcode array.  We then
-	 * choose between the two versions once we know the actual value of
-	 * imm (because we don't know it in the parser module).
-	 *
-	 * A override to force the imm version should just leave this at
-	 * 0.  Then later code won't know the ,1 version even exists.
-	 * TODO: Figure out how this affects CPU flags processing.
-	 */
-	X86_POSTOP_SHIFT,
-
 	/* Instructions that take a sign-extended imm8 as well as imm values
 	 * (eg, the arith instructions and a subset of the imul instructions)
-	 * should set this and put the imm8 form in the second byte of the
-	 * opcode.
+	 * should set this and put the imm8 form as the "normal" opcode (in
+	 * the first one or two bytes) and non-imm8 form in the second or
+	 * third byte of the opcode.
 	 */
 	X86_POSTOP_SIGNEXT_IMM8,
 
-	/* Long (modrm+sib) mov instructions in amd64 can be optimized into
-	 * short mov instructions if a 32-bit address override is applied in
-	 * 64-bit mode to an EA of just an offset (no registers) and the
-	 * target register is al/ax/eax/rax.
-	 */
-	X86_POSTOP_SHORTMOV,
-
 	/* Override any attempt at address-size override to 16 bits, and never
 	 * generate a prefix.  This is used for the ENTER opcode.
 	 */
-	X86_POSTOP_ADDRESS16,
-
-	/* Used for 64-bit mov immediate, which can take a sign-extended
-	 * imm32 as well as imm64 values.  The imm32 form is put in the
-	 * second byte of the opcode and its ModRM byte is put in the third
-	 * byte of the opcode.
-	 */
-	X86_POSTOP_SIGNEXT_IMM32
+	X86_POSTOP_ADDRESS16
     } postop;
 } x86_insn;
 
@@ -251,7 +226,6 @@ typedef struct x86_jmp {
     x86_opcode shortop, nearop;
 
     yasm_value target;		/* jump target */
-    /*@dependent@*/ yasm_bytecode *origin_prevbc;   /* jump origin */
 
     /* which opcode are we using? */
     /* The *FORCED forms are specified in the source as such */
@@ -276,40 +250,37 @@ void yasm_x86__bc_transform_jmpfar(yasm_bytecode *bc, x86_jmpfar *jmpfar);
 
 void yasm_x86__bc_apply_prefixes
     (x86_common *common, unsigned char *rex, int num_prefixes,
-     unsigned long **prefixes, unsigned long line);
+     uintptr_t **prefixes);
 
 /* Check an effective address.  Returns 0 if EA was successfully determined,
  * 1 if invalid EA, or 2 if indeterminate EA.
  */
 int yasm_x86__expr_checkea
     (x86_effaddr *x86_ea, unsigned char *addrsize, unsigned int bits,
-     int address16_op, unsigned char *rex,
-     yasm_calc_bc_dist_func calc_bc_dist, unsigned long line);
+     int address16_op, unsigned char *rex, yasm_bytecode *bc);
 
-void yasm_x86__parse_cpu(yasm_arch *arch, const char *cpuid, size_t cpuid_len,
-			 unsigned long line);
+void yasm_x86__parse_cpu(yasm_arch *arch, const char *cpuid, size_t cpuid_len);
 
 yasm_arch_insnprefix yasm_x86__parse_check_insnprefix
-    (yasm_arch *arch, /*@out@*/ unsigned long data[4], const char *id,
-     size_t id_len, unsigned long line);
+    (yasm_arch *arch, /*@out@*/ uintptr_t data[4], const char *id,
+     size_t id_len);
 yasm_arch_regtmod yasm_x86__parse_check_regtmod
-    (yasm_arch *arch, /*@out@*/ unsigned long *data, const char *id,
-     size_t id_len, unsigned long line);
+    (yasm_arch *arch, /*@out@*/ uintptr_t *data, const char *id,
+     size_t id_len);
 
 void yasm_x86__finalize_insn
     (yasm_arch *arch, yasm_bytecode *bc, yasm_bytecode *prev_bc,
-     const unsigned long data[4], int num_operands,
+     const uintptr_t data[4], int num_operands,
      /*@null@*/ yasm_insn_operands *operands, int num_prefixes,
-     unsigned long **prefixes, int num_segregs, const unsigned long *segregs);
+     uintptr_t **prefixes, int num_segregs, const uintptr_t *segregs);
 
 int yasm_x86__floatnum_tobytes
     (yasm_arch *arch, const yasm_floatnum *flt, unsigned char *buf,
-     size_t destsize, size_t valsize, size_t shift, int warn,
-     unsigned long line);
+     size_t destsize, size_t valsize, size_t shift, int warn);
 int yasm_x86__intnum_tobytes
     (yasm_arch *arch, const yasm_intnum *intn, unsigned char *buf,
      size_t destsize, size_t valsize, int shift, const yasm_bytecode *bc,
-     int warn, unsigned long line);
+     int warn);
 
-unsigned int yasm_x86__get_reg_size(yasm_arch *arch, unsigned long reg);
+unsigned int yasm_x86__get_reg_size(yasm_arch *arch, uintptr_t reg);
 #endif
