@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: x86bc.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: x86bc.c 1795 2007-02-24 22:39:55Z peter $");
 
 #define YASM_LIB_INTERNAL
 #define YASM_BC_INTERNAL
@@ -45,16 +45,24 @@ static void x86_ea_print(const yasm_effaddr *ea, FILE *f, int indent_level);
 static void x86_bc_insn_destroy(void *contents);
 static void x86_bc_insn_print(const void *contents, FILE *f,
 			      int indent_level);
-static yasm_bc_resolve_flags x86_bc_insn_resolve
-    (yasm_bytecode *bc, int save, yasm_calc_bc_dist_func calc_bc_dist);
+static int x86_bc_insn_calc_len(yasm_bytecode *bc,
+				yasm_bc_add_span_func add_span,
+				void *add_span_data);
+static int x86_bc_insn_expand(yasm_bytecode *bc, int span, long old_val,
+			      long new_val, /*@out@*/ long *neg_thres,
+			      /*@out@*/ long *pos_thres);
 static int x86_bc_insn_tobytes(yasm_bytecode *bc, unsigned char **bufp,
 			       void *d, yasm_output_value_func output_value,
 			       /*@null@*/ yasm_output_reloc_func output_reloc);
 
 static void x86_bc_jmp_destroy(void *contents);
 static void x86_bc_jmp_print(const void *contents, FILE *f, int indent_level);
-static yasm_bc_resolve_flags x86_bc_jmp_resolve
-    (yasm_bytecode *bc, int save, yasm_calc_bc_dist_func calc_bc_dist);
+static int x86_bc_jmp_calc_len(yasm_bytecode *bc,
+			       yasm_bc_add_span_func add_span,
+			       void *add_span_data);
+static int x86_bc_jmp_expand(yasm_bytecode *bc, int span, long old_val,
+			     long new_val, /*@out@*/ long *neg_thres,
+			     /*@out@*/ long *pos_thres);
 static int x86_bc_jmp_tobytes(yasm_bytecode *bc, unsigned char **bufp,
 			      void *d, yasm_output_value_func output_value,
 			      /*@null@*/ yasm_output_reloc_func output_reloc);
@@ -62,8 +70,9 @@ static int x86_bc_jmp_tobytes(yasm_bytecode *bc, unsigned char **bufp,
 static void x86_bc_jmpfar_destroy(void *contents);
 static void x86_bc_jmpfar_print(const void *contents, FILE *f,
 				int indent_level);
-static yasm_bc_resolve_flags x86_bc_jmpfar_resolve
-    (yasm_bytecode *bc, int save, yasm_calc_bc_dist_func calc_bc_dist);
+static int x86_bc_jmpfar_calc_len(yasm_bytecode *bc,
+				  yasm_bc_add_span_func add_span,
+				  void *add_span_data);
 static int x86_bc_jmpfar_tobytes
     (yasm_bytecode *bc, unsigned char **bufp, void *d,
      yasm_output_value_func output_value,
@@ -82,29 +91,35 @@ static const yasm_bytecode_callback x86_bc_callback_insn = {
     x86_bc_insn_destroy,
     x86_bc_insn_print,
     yasm_bc_finalize_common,
-    x86_bc_insn_resolve,
-    x86_bc_insn_tobytes
+    x86_bc_insn_calc_len,
+    x86_bc_insn_expand,
+    x86_bc_insn_tobytes,
+    0
 };
 
 static const yasm_bytecode_callback x86_bc_callback_jmp = {
     x86_bc_jmp_destroy,
     x86_bc_jmp_print,
     yasm_bc_finalize_common,
-    x86_bc_jmp_resolve,
-    x86_bc_jmp_tobytes
+    x86_bc_jmp_calc_len,
+    x86_bc_jmp_expand,
+    x86_bc_jmp_tobytes,
+    0
 };
 
 static const yasm_bytecode_callback x86_bc_callback_jmpfar = {
     x86_bc_jmpfar_destroy,
     x86_bc_jmpfar_print,
     yasm_bc_finalize_common,
-    x86_bc_jmpfar_resolve,
-    x86_bc_jmpfar_tobytes
+    x86_bc_jmpfar_calc_len,
+    yasm_bc_expand_common,
+    x86_bc_jmpfar_tobytes,
+    0
 };
 
 int
 yasm_x86__set_rex_from_reg(unsigned char *rex, unsigned char *low3,
-			   unsigned long reg, unsigned int bits,
+			   uintptr_t reg, unsigned int bits,
 			   x86_rex_bit_pos rexbit)
 {
     *low3 = (unsigned char)(reg&7);
@@ -113,14 +128,21 @@ yasm_x86__set_rex_from_reg(unsigned char *rex, unsigned char *low3,
 	x86_expritem_reg_size size = (x86_expritem_reg_size)(reg & ~0xFUL);
 
 	if (size == X86_REG8X || (reg & 0xF) >= 8) {
-	    if (*rex == 0xff)
+	    /* Check to make sure we can set it */
+	    if (*rex == 0xff) {
+		yasm_error_set(YASM_ERROR_TYPE,
+		    N_("cannot use A/B/C/DH with instruction needing REX"));
 		return 1;
+	    }
 	    *rex |= 0x40 | (((reg & 8) >> 3) << rexbit);
 	} else if (size == X86_REG8 && (reg & 7) >= 4) {
 	    /* AH/BH/CH/DH, so no REX allowed */
-	    if (*rex != 0 && *rex != 0xff)
+	    if (*rex != 0 && *rex != 0xff) {
+		yasm_error_set(YASM_ERROR_TYPE,
+		    N_("cannot use A/B/C/DH with instruction needing REX"));
 		return 1;
-	    *rex = 0xff;
+	    }
+	    *rex = 0xff;    /* Flag so we can NEVER set it (see above) */
 	}
     }
 
@@ -146,10 +168,12 @@ yasm_x86__bc_transform_jmpfar(yasm_bytecode *bc, x86_jmpfar *jmpfar)
 }
 
 void
-yasm_x86__ea_init(x86_effaddr *x86_ea, unsigned int spare, unsigned long line)
+yasm_x86__ea_init(x86_effaddr *x86_ea, unsigned int spare,
+		  yasm_bytecode *precbc)
 {
-    if (yasm_value_finalize(&x86_ea->ea.disp))
-	yasm__error(line, N_("effective address too complex"));
+    if (yasm_value_finalize(&x86_ea->ea.disp, precbc))
+	yasm_error_set(YASM_ERROR_TOO_COMPLEX,
+		       N_("effective address too complex"));
     x86_ea->modrm &= 0xC7;		    /* zero spare/reg bits */
     x86_ea->modrm |= (spare << 3) & 0x38;   /* plug in provided bits */
 }
@@ -176,8 +200,8 @@ yasm_x86__ea_create_reg(unsigned long reg, unsigned char *rex,
     x86_ea = yasm_xmalloc(sizeof(x86_effaddr));
 
     x86_ea->ea.callback = &x86_ea_callback;
-    yasm_value_initialize(&x86_ea->ea.disp, NULL);
-    x86_ea->ea.disp_len = 0;
+    yasm_value_initialize(&x86_ea->ea.disp, NULL, 0);
+    x86_ea->ea.need_nonzero_len = 0;
     x86_ea->ea.need_disp = 0;
     x86_ea->ea.nosplit = 0;
     x86_ea->ea.strong = 0;
@@ -216,8 +240,8 @@ yasm_x86__ea_create_expr(yasm_arch *arch, yasm_expr *e)
 				 yasm_expr_reg(X86_RIP), e->line);
 	}
     }
-    yasm_value_initialize(&x86_ea->ea.disp, e);
-    x86_ea->ea.disp_len = 0;
+    yasm_value_initialize(&x86_ea->ea.disp, e, 0);
+    x86_ea->ea.need_nonzero_len = 0;
     x86_ea->ea.need_disp = 1;
     x86_ea->ea.nosplit = 0;
     x86_ea->ea.strong = 0;
@@ -244,8 +268,7 @@ yasm_x86__ea_create_imm(yasm_expr *imm, unsigned int im_len)
     x86_ea = yasm_xmalloc(sizeof(x86_effaddr));
 
     x86_ea->ea.callback = &x86_ea_callback;
-    yasm_value_initialize(&x86_ea->ea.disp, imm);
-    x86_ea->ea.disp_len = (unsigned char)im_len;
+    yasm_value_initialize(&x86_ea->ea.disp, imm, im_len);
     x86_ea->ea.need_disp = 1;
     x86_ea->ea.nosplit = 0;
     x86_ea->ea.strong = 0;
@@ -263,8 +286,7 @@ yasm_x86__ea_create_imm(yasm_expr *imm, unsigned int im_len)
 
 void
 yasm_x86__bc_apply_prefixes(x86_common *common, unsigned char *rex,
-			    int num_prefixes, unsigned long **prefixes,
-			    unsigned long line)
+			    int num_prefixes, uintptr_t **prefixes)
 {
     int i;
     int first = 1;
@@ -273,7 +295,7 @@ yasm_x86__bc_apply_prefixes(x86_common *common, unsigned char *rex,
 	switch ((x86_parse_insn_prefix)prefixes[i][0]) {
 	    case X86_LOCKREP:
 		if (common->lockrep_pre != 0)
-		    yasm__warning(YASM_WARN_GENERAL, line,
+		    yasm_warn_set(YASM_WARN_GENERAL,
 			N_("multiple LOCK or REP prefixes, using leftmost"));
 		common->lockrep_pre = (unsigned char)prefixes[i][1];
 		break;
@@ -291,18 +313,18 @@ yasm_x86__bc_apply_prefixes(x86_common *common, unsigned char *rex,
 		break;
 	    case X86_REX:
 		if (!rex)
-		    yasm__warning(YASM_WARN_GENERAL, line,
-			N_("ignoring REX prefix on jump"));
+		    yasm_warn_set(YASM_WARN_GENERAL,
+				  N_("ignoring REX prefix on jump"));
 		else if (*rex == 0xff)
-		    yasm__warning(YASM_WARN_GENERAL, line,
+		    yasm_warn_set(YASM_WARN_GENERAL,
 			N_("REX prefix not allowed on this instruction, ignoring"));
 		else {
 		    if (*rex != 0) {
 			if (first)
-			    yasm__warning(YASM_WARN_GENERAL, line,
+			    yasm_warn_set(YASM_WARN_GENERAL,
 				N_("overriding generated REX prefix"));
 			else
-			    yasm__warning(YASM_WARN_GENERAL, line,
+			    yasm_warn_set(YASM_WARN_GENERAL,
 				N_("multiple REX prefixes, using leftmost"));
 		    }
 		    /* Here we assume that we can't get this prefix in non
@@ -324,7 +346,7 @@ x86_bc_insn_destroy(void *contents)
     if (insn->x86_ea)
 	yasm_ea_destroy((yasm_effaddr *)insn->x86_ea);
     if (insn->imm) {
-	yasm_value_delete(&insn->imm->val);
+	yasm_value_delete(insn->imm);
 	yasm_xfree(insn->imm);
     }
     yasm_xfree(contents);
@@ -405,10 +427,7 @@ x86_bc_insn_print(const void *contents, FILE *f, int indent_level)
     else {
 	indent_level++;
 	fprintf(f, "\n");
-	yasm_value_print(&insn->imm->val, f, indent_level);
-	fprintf(f, "%*sLen=%u, Sign=%u\n", indent_level, "",
-		(unsigned int)insn->imm->len,
-		(unsigned int)insn->imm->sign);
+	yasm_value_print(insn->imm, f, indent_level);
 	indent_level--;
     }
     x86_opcode_print(&insn->opcode, f, indent_level);
@@ -480,7 +499,7 @@ x86_bc_jmpfar_print(const void *contents, FILE *f, int indent_level)
 }
 
 static unsigned int
-x86_common_resolve(const x86_common *common)
+x86_common_calc_len(const x86_common *common)
 {
     unsigned int len = 0;
 
@@ -496,359 +515,233 @@ x86_common_resolve(const x86_common *common)
     return len;
 }
 
-static yasm_bc_resolve_flags
-x86_bc_insn_resolve(yasm_bytecode *bc, int save,
-		    yasm_calc_bc_dist_func calc_bc_dist)
+static int
+x86_bc_insn_calc_len(yasm_bytecode *bc, yasm_bc_add_span_func add_span,
+		     void *add_span_data)
 {
     x86_insn *insn = (x86_insn *)bc->contents;
     x86_effaddr *x86_ea = insn->x86_ea;
-    yasm_immval *imm = insn->imm;
-    yasm_bc_resolve_flags retval = YASM_BC_RESOLVE_MIN_LEN;
+    yasm_value *imm = insn->imm;
 
     if (x86_ea) {
-	/* Create temp copy of disp, etc. */
-	x86_effaddr eat = *x86_ea;	/* structure copy */
-
-	/* Don't overwrite original expression portion */
-	if (x86_ea->ea.disp.abs)
-	    eat.ea.disp.abs = yasm_expr_copy(x86_ea->ea.disp.abs);
-
-	/* Handle shortmov special-casing */
-	if (insn->postop == X86_POSTOP_SHORTMOV &&
-	    insn->common.mode_bits == 64 && insn->common.addrsize == 32 &&
-	    (!eat.ea.disp.abs ||
-	     !yasm_expr__contains(eat.ea.disp.abs, YASM_EXPR_REG))) {
-	    yasm_x86__ea_set_disponly(&eat);
-
-	    if (save) {
-		/* Make the short form permanent. */
-		insn->opcode.opcode[0] = insn->opcode.opcode[1];
-	    }
-	}
-
 	/* Check validity of effective address and calc R/M bits of
 	 * Mod/RM byte and SIB byte.  We won't know the Mod field
 	 * of the Mod/RM byte until we know more about the
 	 * displacement.
 	 */
-	switch (yasm_x86__expr_checkea(&eat, &insn->common.addrsize,
+	if (yasm_x86__expr_checkea(x86_ea, &insn->common.addrsize,
 		insn->common.mode_bits, insn->postop == X86_POSTOP_ADDRESS16,
-		&insn->rex, calc_bc_dist, bc->line)) {
-	    case 1:
-		yasm_expr_destroy(eat.ea.disp.abs);
-		/* failed, don't bother checking rest of insn */
-		return YASM_BC_RESOLVE_UNKNOWN_LEN|YASM_BC_RESOLVE_ERROR;
-	    case 2:
-		yasm_expr_destroy(eat.ea.disp.abs);
-		/* failed, don't bother checking rest of insn */
-		return YASM_BC_RESOLVE_UNKNOWN_LEN;
-	    default:
-		yasm_expr_destroy(eat.ea.disp.abs);
-		/* okay */
-		break;
-	}
+		&insn->rex, bc))
+	    /* failed, don't bother checking rest of insn */
+	    return -1;
 
-	if (eat.ea.disp_len != 1) {
-	    /* Fits into a word/dword, or unknown. */
-	    retval = YASM_BC_RESOLVE_NONE;  /* may not be smallest size */
-
-	    /* Handle unknown case, make displen word-sized */
-	    if (eat.ea.disp_len == 0xff)
-		eat.ea.disp_len = (insn->common.addrsize == 16) ? 2U : 4U;
+	if (x86_ea->ea.disp.size == 0 && x86_ea->ea.need_nonzero_len) {
+	    /* Handle unknown case, default to byte-sized and set as
+	     * critical expression.
+	     */
+	    x86_ea->ea.disp.size = 8;
+	    add_span(add_span_data, bc, 1, &x86_ea->ea.disp, -128, 127);
 	}
+	bc->len += x86_ea->ea.disp.size/8;
 
 	/* Handle address16 postop case */
 	if (insn->postop == X86_POSTOP_ADDRESS16)
 	    insn->common.addrsize = 0;
 
-	/* If we had forced ea->len but had to override, save it now */
-	if (x86_ea->ea.disp_len != 0 && x86_ea->ea.disp_len != eat.ea.disp_len)
-	    x86_ea->ea.disp_len = eat.ea.disp_len;
-
-	if (save) {
-	    eat.ea.disp.abs = x86_ea->ea.disp.abs; /* Copy back original */
-	    *x86_ea = eat;	/* structure copy */
-	    if (x86_ea->ea.disp_len == 0) {
-		yasm_value_delete(&x86_ea->ea.disp);
-		x86_ea->ea.need_disp = 0;
-	    }
-	}
-
 	/* Compute length of ea and add to total */
-	bc->len += eat.need_modrm + (eat.need_sib ? 1:0) + eat.ea.disp_len;
-	bc->len += (eat.ea.segreg != 0) ? 1 : 0;
+	bc->len += x86_ea->need_modrm + (x86_ea->need_sib ? 1:0);
+	bc->len += (x86_ea->ea.segreg != 0) ? 1 : 0;
     }
 
     if (imm) {
-	/*@null@*/ yasm_expr *temp = NULL;
-	const yasm_intnum *num = NULL;
-	unsigned int immlen = imm->len;
-	long val;
-
-	if (imm->val.abs) {
-	    temp = yasm_expr_copy(imm->val.abs);
-	    assert(temp != NULL);
-	    num = yasm_expr_get_intnum(&temp, calc_bc_dist);
-	}
+	unsigned int immlen = imm->size;
 
 	/* TODO: check imm->len vs. sized len from expr? */
 
-	switch (insn->postop) {
-	    case X86_POSTOP_SIGNEXT_IMM8:
-		/* Handle signext_imm8 postop special-casing */
-		if (imm->val.rel)
-		    val = 1000;	    /* has relative portion, don't collapse */
-		else if (num)
-		    val = yasm_intnum_get_int(num);
-		else
-		    val = 0;
-		if (val >= -128 && val <= 127) {
+	/* Handle signext_imm8 postop special-casing */
+	if (insn->postop == X86_POSTOP_SIGNEXT_IMM8) {
+	    /*@null@*/ /*@only@*/ yasm_intnum *num;
+	    num = yasm_value_get_intnum(imm, NULL, 0);
+
+	    if (!num) {
+		/* Unknown; default to byte form and set as critical
+		 * expression.
+		 */
+		immlen = 8;
+		add_span(add_span_data, bc, 2, imm, -128, 127);
+	    } else {
+		if (yasm_intnum_in_range(num, -128, 127)) {
 		    /* We can use the sign-extended byte form: shorten
-		     * the immediate length to 1.
+		     * the immediate length to 1 and make the byte form
+		     * permanent.
 		     */
-		    immlen = 1;
-		    if (save) {
-			/* Make the byte form permanent. */
-			insn->opcode.opcode[0] = insn->opcode.opcode[1];
-			imm->len = 1;
-			if (insn->opcode.opcode[2] != 0) {
-			    insn->opcode.opcode[1] = insn->opcode.opcode[2];
-			    insn->opcode.len++;
-			}
-		    } else if (insn->opcode.opcode[2] != 0)
-			bc->len++;
+		    imm->size = 8;
+		    imm->sign = 1;
+		    immlen = 8;
+		} else {
+		    /* We can't.  Copy over the word-sized opcode. */
+		    insn->opcode.opcode[0] =
+			insn->opcode.opcode[insn->opcode.len];
+		    insn->opcode.len = 1;
 		}
-		/* Not really necessary, but saves confusion over it. */
-		if (save)
-		    insn->postop = X86_POSTOP_NONE;
-		break;
-
-	    case X86_POSTOP_SIGNEXT_IMM32:
-		/* Handle signext_imm32 postop special-casing */
-		if (!num || yasm_intnum_check_size(num, 32, 0, 1)) {
-		    bc->len++;  /* Due to ModRM byte */
-		    immlen = 4;
-		    if (save) {
-			/* Throwaway REX byte */
-			unsigned char rex_temp = 0;
-
-			/* Build ModRM EA - CAUTION: this depends on
-			 * opcode 0 being a mov instruction!
-			 */
-			insn->x86_ea = yasm_x86__ea_create_reg(
-			    (unsigned long)insn->opcode.opcode[0]-0xB8,
-			    &rex_temp, 64);
-
-			/* Make the imm32s form permanent. */
-			insn->opcode.opcode[0] = insn->opcode.opcode[1];
-			imm->len = 4;
-		    }
-		}
-		/* Not really necessary, but saves confusion over it. */
-		if (save)
-		    insn->postop = X86_POSTOP_NONE;
-		break;
-
-	    case X86_POSTOP_SHIFT:
-		/* Handle shift postop special-casing */
-		if (num && yasm_intnum_get_uint(num) == 1) {
-		    /* We can use the ,1 form: no imm (set to 0 len) */
-		    immlen = 0;
-
-		    if (save) {
-			/* Make the ,1 form permanent. */
-			insn->opcode.opcode[0] = insn->opcode.opcode[1];
-			/* Delete imm, as it's not needed. */
-			yasm_value_delete(&imm->val);
-			yasm_xfree(imm);
-			insn->imm = (yasm_immval *)NULL;
-		    }
-		} else
-		    retval = YASM_BC_RESOLVE_NONE;  /* could still get ,1 */
-
-		/* Not really necessary, but saves confusion over it. */
-		if (save)
-		    insn->postop = X86_POSTOP_NONE;
-		break;
-
-	    default:
-		break;
+		insn->postop = X86_POSTOP_NONE;
+		yasm_intnum_destroy(num);
+	    }
 	}
 
-	yasm_expr_destroy(temp);
-
-	bc->len += immlen;
+	bc->len += immlen/8;
     }
 
     bc->len += insn->opcode.len;
-    bc->len += x86_common_resolve(&insn->common);
+    bc->len += x86_common_calc_len(&insn->common);
     bc->len += (insn->special_prefix != 0) ? 1:0;
     if (insn->rex != 0xff &&
 	(insn->rex != 0 ||
 	 (insn->common.mode_bits == 64 && insn->common.opersize == 64 &&
 	  insn->def_opersize_64 != 64)))
 	bc->len++;
-
-    return retval;
+    return 0;
 }
 
-static yasm_bc_resolve_flags
-x86_bc_jmp_resolve(yasm_bytecode *bc, int save,
-		   yasm_calc_bc_dist_func calc_bc_dist)
+static int
+x86_bc_insn_expand(yasm_bytecode *bc, int span, long old_val, long new_val,
+		   /*@out@*/ long *neg_thres, /*@out@*/ long *pos_thres)
+{
+    x86_insn *insn = (x86_insn *)bc->contents;
+    x86_effaddr *x86_ea = insn->x86_ea;
+    yasm_effaddr *ea = &x86_ea->ea;
+    yasm_value *imm = insn->imm;
+
+    if (ea && span == 1) {
+	/* Change displacement length into word-sized */
+	if (ea->disp.size == 8) {
+	    ea->disp.size = (insn->common.addrsize == 16) ? 16 : 32;
+	    x86_ea->modrm &= ~0300;
+	    x86_ea->modrm |= 0200;
+	    bc->len--;
+	    bc->len += ea->disp.size/8;
+	}
+    }
+
+    if (imm && span == 2) {
+	if (insn->postop == X86_POSTOP_SIGNEXT_IMM8) {
+	    /* Update bc->len for new opcode and immediate size */
+	    bc->len -= insn->opcode.len;
+	    bc->len += imm->size/8;
+
+	    /* Change to the word-sized opcode */
+	    insn->opcode.opcode[0] = insn->opcode.opcode[insn->opcode.len];
+	    insn->opcode.len = 1;
+	    insn->postop = X86_POSTOP_NONE;
+	}
+    }
+
+    return 0;
+}
+
+static int
+x86_bc_jmp_calc_len(yasm_bytecode *bc, yasm_bc_add_span_func add_span,
+		    void *add_span_data)
 {
     x86_jmp *jmp = (x86_jmp *)bc->contents;
-    yasm_bc_resolve_flags retval = YASM_BC_RESOLVE_MIN_LEN;
     yasm_bytecode *target_prevbc;
-    /*@null@*/ yasm_expr *temp;
-    /*@only@*/ yasm_intnum *num;
-    /*@dependent@*/ /*@null@*/ yasm_intnum *num2;
-    long rel;
     unsigned char opersize;
-    x86_jmp_opcode_sel jrtype = JMP_NONE;
 
     /* As opersize may be 0, figure out its "real" value. */
     opersize = (jmp->common.opersize == 0) ?
 	jmp->common.mode_bits : jmp->common.opersize;
 
-    /* We only check to see if forced forms are actually legal if we're in
-     * save mode.  Otherwise we assume that they are legal.
-     */
-    switch (jmp->op_sel) {
-	case JMP_SHORT_FORCED:
-	    /* 1 byte relative displacement */
-	    jrtype = JMP_SHORT;
-	    if (save) {
-		/* does a short form exist? */
-		if (jmp->shortop.len == 0) {
-		    yasm__error(bc->line, N_("short jump does not exist"));
-		    return YASM_BC_RESOLVE_ERROR | YASM_BC_RESOLVE_UNKNOWN_LEN;
-		}
+    bc->len += x86_common_calc_len(&jmp->common);
 
-		if (!jmp->target.rel)
-		    num = yasm_intnum_create_uint(0);
-		else if (!yasm_symrec_get_label(jmp->target.rel, &target_prevbc)
-			 || target_prevbc->section != jmp->origin_prevbc->section
-			 || !(num = calc_bc_dist(jmp->origin_prevbc,
-						 target_prevbc))) {
-		    /* External or out of segment, so we can't check distance.
-		     * This depends on the objfmt supporting 8-bit relocs.
-		     * While most don't, some might, so allow it.  The objfmt
-		     * will error if not supported.
-		     */
-		    break;
-		}
+    if (jmp->op_sel == JMP_NEAR_FORCED || jmp->shortop.len == 0) {
+	if (jmp->nearop.len == 0) {
+	    yasm_error_set(YASM_ERROR_TYPE, N_("near jump does not exist"));
+	    return -1;
+	}
 
-		if (jmp->target.abs) {
-		    temp = yasm_expr_copy(jmp->target.abs);
-		    num2 = yasm_expr_get_intnum(&temp, calc_bc_dist);
-		    if (!num2) {
-			yasm_expr_destroy(temp);
-			yasm__error(bc->line, N_("jump target too complex"));
-			return YASM_BC_RESOLVE_ERROR |
-			    YASM_BC_RESOLVE_UNKNOWN_LEN;
-		    }
-		    yasm_intnum_calc(num, YASM_EXPR_ADD, num2, bc->line);
-		    yasm_expr_destroy(temp);
-		}
-
-		rel = yasm_intnum_get_int(num);
-		yasm_intnum_destroy(num);
-		rel -= jmp->shortop.len+1;
-		/* short displacement must fit in -128 <= rel <= +127 */
-		if (rel < -128 || rel > 127) {
-		    yasm__error(bc->line, N_("short jump out of range"));
-		    return YASM_BC_RESOLVE_ERROR | YASM_BC_RESOLVE_UNKNOWN_LEN;
-		}
-	    }
-	    break;
-	case JMP_NEAR_FORCED:
-	    /* 2/4 byte relative displacement (depending on operand size) */
-	    jrtype = JMP_NEAR;
-	    if (save) {
-		if (jmp->nearop.len == 0) {
-		    yasm__error(bc->line, N_("near jump does not exist"));
-		    return YASM_BC_RESOLVE_ERROR | YASM_BC_RESOLVE_UNKNOWN_LEN;
-		}
-	    }
-	    break;
-	default:
-	    /* Due to code in x86_finalize_jmp(), we can only get here if
-	     * there's BOTH short and near opcodes available, and it wasn't
-	     * forced by the user.
-	     */
-	    if (!jmp->target.rel)
-		num = yasm_intnum_create_uint(0);
-	    else if (!yasm_symrec_get_label(jmp->target.rel, &target_prevbc)
-		     || target_prevbc->section != jmp->origin_prevbc->section
-		     || !(num = calc_bc_dist(jmp->origin_prevbc,
-					     target_prevbc))) {
-		/* It's unknown.  Thus, assume near displacement. */
-		jrtype = JMP_NEAR;
-		retval = YASM_BC_RESOLVE_NONE;
-		break;
-	    }
-
-	    /* Try to find shortest displacement based on difference between
-	     * target expr value and origin offset.
-	     */
-	    if (jmp->target.abs) {
-		temp = yasm_expr_copy(jmp->target.abs);
-		num2 = yasm_expr_get_intnum(&temp, calc_bc_dist);
-		if (!num2) {
-		    yasm_expr_destroy(temp);
-		    yasm__error(bc->line, N_("jump target too complex"));
-		    return YASM_BC_RESOLVE_ERROR | YASM_BC_RESOLVE_UNKNOWN_LEN;
-		}
-		yasm_intnum_calc(num, YASM_EXPR_ADD, num2, bc->line);
-		yasm_expr_destroy(temp);
-	    }
-
-	    rel = yasm_intnum_get_int(num);
-	    yasm_intnum_destroy(num);
-	    rel -= jmp->shortop.len+1;
-	    /* short displacement must fit within -128 <= rel <= +127 */
-	    if (rel >= -128 && rel <= 127) {
-		/* It fits into a short displacement. */
-		jrtype = JMP_SHORT;
-	    } else {
-		/* Near for now, but could get shorter in the future as
-		 * there's a short form available.
-		 */
-		jrtype = JMP_NEAR;
-		retval = YASM_BC_RESOLVE_NONE;
-	    }
-	    break;
+	/* Near jump, no spans needed */
+	if (jmp->shortop.len == 0)
+	    jmp->op_sel = JMP_NEAR;
+	bc->len += jmp->nearop.len;
+	bc->len += (opersize == 16) ? 2 : 4;
+	return 0;
     }
 
-    switch (jrtype) {
-	case JMP_SHORT:
-	    if (save)
+    if (jmp->op_sel == JMP_SHORT_FORCED || jmp->nearop.len == 0) {
+	if (jmp->shortop.len == 0) {
+	    yasm_error_set(YASM_ERROR_TYPE, N_("short jump does not exist"));
+	    return -1;
+	}
+
+	/* We want to be sure to error if we exceed short length, so
+	 * put it in as a dependent expression (falling through).
+	 */
+    }
+
+    if (jmp->target.rel
+	&& (!yasm_symrec_get_label(jmp->target.rel, &target_prevbc)
+	    || target_prevbc->section != bc->section)) {
+	/* External or out of segment, so we can't check distance.
+	 * Allowing short jumps depends on the objfmt supporting
+	 * 8-bit relocs.  While most don't, some might, so allow it here.
+	 * Otherwise default to word-sized.
+	 * The objfmt will error if not supported.
+	 */
+	if (jmp->op_sel == JMP_SHORT_FORCED || jmp->nearop.len == 0) {
+	    if (jmp->op_sel == JMP_NONE)
 		jmp->op_sel = JMP_SHORT;
-	    if (jmp->shortop.len == 0)
-		return YASM_BC_RESOLVE_UNKNOWN_LEN; /* size not available */
-
 	    bc->len += jmp->shortop.len + 1;
-	    break;
-	case JMP_NEAR:
-	    if (save)
-		jmp->op_sel = JMP_NEAR;
-	    if (jmp->nearop.len == 0)
-		return YASM_BC_RESOLVE_UNKNOWN_LEN; /* size not available */
-
+	} else {
+	    jmp->op_sel = JMP_NEAR;
 	    bc->len += jmp->nearop.len;
 	    bc->len += (opersize == 16) ? 2 : 4;
-	    break;
-	default:
-	    yasm_internal_error(N_("unknown jump type"));
+	}
+	return 0;
     }
-    bc->len += x86_common_resolve(&jmp->common);
 
-    return retval;
+    /* Default to short jump and generate span */
+    if (jmp->op_sel == JMP_NONE)
+	jmp->op_sel = JMP_SHORT;
+    bc->len += jmp->shortop.len + 1;
+    add_span(add_span_data, bc, 1, &jmp->target, -128+(long)bc->len,
+	     127+(long)bc->len);
+    return 0;
 }
 
-static yasm_bc_resolve_flags
-x86_bc_jmpfar_resolve(yasm_bytecode *bc, int save,
-		      yasm_calc_bc_dist_func calc_bc_dist)
+static int
+x86_bc_jmp_expand(yasm_bytecode *bc, int span, long old_val, long new_val,
+		  /*@out@*/ long *neg_thres, /*@out@*/ long *pos_thres)
+{
+    x86_jmp *jmp = (x86_jmp *)bc->contents;
+    unsigned char opersize;
+
+    if (span != 1)
+	yasm_internal_error(N_("unrecognized span id"));
+
+    /* As opersize may be 0, figure out its "real" value. */
+    opersize = (jmp->common.opersize == 0) ?
+	jmp->common.mode_bits : jmp->common.opersize;
+
+    if (jmp->op_sel == JMP_SHORT_FORCED || jmp->nearop.len == 0) {
+	yasm_error_set(YASM_ERROR_VALUE, N_("short jump out of range"));
+	return -1;
+    }
+
+    if (jmp->op_sel == JMP_NEAR)
+	yasm_internal_error(N_("trying to expand an already-near jump"));
+
+    /* Upgrade to a near jump */
+    jmp->op_sel = JMP_NEAR;
+    bc->len -= jmp->shortop.len + 1;
+    bc->len += jmp->nearop.len;
+    bc->len += (opersize == 16) ? 2 : 4;
+
+    return 0;
+}
+
+static int
+x86_bc_jmpfar_calc_len(yasm_bytecode *bc, yasm_bc_add_span_func add_span,
+		       void *add_span_data)
 {
     x86_jmpfar *jmpfar = (x86_jmpfar *)bc->contents;
     unsigned char opersize;
@@ -859,9 +752,9 @@ x86_bc_jmpfar_resolve(yasm_bytecode *bc, int save,
     bc->len += jmpfar->opcode.len;
     bc->len += 2;	/* segment */
     bc->len += (opersize == 16) ? 2 : 4;
-    bc->len += x86_common_resolve(&jmpfar->common);
+    bc->len += x86_common_calc_len(&jmpfar->common);
 
-    return YASM_BC_RESOLVE_MIN_LEN;
+    return 0;
 }
 
 static void
@@ -895,14 +788,14 @@ x86_bc_insn_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 {
     x86_insn *insn = (x86_insn *)bc->contents;
     /*@null@*/ x86_effaddr *x86_ea = (x86_effaddr *)insn->x86_ea;
-    yasm_immval *imm = insn->imm;
+    yasm_value *imm = insn->imm;
     unsigned char *bufp_orig = *bufp;
 
     /* Prefixes */
+    x86_common_tobytes(&insn->common, bufp,
+		       x86_ea ? (unsigned int)(x86_ea->ea.segreg>>8) : 0);
     if (insn->special_prefix != 0)
 	YASM_WRITE_8(*bufp, insn->special_prefix);
-    x86_common_tobytes(&insn->common, bufp,
-		       x86_ea ? (x86_ea->ea.segreg>>8) : 0);
     if (insn->rex != 0xff) {
 	if (insn->common.mode_bits == 64 && insn->common.opersize == 64 &&
 	    insn->def_opersize_64 != 64)
@@ -935,23 +828,7 @@ x86_bc_insn_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 	}
 
 	if (x86_ea->ea.need_disp) {
-	    x86_effaddr eat = *x86_ea;  /* structure copy */
-	    unsigned char addrsize = insn->common.addrsize;
-
-	    eat.valid_modrm = 0;    /* force checkea to actually run */
-
-	    if (x86_ea->ea.disp.abs) {
-		/* Call checkea() to simplify the registers out of the
-		 * displacement.  Throw away all of the return values except
-		 * for the modified expr.
-		 */
-		if (yasm_x86__expr_checkea
-		    (&eat, &addrsize, insn->common.mode_bits,
-		     insn->postop == X86_POSTOP_ADDRESS16, &insn->rex,
-		     yasm_common_calc_bc_dist, bc->line))
-		    yasm_internal_error(N_("checkea failed"));
-		x86_ea->ea.disp.abs = eat.ea.disp.abs;
-	    }
+	    unsigned int disp_len = x86_ea->ea.disp.size/8;
 
 	    if (x86_ea->ea.disp.ip_rel) {
 		/* Adjust relative displacement to end of bytecode */
@@ -966,21 +843,29 @@ x86_bc_insn_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 					 yasm_expr_expr(x86_ea->ea.disp.abs),
 					 yasm_expr_int(delta), bc->line);
 	    }
-	    if (output_value(&x86_ea->ea.disp, *bufp, x86_ea->ea.disp_len,
-			     (size_t)(x86_ea->ea.disp_len*8), 0,
+	    if (output_value(&x86_ea->ea.disp, *bufp, disp_len,
 			     (unsigned long)(*bufp-bufp_orig), bc, 1, d))
 		return 1;
-	    *bufp += x86_ea->ea.disp_len;
+	    *bufp += disp_len;
 	}
     }
 
     /* Immediate (if required) */
     if (imm) {
-	if (output_value(&imm->val, *bufp, imm->len, (size_t)(imm->len*8), 0,
-			 (unsigned long)(*bufp-bufp_orig), bc, imm->sign?-1:1,
-			 d))
+	unsigned int imm_len;
+	if (insn->postop == X86_POSTOP_SIGNEXT_IMM8) {
+	    /* If we got here with this postop still set, we need to force
+	     * imm size to 8 here.
+	     */
+	    imm->size = 8;
+	    imm->sign = 1;
+	    imm_len = 1;
+	} else
+	    imm_len = imm->size/8;
+	if (output_value(imm, *bufp, imm_len, (unsigned long)(*bufp-bufp_orig),
+			 bc, 1, d))
 	    return 1;
-	*bufp += imm->len;
+	*bufp += imm_len;
     }
 
     return 0;
@@ -1004,7 +889,7 @@ x86_bc_jmp_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
     opersize = (jmp->common.opersize == 0) ?
 	jmp->common.mode_bits : jmp->common.opersize;
 
-    /* Check here to see if forced forms are actually legal. */
+    /* Check here again to see if forms are actually legal. */
     switch (jmp->op_sel) {
 	case JMP_SHORT_FORCED:
 	case JMP_SHORT:
@@ -1026,7 +911,8 @@ x86_bc_jmp_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 				     yasm_expr_expr(jmp->target.abs),
 				     yasm_expr_int(delta), bc->line);
 
-	    if (output_value(&jmp->target, *bufp, 1, 8, 0,
+	    jmp->target.size = 8;
+	    if (output_value(&jmp->target, *bufp, 1,
 			     (unsigned long)(*bufp-bufp_orig), bc, -1, d))
 		return 1;
 	    *bufp += 1;
@@ -1035,7 +921,8 @@ x86_bc_jmp_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 	case JMP_NEAR:
 	    /* 2/4 byte relative displacement (depending on operand size) */
 	    if (jmp->nearop.len == 0) {
-		yasm__error(bc->line, N_("near jump does not exist"));
+		yasm_error_set(YASM_ERROR_TYPE,
+			       N_("near jump does not exist"));
 		return 1;
 	    }
 
@@ -1055,11 +942,14 @@ x86_bc_jmp_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 				     yasm_expr_expr(jmp->target.abs),
 				     yasm_expr_int(delta), bc->line);
 
-	    if (output_value(&jmp->target, *bufp, i, i*8, 0,
+	    jmp->target.size = i*8;
+	    if (output_value(&jmp->target, *bufp, i,
 			     (unsigned long)(*bufp-bufp_orig), bc, -1, d))
 		return 1;
 	    *bufp += i;
 	    break;
+	case JMP_NONE:
+	    yasm_internal_error(N_("jump op_sel cannot be JMP_NONE in tobytes"));
 	default:
 	    yasm_internal_error(N_("unrecognized relative jump op_sel"));
     }
@@ -1085,11 +975,13 @@ x86_bc_jmpfar_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 
     /* Absolute displacement: segment and offset */
     i = (opersize == 16) ? 2 : 4;
-    if (output_value(&jmpfar->offset, *bufp, i, i*8, 0,
+    jmpfar->offset.size = i*8;
+    if (output_value(&jmpfar->offset, *bufp, i,
 		     (unsigned long)(*bufp-bufp_orig), bc, 1, d))
 	return 1;
     *bufp += i;
-    if (output_value(&jmpfar->segment, *bufp, 2, 2*8, 0,
+    jmpfar->segment.size = 16;
+    if (output_value(&jmpfar->segment, *bufp, 2,
 		     (unsigned long)(*bufp-bufp_orig), bc, 1, d))
 	return 1;
     *bufp += 2;
@@ -1100,11 +992,9 @@ x86_bc_jmpfar_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 int
 yasm_x86__intnum_tobytes(yasm_arch *arch, const yasm_intnum *intn,
 			 unsigned char *buf, size_t destsize, size_t valsize,
-			 int shift, const yasm_bytecode *bc, int warn,
-			 unsigned long line)
+			 int shift, const yasm_bytecode *bc, int warn)
 {
     /* Write value out. */
-    yasm_intnum_get_sized(intn, buf, destsize, valsize, shift, 0, warn,
-			  line);
+    yasm_intnum_get_sized(intn, buf, destsize, valsize, shift, 0, warn);
     return 0;
 }
