@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: x86expr.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: x86expr.c 1795 2007-02-24 22:39:55Z peter $");
 
 #define YASM_LIB_INTERNAL
 #define YASM_EXPR_INTERNAL
@@ -53,12 +53,12 @@ x86_expr_checkea_get_reg3264(yasm_expr__item *ei, int *regnum,
 	case X86_REG32:
 	    if (data->addrsize != 32)
 		return 0;
-	    *regnum = ei->data.reg & 0xF;
+	    *regnum = (unsigned int)(ei->data.reg & 0xF);
 	    break;
 	case X86_REG64:
 	    if (data->addrsize != 64)
 		return 0;
-	    *regnum = ei->data.reg & 0xF;
+	    *regnum = (unsigned int)(ei->data.reg & 0xF);
 	    break;
 	case X86_RIP:
 	    if (data->bits != 64)
@@ -103,7 +103,7 @@ x86_expr_checkea_get_reg16(yasm_expr__item *ei, int *regnum, void *d)
 	return 0;
 
     /* & 7 for sanity check */
-    *regnum = ei->data.reg & 0x7;
+    *regnum = (unsigned int)(ei->data.reg & 0x7);
 
     /* only allow BX, SI, DI, BP */
     if (!reg16[*regnum])
@@ -249,8 +249,7 @@ x86_expr_checkea_distcheck_reg(yasm_expr **ep, unsigned int bits)
 static int
 x86_expr_checkea_getregusage(yasm_expr **ep, /*@null@*/ int *indexreg,
     int *pcrel, unsigned int bits, void *data,
-    int *(*get_reg)(yasm_expr__item *ei, int *regnum, void *d),
-    yasm_calc_bc_dist_func calc_bc_dist)
+    int *(*get_reg)(yasm_expr__item *ei, int *regnum, void *d))
 {
     int i;
     int *reg;
@@ -260,8 +259,7 @@ x86_expr_checkea_getregusage(yasm_expr **ep, /*@null@*/ int *indexreg,
     yasm_expr *e, *wrt;
 
     /*@-unqualifiedtrans@*/
-    *ep = yasm_expr__level_tree(*ep, 1, 1, indexreg == 0, calc_bc_dist, NULL,
-				NULL, NULL, NULL);
+    *ep = yasm_expr__level_tree(*ep, 1, 1, indexreg == 0, 0, NULL, NULL);
 
     /* Check for WRT rip first */
     wrt = yasm_expr_extract_wrt(ep);
@@ -296,8 +294,8 @@ x86_expr_checkea_getregusage(yasm_expr **ep, /*@null@*/ int *indexreg,
 	    return 1;
 	case 2:
 	    /* Need to simplify again */
-	    *ep = yasm_expr__level_tree(*ep, 1, 1, indexreg == 0, NULL, NULL,
-					NULL, NULL, NULL);
+	    *ep = yasm_expr__level_tree(*ep, 1, 1, indexreg == 0, 0, NULL,
+					NULL);
 	    e = *ep;
 	    break;
 	default:
@@ -307,12 +305,7 @@ x86_expr_checkea_getregusage(yasm_expr **ep, /*@null@*/ int *indexreg,
     switch (e->op) {
 	case YASM_EXPR_ADD:
 	    /* Prescan for non-int multipliers against a reg.
-	     * This is because if any of the terms is a more complex
-	     * expr (eg, undetermined value), we don't want to try to
-	     * figure out *any* of the expression, because each register
-	     * lookup overwrites the register with a 0 value!  And storing
-	     * the state of this routine from one excution to the next
-	     * would be a major chore.
+	     * This is invalid due to the optimizer structure.
 	     */
 	    for (i=0; i<e->numterms; i++)
 		if (e->terms[i].type == YASM_EXPR_EXPR) {
@@ -320,10 +313,10 @@ x86_expr_checkea_getregusage(yasm_expr **ep, /*@null@*/ int *indexreg,
 		    if (e->terms[i].data.expn->terms[0].type ==
 			YASM_EXPR_REG) {
 			if (e->terms[i].data.expn->numterms > 2)
-			    return 2;
+			    return 1;
 			if (e->terms[i].data.expn->terms[1].type !=
 			    YASM_EXPR_INT)
-			    return 2;
+			    return 1;
 		    }
 		}
 
@@ -373,9 +366,9 @@ x86_expr_checkea_getregusage(yasm_expr **ep, /*@null@*/ int *indexreg,
 	    yasm_expr__order_terms(e);
 	    if (e->terms[0].type == YASM_EXPR_REG) {
 		if (e->numterms > 2)
-		    return 2;
+		    return 1;
 		if (e->terms[1].type != YASM_EXPR_INT)
-		    return 2;
+		    return 1;
 		reg = get_reg(&e->terms[0], &regnum, data);
 		if (!reg)
 		    return 1;
@@ -392,7 +385,7 @@ x86_expr_checkea_getregusage(yasm_expr **ep, /*@null@*/ int *indexreg,
     /* Simplify expr, which is now really just the displacement. This
      * should get rid of the 0's we put in for registers in the callback.
      */
-    *ep = yasm_expr_simplify(*ep, NULL);
+    *ep = yasm_expr_simplify(*ep, 0);
     /* e = *ep; */
 
     return 0;
@@ -401,7 +394,7 @@ x86_expr_checkea_getregusage(yasm_expr **ep, /*@null@*/ int *indexreg,
 /* Calculate the displacement length, if possible.
  * Takes several extra inputs so it can be used by both 32-bit and 16-bit
  * expressions:
- *  wordsize=2 for 16-bit, =4 for 32-bit.
+ *  wordsize=16 for 16-bit, =32 for 32-bit.
  *  noreg=1 if the *ModRM byte* has no registers used.
  *  dispreq=1 if a displacement value is *required* (even if =0).
  * Returns 0 if successfully calculated, 1 if not.
@@ -409,127 +402,120 @@ x86_expr_checkea_getregusage(yasm_expr **ep, /*@null@*/ int *indexreg,
 /*@-nullstate@*/
 static int
 x86_checkea_calc_displen(x86_effaddr *x86_ea, unsigned int wordsize, int noreg,
-			 int dispreq, unsigned long line)
+			 int dispreq)
 {
-    /*@null@*/ const yasm_intnum *intn = NULL;
-    long dispval;
+    /*@null@*/ /*@only@*/ yasm_intnum *num;
 
     x86_ea->valid_modrm = 0;	/* default to not yet valid */
 
-    switch (x86_ea->ea.disp_len) {
+    switch (x86_ea->ea.disp.size) {
 	case 0:
 	    break;
 	/* If not 0, the displacement length was forced; set the Mod bits
 	 * appropriately and we're done with the ModRM byte.
 	 */
-	case 1:
-	    /* Byte is not valid override in noreg case; fix it. */
+	case 8:
+	    /* Byte is only a valid override if there are registers in the
+	     * EA.  With no registers, we must have a 16/32 value.
+	     */
 	    if (noreg) {
-		x86_ea->ea.disp_len = 0;
-		yasm__warning(YASM_WARN_GENERAL, line,
+		yasm_warn_set(YASM_WARN_GENERAL,
 			      N_("invalid displacement size; fixed"));
+		x86_ea->ea.disp.size = wordsize;
 	    } else
 		x86_ea->modrm |= 0100;
 	    x86_ea->valid_modrm = 1;
-	    break;
-	case 2:
-	case 4:
-	    if (wordsize != x86_ea->ea.disp_len) {
-		yasm__error(line,
+	    return 0;
+	case 16:
+	case 32:
+	    /* Don't allow changing displacement different from BITS setting
+	     * directly; require an address-size override to change it.
+	     */
+	    if (wordsize != x86_ea->ea.disp.size) {
+		yasm_error_set(YASM_ERROR_VALUE,
 		    N_("invalid effective address (displacement size)"));
 		return 1;
 	    }
-	    /* 2/4 is not valid override in noreg case; fix it. */
-	    if (noreg) {
-		if (wordsize != x86_ea->ea.disp_len)
-		    yasm__warning(YASM_WARN_GENERAL, line,
-				  N_("invalid displacement size; fixed"));
-		x86_ea->ea.disp_len = 0;
-	    } else
+	    if (!noreg)
 		x86_ea->modrm |= 0200;
 	    x86_ea->valid_modrm = 1;
-	    break;
+	    return 0;
 	default:
 	    /* we shouldn't ever get any other size! */
 	    yasm_internal_error(N_("strange EA displacement size"));
     }
 
-    if (x86_ea->ea.disp_len == 0) {
-	/* the displacement length hasn't been forced (or the forcing wasn't
-	 * valid), try to determine what it is.
+    /* The displacement length hasn't been forced (or the forcing wasn't
+     * valid), try to determine what it is.
+     */
+    if (noreg) {
+	/* No register in ModRM expression, so it must be disp16/32,
+	 * and as the Mod bits are set to 0 by the caller, we're done
+	 * with the ModRM byte.
 	 */
-	if (noreg) {
-	    /* no register in ModRM expression, so it must be disp16/32,
-	     * and as the Mod bits are set to 0 by the caller, we're done
-	     * with the ModRM byte.
-	     */
-	    x86_ea->ea.disp_len = wordsize;
-	    x86_ea->valid_modrm = 1;
-	    return 0;
-	} else if (dispreq) {
-	    /* for BP/EBP, there *must* be a displacement value, but we
-	     * may not know the size (8 or 16/32) for sure right now.
-	     * We can't leave displen at 0, because that just means
-	     * unknown displacement, including none.
-	     */
-	    x86_ea->ea.disp_len = 0xff;
-	}
-
-	if (x86_ea->ea.disp.rel ||
-	    (x86_ea->ea.disp.abs &&
-	     !(intn = yasm_expr_get_intnum(&x86_ea->ea.disp.abs, NULL)))) {
-	    /* expr still has unknown values or is relative:
-	     * assume 16/32-bit disp
-	     */
-	    x86_ea->ea.disp_len = wordsize;
-	    x86_ea->modrm |= 0200;
-	    x86_ea->valid_modrm = 1;
-	    return 0;
-	}	
-
-	/* don't try to find out what size displacement we have if
-	 * displen is known.
-	 */
-	if (x86_ea->ea.disp_len != 0 && x86_ea->ea.disp_len != 0xff) {
-	    if (x86_ea->ea.disp_len == 1)
-		x86_ea->modrm |= 0100;
-	    else
-		x86_ea->modrm |= 0200;
-	    x86_ea->valid_modrm = 1;
-	    return 0;
-	}
-
-	if (intn)
-	    dispval = yasm_intnum_get_int(intn);
-	else
-	    dispval = 0;
-
-	/* Figure out what size displacement we will have. */
-	if (x86_ea->ea.disp_len != 0xff && dispval == 0) {
-	    /* if we know that the displacement is 0 right now,
-	     * go ahead and delete the expr and make it so no
-	     * displacement value is included in the output.
-	     * The Mod bits of ModRM are set to 0 above, and
-	     * we're done with the ModRM byte!
-	     *
-	     * Don't do this if we came from dispreq check above, so
-	     * check *displen.
-	     */
-	    yasm_expr_destroy(x86_ea->ea.disp.abs);
-	    x86_ea->ea.disp.abs = NULL;
-	    x86_ea->ea.need_disp = 0;
-	} else if (dispval >= -128 && dispval <= 127) {
-	    /* It fits into a signed byte */
-	    x86_ea->ea.disp_len = 1;
-	    x86_ea->modrm |= 0100;
-	} else {
-	    /* It's a 16/32-bit displacement */
-	    x86_ea->ea.disp_len = wordsize;
-	    x86_ea->modrm |= 0200;
-	}
-	x86_ea->valid_modrm = 1;	/* We're done with ModRM */
+	x86_ea->ea.disp.size = wordsize;
+	x86_ea->valid_modrm = 1;
+	return 0;
     }
 
+    if (dispreq) {
+	/* for BP/EBP, there *must* be a displacement value, but we
+	 * may not know the size (8 or 16/32) for sure right now.
+	 */
+	x86_ea->ea.need_nonzero_len = 1;
+    }
+
+    if (x86_ea->ea.disp.rel) {
+	/* Relative displacement; basically all object formats need non-byte
+	 * for relocation here, so just do that. (TODO: handle this
+	 * differently?)
+	 */
+	x86_ea->ea.disp.size = wordsize;
+	x86_ea->modrm |= 0200;
+	x86_ea->valid_modrm = 1;
+	return 0;
+    }
+
+    /* At this point there's 3 possibilities for the displacement:
+     *  - None (if =0)
+     *  - signed 8 bit (if in -128 to 127 range)
+     *  - 16/32 bit (word size)
+     * For now, check intnum value right now; if it's not 0,
+     * assume 8 bit and set up for allowing 16 bit later.
+     * FIXME: The complex expression equaling zero is probably a rare case,
+     * so we ignore it for now.
+     */
+    num = yasm_value_get_intnum(&x86_ea->ea.disp, NULL, 0);
+    if (!num) {
+	/* Still has unknown values. */
+	x86_ea->ea.need_nonzero_len = 1;
+	x86_ea->modrm |= 0100;
+	x86_ea->valid_modrm = 1;
+	return 0;
+    }
+
+    /* Figure out what size displacement we will have. */
+    if (yasm_intnum_is_zero(num) && !x86_ea->ea.need_nonzero_len) {
+	/* If we know that the displacement is 0 right now,
+	 * go ahead and delete the expr and make it so no
+	 * displacement value is included in the output.
+	 * The Mod bits of ModRM are set to 0 above, and
+	 * we're done with the ModRM byte!
+	 */
+	yasm_value_delete(&x86_ea->ea.disp);
+	x86_ea->ea.need_disp = 0;
+    } else if (yasm_intnum_in_range(num, -128, 127)) {
+	/* It fits into a signed byte */
+	x86_ea->ea.disp.size = 8;
+	x86_ea->modrm |= 0100;
+    } else {
+	/* It's a 16/32-bit displacement */
+	x86_ea->ea.disp.size = wordsize;
+	x86_ea->modrm |= 0200;
+    }
+    x86_ea->valid_modrm = 1;	/* We're done with ModRM */
+
+    yasm_intnum_destroy(num);
     return 0;
 }
 /*@=nullstate@*/
@@ -562,7 +548,7 @@ x86_expr_checkea_getregsize_callback(yasm_expr__item *ei, void *d)
 int
 yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 		       unsigned int bits, int address16_op, unsigned char *rex,
-		       yasm_calc_bc_dist_func calc_bc_dist, unsigned long line)
+		       yasm_bytecode *bc)
 {
     int retval;
 
@@ -572,23 +558,23 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 	 * - what registers are used in the expression
 	 * - the bits setting
 	 */
-	switch (x86_ea->ea.disp_len) {
-	    case 2:
+	switch (x86_ea->ea.disp.size) {
+	    case 16:
 		/* must be 16-bit */
 		*addrsize = 16;
 		break;
-	    case 8:
+	    case 64:
 		/* We have to support this for the MemOffs case, but it's
 		 * otherwise illegal.  It's also illegal in non-64-bit mode.
 		 */
 		if (x86_ea->need_modrm || x86_ea->need_sib) {
-		    yasm__error(line,
+		    yasm_error_set(YASM_ERROR_VALUE,
 			N_("invalid effective address (displacement size)"));
 		    return 1;
 		}
 		*addrsize = 64;
 		break;
-	    case 4:
+	    case 32:
 		/* Must be 32-bit in 16-bit or 32-bit modes.  In 64-bit mode,
 		 * we don't know unless we look at the registers, except in the
 		 * MemOffs case (see the end of this function).
@@ -646,7 +632,7 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 
 	/* We can only do 64-bit addresses in 64-bit mode. */
 	if (*addrsize == 64 && bits != 64) {
-	    yasm__error(line,
+	    yasm_error_set(YASM_ERROR_TYPE,
 		N_("invalid effective address (64-bit in non-64-bit mode)"));
 	    return 1;
 	}
@@ -658,22 +644,18 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 	    int pcrel = 0;
 	    switch (x86_expr_checkea_getregusage
 		    (&x86_ea->ea.disp.abs, &indexreg, &pcrel, bits,
-		     &reg3264_data, x86_expr_checkea_get_reg3264,
-		     calc_bc_dist)) {
+		     &reg3264_data, x86_expr_checkea_get_reg3264)) {
 		case 1:
-		    yasm__error(line, N_("invalid effective address"));
+		    yasm_error_set(YASM_ERROR_VALUE,
+				   N_("invalid effective address"));
 		    return 1;
 		case 2:
-		    if (pcrel) {
-			x86_ea->ea.disp.curpos_rel = 1;
-			x86_ea->ea.disp.ip_rel = 1;
-		    }
+		    if (pcrel)
+			yasm_value_set_curpos_rel(&x86_ea->ea.disp, bc, 1);
 		    return 2;
 		default:
-		    if (pcrel) {
-			x86_ea->ea.disp.curpos_rel = 1;
-			x86_ea->ea.disp.ip_rel = 1;
-		    }
+		    if (pcrel)
+			yasm_value_set_curpos_rel(&x86_ea->ea.disp, bc, 1);
 		    break;
 	    }
 	}
@@ -691,7 +673,8 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 	 */
 	for (i=0; i<17; i++) {
 	    if (reg3264mult[i] < 0) {
-		yasm__error(line, N_("invalid effective address"));
+		yasm_error_set(YASM_ERROR_VALUE,
+			       N_("invalid effective address"));
 		return 1;
 	    }
 	    if (i != indexreg && reg3264mult[i] == 1 &&
@@ -733,7 +716,8 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 	 */
 	for (i=0; i<17; i++)
 	    if (i != basereg && i != indexreg && reg3264mult[i] != 0) {
-		yasm__error(line, N_("invalid effective address"));
+		yasm_error_set(YASM_ERROR_VALUE,
+			       N_("invalid effective address"));
 		return 1;
 	    }
 
@@ -741,7 +725,7 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 	if (indexreg != REG3264_NONE && reg3264mult[indexreg] != 1 &&
 	    reg3264mult[indexreg] != 2 && reg3264mult[indexreg] != 4 &&
 	    reg3264mult[indexreg] != 8) {
-	    yasm__error(line, N_("invalid effective address"));
+	    yasm_error_set(YASM_ERROR_VALUE, N_("invalid effective address"));
 	    return 1;
 	}
 
@@ -751,7 +735,8 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 	     * legal.
 	     */
 	    if (reg3264mult[REG3264_ESP] > 1 || basereg == REG3264_ESP) {
-		yasm__error(line, N_("invalid effective address"));
+		yasm_error_set(YASM_ERROR_VALUE,
+			       N_("invalid effective address"));
 		return 1;
 	    }
 	    /* If mult==1 and basereg is not ESP, swap indexreg w/basereg. */
@@ -762,7 +747,7 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 	/* RIP is only legal if it's the ONLY register used. */
 	if (indexreg == REG64_RIP ||
 	    (basereg == REG64_RIP && indexreg != REG3264_NONE)) {
-	    yasm__error(line, N_("invalid effective address"));
+	    yasm_error_set(YASM_ERROR_VALUE, N_("invalid effective address"));
 	    return 1;
 	}
 
@@ -793,7 +778,7 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 	    x86_ea->need_sib = 0;
 	    /* RIP always requires a 32-bit displacement */
 	    x86_ea->valid_modrm = 1;
-	    x86_ea->ea.disp_len = 4;
+	    x86_ea->ea.disp.size = 32;
 	    return 0;
 	} else if (indexreg == REG3264_NONE) {
 	    /* basereg only */
@@ -803,11 +788,8 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 	     */
 	    if (yasm_x86__set_rex_from_reg(rex, &low3,
 					   (unsigned int)(X86_REG64 | basereg),
-					   bits, X86_REX_B)) {
-		yasm__error(line,
-		    N_("invalid combination of operands and effective address"));
+					   bits, X86_REX_B))
 		return 1;
-	    }
 	    x86_ea->modrm |= low3;
 	    /* we don't need an SIB *unless* basereg is ESP or R12 */
 	    if (basereg == REG3264_ESP || basereg == REG64_R12)
@@ -833,11 +815,8 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 	    else {
 		if (yasm_x86__set_rex_from_reg(rex, &low3, (unsigned int)
 					       (X86_REG64 | basereg), bits,
-					       X86_REX_B)) {
-		    yasm__error(line,
-			N_("invalid combination of operands and effective address"));
+					       X86_REX_B))
 		    return 1;
-		}
 		x86_ea->sib |= low3;
 	    }
 	    
@@ -848,11 +827,8 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 	    else {
 		if (yasm_x86__set_rex_from_reg(rex, &low3, (unsigned int)
 					       (X86_REG64 | indexreg), bits,
-					       X86_REX_X)) {
-		    yasm__error(line,
-			N_("invalid combination of operands and effective address"));
+					       X86_REX_X))
 		    return 1;
-		}
 		x86_ea->sib |= low3 << 3;
 		/* Set scale field, 1 case -> 0, so don't bother. */
 		switch (reg3264mult[indexreg]) {
@@ -873,8 +849,8 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 
 	/* Calculate displacement length (if possible) */
 	retval = x86_checkea_calc_displen
-	    (x86_ea, 4, basereg == REG3264_NONE,
-	     basereg == REG3264_EBP || basereg == REG64_R13, line);
+	    (x86_ea, 32, basereg == REG3264_NONE,
+	     basereg == REG3264_EBP || basereg == REG64_R13);
 	return retval;
     } else if (*addrsize == 16 && x86_ea->need_modrm && !x86_ea->valid_modrm) {
 	static const unsigned char modrm16[16] = {
@@ -896,7 +872,7 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 
 	/* 64-bit mode does not allow 16-bit addresses */
 	if (bits == 64 && !address16_op) {
-	    yasm__error(line,
+	    yasm_error_set(YASM_ERROR_TYPE,
 		N_("16-bit addresses not supported in 64-bit mode"));
 	    return 1;
 	}
@@ -910,21 +886,18 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 	    int pcrel = 0;
 	    switch (x86_expr_checkea_getregusage
 		    (&x86_ea->ea.disp.abs, (int *)NULL, &pcrel, bits,
-		     &reg16mult, x86_expr_checkea_get_reg16, calc_bc_dist)) {
+		     &reg16mult, x86_expr_checkea_get_reg16)) {
 		case 1:
-		    yasm__error(line, N_("invalid effective address"));
+		    yasm_error_set(YASM_ERROR_VALUE,
+				   N_("invalid effective address"));
 		    return 1;
 		case 2:
-		    if (pcrel) {
-			x86_ea->ea.disp.curpos_rel = 1;
-			x86_ea->ea.disp.ip_rel = 1;
-		    }
+		    if (pcrel)
+			yasm_value_set_curpos_rel(&x86_ea->ea.disp, bc, 1);
 		    return 2;
 		default:
-		    if (pcrel) {
-			x86_ea->ea.disp.curpos_rel = 1;
-			x86_ea->ea.disp.ip_rel = 1;
-		    }
+		    if (pcrel)
+			yasm_value_set_curpos_rel(&x86_ea->ea.disp, bc, 1);
 		    break;
 	    }
 	}
@@ -932,7 +905,7 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 	/* reg multipliers not 0 or 1 are illegal. */
 	if (reg16mult.bx & ~1 || reg16mult.si & ~1 || reg16mult.di & ~1 ||
 	    reg16mult.bp & ~1) {
-	    yasm__error(line, N_("invalid effective address"));
+	    yasm_error_set(YASM_ERROR_VALUE, N_("invalid effective address"));
 	    return 1;
 	}
 
@@ -948,7 +921,7 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 
 	/* Check the modrm value for invalid combinations. */
 	if (modrm16[havereg] & 0070) {
-	    yasm__error(line, N_("invalid effective address"));
+	    yasm_error_set(YASM_ERROR_VALUE, N_("invalid effective address"));
 	    return 1;
 	}
 
@@ -957,30 +930,30 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 
 	/* Calculate displacement length (if possible) */
 	retval = x86_checkea_calc_displen
-	    (x86_ea, 2, havereg == HAVE_NONE, havereg == HAVE_BP, line);
+	    (x86_ea, 16, havereg == HAVE_NONE, havereg == HAVE_BP);
 	return retval;
     } else if (!x86_ea->need_modrm && !x86_ea->need_sib) {
 	/* Special case for MOV MemOffs opcode: displacement but no modrm. */
 	switch (*addrsize) {
 	    case 64:
 		if (bits != 64) {
-		    yasm__error(line,
+		    yasm_error_set(YASM_ERROR_TYPE,
 			N_("invalid effective address (64-bit in non-64-bit mode)"));
 		    return 1;
 		}
-		x86_ea->ea.disp_len = 8;
+		x86_ea->ea.disp.size = 64;
 		break;
 	    case 32:
-		x86_ea->ea.disp_len = 4;
+		x86_ea->ea.disp.size = 32;
 		break;
 	    case 16:
 		/* 64-bit mode does not allow 16-bit addresses */
 		if (bits == 64 && !address16_op) {
-		    yasm__error(line,
+		    yasm_error_set(YASM_ERROR_TYPE,
 			N_("16-bit addresses not supported in 64-bit mode"));
 		    return 1;
 		}
-		x86_ea->ea.disp_len = 2;
+		x86_ea->ea.disp.size = 16;
 		break;
 	}
     }
@@ -990,13 +963,14 @@ yasm_x86__expr_checkea(x86_effaddr *x86_ea, unsigned char *addrsize,
 int
 yasm_x86__floatnum_tobytes(yasm_arch *arch, const yasm_floatnum *flt,
 			   unsigned char *buf, size_t destsize, size_t valsize,
-			   size_t shift, int warn, unsigned long line)
+			   size_t shift, int warn)
 {
     if (!yasm_floatnum_check_size(flt, valsize)) {
-	yasm__error(line, N_("invalid floating point constant size"));
+	yasm_error_set(YASM_ERROR_FLOATING_POINT,
+		       N_("invalid floating point constant size"));
 	return 1;
     }
 
-    yasm_floatnum_get_sized(flt, buf, destsize, valsize, shift, 0, warn, line);
+    yasm_floatnum_get_sized(flt, buf, destsize, valsize, shift, 0, warn);
     return 0;
 }
