@@ -28,7 +28,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: cv-symline.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: cv-symline.c 1795 2007-02-24 22:39:55Z peter $");
 
 #define YASM_LIB_INTERNAL
 #define YASM_BC_INTERNAL
@@ -166,8 +166,8 @@ typedef struct cv_sym {
 static void cv8_symhead_bc_destroy(void *contents);
 static void cv8_symhead_bc_print(const void *contents, FILE *f,
 				 int indent_level);
-static yasm_bc_resolve_flags cv8_symhead_bc_resolve
-    (yasm_bytecode *bc, int save, yasm_calc_bc_dist_func calc_bc_dist);
+static int cv8_symhead_bc_calc_len
+    (yasm_bytecode *bc, yasm_bc_add_span_func add_span, void *add_span_data);
 static int cv8_symhead_bc_tobytes
     (yasm_bytecode *bc, unsigned char **bufp, void *d,
      yasm_output_value_func output_value,
@@ -176,8 +176,8 @@ static int cv8_symhead_bc_tobytes
 static void cv8_fileinfo_bc_destroy(void *contents);
 static void cv8_fileinfo_bc_print(const void *contents, FILE *f,
 				  int indent_level);
-static yasm_bc_resolve_flags cv8_fileinfo_bc_resolve
-    (yasm_bytecode *bc, int save, yasm_calc_bc_dist_func calc_bc_dist);
+static int cv8_fileinfo_bc_calc_len
+    (yasm_bytecode *bc, yasm_bc_add_span_func add_span, void *add_span_data);
 static int cv8_fileinfo_bc_tobytes
     (yasm_bytecode *bc, unsigned char **bufp, void *d,
      yasm_output_value_func output_value,
@@ -186,8 +186,8 @@ static int cv8_fileinfo_bc_tobytes
 static void cv8_lineinfo_bc_destroy(void *contents);
 static void cv8_lineinfo_bc_print(const void *contents, FILE *f,
 				  int indent_level);
-static yasm_bc_resolve_flags cv8_lineinfo_bc_resolve
-    (yasm_bytecode *bc, int save, yasm_calc_bc_dist_func calc_bc_dist);
+static int cv8_lineinfo_bc_calc_len
+    (yasm_bytecode *bc, yasm_bc_add_span_func add_span, void *add_span_data);
 static int cv8_lineinfo_bc_tobytes
     (yasm_bytecode *bc, unsigned char **bufp, void *d,
      yasm_output_value_func output_value,
@@ -195,8 +195,8 @@ static int cv8_lineinfo_bc_tobytes
 
 static void cv_sym_bc_destroy(void *contents);
 static void cv_sym_bc_print(const void *contents, FILE *f, int indent_level);
-static yasm_bc_resolve_flags cv_sym_bc_resolve
-    (yasm_bytecode *bc, int save, yasm_calc_bc_dist_func calc_bc_dist);
+static int cv_sym_bc_calc_len
+    (yasm_bytecode *bc, yasm_bc_add_span_func add_span, void *add_span_data);
 static int cv_sym_bc_tobytes
     (yasm_bytecode *bc, unsigned char **bufp, void *d,
      yasm_output_value_func output_value,
@@ -207,32 +207,40 @@ static const yasm_bytecode_callback cv8_symhead_bc_callback = {
     cv8_symhead_bc_destroy,
     cv8_symhead_bc_print,
     yasm_bc_finalize_common,
-    cv8_symhead_bc_resolve,
-    cv8_symhead_bc_tobytes
+    cv8_symhead_bc_calc_len,
+    yasm_bc_expand_common,
+    cv8_symhead_bc_tobytes,
+    0
 };
 
 static const yasm_bytecode_callback cv8_fileinfo_bc_callback = {
     cv8_fileinfo_bc_destroy,
     cv8_fileinfo_bc_print,
     yasm_bc_finalize_common,
-    cv8_fileinfo_bc_resolve,
-    cv8_fileinfo_bc_tobytes
+    cv8_fileinfo_bc_calc_len,
+    yasm_bc_expand_common,
+    cv8_fileinfo_bc_tobytes,
+    0
 };
 
 static const yasm_bytecode_callback cv8_lineinfo_bc_callback = {
     cv8_lineinfo_bc_destroy,
     cv8_lineinfo_bc_print,
     yasm_bc_finalize_common,
-    cv8_lineinfo_bc_resolve,
-    cv8_lineinfo_bc_tobytes
+    cv8_lineinfo_bc_calc_len,
+    yasm_bc_expand_common,
+    cv8_lineinfo_bc_tobytes,
+    0
 };
 
 static const yasm_bytecode_callback cv_sym_bc_callback = {
     cv_sym_bc_destroy,
     cv_sym_bc_print,
     yasm_bc_finalize_common,
-    cv_sym_bc_resolve,
-    cv_sym_bc_tobytes
+    cv_sym_bc_calc_len,
+    yasm_bc_expand_common,
+    cv_sym_bc_tobytes,
+    0
 };
 
 static cv8_symhead *cv8_add_symhead(yasm_dbgfmt_cv *dbgfmt_cv,
@@ -376,7 +384,7 @@ cv_dbgfmt_add_file(yasm_dbgfmt_cv *dbgfmt_cv, size_t filenum,
     if (!f)
 	yasm__fatal(N_("codeview: could not open source file"));
     while ((len = fread(buf, 1, 1024, f)) > 0)
-	yasm_md5_update(&context, buf, len);
+	yasm_md5_update(&context, buf, (unsigned long)len);
     yasm_md5_final(dbgfmt_cv->filenames[filenum].digest, &context);
     fclose(f);
     yasm_xfree(buf);
@@ -407,15 +415,16 @@ cv_append_str(yasm_section *sect, const char *str)
     yasm_dvs_initialize(&dvs);
     yasm_dvs_append(&dvs, yasm_dv_create_string(yasm__xstrdup(str),
 						strlen(str)));
-    bc = yasm_bc_create_data(&dvs, 1, 1, 0);
+    bc = yasm_bc_create_data(&dvs, 1, 1, NULL, 0);
     yasm_bc_finalize(bc, yasm_cv__append_bc(sect, bc));
-    yasm_bc_resolve(bc, 0, NULL);
+    yasm_bc_calc_len(bc, NULL, NULL);
     return bc;
 }
 
 typedef struct cv_line_info {
     yasm_section *debug_symline;
     yasm_dbgfmt_cv *dbgfmt_cv;
+    yasm_errwarns *errwarns;
     unsigned int num_lineinfos;
     STAILQ_HEAD(, cv8_lineinfo) cv8_lineinfos;
     /*@null@*/ cv8_lineinfo *cv8_cur_li;
@@ -508,7 +517,6 @@ static int
 cv_generate_line_section(yasm_section *sect, /*@null@*/ void *d)
 {
     cv_line_info *info = (cv_line_info *)d;
-    yasm_dbgfmt_cv *dbgfmt_cv = info->dbgfmt_cv;
 
     if (!yasm_section_is_code(sect))
 	return 0;	/* not code, so no line data for this section */
@@ -516,7 +524,7 @@ cv_generate_line_section(yasm_section *sect, /*@null@*/ void *d)
     info->cv8_cur_li = NULL;
     info->cv8_cur_ls = NULL;
 
-    yasm_section_bcs_traverse(sect, info, cv_generate_line_bc);
+    yasm_section_bcs_traverse(sect, info->errwarns, info, cv_generate_line_bc);
 
     return 0;
 }
@@ -534,9 +542,13 @@ cv_generate_sym(yasm_symrec *sym, void *d)
 {
     cv_line_info *info = (cv_line_info *)d;
     yasm_bytecode *precbc;
+    const char *name = yasm_symrec_get_name(sym);
 
-    /* only care about labels (for now) */
-    if (!yasm_symrec_get_label(sym, &precbc))
+    /* only care about labels (for now).  Don't put in symbols starting with
+     * ".", as these are typically internally generated ones (like section
+     * symbols).
+     */
+    if (name[0] == '.' || !yasm_symrec_get_label(sym, &precbc))
 	return 0;
 
     /* TODO: add data types; until then, just mark everything as UBYTE */
@@ -549,7 +561,7 @@ cv_generate_sym(yasm_symrec *sym, void *d)
 }
 
 yasm_section *
-yasm_cv__generate_symline(yasm_dbgfmt_cv *dbgfmt_cv)
+yasm_cv__generate_symline(yasm_dbgfmt_cv *dbgfmt_cv, yasm_errwarns *errwarns)
 {
     cv_line_info info;
     int new;
@@ -564,6 +576,7 @@ yasm_cv__generate_symline(yasm_dbgfmt_cv *dbgfmt_cv)
 				    cv_generate_filename);
 
     info.dbgfmt_cv = dbgfmt_cv;
+    info.errwarns = errwarns;
     info.debug_symline = yasm_object_get_general(dbgfmt_cv->object,
 						 ".debug$S", 0, 1, 0, 0, &new,
 						 0);
@@ -578,7 +591,9 @@ yasm_cv__generate_symline(yasm_dbgfmt_cv *dbgfmt_cv)
     off = 1;
     for (i=0; i<dbgfmt_cv->filenames_size; i++) {
 	if (!dbgfmt_cv->filenames[i].pathname) {
-	    yasm__error(0, N_("codeview file number %d unassigned"), i+1);
+	    yasm_error_set(YASM_ERROR_GENERAL,
+			   N_("codeview file number %d unassigned"), i+1);
+	    yasm_errwarn_propagate(errwarns, 0);
 	    continue;
 	}
 	bc = cv_append_str(info.debug_symline,
@@ -593,7 +608,7 @@ yasm_cv__generate_symline(yasm_dbgfmt_cv *dbgfmt_cv)
 	(yasm_expr_create_ident(yasm_expr_int(yasm_intnum_create_uint(4)), 0),
 	 NULL, NULL, NULL, 0);
     yasm_bc_finalize(bc, yasm_cv__append_bc(info.debug_symline, bc));
-    yasm_bc_resolve(bc, 0, NULL);
+    yasm_bc_calc_len(bc, NULL, NULL);
 
     /* source file info table */
     head = cv8_add_symhead(dbgfmt_cv, info.debug_symline, CV8_FILE_INFO, 0);
@@ -643,7 +658,7 @@ yasm_cv__generate_symline(yasm_dbgfmt_cv *dbgfmt_cv)
 	(yasm_expr_create_ident(yasm_expr_int(yasm_intnum_create_uint(4)), 0),
 	 NULL, NULL, NULL, 0);
     yasm_bc_finalize(bc, yasm_cv__append_bc(info.debug_symline, bc));
-    yasm_bc_resolve(bc, 0, NULL);
+    yasm_bc_calc_len(bc, NULL, NULL);
 
     return info.debug_symline;
 }
@@ -655,15 +670,15 @@ cv_out_sym(yasm_symrec *sym, unsigned long off, yasm_bytecode *bc,
     yasm_value val;
 
     /* sym in its section */
-    yasm_value_init_sym(&val, sym);
+    yasm_value_init_sym(&val, sym, 32);
     val.section_rel = 1;
-    output_value(&val, *bufp, 4, 32, 0, off, bc, 0, d);
+    output_value(&val, *bufp, 4, off, bc, 0, d);
     *bufp += 4;
 
     /* section index */
-    yasm_value_init_sym(&val, sym);
+    yasm_value_init_sym(&val, sym, 16);
     val.seg_of = 1;
-    output_value(&val, *bufp, 2, 16, 0, off+4, bc, 0, d);
+    output_value(&val, *bufp, 2, off+4, bc, 0, d);
     *bufp += 2;
 }
 
@@ -709,13 +724,13 @@ cv8_symhead_bc_print(const void *contents, FILE *f, int indent_level)
     /* TODO */
 }
 
-static yasm_bc_resolve_flags
-cv8_symhead_bc_resolve(yasm_bytecode *bc, int save,
-		       yasm_calc_bc_dist_func calc_bc_dist)
+static int
+cv8_symhead_bc_calc_len(yasm_bytecode *bc, yasm_bc_add_span_func add_span,
+			void *add_span_data)
 {
-    yasm_internal_error(N_("tried to resolve a codeview symhead bytecode"));
+    yasm_internal_error(N_("tried to calc_len a codeview symhead bytecode"));
     /*@notreached@*/
-    return YASM_BC_RESOLVE_MIN_LEN;
+    return 0;
 }
 
 static int
@@ -731,21 +746,20 @@ cv8_symhead_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
     cval = yasm_intnum_create_uint(4);
     /* Output "version" if first */
     if (head->first) {
-	yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0,
-				 0);
+	yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0);
 	buf += 4;
     }
 
     /* Type contained - 4 bytes */
     yasm_intnum_set_uint(cval, head->type);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0, 0);
+    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0);
     buf += 4;
 
     /* Total length of info (following this field) - 4 bytes */
     yasm_intnum_set_uint(cval, bc->len);
-    intn = yasm_common_calc_bc_dist(head->start_prevbc, head->end_prevbc);
-    yasm_intnum_calc(intn, YASM_EXPR_SUB, cval, bc->line);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, intn, buf, 4, 32, 0, bc, 0, 0);
+    intn = yasm_calc_bc_dist(head->start_prevbc, head->end_prevbc);
+    yasm_intnum_calc(intn, YASM_EXPR_SUB, cval);
+    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, intn, buf, 4, 32, 0, bc, 0);
     buf += 4;
     yasm_intnum_destroy(intn);
 
@@ -785,13 +799,13 @@ cv8_fileinfo_bc_print(const void *contents, FILE *f, int indent_level)
     /* TODO */
 }
 
-static yasm_bc_resolve_flags
-cv8_fileinfo_bc_resolve(yasm_bytecode *bc, int save,
-			yasm_calc_bc_dist_func calc_bc_dist)
+static int
+cv8_fileinfo_bc_calc_len(yasm_bytecode *bc, yasm_bc_add_span_func add_span,
+			 void *add_span_data)
 {
-    yasm_internal_error(N_("tried to resolve a codeview fileinfo bytecode"));
+    yasm_internal_error(N_("tried to calc_len a codeview fileinfo bytecode"));
     /*@notreached@*/
-    return YASM_BC_RESOLVE_MIN_LEN;
+    return 0;
 }
 
 static int
@@ -807,12 +821,12 @@ cv8_fileinfo_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 
     /* Offset in filename string table */
     cval = yasm_intnum_create_uint(fi->fn->str_off);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0, 0);
+    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0);
     buf += 4;
 
     /* Checksum type/length */
     yasm_intnum_set_uint(cval, 0x0110);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 2, 16, 0, bc, 0, 0);
+    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 2, 16, 0, bc, 0);
     buf += 2;
 
     /* Checksum */
@@ -852,13 +866,13 @@ cv8_lineinfo_bc_print(const void *contents, FILE *f, int indent_level)
     /* TODO */
 }
 
-static yasm_bc_resolve_flags
-cv8_lineinfo_bc_resolve(yasm_bytecode *bc, int save,
-			yasm_calc_bc_dist_func calc_bc_dist)
+static int
+cv8_lineinfo_bc_calc_len(yasm_bytecode *bc, yasm_bc_add_span_func add_span,
+			 void *add_span_data)
 {
-    yasm_internal_error(N_("tried to resolve a codeview linehead bytecode"));
+    yasm_internal_error(N_("tried to calc_len a codeview linehead bytecode"));
     /*@notreached@*/
-    return YASM_BC_RESOLVE_MIN_LEN;
+    return 0;
 }
 
 static int
@@ -881,24 +895,24 @@ cv8_lineinfo_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
     YASM_WRITE_8(buf, 0);
 
     /* Section length covered by line number info */
-    cval = yasm_common_calc_bc_dist(yasm_section_bcs_first(li->sect),
-				    yasm_section_bcs_last(li->sect));
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0, 0);
+    cval = yasm_calc_bc_dist(yasm_section_bcs_first(li->sect),
+			     yasm_section_bcs_last(li->sect));
+    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0);
     buf += 4;
 
     /* Offset of source file in info table */
     yasm_intnum_set_uint(cval, li->fn->info_off);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0, 0);
+    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0);
     buf += 4;
 
     /* Number of line number pairs */
     yasm_intnum_set_uint(cval, li->num_linenums);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0, 0);
+    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0);
     buf += 4;
 
     /* Number of bytes of line number pairs + 12 (no, I don't know why) */
     yasm_intnum_set_uint(cval, li->num_linenums*8+12);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0, 0);
+    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0);
     buf += 4;
 
     /* Offset / line number pairs */
@@ -909,13 +923,13 @@ cv8_lineinfo_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 	    /* offset in section */
 	    yasm_intnum_set_uint(cval, ls->pairs[j].offset);
 	    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc,
-				     0, 0);
+				     0);
 	    buf += 4;
 
 	    /* line number in file */
 	    yasm_intnum_set_uint(cval, ls->pairs[j].line);
 	    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc,
-				     0, 0);
+				     0);
 	    buf += 4;
 	}
     }
@@ -958,11 +972,12 @@ cv_sym_size(const cv_sym *cvs)
 		break;
 	    case 'S':
 		len += 1;	/* XXX: is this 1 or 2? */
-		slen = strlen((const char *)cvs->args[arg++].p);
+		slen = (unsigned long)strlen((const char *)cvs->args[arg++].p);
 		len += slen <= 0xff ? slen : 0xff;
 		break;
 	    case 'Z':
-		len += strlen((const char *)cvs->args[arg++].p) + 1;
+		len +=
+		    (unsigned long)strlen((const char *)cvs->args[arg++].p) + 1;
 		break;
 	    default:
 		yasm_internal_error(N_("unknown sym format character"));
@@ -1008,13 +1023,13 @@ cv_sym_bc_print(const void *contents, FILE *f, int indent_level)
     /* TODO */
 }
 
-static yasm_bc_resolve_flags
-cv_sym_bc_resolve(yasm_bytecode *bc, int save,
-		  yasm_calc_bc_dist_func calc_bc_dist)
+static int
+cv_sym_bc_calc_len(yasm_bytecode *bc, yasm_bc_add_span_func add_span,
+		   void *add_span_data)
 {
-    yasm_internal_error(N_("tried to resolve a codeview sym bytecode"));
+    yasm_internal_error(N_("tried to calc_len a codeview sym bytecode"));
     /*@notreached@*/
-    return YASM_BC_RESOLVE_MIN_LEN;
+    return 0;
 }
 
 static int
@@ -1032,12 +1047,12 @@ cv_sym_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 
     /* Total length of record (following this field) - 2 bytes */
     cval = yasm_intnum_create_uint(bc->len-2);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 2, 16, 0, bc, 1, 0);
+    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 2, 16, 0, bc, 1);
     buf += 2;
 
     /* Type contained - 2 bytes */
     yasm_intnum_set_uint(cval, cvs->type);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 2, 16, 0, bc, 0, 0);
+    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 2, 16, 0, bc, 0);
     buf += 2;
 
     while (*ch) {
@@ -1049,23 +1064,24 @@ cv_sym_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 	    case 'h':
 		yasm_intnum_set_uint(cval, cvs->args[arg++].i);
 		yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 2, 16, 0,
-					 bc, 0, 0);
+					 bc, 0);
 		buf += 2;
 		break;
 	    case 'w':
 		yasm_intnum_set_uint(cval, cvs->args[arg++].i);
 		yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0,
-					 bc, 0, 0);
+					 bc, 0);
 		buf += 4;
 		break;
 	    case 'Y':
-		cv_out_sym((yasm_symrec *)cvs->args[arg++].p, buf-(*bufp), bc,
-			   &buf, d, output_value);
+		cv_out_sym((yasm_symrec *)cvs->args[arg++].p,
+			   (unsigned long)(buf-(*bufp)), bc, &buf, d,
+			   output_value);
 		break;
 	    case 'T':
 		yasm_intnum_set_uint(cval, cvs->args[arg++].i);
 		yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0,
-					 bc, 0, 0);
+					 bc, 0);
 		buf += 4;	/* XXX: will be 2 in CV4 */
 		break;
 	    case 'S':
