@@ -28,7 +28,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: cv-type.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: cv-type.c 1795 2007-02-24 22:39:55Z peter $");
 
 #define YASM_LIB_INTERNAL
 #define YASM_BC_INTERNAL
@@ -483,8 +483,8 @@ typedef struct cv_type {
 /* Bytecode callback function prototypes */
 static void cv_type_bc_destroy(void *contents);
 static void cv_type_bc_print(const void *contents, FILE *f, int indent_level);
-static yasm_bc_resolve_flags cv_type_bc_resolve
-    (yasm_bytecode *bc, int save, yasm_calc_bc_dist_func calc_bc_dist);
+static int cv_type_bc_calc_len
+    (yasm_bytecode *bc, yasm_bc_add_span_func add_span, void *add_span_data);
 static int cv_type_bc_tobytes
     (yasm_bytecode *bc, unsigned char **bufp, void *d,
      yasm_output_value_func output_value,
@@ -495,8 +495,10 @@ static const yasm_bytecode_callback cv_type_bc_callback = {
     cv_type_bc_destroy,
     cv_type_bc_print,
     yasm_bc_finalize_common,
-    cv_type_bc_resolve,
-    cv_type_bc_tobytes
+    cv_type_bc_calc_len,
+    yasm_bc_expand_common,
+    cv_type_bc_tobytes,
+    0
 };
 
 static cv_type *cv_type_create(yasm_dbgfmt_cv *dbgfmt_cv, unsigned long indx);
@@ -530,7 +532,7 @@ yasm_cv__generate_type(yasm_dbgfmt_cv *dbgfmt_cv)
     cv_type_append_leaf(type, cv_leaf_create_label(0));
     bc = yasm_bc_create_common(&cv_type_bc_callback, type, 0);
     yasm_bc_finalize(bc, yasm_cv__append_bc(debug_type, bc));
-    yasm_bc_resolve(bc, 0, NULL);
+    yasm_bc_calc_len(bc, NULL, NULL);
 
     return debug_type;
 }
@@ -595,7 +597,7 @@ cv_leaf_size(const cv_leaf *leaf)
 		break;
 	    case 'S':
 		len += 1;	/* XXX: is this 1 or 2? */
-		slen = strlen((const char *)leaf->args[arg++].p);
+		slen = (unsigned long)strlen((const char *)leaf->args[arg++].p);
 		len += slen <= 0xff ? slen : 0xff;
 		break;
 	    default:
@@ -618,7 +620,7 @@ cv_leaf_tobytes(const cv_leaf *leaf, yasm_bytecode *bc, yasm_arch *arch,
 
     /* leaf type */
     yasm_intnum_set_uint(cval, leaf->type);
-    yasm_arch_intnum_tobytes(arch, cval, buf, 2, 16, 0, bc, 0, 0);
+    yasm_arch_intnum_tobytes(arch, cval, buf, 2, 16, 0, bc, 0);
     buf += 2;
 
     while (*ch) {
@@ -629,12 +631,12 @@ cv_leaf_tobytes(const cv_leaf *leaf, yasm_bytecode *bc, yasm_arch *arch,
 		break;
 	    case 'h':
 		yasm_intnum_set_uint(cval, leaf->args[arg++].i);
-		yasm_arch_intnum_tobytes(arch, cval, buf, 2, 16, 0, bc, 0, 0);
+		yasm_arch_intnum_tobytes(arch, cval, buf, 2, 16, 0, bc, 0);
 		buf += 2;
 		break;
 	    case 'w':
 		yasm_intnum_set_uint(cval, leaf->args[arg++].i);
-		yasm_arch_intnum_tobytes(arch, cval, buf, 4, 32, 0, bc, 0, 0);
+		yasm_arch_intnum_tobytes(arch, cval, buf, 4, 32, 0, bc, 0);
 		buf += 4;
 		break;
 	    case 'L':
@@ -644,7 +646,7 @@ cv_leaf_tobytes(const cv_leaf *leaf, yasm_bytecode *bc, yasm_arch *arch,
 	    case 'T':
 		yasm_intnum_set_uint(cval,
 		    ((const cv_type *)leaf->args[arg++].p)->indx);
-		yasm_arch_intnum_tobytes(arch, cval, buf, 4, 32, 0, bc, 0, 0);
+		yasm_arch_intnum_tobytes(arch, cval, buf, 4, 32, 0, bc, 0);
 		buf += 4;	/* XXX: will be 2 in CV4 */
 		break;
 	    case 'S':
@@ -710,9 +712,9 @@ cv_type_bc_print(const void *contents, FILE *f, int indent_level)
     /* TODO */
 }
 
-static yasm_bc_resolve_flags
-cv_type_bc_resolve(yasm_bytecode *bc, int save,
-		   yasm_calc_bc_dist_func calc_bc_dist)
+static int
+cv_type_bc_calc_len(yasm_bytecode *bc, yasm_bc_add_span_func add_span,
+		    void *add_span_data)
 {
     cv_type *type = (cv_type *)bc->contents;
     size_t i;
@@ -729,7 +731,7 @@ cv_type_bc_resolve(yasm_bytecode *bc, int save,
     if (bc->len & 0x3)
 	bc->len += 4-(bc->len & 0x3);
 
-    return YASM_BC_RESOLVE_MIN_LEN;
+    return 0;
 }
 
 static int
@@ -746,15 +748,14 @@ cv_type_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 
     cval = yasm_intnum_create_uint(4);	    /* version */
     if (type->indx == CV_FIRST_NONPRIM) {
-	yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 1,
-				 0);
+	yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 1);
 	buf += 4;
 	reclen -= 4;
     }
 
     /* Total length of record (following this field) - 2 bytes */
     yasm_intnum_set_uint(cval, reclen);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 2, 16, 0, bc, 1, 0);
+    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 2, 16, 0, bc, 1);
     buf += 2;
 
     /* Leaves */
