@@ -28,7 +28,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: dwarf2-line.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: dwarf2-line.c 1795 2007-02-24 22:39:55Z peter $");
 
 #define YASM_LIB_INTERNAL
 #define YASM_BC_INTERNAL
@@ -130,8 +130,8 @@ typedef struct dwarf2_line_op {
 static void dwarf2_spp_bc_destroy(void *contents);
 static void dwarf2_spp_bc_print(const void *contents, FILE *f,
 				int indent_level);
-static yasm_bc_resolve_flags dwarf2_spp_bc_resolve
-    (yasm_bytecode *bc, int save, yasm_calc_bc_dist_func calc_bc_dist);
+static int dwarf2_spp_bc_calc_len
+    (yasm_bytecode *bc, yasm_bc_add_span_func add_span, void *add_span_data);
 static int dwarf2_spp_bc_tobytes
     (yasm_bytecode *bc, unsigned char **bufp, void *d,
      yasm_output_value_func output_value,
@@ -140,8 +140,8 @@ static int dwarf2_spp_bc_tobytes
 static void dwarf2_line_op_bc_destroy(void *contents);
 static void dwarf2_line_op_bc_print(const void *contents, FILE *f,
 				    int indent_level);
-static yasm_bc_resolve_flags dwarf2_line_op_bc_resolve
-    (yasm_bytecode *bc, int save, yasm_calc_bc_dist_func calc_bc_dist);
+static int dwarf2_line_op_bc_calc_len
+    (yasm_bytecode *bc, yasm_bc_add_span_func add_span, void *add_span_data);
 static int dwarf2_line_op_bc_tobytes
     (yasm_bytecode *bc, unsigned char **bufp, void *d,
      yasm_output_value_func output_value,
@@ -152,26 +152,30 @@ static const yasm_bytecode_callback dwarf2_spp_bc_callback = {
     dwarf2_spp_bc_destroy,
     dwarf2_spp_bc_print,
     yasm_bc_finalize_common,
-    dwarf2_spp_bc_resolve,
-    dwarf2_spp_bc_tobytes
+    dwarf2_spp_bc_calc_len,
+    yasm_bc_expand_common,
+    dwarf2_spp_bc_tobytes,
+    0
 };
 
 static const yasm_bytecode_callback dwarf2_line_op_bc_callback = {
     dwarf2_line_op_bc_destroy,
     dwarf2_line_op_bc_print,
     yasm_bc_finalize_common,
-    dwarf2_line_op_bc_resolve,
-    dwarf2_line_op_bc_tobytes
+    dwarf2_line_op_bc_calc_len,
+    yasm_bc_expand_common,
+    dwarf2_line_op_bc_tobytes,
+    0
 };
 
 
 static size_t
-dwarf2_dbgfmt_add_file(yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2, size_t filenum,
+dwarf2_dbgfmt_add_file(yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2, unsigned long filenum,
 		       const char *pathname)
 {
     size_t dirlen;
     const char *filename;
-    size_t i, dir;
+    unsigned long i, dir;
 
     /* Put the directory into the directory table */
     dir = 0;
@@ -210,7 +214,7 @@ dwarf2_dbgfmt_add_file(yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2, size_t filenum,
 
     /* Realloc table if necessary */
     if (filenum >= dbgfmt_dwarf2->filenames_allocated) {
-	size_t old_allocated = dbgfmt_dwarf2->filenames_allocated;
+	unsigned long old_allocated = dbgfmt_dwarf2->filenames_allocated;
 	dbgfmt_dwarf2->filenames_allocated = filenum+32;
 	dbgfmt_dwarf2->filenames = yasm_xrealloc(dbgfmt_dwarf2->filenames,
 	    sizeof(dwarf2_filename)*dbgfmt_dwarf2->filenames_allocated);
@@ -376,7 +380,8 @@ dwarf2_dbgfmt_gen_line_op(yasm_section *debug_line, dwarf2_line_state *state,
 	/* Set the starting address for the section */
 	if (!loc->sym) {
 	    /* shouldn't happen! */
-	    yasm__error(loc->line, N_("could not find label prior to loc"));
+	    yasm_error_set(YASM_ERROR_GENERAL,
+			   N_("could not find label prior to loc"));
 	    return 1;
 	}
 	dwarf2_dbgfmt_append_line_ext_op(debug_line, DW_LNE_set_address,
@@ -443,7 +448,7 @@ typedef struct dwarf2_line_bc_info {
     yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2;
     dwarf2_line_state *state;
     dwarf2_loc loc;
-    size_t lastfile;
+    unsigned long lastfile;
 } dwarf2_line_bc_info;
 
 static int
@@ -451,7 +456,7 @@ dwarf2_generate_line_bc(yasm_bytecode *bc, /*@null@*/ void *d)
 {
     dwarf2_line_bc_info *info = (dwarf2_line_bc_info *)d;
     yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2 = info->dbgfmt_dwarf2;
-    size_t i;
+    unsigned long i;
     const char *filename;
     /*@null@*/ yasm_bytecode *nextbc = yasm_bc__next(bc);
 
@@ -496,6 +501,7 @@ dwarf2_generate_line_bc(yasm_bytecode *bc, /*@null@*/ void *d)
 typedef struct dwarf2_line_info {
     yasm_section *debug_line;	/* section to which line number info goes */
     yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2;
+    yasm_errwarns *errwarns;
 
     /* Generate based on bytecodes (1) or locs (0)?  Use bytecodes if we're
      * generating line numbers for the actual assembly source file.
@@ -569,7 +575,8 @@ dwarf2_generate_line_section(yasm_section *sect, /*@null@*/ void *d)
 	    }
 	}
 
-	yasm_section_bcs_traverse(sect, &bcinfo, dwarf2_generate_line_bc);
+	yasm_section_bcs_traverse(sect, info->errwarns, &bcinfo,
+				  dwarf2_generate_line_bc);
     } else {
 	/*@null@*/ dwarf2_loc *loc;
 
@@ -589,7 +596,7 @@ dwarf2_generate_line_section(yasm_section *sect, /*@null@*/ void *d)
     if (!state.precbc)
 	state.precbc = yasm_section_bcs_first(sect);
     bc = yasm_section_bcs_last(sect);
-    addr_delta = bc->offset + bc->len - state.precbc->offset;
+    addr_delta = yasm_bc_next_offset(bc) - state.precbc->offset;
     if (addr_delta == DWARF2_MAX_SPECIAL_ADDR_DELTA)
 	dwarf2_dbgfmt_append_line_op(info->debug_line, DW_LNS_const_add_pc,
 				     NULL);
@@ -611,7 +618,8 @@ dwarf2_generate_filename(const char *filename, void *d)
 }
 
 yasm_section *
-yasm_dwarf2__generate_line(yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2, int asm_source,
+yasm_dwarf2__generate_line(yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2,
+			   yasm_errwarns *errwarns, int asm_source,
 			   /*@out@*/ yasm_section **main_code,
 			   /*@out@*/ size_t *num_line_sections)
 {
@@ -649,16 +657,19 @@ yasm_dwarf2__generate_line(yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2, int asm_source,
 
     /* directory list */
     for (i=0; i<dbgfmt_dwarf2->dirs_size; i++)
-	sppbc->len += strlen(dbgfmt_dwarf2->dirs[i])+1;
+	sppbc->len += (unsigned long)strlen(dbgfmt_dwarf2->dirs[i])+1;
     sppbc->len++;
 
     /* filename list */
     for (i=0; i<dbgfmt_dwarf2->filenames_size; i++) {
 	if (!dbgfmt_dwarf2->filenames[i].filename) {
-	    yasm__error(0, N_("dwarf2 file number %d unassigned"), i+1);
+	    yasm_error_set(YASM_ERROR_GENERAL,
+			   N_("dwarf2 file number %d unassigned"), i+1);
+	    yasm_errwarn_propagate(errwarns, 0);
 	    continue;
 	}
-	sppbc->len += strlen(dbgfmt_dwarf2->filenames[i].filename) + 1 +
+	sppbc->len +=
+	    (unsigned long)strlen(dbgfmt_dwarf2->filenames[i].filename) + 1 +
 	    yasm_size_uleb128(dbgfmt_dwarf2->filenames[i].dir) + 2;
     }
     sppbc->len++;
@@ -691,13 +702,13 @@ dwarf2_spp_bc_print(const void *contents, FILE *f, int indent_level)
     /* TODO */
 }
 
-static yasm_bc_resolve_flags
-dwarf2_spp_bc_resolve(yasm_bytecode *bc, int save,
-		      yasm_calc_bc_dist_func calc_bc_dist)
+static int
+dwarf2_spp_bc_calc_len(yasm_bytecode *bc, yasm_bc_add_span_func add_span,
+		       void *add_span_data)
 {
-    yasm_internal_error(N_("tried to resolve a dwarf2 spp bytecode"));
+    yasm_internal_error(N_("tried to calc_len a dwarf2 spp bytecode"));
     /*@notreached@*/
-    return YASM_BC_RESOLVE_MIN_LEN;
+    return 0;
 }
 
 static int
@@ -712,11 +723,11 @@ dwarf2_spp_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
     size_t i, len;
 
     /* Prologue length (following this field) */
-    cval = yasm_intnum_create_uint(bc->len - (buf-*bufp) -
+    cval = yasm_intnum_create_uint(bc->len - (unsigned long)(buf-*bufp) -
 				   dbgfmt_dwarf2->sizeof_offset);
     yasm_arch_intnum_tobytes(dbgfmt_dwarf2->arch, cval, buf,
 			     dbgfmt_dwarf2->sizeof_offset,
-			     dbgfmt_dwarf2->sizeof_offset*8, 0, bc, 0, 0);
+			     dbgfmt_dwarf2->sizeof_offset*8, 0, bc, 0);
     buf += dbgfmt_dwarf2->sizeof_offset;
 
     YASM_WRITE_8(buf, dbgfmt_dwarf2->min_insn_len);	/* minimum_instr_len */
@@ -773,13 +784,13 @@ dwarf2_line_op_bc_print(const void *contents, FILE *f, int indent_level)
     /* TODO */
 }
 
-static yasm_bc_resolve_flags
-dwarf2_line_op_bc_resolve(yasm_bytecode *bc, int save,
-			  yasm_calc_bc_dist_func calc_bc_dist)
+static int
+dwarf2_line_op_bc_calc_len(yasm_bytecode *bc, yasm_bc_add_span_func add_span,
+			   void *add_span_data)
 {
-    yasm_internal_error(N_("tried to resolve a dwarf2 line_op bytecode"));
+    yasm_internal_error(N_("tried to calc_len a dwarf2 line_op bytecode"));
     /*@notreached@*/
-    return YASM_BC_RESOLVE_MIN_LEN;
+    return 0;
 }
 
 static int
@@ -798,9 +809,9 @@ dwarf2_line_op_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 	YASM_WRITE_8(buf, line_op->ext_opcode);
 	if (line_op->ext_operand) {
 	    yasm_value value;
-	    yasm_value_init_sym(&value, line_op->ext_operand);
+	    yasm_value_init_sym(&value, line_op->ext_operand,
+				line_op->ext_operandsize*8);
 	    output_value(&value, buf, line_op->ext_operandsize,
-			 line_op->ext_operandsize*8, 0,
 			 (unsigned long)(buf-*bufp), bc, 0, d);
 	    buf += line_op->ext_operandsize;
 	}
@@ -815,26 +826,28 @@ yasm_dwarf2__line_directive(yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2,
 			    const char *name, yasm_section *sect,
 			    yasm_valparamhead *valparams, unsigned long line)
 {
+    yasm_valparam *vp;
     if (yasm__strcasecmp(name, "loc") == 0) {
 	/*@dependent@*/ /*@null@*/ const yasm_intnum *intn;
 	dwarf2_section_data *dsd;
 	dwarf2_loc *loc = yasm_xmalloc(sizeof(dwarf2_loc));
 
 	/* File number (required) */
-	yasm_valparam *vp = yasm_vps_first(valparams);
-	if (!vp || !vp->param) {
-	    yasm__error(line, N_("file number required"));
+	if (!valparams || !(vp = yasm_vps_first(valparams)) || !vp->param) {
+	    yasm_error_set(YASM_ERROR_SYNTAX, N_("file number required"));
 	    yasm_xfree(loc);
 	    return 0;
 	}
-	intn = yasm_expr_get_intnum(&vp->param, NULL);
+	intn = yasm_expr_get_intnum(&vp->param, 0);
 	if (!intn) {
-	    yasm__error(line, N_("file number is not a constant"));
+	    yasm_error_set(YASM_ERROR_NOT_CONSTANT,
+			   N_("file number is not a constant"));
 	    yasm_xfree(loc);
 	    return 0;
 	}
 	if (yasm_intnum_sign(intn) != 1) {
-	    yasm__error(line, N_("file number less than one"));
+	    yasm_error_set(YASM_ERROR_VALUE,
+			   N_("file number less than one"));
 	    yasm_xfree(loc);
 	    return 0;
 	}
@@ -843,13 +856,14 @@ yasm_dwarf2__line_directive(yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2,
 	/* Line number (required) */
 	vp = yasm_vps_next(vp);
 	if (!vp || !vp->param) {
-	    yasm__error(line, N_("line number required"));
+	    yasm_error_set(YASM_ERROR_SYNTAX, N_("line number required"));
 	    yasm_xfree(loc);
 	    return 0;
 	}
-	intn = yasm_expr_get_intnum(&vp->param, NULL);
+	intn = yasm_expr_get_intnum(&vp->param, 0);
 	if (!intn) {
-	    yasm__error(line, N_("file number is not a constant"));
+	    yasm_error_set(YASM_ERROR_NOT_CONSTANT,
+			   N_("file number is not a constant"));
 	    yasm_xfree(loc);
 	    return 0;
 	}
@@ -875,9 +889,10 @@ yasm_dwarf2__line_directive(yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2,
 	/* Optional column number */
 	vp = yasm_vps_next(vp);
 	if (vp && vp->param) {
-	    intn = yasm_expr_get_intnum(&vp->param, NULL);
+	    intn = yasm_expr_get_intnum(&vp->param, 0);
 	    if (!intn) {
-		yasm__error(line, N_("column number is not a constant"));
+		yasm_error_set(YASM_ERROR_NOT_CONSTANT,
+			       N_("column number is not a constant"));
 		yasm_xfree(loc);
 		return 0;
 	    }
@@ -895,13 +910,15 @@ yasm_dwarf2__line_directive(yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2,
 		loc->epilogue_begin = 1;
 	    else if (yasm__strcasecmp(vp->val, "is_stmt") == 0) {
 		if (!vp->param) {
-		    yasm__error(line, N_("is_stmt requires value"));
+		    yasm_error_set(YASM_ERROR_SYNTAX,
+				   N_("is_stmt requires value"));
 		    yasm_xfree(loc);
 		    return 0;
 		}
-		intn = yasm_expr_get_intnum(&vp->param, NULL);
+		intn = yasm_expr_get_intnum(&vp->param, 0);
 		if (!intn) {
-		    yasm__error(line, N_("is_stmt value is not a constant"));
+		    yasm_error_set(YASM_ERROR_NOT_CONSTANT,
+				   N_("is_stmt value is not a constant"));
 		    yasm_xfree(loc);
 		    return 0;
 		}
@@ -910,31 +927,34 @@ yasm_dwarf2__line_directive(yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2,
 		else if (yasm_intnum_is_pos1(intn))
 		    loc->is_stmt = IS_STMT_CLEAR;
 		else {
-		    yasm__error(line, N_("is_stmt value not 0 or 1"));
+		    yasm_error_set(YASM_ERROR_VALUE,
+				   N_("is_stmt value not 0 or 1"));
 		    yasm_xfree(loc);
 		    return 0;
 		}
 	    } else if (yasm__strcasecmp(vp->val, "isa") == 0) {
 		if (!vp->param) {
-		    yasm__error(line, N_("isa requires value"));
+		    yasm_error_set(YASM_ERROR_SYNTAX, N_("isa requires value"));
 		    yasm_xfree(loc);
 		    return 0;
 		}
-		intn = yasm_expr_get_intnum(&vp->param, NULL);
+		intn = yasm_expr_get_intnum(&vp->param, 0);
 		if (!intn) {
-		    yasm__error(line, N_("isa value is not a constant"));
+		    yasm_error_set(YASM_ERROR_NOT_CONSTANT,
+				   N_("isa value is not a constant"));
 		    yasm_xfree(loc);
 		    return 0;
 		}
 		if (yasm_intnum_sign(intn) < 0) {
-		    yasm__error(line, N_("isa value less than zero"));
+		    yasm_error_set(YASM_ERROR_VALUE,
+				   N_("isa value less than zero"));
 		    yasm_xfree(loc);
 		    return 0;
 		}
 		loc->isa_change = 1;
 		loc->isa = yasm_intnum_get_uint(intn);
 	    } else
-		yasm__warning(YASM_WARN_GENERAL, line,
+		yasm_warn_set(YASM_WARN_GENERAL,
 			      N_("unrecognized loc option `%s'"), vp->val);
 	}
 
@@ -947,10 +967,15 @@ yasm_dwarf2__line_directive(yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2,
 	return 0;
     } else if (yasm__strcasecmp(name, "file") == 0) {
 	/*@dependent@*/ /*@null@*/ const yasm_intnum *file_intn;
-	size_t filenum;
+	unsigned long filenum;
 
-	yasm_valparam *vp = yasm_vps_first(valparams);
+	if (!valparams) {
+	    yasm_error_set(YASM_ERROR_SYNTAX, N_("[%s] requires an argument"),
+			   "FILE");
+	    return 0;
+	}
 
+	vp = yasm_vps_first(valparams);
 	if (vp->val) {
 	    /* Just a bare filename */
 	    yasm_object_set_source_fn(dbgfmt_dwarf2->object, vp->val);
@@ -958,16 +983,18 @@ yasm_dwarf2__line_directive(yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2,
 	}
 
 	/* Otherwise.. first vp is the file number */
-	file_intn = yasm_expr_get_intnum(&vp->param, NULL);
+	file_intn = yasm_expr_get_intnum(&vp->param, 0);
 	if (!file_intn) {
-	    yasm__error(line, N_("file number is not a constant"));
+	    yasm_error_set(YASM_ERROR_NOT_CONSTANT,
+			   N_("file number is not a constant"));
 	    return 0;
 	}
-	filenum = (size_t)yasm_intnum_get_uint(file_intn);
+	filenum = yasm_intnum_get_uint(file_intn);
 
 	vp = yasm_vps_next(vp);
 	if (!vp || !vp->val) {
-	    yasm__error(line, N_("file number given but no filename"));
+	    yasm_error_set(YASM_ERROR_SYNTAX,
+			   N_("file number given but no filename"));
 	    return 0;
 	}
 
