@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: null-dbgfmt.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: null-dbgfmt.c 1521 2006-05-04 07:55:32Z peter $");
 
 #define YASM_LIB_INTERNAL
 #include <libyasm.h>
@@ -57,7 +57,7 @@ null_dbgfmt_directive(yasm_dbgfmt *dbgfmt, const char *name,
 }
 
 static void
-null_dbgfmt_generate(yasm_dbgfmt *dbgfmt)
+null_dbgfmt_generate(yasm_dbgfmt *dbgfmt, yasm_errwarns *errwarns)
 {
 }
 
