@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: stabs-dbgfmt.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: stabs-dbgfmt.c 1795 2007-02-24 22:39:55Z peter $");
 
 #define YASM_LIB_INTERNAL
 #define YASM_BC_INTERNAL
@@ -104,6 +104,7 @@ typedef struct {
     yasm_bytecode *basebc;      /* base bytecode from which to track SLINEs */
 
     yasm_dbgfmt_stabs *dbgfmt_stabs;
+    yasm_errwarns *errwarns;
 } stabs_info;
 
 typedef struct {
@@ -121,8 +122,8 @@ typedef struct {
 static void stabs_bc_str_destroy(void *contents);
 static void stabs_bc_str_print(const void *contents, FILE *f, int
 			       indent_level);
-static yasm_bc_resolve_flags stabs_bc_str_resolve
-    (yasm_bytecode *bc, int save, yasm_calc_bc_dist_func calc_bc_dist);
+static int stabs_bc_str_calc_len
+    (yasm_bytecode *bc, yasm_bc_add_span_func add_span, void *add_span_data);
 static int stabs_bc_str_tobytes
     (yasm_bytecode *bc, unsigned char **bufp, void *d,
      yasm_output_value_func output_value,
@@ -131,8 +132,8 @@ static int stabs_bc_str_tobytes
 static void stabs_bc_stab_destroy(void *contents);
 static void stabs_bc_stab_print(const void *contents, FILE *f, int
 				indent_level);
-static yasm_bc_resolve_flags stabs_bc_stab_resolve
-    (yasm_bytecode *bc, int save, yasm_calc_bc_dist_func calc_bc_dist);
+static int stabs_bc_stab_calc_len
+    (yasm_bytecode *bc, yasm_bc_add_span_func add_span, void *add_span_data);
 static int stabs_bc_stab_tobytes
     (yasm_bytecode *bc, unsigned char **bufp, void *d,
      yasm_output_value_func output_value,
@@ -144,16 +145,20 @@ static const yasm_bytecode_callback stabs_bc_str_callback = {
     stabs_bc_str_destroy,
     stabs_bc_str_print,
     yasm_bc_finalize_common,
-    stabs_bc_str_resolve,
-    stabs_bc_str_tobytes
+    stabs_bc_str_calc_len,
+    yasm_bc_expand_common,
+    stabs_bc_str_tobytes,
+    0
 };
 
 static const yasm_bytecode_callback stabs_bc_stab_callback = {
     stabs_bc_stab_destroy,
     stabs_bc_stab_print,
     yasm_bc_finalize_common,
-    stabs_bc_stab_resolve,
-    stabs_bc_stab_tobytes
+    stabs_bc_stab_calc_len,
+    yasm_bc_expand_common,
+    stabs_bc_stab_tobytes,
+    0
 };
 
 yasm_dbgfmt_module yasm_stabs_LTX_dbgfmt;
@@ -183,12 +188,11 @@ stabs_dbgfmt_destroy(/*@only@*/ yasm_dbgfmt *dbgfmt)
 static yasm_bytecode *
 stabs_dbgfmt_append_bcstr(yasm_section *sect, const char *str)
 {
-    yasm_bytecode *bc, *precbc;
+    yasm_bytecode *bc;
    
-    precbc = yasm_section_bcs_last(sect);
     bc = yasm_bc_create_common(&stabs_bc_str_callback, yasm__xstrdup(str), 0);
-    bc->len = strlen(str)+1;
-    bc->offset = precbc ? precbc->offset + precbc->len : 0;
+    bc->len = (unsigned long)(strlen(str)+1);
+    bc->offset = yasm_bc_next_offset(yasm_section_bcs_last(sect));
 
     yasm_section_bcs_append(sect, bc);
 
@@ -204,7 +208,7 @@ stabs_dbgfmt_append_stab(stabs_info *info, yasm_section *sect,
 			 unsigned long desc, /*@null@*/ yasm_symrec *symvalue,
 			 /*@null@*/ yasm_bytecode *bcvalue, unsigned long value)
 {
-    yasm_bytecode *bc, *precbc;
+    yasm_bytecode *bc;
     stabs_stab *stab = yasm_xmalloc(sizeof(stabs_stab));
 
     stab->other = 0;
@@ -215,11 +219,10 @@ stabs_dbgfmt_append_stab(stabs_info *info, yasm_section *sect,
     stab->bcvalue = bcvalue;
     stab->value = value;
 
-    precbc = yasm_section_bcs_last(sect);
     bc = yasm_bc_create_common(&stabs_bc_stab_callback, stab,
 			       bcvalue ? bcvalue->line : 0);
     bc->len = info->stablen;
-    bc->offset = precbc ? precbc->offset + precbc->len : 0;
+    bc->offset = yasm_bc_next_offset(yasm_section_bcs_last(sect));
 
     yasm_section_bcs_append(sect, bc);
 
@@ -299,7 +302,8 @@ stabs_dbgfmt_generate_sections(yasm_section *sect, /*@null@*/ void *d)
     /* handle first (pseudo) bc separately */
     stabs_dbgfmt_generate_n_fun(d, yasm_section_bcs_first(sect));
 
-    yasm_section_bcs_traverse(sect, d, stabs_dbgfmt_generate_bcs);
+    yasm_section_bcs_traverse(sect, info->errwarns, d,
+			      stabs_dbgfmt_generate_bcs);
 
     if (yasm__strcasecmp(sectname, ".text")==0) {
         /* Close out last function by appending a null SO stab after last bc */
@@ -314,7 +318,7 @@ stabs_dbgfmt_generate_sections(yasm_section *sect, /*@null@*/ void *d)
 }
 
 static void
-stabs_dbgfmt_generate(yasm_dbgfmt *dbgfmt)
+stabs_dbgfmt_generate(yasm_dbgfmt *dbgfmt, yasm_errwarns *errwarns)
 {
     yasm_dbgfmt_stabs *dbgfmt_stabs = (yasm_dbgfmt_stabs *)dbgfmt;
     stabs_info info;
@@ -333,38 +337,46 @@ stabs_dbgfmt_generate(yasm_dbgfmt *dbgfmt)
 	return;
 
     info.dbgfmt_stabs = dbgfmt_stabs;
+    info.errwarns = errwarns;
     info.lastline = 0;
     info.stabcount = 0;
     info.stab = yasm_object_get_general(dbgfmt_stabs->object, ".stab", 0, 4, 0,
 					0, &new, 0);
     if (!new) {
 	yasm_bytecode *last = yasm_section_bcs_last(info.stab);
-	if (last == NULL)
-	    yasm__error(yasm_section_bcs_first(info.stab)->line,
+	if (last == NULL) {
+	    yasm_error_set(YASM_ERROR_GENERAL,
 		N_("stabs debugging conflicts with user-defined section .stab"));
-	else
-	    yasm__warning(YASM_WARN_GENERAL, 0,
+	    yasm_errwarn_propagate(errwarns,
+				   yasm_section_bcs_first(info.stab)->line);
+	} else {
+	    yasm_warn_set(YASM_WARN_GENERAL,
 		N_("stabs debugging overrides empty section .stab"));
+	    yasm_errwarn_propagate(errwarns, 0);
+	}
     }
 
     info.stabstr = yasm_object_get_general(dbgfmt_stabs->object, ".stabstr", 0,
 					   1, 0, 0, &new, 0);
     if (!new) {
 	yasm_bytecode *last = yasm_section_bcs_last(info.stabstr);
-	if (last == NULL)
-	    yasm__error(yasm_section_bcs_first(info.stabstr)->line,
+	if (last == NULL) {
+	    yasm_error_set(YASM_ERROR_GENERAL,
 		N_("stabs debugging conflicts with user-defined section .stabstr"));
-	else
-	    yasm__warning(YASM_WARN_GENERAL, 0,
+	    yasm_errwarn_propagate(errwarns,
+				   yasm_section_bcs_first(info.stab)->line);
+	} else {
+	    yasm_warn_set(YASM_WARN_GENERAL,
 		N_("stabs debugging overrides empty section .stabstr"));
+	    yasm_errwarn_propagate(errwarns, 0);
+	}
     }
 
-
-
     /* initial pseudo-stab */
     stab = yasm_xmalloc(sizeof(stabs_stab));
     dbgbc = yasm_bc_create_common(&stabs_bc_stab_callback, stab, 0);
     dbgbc->len = info.stablen;
+    dbgbc->offset = 0;
     yasm_section_bcs_append(info.stab, dbgbc);
 
     /* initial strtab bytecodes */
@@ -389,11 +401,16 @@ stabs_dbgfmt_generate(yasm_dbgfmt *dbgfmt)
 
     stab->bcvalue = NULL;
     stab->symvalue = NULL;
-    stab->value = laststr->offset + laststr->len;
+    stab->value = yasm_bc_next_offset(laststr);
     stab->bcstr = filebc;
     stab->type = N_UNDF;
     stab->other = 0;
-    stab->desc = info.stabcount;
+    if (info.stabcount > 0xffff) {
+	yasm_warn_set(YASM_WARN_GENERAL, N_("over 65535 stabs"));
+	yasm_errwarn_propagate(errwarns, 0);
+	stab->desc = 0xffff;
+    } else
+	stab->desc = (unsigned short)info.stabcount;
 }
 
 static int
@@ -473,22 +490,22 @@ stabs_bc_str_print(const void *contents, FILE *f, int indent_level)
     fprintf(f, "%*s\"%s\"\n", indent_level, "", (const char *)contents);
 }
 
-static yasm_bc_resolve_flags
-stabs_bc_stab_resolve(yasm_bytecode *bc, int save,
-		      yasm_calc_bc_dist_func calc_bc_dist)
+static int
+stabs_bc_stab_calc_len(yasm_bytecode *bc, yasm_bc_add_span_func add_span,
+		       void *add_span_data)
 {
     yasm_internal_error(N_("tried to resolve a stabs stab bytecode"));
     /*@notreached@*/
-    return YASM_BC_RESOLVE_MIN_LEN;
+    return 0;
 }
 
-static yasm_bc_resolve_flags
-stabs_bc_str_resolve(yasm_bytecode *bc, int save,
-		     yasm_calc_bc_dist_func calc_bc_dist)
+static int
+stabs_bc_str_calc_len(yasm_bytecode *bc, yasm_bc_add_span_func add_span,
+		      void *add_span_data)
 {
     yasm_internal_error(N_("tried to resolve a stabs str bytecode"));
     /*@notreached@*/
-    return YASM_BC_RESOLVE_MIN_LEN;
+    return 0;
 }
 
 static int
