@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: nasm-listfmt.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: nasm-listfmt.c 1795 2007-02-24 22:39:55Z peter $");
 
 #define YASM_LIB_INTERNAL
 #define YASM_BC_INTERNAL
@@ -82,18 +82,18 @@ nasm_listfmt_destroy(/*@only@*/ yasm_listfmt *listfmt)
 
 static int
 nasm_listfmt_output_value(yasm_value *value, unsigned char *buf,
-			  size_t destsize, size_t valsize, int shift,
-			  unsigned long offset, yasm_bytecode *bc, int warn,
-			  /*@null@*/ void *d)
+			  unsigned int destsize, unsigned long offset,
+			  yasm_bytecode *bc, int warn, /*@null@*/ void *d)
 {
     /*@null@*/ nasm_listfmt_output_info *info = (nasm_listfmt_output_info *)d;
     /*@dependent@*/ /*@null@*/ yasm_intnum *intn;
+    unsigned int valsize = value->size;
 
     assert(info != NULL);
 
     /* Output */
-    switch (yasm_value_output_basic(value, buf, destsize, valsize, shift, bc,
-				    warn, info->arch, NULL)) {
+    switch (yasm_value_output_basic(value, buf, destsize, bc, warn,
+				    info->arch)) {
 	case -1:
 	    return 1;
 	case 0:
@@ -121,19 +121,20 @@ nasm_listfmt_output_value(yasm_value *value, unsigned char *buf,
     }
 
     if (value->abs) {
-	intn = yasm_expr_get_intnum(&value->abs, NULL);
+	intn = yasm_expr_get_intnum(&value->abs, 0);
 	if (intn)
 	    return yasm_arch_intnum_tobytes(info->arch, intn, buf, destsize,
-					    valsize, shift, bc, 0, bc->line);
+					    valsize, 0, bc, 0);
 	else {
-	    yasm__error(bc->line, N_("relocation too complex"));
+	    yasm_error_set(YASM_ERROR_TOO_COMPLEX,
+			   N_("relocation too complex"));
 	    return 1;
 	}
     } else {
 	int retval;
 	intn = yasm_intnum_create_uint(0);
 	retval = yasm_arch_intnum_tobytes(info->arch, intn, buf, destsize,
-					  valsize, shift, bc, 0, bc->line);
+					  valsize, 0, bc, 0);
 	yasm_intnum_destroy(intn);
 	return retval;
     }
@@ -205,7 +206,7 @@ nasm_listfmt_output(yasm_listfmt *listfmt, FILE *f, yasm_linemap *linemap,
 	    while (bc && bc->line == line) {
 		/*@null@*/ /*@only@*/ unsigned char *bigbuf;
 		unsigned long size = REGULAR_BUF_SIZE;
-		unsigned long multiple;
+		long multiple;
 		unsigned long offset = bc->offset;
 		unsigned char *origp, *p;
 		int gap;
@@ -215,8 +216,11 @@ nasm_listfmt_output(yasm_listfmt *listfmt, FILE *f, yasm_linemap *linemap,
 		 */
 		bigbuf = yasm_bc_tobytes(bc, buf, &size, &gap, &info,
 					 nasm_listfmt_output_value, NULL);
-		yasm_bc_get_multiple(bc, &multiple, NULL);
-		size /= multiple;
+		yasm_bc_get_multiple(bc, &multiple, 1);
+		if (multiple <= 0)
+		    size = 0;
+		else
+		    size /= multiple;
 
 		/* output bytes with reloc information */
 		origp = bigbuf ? bigbuf : buf;
