@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: bin-objfmt.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: bin-objfmt.c 1795 2007-02-24 22:39:55Z peter $");
 
 #define YASM_LIB_INTERNAL
 #define YASM_BC_INTERNAL
@@ -67,18 +67,13 @@ bin_objfmt_align_section(yasm_section *sect, yasm_section *prevsect,
 			 /*@out@*/ unsigned long *prevsectlen,
 			 /*@out@*/ unsigned long *padamt)
 {
-    /*@dependent@*/ /*@null@*/ yasm_bytecode *last;
     unsigned long start;
     unsigned long align;
 
     /* Figure out the size of .text by looking at the last bytecode's offset
      * plus its length.  Add the start and size together to get the new start.
      */
-    last = yasm_section_bcs_last(prevsect);
-    if (last)
-	*prevsectlen = last->offset + last->len;
-    else
-	*prevsectlen = 0;
+    *prevsectlen = yasm_bc_next_offset(yasm_section_bcs_last(prevsect));
     start = base + *prevsectlen;
 
     /* Round new start up to alignment of .data section, and adjust textlen to
@@ -100,7 +95,8 @@ typedef struct bin_objfmt_output_info {
     /*@dependent@*/ FILE *f;
     /*@only@*/ unsigned char *buf;
     /*@observer@*/ const yasm_section *sect;
-    unsigned long start;
+    unsigned long start;	/* what normal variables go against */
+    unsigned long abs_start;	/* what absolutes go against */
 } bin_objfmt_output_info;
 
 static /*@only@*/ yasm_expr *
@@ -116,12 +112,10 @@ bin_objfmt_expr_xform(/*@returned@*/ /*@only@*/ yasm_expr *e,
 	/* Transform symrecs that reference sections into
 	 * start expr + intnum(dist).
 	 */
-	if ((e->terms[i].type == YASM_EXPR_SYM ||
-	     e->terms[i].type == YASM_EXPR_SYMEXP) &&
+	if (e->terms[i].type == YASM_EXPR_SYM &&
 	    yasm_symrec_get_label(e->terms[i].data.sym, &precbc) &&
 	    (sect = yasm_bc_get_section(precbc)) &&
-	    (dist = yasm_common_calc_bc_dist(yasm_section_bcs_first(sect),
-					     precbc))) {
+	    (dist = yasm_calc_bc_dist(yasm_section_bcs_first(sect), precbc))) {
 	    const yasm_expr *start = yasm_section_get_start(sect);
 	    e->terms[i].type = YASM_EXPR_EXPR;
 	    e->terms[i].data.expn =
@@ -135,8 +129,8 @@ bin_objfmt_expr_xform(/*@returned@*/ /*@only@*/ yasm_expr *e,
 }
 
 static int
-bin_objfmt_output_value(yasm_value *value, unsigned char *buf, size_t destsize,
-			size_t valsize, int shift,
+bin_objfmt_output_value(yasm_value *value, unsigned char *buf,
+			unsigned int destsize,
 			/*@unused@*/ unsigned long offset, yasm_bytecode *bc,
 			int warn, /*@null@*/ void *d)
 {
@@ -150,25 +144,32 @@ bin_objfmt_output_value(yasm_value *value, unsigned char *buf, size_t destsize,
     if (value->rel && !value->curpos_rel
 	&& yasm_symrec_get_label(value->rel, &precbc)
 	&& (sect = yasm_bc_get_section(precbc))) {
+	unsigned int rshift = (unsigned int)value->rshift;
+	yasm_expr *syme;
+	if (value->rshift > 0)
+	    syme = yasm_expr_create(YASM_EXPR_SHR, yasm_expr_sym(value->rel),
+		yasm_expr_int(yasm_intnum_create_uint(rshift)), bc->line);
+	else
+	    syme = yasm_expr_create_ident(yasm_expr_sym(value->rel), bc->line);
+
 	if (!value->abs)
-	    value->abs = yasm_expr_create_ident(yasm_expr_sym(value->rel),
-						bc->line);
+	    value->abs = syme;
 	else
 	    value->abs =
 		yasm_expr_create(YASM_EXPR_ADD, yasm_expr_expr(value->abs),
-				 yasm_expr_sym(value->rel), bc->line);
+				 yasm_expr_expr(syme), bc->line);
 	value->rel = NULL;
+	value->rshift = 0;
     }
 
     /* Simplify absolute portion of value, transforming symrecs */
     if (value->abs)
 	value->abs = yasm_expr__level_tree
-	    (value->abs, 1, 1, 1, NULL, bin_objfmt_expr_xform, NULL, NULL,
-	     NULL);
+	    (value->abs, 1, 1, 1, 0, bin_objfmt_expr_xform, NULL);
 
     /* Output */
-    switch (yasm_value_output_basic(value, buf, destsize, valsize, shift,
-				    bc, warn, info->objfmt_bin->arch, NULL)) {
+    switch (yasm_value_output_basic(value, buf, destsize, bc, warn,
+				    info->objfmt_bin->arch)) {
 	case -1:
 	    return 1;
 	case 0:
@@ -177,8 +178,38 @@ bin_objfmt_output_value(yasm_value *value, unsigned char *buf, size_t destsize,
 	    return 0;
     }
 
+    /* Absolute value; handle it here as output_basic won't understand it */
+    if (value->rel && yasm_symrec_is_abs(value->rel)) {
+	if (value->curpos_rel) {
+	    /* Calculate value relative to current assembly position */
+	    /*@only@*/ yasm_intnum *outval;
+	    unsigned int valsize = value->size;
+	    int retval = 0;
+
+	    outval = yasm_intnum_create_uint(bc->offset + info->abs_start);
+	    yasm_intnum_calc(outval, YASM_EXPR_NEG, NULL);
+
+	    if (value->rshift > 0) {
+		/*@only@*/ yasm_intnum *shamt =
+		    yasm_intnum_create_uint((unsigned long)value->rshift);
+		yasm_intnum_calc(outval, YASM_EXPR_SHR, shamt);
+		yasm_intnum_destroy(shamt);
+	    }
+	    /* Add in absolute portion */
+	    if (value->abs)
+		yasm_intnum_calc(outval, YASM_EXPR_ADD,
+				 yasm_expr_get_intnum(&value->abs, 1));
+	    /* Output! */
+	    if (yasm_arch_intnum_tobytes(info->objfmt_bin->arch, outval, buf,
+					 destsize, valsize, 0, bc, warn))
+		retval = 1;
+	    yasm_intnum_destroy(outval);
+	    return retval;
+	}
+    }
+
     /* Couldn't output, assume it contains an external reference. */
-    yasm__error(bc->line,
+    yasm_error_set(YASM_ERROR_GENERAL,
 	N_("binary object format does not support external references"));
     return 1;
 }
@@ -206,7 +237,7 @@ bin_objfmt_output_bytecode(yasm_bytecode *bc, /*@null@*/ void *d)
     /* Warn that gaps are converted to 0 and write out the 0's. */
     if (gap) {
 	unsigned long left;
-	yasm__warning(YASM_WARN_UNINIT_CONTENTS, bc->line,
+	yasm_warn_set(YASM_WARN_UNINIT_CONTENTS,
 	    N_("uninitialized space declared in code/data section: zeroing"));
 	/* Write out in chunks */
 	memset(info->buf, 0, REGULAR_OUTBUF_SIZE);
@@ -230,7 +261,7 @@ bin_objfmt_output_bytecode(yasm_bytecode *bc, /*@null@*/ void *d)
 
 static void
 bin_objfmt_output(yasm_objfmt *objfmt, FILE *f, /*@unused@*/ int all_syms,
-		  /*@unused@*/ yasm_dbgfmt *df)
+		  /*@unused@*/ yasm_dbgfmt *df, yasm_errwarns *errwarns)
 {
     yasm_objfmt_bin *objfmt_bin = (yasm_objfmt_bin *)objfmt;
     /*@observer@*/ /*@null@*/ yasm_section *text, *data, *bss, *prevsect;
@@ -263,13 +294,16 @@ bin_objfmt_output(yasm_objfmt *objfmt, FILE *f, /*@unused@*/ int all_syms,
     /* Find out the start of .text */
     startexpr = yasm_expr_copy(yasm_section_get_start(text));
     assert(startexpr != NULL);
-    startnum = yasm_expr_get_intnum(&startexpr, NULL);
+    startnum = yasm_expr_get_intnum(&startexpr, 0);
     if (!startnum) {
-	yasm__error(startexpr->line, N_("ORG expression too complex"));
+	yasm_error_set(YASM_ERROR_TOO_COMPLEX,
+		       N_("ORG expression too complex"));
+	yasm_errwarn_propagate(errwarns, startexpr->line);
 	return;
     }
     start = yasm_intnum_get_uint(startnum);
     yasm_expr_destroy(startexpr);
+    info.abs_start = start;
     textstart = start;
 
     /* Align .data and .bss (if present) by adjusting their starts. */
@@ -296,7 +330,8 @@ bin_objfmt_output(yasm_objfmt *objfmt, FILE *f, /*@unused@*/ int all_syms,
     /* Output .text first. */
     info.sect = text;
     info.start = textstart;
-    yasm_section_bcs_traverse(text, &info, bin_objfmt_output_bytecode);
+    yasm_section_bcs_traverse(text, errwarns, &info,
+			      bin_objfmt_output_bytecode);
 
     /* If .data is present, output it */
     if (data) {
@@ -309,7 +344,8 @@ bin_objfmt_output(yasm_objfmt *objfmt, FILE *f, /*@unused@*/ int all_syms,
 	/* Output .data bytecodes */
 	info.sect = data;
 	info.start = datastart;
-	yasm_section_bcs_traverse(data, &info, bin_objfmt_output_bytecode);
+	yasm_section_bcs_traverse(data, errwarns,
+				  &info, bin_objfmt_output_bytecode);
     }
 
     /* If .bss is present, check it for non-reserve bytecodes */
@@ -379,15 +415,15 @@ bin_objfmt_section_switch(yasm_objfmt *objfmt, yasm_valparamhead *valparams,
 	    resonly = 1;
 	} else {
 	    /* other section names not recognized. */
-	    yasm__error(line, N_("segment name `%s' not recognized"),
-			sectname);
+	    yasm_error_set(YASM_ERROR_GENERAL,
+			   N_("segment name `%s' not recognized"), sectname);
 	    return NULL;
 	}
 
 	/* Check for ALIGN qualifier */
 	while ((vp = yasm_vps_next(vp))) {
 	    if (!vp->val) {
-		yasm__warning(YASM_WARN_GENERAL, line,
+		yasm_warn_set(YASM_WARN_GENERAL,
 			      N_("Unrecognized numeric qualifier"));
 		continue;
 	    }
@@ -396,24 +432,24 @@ bin_objfmt_section_switch(yasm_objfmt *objfmt, yasm_valparamhead *valparams,
 		/*@dependent@*/ /*@null@*/ const yasm_intnum *align_expr;
 
 		if (strcmp(sectname, ".text") == 0) {
-		    yasm__error(line,
+		    yasm_error_set(YASM_ERROR_GENERAL,
 			N_("cannot specify an alignment to the `%s' section"),
 			sectname);
 		    return NULL;
 		}
 		
-		align_expr = yasm_expr_get_intnum(&vp->param, NULL);
+		align_expr = yasm_expr_get_intnum(&vp->param, 0);
 		if (!align_expr) {
-		    yasm__error(line,
-				N_("argument to `%s' is not a power of two"),
+		    yasm_error_set(YASM_ERROR_VALUE,
+				N_("argument to `%s' is not an integer"),
 				vp->val);
 		    return NULL;
 		}
 		align = yasm_intnum_get_uint(align_expr);
 
 		/* Alignments must be a power of two. */
-		if ((align & (align - 1)) != 0) {
-		    yasm__error(line,
+		if (!is_exp2(align)) {
+		    yasm_error_set(YASM_ERROR_VALUE,
 				N_("argument to `%s' is not a power of two"),
 				vp->val);
 		    return NULL;
@@ -435,7 +471,7 @@ bin_objfmt_section_switch(yasm_objfmt *objfmt, yasm_valparamhead *valparams,
 	    yasm_section_set_default(retval, 0);
 	    yasm_section_set_align(retval, align, line);
 	} else if (have_align)
-	    yasm__warning(YASM_WARN_GENERAL, line,
+	    yasm_warn_set(YASM_WARN_GENERAL,
 		N_("alignment value ignored on section redeclaration"));
 
 	return retval;
@@ -452,7 +488,7 @@ bin_objfmt_extern_declare(yasm_objfmt *objfmt, const char *name,
     yasm_objfmt_bin *objfmt_bin = (yasm_objfmt_bin *)objfmt;
     yasm_symrec *sym;
 
-    yasm__warning(YASM_WARN_GENERAL, line,
+    yasm_warn_set(YASM_WARN_GENERAL,
 		  N_("binary object format does not support extern variables"));
 
     sym = yasm_symtab_declare(objfmt_bin->symtab, name, YASM_SYM_EXTERN, line);
@@ -468,7 +504,7 @@ bin_objfmt_global_declare(yasm_objfmt *objfmt, const char *name,
     yasm_objfmt_bin *objfmt_bin = (yasm_objfmt_bin *)objfmt;
     yasm_symrec *sym;
 
-    yasm__warning(YASM_WARN_GENERAL, line,
+    yasm_warn_set(YASM_WARN_GENERAL,
 		  N_("binary object format does not support global variables"));
 
     sym = yasm_symtab_declare(objfmt_bin->symtab, name, YASM_SYM_GLOBAL, line);
@@ -485,7 +521,7 @@ bin_objfmt_common_declare(yasm_objfmt *objfmt, const char *name,
     yasm_symrec *sym;
 
     yasm_expr_destroy(size);
-    yasm__error(line,
+    yasm_error_set(YASM_ERROR_TYPE,
 	N_("binary object format does not support common variables"));
 
     sym = yasm_symtab_declare(objfmt_bin->symtab, name, YASM_SYM_COMMON, line);
@@ -494,7 +530,7 @@ bin_objfmt_common_declare(yasm_objfmt *objfmt, const char *name,
 
 static int
 bin_objfmt_directive(yasm_objfmt *objfmt, const char *name,
-		     yasm_valparamhead *valparams,
+		     /*@null@*/ yasm_valparamhead *valparams,
 		     /*@unused@*/ /*@null@*/
 		     yasm_valparamhead *objext_valparams, unsigned long line)
 {
@@ -505,6 +541,12 @@ bin_objfmt_directive(yasm_objfmt *objfmt, const char *name,
     if (yasm__strcasecmp(name, "org") == 0) {
 	/*@null@*/ yasm_expr *start = NULL;
 
+	if (!valparams) {
+	    yasm_error_set(YASM_ERROR_SYNTAX, N_("[%s] requires an argument"),
+			   "ORG");
+	    return 0;
+	}
+
 	/* ORG takes just a simple integer as param */
 	vp = yasm_vps_first(valparams);
 	if (vp->val)
@@ -517,7 +559,8 @@ bin_objfmt_directive(yasm_objfmt *objfmt, const char *name,
 	}
 
 	if (!start) {
-	    yasm__error(line, N_("argument to ORG must be expression"));
+	    yasm_error_set(YASM_ERROR_SYNTAX,
+			   N_("argument to ORG must be expression"));
 	    return 0;
 	}
 
