@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: dbg-objfmt.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: dbg-objfmt.c 1795 2007-02-24 22:39:55Z peter $");
 
 #define YASM_LIB_INTERNAL
 #include <libyasm.h>
@@ -61,11 +61,12 @@ dbg_objfmt_create(yasm_object *object, yasm_arch *a)
 }
 
 static void
-dbg_objfmt_output(yasm_objfmt *objfmt, FILE *f, int all_syms, yasm_dbgfmt *df)
+dbg_objfmt_output(yasm_objfmt *objfmt, FILE *f, int all_syms, yasm_dbgfmt *df,
+		  yasm_errwarns *errwarns)
 {
     yasm_objfmt_dbg *objfmt_dbg = (yasm_objfmt_dbg *)objfmt;
     char buf[1024];
-    unsigned int i;
+    size_t i;
 
     /* Copy temp file to real output file */
     rewind(objfmt_dbg->dbgfile);
@@ -210,7 +211,7 @@ dbg_objfmt_common_declare(yasm_objfmt *objfmt, const char *name,
 
 static int
 dbg_objfmt_directive(yasm_objfmt *objfmt, const char *name,
-		     yasm_valparamhead *valparams,
+		     /*@null@*/ yasm_valparamhead *valparams,
 		     /*@null@*/ yasm_valparamhead *objext_valparams,
 		     unsigned long line)
 {
