@@ -26,7 +26,7 @@
  */
 
 #include <util.h>
-/*@unused@*/ RCSID("$Id: elf.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: elf.c 1795 2007-02-24 22:39:55Z peter $");
 
 #define YASM_LIB_INTERNAL
 #define YASM_EXPR_INTERNAL
@@ -176,7 +176,7 @@ elf_strtab_entry_set_str(elf_strtab_entry *entry, const char *str)
     last = entry;
     entry = STAILQ_NEXT(last, qlink);
     while (entry) {
-	entry->index = last->index + strlen(last->str) + 1;
+	entry->index = last->index + (unsigned long)strlen(last->str) + 1;
 	last = entry;
 	entry = STAILQ_NEXT(last, qlink);
     }
@@ -209,7 +209,7 @@ elf_strtab_append_str(elf_strtab_head *strtab, const char *str)
     last = STAILQ_LAST(strtab, elf_strtab_entry, qlink);
 
     entry = elf_strtab_entry_create(str);
-    entry->index = last->index + strlen(last->str) + 1;
+    entry->index = last->index + (unsigned long)strlen(last->str) + 1;
 
     STAILQ_INSERT_TAIL(strtab, entry, qlink);
     return entry;
@@ -248,7 +248,7 @@ elf_strtab_output_to_file(FILE *f, elf_strtab_head *strtab)
     STAILQ_FOREACH(entry, strtab, qlink) {
 	size_t len = 1 + strlen(entry->str);
 	fwrite(entry->str, len, 1, f);
-	size += len;
+	size += (unsigned long)len;
     }
     return size;
 }
@@ -422,7 +422,8 @@ elf_symtab_assign_indices(elf_symtab_head *symtab)
 }
 
 unsigned long
-elf_symtab_write_to_file(FILE *f, elf_symtab_head *symtab)
+elf_symtab_write_to_file(FILE *f, elf_symtab_head *symtab,
+			 yasm_errwarns *errwarns)
 {
     unsigned char buf[SYMTAB_MAXSIZE], *bufp;
     elf_symtab_entry *entry, *prev;
@@ -440,10 +441,12 @@ elf_symtab_write_to_file(FILE *f, elf_symtab_head *symtab)
 	/* get size (if specified); expr overrides stored integer */
 	if (entry->xsize) {
 	    size_intn = yasm_intnum_copy(
-		yasm_expr_get_intnum(&entry->xsize, yasm_common_calc_bc_dist));
-	    if (!size_intn)
-		yasm__error(entry->xsize->line,
-		    N_("size specifier not an integer expression"));
+		yasm_expr_get_intnum(&entry->xsize, 1));
+	    if (!size_intn) {
+		yasm_error_set(YASM_ERROR_VALUE,
+			       N_("size specifier not an integer expression"));
+		yasm_errwarn_propagate(errwarns, entry->xsize->line);
+	    }
 	}
 	else
 	    size_intn = yasm_intnum_create_uint(entry->size);
@@ -456,15 +459,14 @@ elf_symtab_write_to_file(FILE *f, elf_symtab_head *symtab)
 	    if (equ_expr_c != NULL) {
 		const yasm_intnum *equ_intn;
 		yasm_expr *equ_expr = yasm_expr_copy(equ_expr_c);
-		equ_intn = yasm_expr_get_intnum(&equ_expr,
-						yasm_common_calc_bc_dist);
+		equ_intn = yasm_expr_get_intnum(&equ_expr, 1);
 
 		if (equ_intn == NULL) {
-		    yasm__error(equ_expr->line,
-				N_("EQU value not an integer expression"));
-		}
-
-		value_intn = yasm_intnum_copy(equ_intn);
+		    yasm_error_set(YASM_ERROR_VALUE,
+				   N_("EQU value not an integer expression"));
+		    yasm_errwarn_propagate(errwarns, equ_expr->line);
+		} else
+		    value_intn = yasm_intnum_copy(equ_intn);
 		entry->index = SHN_ABS;
 		yasm_expr_destroy(equ_expr);
 	    }
@@ -658,7 +660,7 @@ elf_secthead_name_reloc_section(const char *basesect)
     }
     else
     {
-        int prepend_length = strlen(elf_march->reloc_section_prefix);
+        size_t prepend_length = strlen(elf_march->reloc_section_prefix);
         char *sectname = yasm_xmalloc(prepend_length + strlen(basesect) + 1);
         strcpy(sectname, elf_march->reloc_section_prefix);
         strcat(sectname, basesect);
@@ -700,7 +702,7 @@ elf_secthead_write_rel_to_file(FILE *f, elf_section_index symtab_idx,
 
 unsigned long
 elf_secthead_write_relocs_to_file(FILE *f, yasm_section *sect,
-				  elf_secthead *shead)
+				  elf_secthead *shead, yasm_errwarns *errwarns)
 {
     elf_reloc_entry *reloc;
     unsigned char buf[RELOC_MAXSIZE], *bufp;
@@ -716,11 +718,16 @@ elf_secthead_write_relocs_to_file(FILE *f, yasm_section *sect,
 
     /* first align section to multiple of 4 */
     pos = ftell(f);
-    if (pos == -1)
-	yasm__error(0, N_("couldn't read position on output stream"));
+    if (pos == -1) {
+	yasm_error_set(YASM_ERROR_IO,
+		       N_("couldn't read position on output stream"));
+	yasm_errwarn_propagate(errwarns, 0);
+    }
     pos = (pos + 3) & ~3;
-    if (fseek(f, pos, SEEK_SET) < 0)
-	yasm__error(0, N_("couldn't seek on output stream"));
+    if (fseek(f, pos, SEEK_SET) < 0) {
+	yasm_error_set(YASM_ERROR_IO, N_("couldn't seek on output stream"));
+	yasm_errwarn_propagate(errwarns, 0);
+    }
     shead->rel_offset = (unsigned long)pos;
 
 
@@ -843,7 +850,7 @@ void
 elf_secthead_add_size(elf_secthead *shead, yasm_intnum *size)
 {
     if (size) {
-	yasm_intnum_calc(shead->size, YASM_EXPR_ADD, size, 0);
+	yasm_intnum_calc(shead->size, YASM_EXPR_ADD, size);
     }
 }
 
