@@ -1,4 +1,4 @@
-/* $Id: elf.h 1591 2006-07-13 03:25:38Z peter $
+/* $Id: elf.h 1521 2006-05-04 07:55:32Z peter $
  * ELF object format helpers
  *
  *  Copyright (C) 2003  Michael Urman
@@ -436,7 +436,8 @@ void elf_symtab_insert_local_sym(elf_symtab_head *symtab,
 				 elf_symtab_entry *entry);
 void elf_symtab_destroy(elf_symtab_head *head);
 unsigned long elf_symtab_assign_indices(elf_symtab_head *symtab);
-unsigned long elf_symtab_write_to_file(FILE *f, elf_symtab_head *symtab);
+unsigned long elf_symtab_write_to_file(FILE *f, elf_symtab_head *symtab,
+				       yasm_errwarns *errwarns);
 void elf_symtab_set_nonzero(elf_symtab_entry	*entry,
 			    struct yasm_section *sect,
 			    elf_section_index	 sectidx,
@@ -490,7 +491,8 @@ unsigned long elf_secthead_write_rel_to_file(FILE *f, elf_section_index symtab,
 					     elf_secthead *esd,
 					     elf_section_index sindex);
 unsigned long elf_secthead_write_relocs_to_file(FILE *f, yasm_section *sect,
-						elf_secthead *shead);
+						elf_secthead *shead,
+						yasm_errwarns *errwarns);
 long elf_secthead_set_file_offset(elf_secthead *shead, long pos);
 
 /* program header function */
