@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: xdf-objfmt.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: xdf-objfmt.c 1795 2007-02-24 22:39:55Z peter $");
 
 #define YASM_LIB_INTERNAL
 #define YASM_BC_INTERNAL
@@ -41,8 +41,6 @@
 #define XDF_SYM_GLOBAL	2
 #define XDF_SYM_EQU	4
 
-typedef STAILQ_HEAD(xdf_reloc_head, xdf_reloc) xdf_reloc_head;
-
 typedef struct xdf_reloc {
     yasm_reloc reloc;
     /*@null@*/ yasm_symrec *base;   /* base symbol (for WRT) */
@@ -79,7 +77,6 @@ typedef struct xdf_section_data {
     unsigned long size;	    /* size of raw data (section data) in bytes */
     unsigned long relptr;   /* file ptr to relocation */
     unsigned long nreloc;   /* number of relocation entries >64k -> error */
-    /*@owned@*/ xdf_reloc_head relocs;
 } xdf_section_data;
 
 typedef struct xdf_symrec_data {
@@ -98,6 +95,7 @@ typedef struct yasm_objfmt_xdf {
 
 typedef struct xdf_objfmt_output_info {
     yasm_objfmt_xdf *objfmt_xdf;
+    yasm_errwarns *errwarns;
     /*@dependent@*/ FILE *f;
     /*@only@*/ unsigned char *buf;
     yasm_section *sect;
@@ -157,8 +155,8 @@ xdf_objfmt_create(yasm_object *object, yasm_arch *a)
 }
 
 static int
-xdf_objfmt_output_value(yasm_value *value, unsigned char *buf, size_t destsize,
-			size_t valsize, int shift, unsigned long offset,
+xdf_objfmt_output_value(yasm_value *value, unsigned char *buf,
+			unsigned int destsize, unsigned long offset,
 			yasm_bytecode *bc, int warn, /*@null@*/ void *d)
 {
     /*@null@*/ xdf_objfmt_output_info *info = (xdf_objfmt_output_info *)d;
@@ -166,20 +164,20 @@ xdf_objfmt_output_value(yasm_value *value, unsigned char *buf, size_t destsize,
     /*@dependent@*/ /*@null@*/ yasm_intnum *intn;
     unsigned long intn_minus;
     int retval;
+    unsigned int valsize = value->size;
 
     assert(info != NULL);
     objfmt_xdf = info->objfmt_xdf;
 
     if (value->abs)
-	value->abs = yasm_expr_simplify(value->abs, yasm_common_calc_bc_dist);
+	value->abs = yasm_expr_simplify(value->abs, 1);
 
     /* Try to output constant and PC-relative section-local first.
      * Note this does NOT output any value with a SEG, WRT, external,
      * cross-section, or non-PC-relative reference (those are handled below).
      */
-    switch (yasm_value_output_basic(value, buf, destsize, valsize, shift, bc,
-				    warn, info->objfmt_xdf->arch,
-				    yasm_common_calc_bc_dist)) {
+    switch (yasm_value_output_basic(value, buf, destsize, bc, warn,
+				    info->objfmt_xdf->arch)) {
 	case -1:
 	    return 1;
 	case 0:
@@ -189,7 +187,8 @@ xdf_objfmt_output_value(yasm_value *value, unsigned char *buf, size_t destsize,
     }
 
     if (value->section_rel) {
-	yasm__error(bc->line, N_("xdf: relocation too complex"));
+	yasm_error_set(YASM_ERROR_TOO_COMPLEX,
+		       N_("xdf: relocation too complex"));
 	return 1;
     }
 
@@ -223,22 +222,23 @@ xdf_objfmt_output_value(yasm_value *value, unsigned char *buf, size_t destsize,
 
     if (intn_minus > 0) {
 	intn = yasm_intnum_create_uint(intn_minus);
-	yasm_intnum_calc(intn, YASM_EXPR_NEG, NULL, bc->line);
+	yasm_intnum_calc(intn, YASM_EXPR_NEG, NULL);
     } else
 	intn = yasm_intnum_create_uint(0);
 
     if (value->abs) {
-	yasm_intnum *intn2 = yasm_expr_get_intnum(&value->abs, NULL);
+	yasm_intnum *intn2 = yasm_expr_get_intnum(&value->abs, 0);
 	if (!intn2) {
-	    yasm__error(bc->line, N_("xdf: relocation too complex"));
+	    yasm_error_set(YASM_ERROR_TOO_COMPLEX,
+			   N_("xdf: relocation too complex"));
 	    yasm_intnum_destroy(intn);
 	    return 1;
 	}
-	yasm_intnum_calc(intn, YASM_EXPR_ADD, intn2, bc->line);
+	yasm_intnum_calc(intn, YASM_EXPR_ADD, intn2);
     }
 
     retval = yasm_arch_intnum_tobytes(objfmt_xdf->arch, intn, buf, destsize,
-				      valsize, shift, bc, warn, bc->line);
+				      valsize, 0, bc, warn);
     yasm_intnum_destroy(intn);
     return retval;
 }
@@ -268,8 +268,8 @@ xdf_objfmt_output_bytecode(yasm_bytecode *bc, /*@null@*/ void *d)
     /* Warn that gaps are converted to 0 and write out the 0's. */
     if (gap) {
 	unsigned long left;
-	yasm__warning(YASM_WARN_UNINIT_CONTENTS, bc->line,
-	    N_("uninitialized space: zeroing"));
+	yasm_warn_set(YASM_WARN_UNINIT_CONTENTS,
+		      N_("uninitialized space: zeroing"));
 	/* Write out in chunks */
 	memset(info->buf, 0, REGULAR_OUTBUF_SIZE);
 	left = size;
@@ -297,7 +297,6 @@ xdf_objfmt_output_section(yasm_section *sect, /*@null@*/ void *d)
     /*@dependent@*/ /*@null@*/ xdf_section_data *xsd;
     long pos;
     xdf_reloc *reloc;
-    yasm_bytecode *last;
 
     /* FIXME: Don't output absolute sections into the section table */
     if (yasm_section_is_absolute(sect))
@@ -307,13 +306,12 @@ xdf_objfmt_output_section(yasm_section *sect, /*@null@*/ void *d)
     xsd = yasm_section_get_data(sect, &xdf_section_data_cb);
     assert(xsd != NULL);
 
-    last = yasm_section_bcs_last(sect);
     if (xsd->flags & XDF_SECT_BSS) {
 	/* Don't output BSS sections.
 	 * TODO: Check for non-reserve bytecodes?
 	 */
 	pos = 0;    /* position = 0 because it's not in the file */
-	xsd->size = last->offset + last->len;
+	xsd->size = yasm_bc_next_offset(yasm_section_bcs_last(sect));
     } else {
 	pos = ftell(info->f);
 	if (pos == -1) {
@@ -324,10 +322,11 @@ xdf_objfmt_output_section(yasm_section *sect, /*@null@*/ void *d)
 
 	info->sect = sect;
 	info->xsd = xsd;
-	yasm_section_bcs_traverse(sect, info, xdf_objfmt_output_bytecode);
+	yasm_section_bcs_traverse(sect, info->errwarns, info,
+				  xdf_objfmt_output_bytecode);
 
 	/* Sanity check final section size */
-	if (xsd->size != (last->offset + last->len))
+	if (xsd->size != yasm_bc_next_offset(yasm_section_bcs_last(sect)))
 	    yasm_internal_error(
 		N_("xdf: section computed size did not match actual size"));
     }
@@ -360,7 +359,7 @@ xdf_objfmt_output_section(yasm_section *sect, /*@null@*/ void *d)
 	    yasm_internal_error(
 		N_("xdf: no symbol data for relocated symbol"));
 
-	yasm_intnum_get_sized(reloc->reloc.addr, localbuf, 4, 32, 0, 0, 0, 0);
+	yasm_intnum_get_sized(reloc->reloc.addr, localbuf, 4, 32, 0, 0, 0);
 	localbuf += 4;				/* address of relocation */
 	YASM_WRITE_32_L(localbuf, xsymd->index);    /* relocated symbol */
 	if (reloc->base) {
@@ -411,17 +410,17 @@ xdf_objfmt_output_secthead(yasm_section *sect, /*@null@*/ void *d)
 
     YASM_WRITE_32_L(localbuf, xsymd->index);	/* section name symbol */
     if (xsd->addr) {
-	yasm_intnum_get_sized(xsd->addr, localbuf, 8, 64, 0, 0, 0, 0);
+	yasm_intnum_get_sized(xsd->addr, localbuf, 8, 64, 0, 0, 0);
 	localbuf += 8;				/* physical address */
     } else {
 	YASM_WRITE_32_L(localbuf, 0);
 	YASM_WRITE_32_L(localbuf, 0);
     }
     if (xsd->vaddr) {
-	yasm_intnum_get_sized(xsd->vaddr, localbuf, 8, 64, 0, 0, 0, 0);
+	yasm_intnum_get_sized(xsd->vaddr, localbuf, 8, 64, 0, 0, 0);
 	localbuf += 8;				/* virtual address */
     } else if (xsd->addr) {
-	yasm_intnum_get_sized(xsd->addr, localbuf, 8, 64, 0, 0, 0, 0);
+	yasm_intnum_get_sized(xsd->addr, localbuf, 8, 64, 0, 0, 0);
 	localbuf += 8;				/* VA=PA */
     } else {
 	YASM_WRITE_32_L(localbuf, 0);
@@ -495,12 +494,12 @@ xdf_objfmt_output_sym(yasm_symrec *sym, /*@null@*/ void *d)
 		    yasm_expr *abs_start;
 
 		    abs_start = yasm_expr_copy(yasm_section_get_start(sect));
-		    intn = yasm_expr_get_intnum(&abs_start,
-						yasm_common_calc_bc_dist);
-		    if (!intn)
-			yasm__error(abs_start->line,
+		    intn = yasm_expr_get_intnum(&abs_start, 1);
+		    if (!intn) {
+			yasm_error_set(YASM_ERROR_NOT_CONSTANT,
 			    N_("absolute section start not an integer expression"));
-		    else
+			yasm_errwarn_propagate(info->errwarns, abs_start->line);
+		    } else
 			value = yasm_intnum_get_uint(intn);
 		    yasm_expr_destroy(abs_start);
 
@@ -509,16 +508,17 @@ xdf_objfmt_output_sym(yasm_symrec *sym, /*@null@*/ void *d)
 		} else
 		    yasm_internal_error(N_("didn't understand section"));
 		if (precbc)
-		    value += precbc->offset + precbc->len;
+		    value += yasm_bc_next_offset(precbc);
 	    }
 	} else if ((equ_val = yasm_symrec_get_equ(sym))) {
 	    yasm_expr *equ_val_copy = yasm_expr_copy(equ_val);
-	    intn = yasm_expr_get_intnum(&equ_val_copy,
-					yasm_common_calc_bc_dist);
+	    intn = yasm_expr_get_intnum(&equ_val_copy, 1);
 	    if (!intn) {
-		if (vis & YASM_SYM_GLOBAL)
-		    yasm__error(equ_val->line,
+		if (vis & YASM_SYM_GLOBAL) {
+		    yasm_error_set(YASM_ERROR_NOT_CONSTANT,
 			N_("global EQU value not an integer expression"));
+		    yasm_errwarn_propagate(info->errwarns, equ_val->line);
+		}
 	    } else
 		value = yasm_intnum_get_uint(intn);
 	    yasm_expr_destroy(equ_val_copy);
@@ -536,7 +536,7 @@ xdf_objfmt_output_sym(yasm_symrec *sym, /*@null@*/ void *d)
 	YASM_WRITE_32_L(localbuf, scnum);	/* section number */
 	YASM_WRITE_32_L(localbuf, value);	/* value */
 	YASM_WRITE_32_L(localbuf, info->strtab_offset);
-	info->strtab_offset += len+1;
+	info->strtab_offset += (unsigned long)(len+1);
 	YASM_WRITE_32_L(localbuf, flags);	/* flags */
 	fwrite(info->buf, 16, 1, info->f);
     }
@@ -561,7 +561,7 @@ xdf_objfmt_output_str(yasm_symrec *sym, /*@null@*/ void *d)
 
 static void
 xdf_objfmt_output(yasm_objfmt *objfmt, FILE *f, int all_syms,
-		  /*@unused@*/ yasm_dbgfmt *df)
+		  /*@unused@*/ yasm_dbgfmt *df, yasm_errwarns *errwarns)
 {
     yasm_objfmt_xdf *objfmt_xdf = (yasm_objfmt_xdf *)objfmt;
     xdf_objfmt_output_info info;
@@ -569,6 +569,7 @@ xdf_objfmt_output(yasm_objfmt *objfmt, FILE *f, int all_syms,
     unsigned long symtab_count = 0;
 
     info.objfmt_xdf = objfmt_xdf;
+    info.errwarns = errwarns;
     info.f = f;
     info.buf = yasm_xmalloc(REGULAR_OUTBUF_SIZE);
 
@@ -642,7 +643,6 @@ xdf_objfmt_init_new_section(yasm_objfmt_xdf *objfmt_xdf, yasm_section *sect,
     data->size = 0;
     data->relptr = 0;
     data->nreloc = 0;
-    STAILQ_INIT(&data->relocs);
     yasm_section_add_data(sect, &xdf_section_data_cb, data);
 
     sym = yasm_symtab_define_label(objfmt_xdf->symtab, sectname,
@@ -694,7 +694,7 @@ xdf_objfmt_section_switch(yasm_objfmt *objfmt, yasm_valparamhead *valparams,
 
     while ((vp = yasm_vps_next(vp))) {
 	if (!vp->val) {
-	    yasm__warning(YASM_WARN_GENERAL, line,
+	    yasm_warn_set(YASM_WARN_GENERAL,
 			  N_("Unrecognized numeric qualifier"));
 	    continue;
 	}
@@ -718,46 +718,50 @@ xdf_objfmt_section_switch(yasm_objfmt *objfmt, yasm_valparamhead *valparams,
 	    flags |= XDF_SECT_FLAT;
 	} else if (yasm__strcasecmp(vp->val, "absolute") == 0 && vp->param) {
 	    flags |= XDF_SECT_ABSOLUTE;
-	    absaddr = yasm_expr_get_intnum(&vp->param, NULL);
+	    absaddr = yasm_expr_get_intnum(&vp->param, 0);
 	    if (!absaddr) {
-		yasm__error(line, N_("argument to `%s' is not an integer"),
-			    vp->val);
+		yasm_error_set(YASM_ERROR_NOT_CONSTANT,
+			       N_("argument to `%s' is not an integer"),
+			       vp->val);
 		return NULL;
 	    }
 	} else if (yasm__strcasecmp(vp->val, "virtual") == 0 && vp->param) {
-	    vaddr = yasm_expr_get_intnum(&vp->param, NULL);
+	    vaddr = yasm_expr_get_intnum(&vp->param, 0);
 	    if (!vaddr) {
-		yasm__error(line, N_("argument to `%s' is not an integer"),
-			    vp->val);
+		yasm_error_set(YASM_ERROR_NOT_CONSTANT,
+			       N_("argument to `%s' is not an integer"),
+			       vp->val);
 		return NULL;
 	    }
 	} else if (yasm__strcasecmp(vp->val, "align") == 0 && vp->param) {
 	    /*@dependent@*/ /*@null@*/ const yasm_intnum *align_expr;
 
-	    align_expr = yasm_expr_get_intnum(&vp->param, NULL);
+	    align_expr = yasm_expr_get_intnum(&vp->param, 0);
 	    if (!align_expr) {
-		yasm__error(line, N_("argument to `%s' is not a power of two"),
-			    vp->val);
+		yasm_error_set(YASM_ERROR_VALUE,
+			       N_("argument to `%s' is not an integer"),
+			       vp->val);
 		return NULL;
 	    }
 	    align = yasm_intnum_get_uint(align_expr);
 
             /* Alignments must be a power of two. */
-            if ((align & (align - 1)) != 0) {
-		yasm__error(line, N_("argument to `%s' is not a power of two"),
-			    vp->val);
+            if (!is_exp2(align)) {
+		yasm_error_set(YASM_ERROR_VALUE,
+			       N_("argument to `%s' is not a power of two"),
+			       vp->val);
 		return NULL;
 	    }
 
 	    /* Check to see if alignment is supported size */
 	    if (align > 4096) {
-		yasm__error(line,
-			    N_("XDF does not support alignments > 4096"));
+		yasm_error_set(YASM_ERROR_VALUE,
+			       N_("XDF does not support alignments > 4096"));
 		return NULL;
 	    }
 	} else
-	    yasm__warning(YASM_WARN_GENERAL, line,
-			  N_("Unrecognized qualifier `%s'"), vp->val);
+	    yasm_warn_set(YASM_WARN_GENERAL, N_("Unrecognized qualifier `%s'"),
+			  vp->val);
     }
 
     retval = yasm_object_get_general(objfmt_xdf->object, sectname, 0, align, 1,
@@ -783,7 +787,7 @@ xdf_objfmt_section_switch(yasm_objfmt *objfmt, yasm_valparamhead *valparams,
 	}
 	yasm_section_set_align(retval, align, line);
     } else if (flags_override)
-	yasm__warning(YASM_WARN_GENERAL, line,
+	yasm_warn_set(YASM_WARN_GENERAL,
 		      N_("section flags ignored on section redeclaration"));
     return retval;
 }
@@ -816,7 +820,6 @@ xdf_section_data_print(void *data, FILE *f, int indent_level)
     fprintf(f, "%*ssize=%ld\n", indent_level, "", xsd->size);
     fprintf(f, "%*srelptr=0x%lx\n", indent_level, "", xsd->relptr);
     fprintf(f, "%*snreloc=%ld\n", indent_level, "", xsd->nreloc);
-    fprintf(f, "%*srelocs:\n", indent_level, "");
 }
 
 static yasm_symrec *
@@ -850,7 +853,7 @@ xdf_objfmt_common_declare(yasm_objfmt *objfmt, const char *name,
     yasm_objfmt_xdf *objfmt_xdf = (yasm_objfmt_xdf *)objfmt;
 
     yasm_expr_destroy(size);
-    yasm__error(line,
+    yasm_error_set(YASM_ERROR_GENERAL,
 	N_("XDF object format does not support common variables"));
 
     return yasm_symtab_declare(objfmt_xdf->symtab, name, YASM_SYM_COMMON,
@@ -874,7 +877,7 @@ xdf_symrec_data_print(void *data, FILE *f, int indent_level)
 static int
 xdf_objfmt_directive(/*@unused@*/ yasm_objfmt *objfmt,
 		      /*@unused@*/ const char *name,
-		      /*@unused@*/ yasm_valparamhead *valparams,
+		      /*@unused@*/ /*@null@*/ yasm_valparamhead *valparams,
 		      /*@unused@*/ /*@null@*/
 		      yasm_valparamhead *objext_valparams,
 		      /*@unused@*/ unsigned long line)
