@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: nasm-parser.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: nasm-parser.c 1716 2006-12-23 10:20:51Z peter $");
 
 #define YASM_LIB_INTERNAL
 #include <libyasm.h>
@@ -37,7 +37,7 @@ static void
 nasm_parser_do_parse(yasm_object *object, yasm_preproc *pp, yasm_arch *a,
 		     yasm_objfmt *of, yasm_dbgfmt *df, FILE *f,
 		     const char *in_filename, int save_input,
-		     yasm_section *def_sect)
+		     yasm_section *def_sect, yasm_errwarns *errwarns)
 {
     yasm_parser_nasm parser_nasm;
 
@@ -54,6 +54,7 @@ nasm_parser_do_parse(yasm_object *object, yasm_preproc *pp, yasm_arch *a,
     parser_nasm.arch = a;
     parser_nasm.objfmt = of;
     parser_nasm.dbgfmt = df;
+    parser_nasm.errwarns = errwarns;
 
     parser_nasm.cur_section = def_sect;
     parser_nasm.prev_bc = yasm_section_bcs_first(def_sect);
@@ -61,18 +62,10 @@ nasm_parser_do_parse(yasm_object *object, yasm_preproc *pp, yasm_arch *a,
     parser_nasm.save_input = save_input;
     parser_nasm.save_last = 0;
 
+    parser_nasm.peek_token = NONE;
+
     /* initialize scanner structure */
-    parser_nasm.s.bot = NULL;
-    parser_nasm.s.tok = NULL;
-    parser_nasm.s.ptr = NULL;
-    parser_nasm.s.cur = NULL;
-    parser_nasm.s.pos = NULL;
-    parser_nasm.s.lim = NULL;
-    parser_nasm.s.top = NULL;
-    parser_nasm.s.eof = NULL;
-    parser_nasm.s.tchar = 0;
-    parser_nasm.s.tline = 0;
-    parser_nasm.s.cline = 1;
+    yasm_scanner_initialize(&parser_nasm.s);
 
     parser_nasm.state = INITIAL;
 
@@ -81,7 +74,7 @@ nasm_parser_do_parse(yasm_object *object, yasm_preproc *pp, yasm_arch *a,
 
     nasm_parser_parse(&parser_nasm);
 
-    nasm_parser_cleanup(&parser_nasm);
+    yasm_scanner_delete(&parser_nasm.s);
 
     /* Free locallabel base if necessary */
     if (parser_nasm.locallabel_base)
