@@ -1,4 +1,4 @@
-/* $Id: nasm-parser.h 1591 2006-07-13 03:25:38Z peter $
+/* $Id: nasm-parser.h 1774 2007-02-18 20:42:55Z peter $
  * NASM-compatible parser header file
  *
  *  Copyright (C) 2002  Peter Johnson
@@ -27,16 +27,56 @@
 #ifndef YASM_NASM_PARSER_H
 #define YASM_NASM_PARSER_H
 
-#include "nasm-bison.h"
-
 #define YYCTYPE		unsigned char
-typedef struct Scanner {
-    YYCTYPE		*bot, *tok, *ptr, *cur, *pos, *lim, *top, *eof;
-    unsigned int	tchar, tline, cline;
-} Scanner;
 
 #define MAX_SAVED_LINE_LEN  80
 
+enum tokentype {
+    INTNUM = 258,
+    FLTNUM,
+    DIRECTIVE_NAME,
+    FILENAME,
+    STRING,
+    SIZE_OVERRIDE,
+    DECLARE_DATA,
+    RESERVE_SPACE,
+    INCBIN,
+    EQU,
+    TIMES,
+    SEG,
+    WRT,
+    NOSPLIT,
+    STRICT,
+    INSN,
+    PREFIX,
+    REG,
+    SEGREG,
+    TARGETMOD,
+    LEFT_OP,
+    RIGHT_OP,
+    SIGNDIV,
+    SIGNMOD,
+    START_SECTION_ID,
+    ID,
+    LOCAL_ID,
+    SPECIAL_ID,
+    LINE,
+    NONE		/* special token for lookahead */
+};
+
+typedef union {
+    unsigned int int_info;
+    char *str_val;
+    yasm_intnum *intn;
+    yasm_floatnum *flt;
+    uintptr_t arch_data[4];
+    struct {
+	char *contents;
+	size_t len;
+    } str;
+} yystype;
+#define YYSTYPE yystype
+
 typedef struct yasm_parser_nasm {
     FILE *in;
     int debug;
@@ -52,30 +92,57 @@ typedef struct yasm_parser_nasm {
     /*@dependent@*/ yasm_arch *arch;
     /*@dependent@*/ yasm_objfmt *objfmt;
     /*@dependent@*/ yasm_dbgfmt *dbgfmt;
+    /*@dependent@*/ yasm_errwarns *errwarns;
 
     /*@dependent@*/ yasm_linemap *linemap;
     /*@dependent@*/ yasm_symtab *symtab;
 
     /*@null@*/ yasm_bytecode *prev_bc;
-    yasm_bytecode *temp_bc;
 
     int save_input;
     YYCTYPE save_line[2][MAX_SAVED_LINE_LEN];
     int save_last;
 
-    Scanner s;
+    yasm_scanner s;
     enum {
 	INITIAL,
 	DIRECTIVE,
+	SECTION_DIRECTIVE,
 	DIRECTIVE2,
 	LINECHG,
 	LINECHG2,
 	INSTRUCTION
     } state;
+
+    int token;		/* enum tokentype or any character */
+    yystype tokval;
+    char tokch;		/* first character of token */
+
+    /* one token of lookahead; used sparingly */
+    int peek_token;	/* NONE if none */
+    yystype peek_tokval;
+    char peek_tokch;
 } yasm_parser_nasm;
 
 /* shorter access names to commonly used parser_nasm fields */
 #define p_symtab	(parser_nasm->symtab)
+#define curtok		(parser_nasm->token)
+#define curval		(parser_nasm->tokval)
+
+#define INTNUM_val		(curval.intn)
+#define FLTNUM_val		(curval.flt)
+#define DIRECTIVE_NAME_val	(curval.str_val)
+#define FILENAME_val		(curval.str_val)
+#define STRING_val		(curval.str)
+#define SIZE_OVERRIDE_val	(curval.int_info)
+#define DECLARE_DATA_val	(curval.int_info)
+#define RESERVE_SPACE_val	(curval.int_info)
+#define INSN_val		(curval.arch_data)
+#define PREFIX_val		(curval.arch_data)
+#define REG_val			(curval.arch_data)
+#define SEGREG_val		(curval.arch_data)
+#define TARGETMOD_val		(curval.arch_data)
+#define ID_val			(curval.str_val)
 
 #define cur_line	(yasm_linemap_get_current(parser_nasm->linemap))
 
@@ -83,7 +150,7 @@ typedef struct yasm_parser_nasm {
 #define p_expr_new_branch(o,r)	yasm_expr_create_branch(o,r,cur_line)
 #define p_expr_new_ident(r)	yasm_expr_create_ident(r,cur_line)
 
-int nasm_parser_parse(void *parser_nasm_arg);
+void nasm_parser_parse(yasm_parser_nasm *parser_nasm);
 void nasm_parser_cleanup(yasm_parser_nasm *parser_nasm);
 int nasm_parser_lex(YYSTYPE *lvalp, yasm_parser_nasm *parser_nasm);
 
