@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: nasm-preproc.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: nasm-preproc.c 1669 2006-10-28 21:31:38Z peter $");
 
 #define YASM_LIB_INTERNAL
 #include <libyasm.h>
@@ -46,6 +46,7 @@ typedef struct yasm_preproc_nasm {
     int lineinc;
 } yasm_preproc_nasm;
 static yasm_linemap *cur_lm;
+static yasm_errwarns *cur_errwarns;
 int tasm_compatible_mode = 0;
 
 typedef struct preproc_dep {
@@ -106,11 +107,10 @@ nasm_efunc(int severity, const char *fmt, ...)
     va_start(va, fmt);
     switch (severity & ERR_MASK) {
 	case ERR_WARNING:
-	    yasm__warning_va(YASM_WARN_PREPROC,
-			     yasm_linemap_get_current(cur_lm), fmt, va);
+	    yasm_warn_set_va(YASM_WARN_PREPROC, fmt, va);
 	    break;
 	case ERR_NONFATAL:
-	    yasm__error_va(yasm_linemap_get_current(cur_lm), fmt, va);
+	    yasm_error_set_va(YASM_ERROR_GENERAL, fmt, va);
 	    break;
 	case ERR_FATAL:
 	    yasm_fatal(fmt, va);
@@ -123,10 +123,14 @@ nasm_efunc(int severity, const char *fmt, ...)
 	    break;
     }
     va_end(va);
+    yasm_errwarn_propagate(cur_errwarns,
+	yasm_linemap_poke(cur_lm, nasm_src_get_fname(),
+			  (unsigned long)nasm_src_get_linnum()));
 }
 
 static yasm_preproc *
-nasm_preproc_create(FILE *f, const char *in_filename, yasm_linemap *lm)
+nasm_preproc_create(FILE *f, const char *in_filename, yasm_linemap *lm,
+		    yasm_errwarns *errwarns)
 {
     yasm_preproc_nasm *preproc_nasm = yasm_xmalloc(sizeof(yasm_preproc_nasm));
 
@@ -134,6 +138,7 @@ nasm_preproc_create(FILE *f, const char *in_filename, yasm_linemap *lm)
 
     preproc_nasm->in = f;
     cur_lm = lm;
+    cur_errwarns = errwarns;
     preproc_deps = NULL;
     done_dep_preproc = 0;
     preproc_nasm->line = NULL;
@@ -148,8 +153,10 @@ nasm_preproc_create(FILE *f, const char *in_filename, yasm_linemap *lm)
 static void
 nasm_preproc_destroy(yasm_preproc *preproc)
 {
+    yasm_preproc_nasm *preproc_nasm = (yasm_preproc_nasm *)preproc;
     nasmpp.cleanup(0);
-    nasm_eval_cleanup();
+    if (preproc_nasm->file_name)
+	yasm_xfree(preproc_nasm->file_name);
     yasm_xfree(preproc);
     if (preproc_deps)
 	yasm_xfree(preproc_deps);
@@ -188,10 +195,6 @@ nasm_preproc_input(yasm_preproc *preproc, char *buf, size_t max_size)
 	    }
 	    preproc_nasm->prior_linnum = linnum;
 	}
-	if (preproc_nasm->file_name) {
-	    yasm_xfree(preproc_nasm->file_name);
-	    preproc_nasm->file_name = NULL;
-	}
     }
 
     n = preproc_nasm->lineleft<max_size?preproc_nasm->lineleft:max_size;
@@ -263,12 +266,6 @@ nasm_preproc_get_included_file(yasm_preproc *preproc, /*@out@*/ char *buf,
     }
 }
 
-static void
-nasm_preproc_add_include_path(yasm_preproc *preproc, const char *path)
-{
-    pp_include_path(path);
-}
-
 static void
 nasm_preproc_add_include_file(yasm_preproc *preproc, const char *filename)
 {
@@ -307,7 +304,6 @@ yasm_preproc_module yasm_nasm_LTX_preproc = {
     nasm_preproc_destroy,
     nasm_preproc_input,
     nasm_preproc_get_included_file,
-    nasm_preproc_add_include_path,
     nasm_preproc_add_include_file,
     nasm_preproc_predefine_macro,
     nasm_preproc_undefine_macro,
