@@ -1,5 +1,8 @@
 #! /bin/sh
-# $Id: out_test.sh 1591 2006-07-13 03:25:38Z peter $
+# $Id: out_test.sh 1743 2007-01-28 00:40:36Z peter $
+
+YASM_TEST_SUITE=1
+export YASM_TEST_SUITE
 
 case `echo "testing\c"; echo 1,2,3`,`echo -n testing; echo 1,2,3` in
   *c*,-n*) ECHO_N= ECHO_C='
@@ -17,7 +20,7 @@ mkdir results >/dev/null 2>&1
 passedct=0
 failedct=0
 
-echo $ECHO_N "Test $1: "
+echo $ECHO_N "Test $1: $ECHO_C"
 for asm in ${srcdir}/$2/*.asm
 do
     a=`echo ${asm} | sed 's,^.*/,,;s,.asm$,,'`
@@ -26,32 +29,34 @@ do
     og=`echo ${asm} | sed 's,.asm$,.hex,'`
     e=${a}.ew
     eg=`echo ${asm} | sed 's,.asm$,.errwarn,'`
+    if test \! -e ${eg}; then
+	eg=/dev/null
+    fi
 
     # Run within a subshell to prevent signal messages from displaying.
-    sh -c "cat ${asm} | ./yasm $4 -o results/${o} 2>results/${e}" >/dev/null 2>/dev/null
+    sh -c "cat ${asm} | ./yasm $4 -o results/${o} - 2>results/${e}" >/dev/null 2>/dev/null
     status=$?
     if test $status -gt 128; then
 	# We should never get a coredump!
-	echo $ECHO_N "C"
+	echo $ECHO_N "C$ECHO_C"
         eval "failed$failedct='C: ${a} crashed!'"
 	failedct=`expr $failedct + 1`
     elif test $status -gt 0; then
 	echo ${asm} | grep err >/dev/null
        	if test $? -gt 0; then
 	    # YASM detected errors but shouldn't have!
-	    echo $ECHO_N "E"
+	    echo $ECHO_N "E$ECHO_C"
             eval "failed$failedct='E: ${a} returned an error code!'"
 	    failedct=`expr $failedct + 1`
 	else
 	    # We got errors, check to see if they match:
-	    diff -w ${eg} results/${e} > /dev/null
-	    if test $? -eq 0; then
+	    if diff -w ${eg} results/${e} >/dev/null; then
 		# Error/warnings match, it passes!
-		echo $ECHO_N "."
+		echo $ECHO_N ".$ECHO_C"
 		passedct=`expr $passedct + 1`
 	    else
 		# Error/warnings don't match.
-		echo $ECHO_N "W"
+		echo $ECHO_N "W$ECHO_C"
                 eval "failed$failedct='W: ${a} did not match errors and warnings!'"
 		failedct=`expr $failedct + 1`
 	    fi
@@ -59,28 +64,26 @@ do
     else
 	echo ${asm} | grep -v err >/dev/null
        	if test $? -gt 0; then
-	    # YASM detected errors but shouldn't have!
-	    echo $ECHO_N "E"
-            eval "failed$failedct='E: ${a} returned an error code!'"
+	    # YASM didn't detect errors but should have!
+	    echo $ECHO_N "E$ECHO_C"
+            eval "failed$failedct='E: ${a} did not return an error code!'"
 	    failedct=`expr $failedct + 1`
 	else
 	    ./test_hd results/${o} > results/${oh}
-	    diff ${og} results/${oh} > /dev/null
-	    if test $? -eq 0; then
-		diff -w ${eg} results/${e} > /dev/null
-		if test $? -eq 0; then
+	    if diff ${og} results/${oh} >/dev/null; then
+		if diff -w ${eg} results/${e} >/dev/null; then
 		    # Both object file and error/warnings match, it passes!
-		    echo $ECHO_N "."
+		    echo $ECHO_N ".$ECHO_C"
 		    passedct=`expr $passedct + 1`
 		else
 		    # Error/warnings don't match.
-		    echo $ECHO_N "W"
+		    echo $ECHO_N "W$ECHO_C"
                     eval "failed$failedct='W: ${a} did not match errors and warnings!'"
 		    failedct=`expr $failedct + 1`
 		fi
 	    else
 		# Object file doesn't match.
-		echo $ECHO_N "O"
+		echo $ECHO_N "O$ECHO_C"
                 eval "failed$failedct='O: ${a} did not match object file!'"
 		failedct=`expr $failedct + 1`
 	    fi
