@@ -1,4 +1,4 @@
-/* $Id: gap.c 1591 2006-07-13 03:25:38Z peter $
+/* $Id: gap.c 1795 2007-02-24 22:39:55Z peter $
  *
  * Generate Arch Parser (GAP): generates ARCHparse.c from ARCHparse.gap.
  *
@@ -642,7 +642,7 @@ perfect_dir(FILE *out, const char *which, const char *parser, dir_list *dirs)
 	key *k = yasm_xmalloc(sizeof(key));
 
 	k->name_k = yasm__xstrdup(d->name);
-	k->len_k = strlen(d->name);
+	k->len_k = (ub4)strlen(d->name);
 	k->next_k = keys;
 	keys = k;
 	nkeys++;
