@@ -1,4 +1,4 @@
-/* $Id: util.h 1591 2006-07-13 03:25:38Z peter $
+/* $Id: util.h 1786 2007-02-24 04:16:05Z peter $
  * YASM utility functions.
  *
  * Includes standard headers and defines prototypes for replacement functions
@@ -34,7 +34,7 @@
 #include <config.h>
 #endif
 
-#ifdef HAVE_GNU_C_LIBRARY
+#if defined(HAVE_GNU_C_LIBRARY) || defined(__MINGW32__) || defined(__DJGPP__)
 
 /* Work around glibc's non-defining of certain things when using gcc -ansi */
 # ifdef __STRICT_ANSI__
@@ -151,6 +151,13 @@
 	d = BC_COUNT(d, 4);			\
     } while (0)
 
+/** Determine if a value is exactly a power of 2.  Zero is treated as a power
+ * of two.
+ * \param x	value
+ * \return Nonzero if x is a power of 2.
+ */
+#define is_exp2(x)  ((x & (x - 1)) == 0)
+
 #ifndef NELEMS
 /** Get the number of elements in an array.
  * \internal
